@@ -1,6 +1,6 @@
 #!/bin/sh
 
-# $Id: build.sh,v 1.1.1.1 2003/03/03 07:05:44 kvisco Exp $
+# $Id: build.sh,v 1.4 2005/05/17 14:00:25 wguttmn Exp $
 
 if [ -z "$JAVA_HOME" ] ; then
   JAVA=`which java`
@@ -14,8 +14,11 @@ fi
 
 JAVA=$JAVA_HOME/bin/java
 
-CLASSPATH=`echo lib/*.jar | tr ' ' ':'`
-CLASSPATH=lib/ant_1.3.jar:build/classes/:$CLASSPATH:$JAVA_HOME/lib/tools.jar
+CLASSPATH=lib/ant-1.6.jar:lib/ant-1.6-launcher.jar:lib/xerces-J_1.4.0.jar:build/classes/:$CLASSPATH:$JAVA_HOME/lib/tools.jar
 
+for i in `ls ./lib/*.jar`
+   do 
+       CLASSPATH=$CLASSPATH:$i
+   done
 
 $JAVA -classpath $CLASSPATH -Dant.home=lib org.apache.tools.ant.Main "$@" -buildfile src/build.xml
