@@ -46,7 +46,7 @@
  *     - Moved creation of access methods into an appropriate
  *       set of separate methods, for extensibility
  *
- * $Id: CollectionInfo.java,v 1.1.1.1 2003/03/03 07:07:41 kvisco Exp $
+ * $Id: CollectionInfo.java,v 1.5 2005/04/09 04:41:20 kvisco Exp $
  */
 
 package org.exolab.castor.builder;
@@ -55,12 +55,10 @@ import org.exolab.castor.builder.types.*;
 import org.exolab.castor.xml.JavaNaming;
 import org.exolab.javasource.*;
 
-import java.util.Vector;
-
 /**
  * A helper used for generating source that deals with Collections
  * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:07:41 $
+ * @version $Revision: 1.5 $ $Date: 2005/04/09 04:41:20 $
 **/
 public class CollectionInfo extends FieldInfo {
 
@@ -71,6 +69,8 @@ public class CollectionInfo extends FieldInfo {
     public static final String REFERENCE_SUFFIX_PROPERTY
         = "org.exolab.castor.builder.collections.reference.suffix";
 
+    public static final String DEFAULT_REFERENCE_SUFFIX = "AsReference";
+    
     protected XSList xsList      = null;
     private String contentName = null;
     private XSType contentType = null;
@@ -78,9 +78,22 @@ public class CollectionInfo extends FieldInfo {
     private FieldInfo content   = null;
     private String elementName;
 
+    /**
+     * A flag indicating that "extra" accessor methods
+     * should be created for returning and setting a
+     * reference to the underlying collection
+     */
+    private boolean _extraMethods = false;
+    
+    /**
+     * The reference suffix to use.
+     */
+    private String _referenceSuffix = DEFAULT_REFERENCE_SUFFIX;
+    
+    
     /**
      * Creates a new CollectionInfo
-     * @param contextType the content type of the collection, ie. the type
+     * @param contentType the content type of the collection, ie. the type
      * of objects that the collection will contain
      * @param name the name of the Collection
      * @param elementName the element name for each element in collection
@@ -90,6 +103,9 @@ public class CollectionInfo extends FieldInfo {
         super(new XSList(contentType), name);
         xsList = (XSList) getSchemaType();
         this.contentType = contentType;
+        if (elementName.charAt(0) == '_') {
+            elementName = elementName.substring(1);
+        }
         this.contentName = "v" + JavaNaming.toJavaClassName(elementName);
         this.elementName = elementName;
         content = new FieldInfo(contentType, contentName);
@@ -138,11 +154,12 @@ public class CollectionInfo extends FieldInfo {
         
         JSourceCode jsc = null;
 
-          //---------------------/
+          //----------------------/
          //- Create add methods -/
-        //---------------------/
+        //----------------------/
 
         String cName = JavaNaming.toJavaClassName(getElementName());
+
         method = new JMethod(null, "add"+cName);
         jClass.addMethod(method);
         method.addException(SGTypes.IndexOutOfBoundsException);
@@ -182,9 +199,7 @@ public class CollectionInfo extends FieldInfo {
 
         if (extraMethods()) {
             //-- Reference getter (non type-safe)
-            String suffix = SourceGenerator.getProperty(this.REFERENCE_SUFFIX_PROPERTY, 
-                "AsReference");
-            method = new JMethod(SGTypes.Vector, "get" + cName + suffix);
+            method = new JMethod(SGTypes.Vector, "get" + cName + _referenceSuffix);
             jClass.addMethod(method);
             createGetCollectionReferenceMethod(method);
         }
@@ -218,9 +233,7 @@ public class CollectionInfo extends FieldInfo {
             createSetCollectionMethod(method);
             
             //-- Reference setter (non type-safe
-            String suffix = SourceGenerator.getProperty(this.REFERENCE_SUFFIX_PROPERTY, 
-                "AsReference");
-            method = new JMethod(null, "set" + cName + suffix);
+            method = new JMethod(null, "set" + cName + _referenceSuffix);
             method.addParameter(vParam);
             jClass.addMethod(method);
             createSetCollectionReferenceMethod(method);
@@ -348,7 +361,11 @@ public class CollectionInfo extends FieldInfo {
             jsc.append(Integer.toString(maxSize));
             jsc.append(")) {");
             jsc.indent();
-            jsc.add("throw new IndexOutOfBoundsException();");
+            jsc.add("throw new IndexOutOfBoundsException(\"");
+            jsc.append(method.getName());
+            jsc.append(" has a maximum of ");
+            jsc.append(Integer.toString(maxSize));
+            jsc.append("\");"); 
             jsc.unindent();
             jsc.add("}");
         }
@@ -381,7 +398,11 @@ public class CollectionInfo extends FieldInfo {
             jsc.append(Integer.toString(maxSize));
             jsc.append(")) {");
             jsc.indent();
-            jsc.add("throw new IndexOutOfBoundsException();");
+            jsc.add("throw new IndexOutOfBoundsException(\"");
+            jsc.append(method.getName());
+            jsc.append(" has a maximum of ");
+            jsc.append(Integer.toString(maxSize));
+            jsc.append("\");"); 
             jsc.unindent();
             jsc.add("}");
         }
@@ -490,7 +511,12 @@ public class CollectionInfo extends FieldInfo {
         jsc.append(getName());
         jsc.append(".size())) {");
         jsc.indent();
-        jsc.add("throw new IndexOutOfBoundsException();");
+        jsc.add("throw new IndexOutOfBoundsException(\"");
+        jsc.append(method.getName());
+        jsc.append(": Index value '\"+index+\"' not in range [0..\"+");
+        jsc.append(getName());
+        jsc.append(".size()+ \"]");        
+        jsc.append("\");"); 
         jsc.unindent();
         jsc.add("}");
 
@@ -566,7 +592,12 @@ public class CollectionInfo extends FieldInfo {
         jsc.append(getName());
         jsc.append(".size())) {");
         jsc.indent();
-        jsc.add("throw new IndexOutOfBoundsException();");
+        jsc.add("throw new IndexOutOfBoundsException(\"");
+        jsc.append(method.getName());
+        jsc.append(": Index value '\"+index+\"' not in range [0..\"+");
+        jsc.append(getName());
+        jsc.append(".size()+ \"]");        
+        jsc.append("\");"); 
         jsc.unindent();
         jsc.add("}");
 
@@ -577,7 +608,11 @@ public class CollectionInfo extends FieldInfo {
             jsc.append(Integer.toString(maxSize));
             jsc.append(")) {");
             jsc.indent();
-            jsc.add("throw new IndexOutOfBoundsException();");
+            jsc.add("throw new IndexOutOfBoundsException(\"");
+            jsc.append(method.getName());
+            jsc.append(" has a maximum of ");
+            jsc.append(Integer.toString(maxSize));
+            jsc.append("\");"); 
             jsc.unindent();
             jsc.add("}");
         }
@@ -685,6 +720,38 @@ public class CollectionInfo extends FieldInfo {
 
     } //-- createSetCollectionReferenceMethod
 
+
+    /**
+     * Sets whether or not to create extra collection methods
+     * for accessing the actual collection
+     *
+     * @param extraMethods a boolean that when true indicates that
+     * extra collection accessor methods should be created. False
+     * by default.
+     * @see #setReferenceMethodSuffix
+     */
+    public void setCreateExtraMethods(boolean extraMethods) {
+        _extraMethods = extraMethods;
+    } //-- setCreateExtraMethods
+    
+    /**
+     * Sets the method suffix (ending) to use when creating
+     * the extra collection methods.
+     *
+     * @param suffix the method suffix to use when creating
+     * the extra collection methods. If null or emtpty the default
+     * value, as specified by DEFAULT_REFERENCE_SUFFIX will
+     * used.
+     * @see #setCreateExtraMethods
+     */
+    public void setReferenceMethodSuffix(String suffix) {
+        if ((suffix == null) || (suffix.length() == 0)) {
+            this._referenceSuffix = DEFAULT_REFERENCE_SUFFIX;
+        }
+        else _referenceSuffix = suffix;
+    } //-- setReferenceMethodSuffix
+    
+    
     /**
      * Creates the necessary source code for notifying
      * PropertyChangeListeners when the collection has
@@ -811,13 +878,27 @@ public class CollectionInfo extends FieldInfo {
     } //-- createRemoveAllMethod
 
     /**
-     * Returns true if extra collection methods should be generated
+     * Returns true if extra collection methods should be generated.
+     * The extra collection methods are methods which return an
+     * actual reference to the underlying collection as opposed to
+     * an enumeration, iterator, or copy.
      *
      * @return true if extra collection methods should be generated
      */
     protected final boolean extraMethods() {
-        return SourceGenerator.generateExtraCollectionMethods();
+        return _extraMethods;
     } //-- extraMethods
+    
+    /**
+     * Returns the suffix (ending) that should be used when
+     * creating the extra collection methods
+     *
+     * @return the suffix for the reference methods
+     */
+    protected final String getReferenceMethodSuffix() {
+        return _referenceSuffix;
+    } //-- getReferenceMethodSuffix
+    
 
 } //-- CollectionInfo
 
