@@ -45,13 +45,12 @@
  * - Frank Thelen, frank.thelen@poet.de
  *     - initial contributor
  *
- * $Id: CollectionInfoJ2.java,v 1.1.1.1 2003/03/03 07:07:42 kvisco Exp $
+ * $Id: CollectionInfoJ2.java,v 1.4 2005/05/03 08:12:13 bsnyder Exp $
  */
 
 package org.exolab.castor.builder;
 
 
-import org.exolab.castor.builder.*;
 import org.exolab.castor.builder.types.*;
 import org.exolab.castor.xml.JavaNaming;
 import org.exolab.javasource.*;
@@ -60,12 +59,12 @@ import org.exolab.javasource.*;
 /**
  * A helper used for generating source that deals with Java 2 Collections.
  * @author <a href="mailto:arkin@intalio.com">Assaf Arkin</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:07:42 $
+ * @version $Revision: 1.4 $ $Date: 2005/05/03 08:12:13 $
 **/
 public class CollectionInfoJ2 extends CollectionInfo {
 
     /**
-     * @param contextType the content type of the collection, ie. the type
+     * @param contentType the content type of the collection, ie. the type
      * of objects that the collection will contain
      * @param name the name of the Collection
      * @param elementName the element name for each element in collection
@@ -139,8 +138,7 @@ public class CollectionInfoJ2 extends CollectionInfo {
         
         if (extraMethods()) {
             //-- Reference getter (non type-safe)
-            String suffix = SourceGenerator.getProperty(this.REFERENCE_SUFFIX_PROPERTY, 
-                "AsReference");
+            String suffix = getReferenceMethodSuffix();
             method = new JMethod(SGTypes.ArrayList, "get" + cName + suffix);
             jClass.addMethod(method);
             createGetCollectionReferenceMethod(method);
@@ -178,8 +176,7 @@ public class CollectionInfoJ2 extends CollectionInfo {
             createSetCollectionMethod(method);
             
             //-- Reference setter (non type-safe)
-            String suffix = SourceGenerator.getProperty(this.REFERENCE_SUFFIX_PROPERTY, 
-                "AsReference");
+            String suffix = getReferenceMethodSuffix();
             method = new JMethod(null, "set" + cName + suffix);
             method.addParameter(vParam);
             jClass.addMethod(method);
@@ -525,7 +522,7 @@ public class CollectionInfoJ2 extends CollectionInfo {
         comment.appendComment(cName);
         comment.appendComment("' by copying the given ArrayList.");
         JDocDescriptor jDesc = comment.getParamDescriptor(paramName);
-        jDesc.setDescription("the Vector to copy.");
+        jDesc.setDescription("the ArrayList to copy.");
         
         //-- create copy code
         JSourceCode jsc = method.getSourceCode();
