@@ -38,9 +38,14 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
+ * Copyright 1999-2004 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: DescriptorSourceFactory.java,v 1.2 2003/03/03 17:09:16 blandin Exp $
+ * This file was originally developed by Keith Visco during the
+ * course of employment at Intalio Inc.
+ * All portions of this file developed by Keith Visco after Jan 19 2005 are
+ * Copyright (C) 2005 Keith Visco. All Rights Reserverd.
+ *
+ * $Id: DescriptorSourceFactory.java,v 1.9 2005/09/12 04:09:22 kvisco Exp $
  */
 
 package org.exolab.castor.builder;
@@ -52,9 +57,10 @@ import org.exolab.castor.builder.types.*;
 
 /**
  * A factory for creating the source code of descriptor classes
- * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.2 $ $Date: 2003/03/03 17:09:16 $
-**/
+ * 
+ * @author <a href="mailto:keith AT kvisco DOT com">Keith Visco</a>
+ * @version $Revision: 1.9 $ $Date: 2005/09/12 04:09:22 $
+ */
 public class DescriptorSourceFactory {
 
 	//-- org.exolab.castor.mapping
@@ -64,6 +70,12 @@ public class DescriptorSourceFactory {
 	private static JClass _FieldDescriptorClass =
 		new JClass("org.exolab.castor.mapping.FieldDescriptor");
 
+    /**
+     * GeneralizedFieldHandler
+     */
+    private static final JClass GENERALIZED_FIELD_HANDLER_CLASS =
+        new JClass("org.exolab.castor.mapping.GeneralizedFieldHandler");
+    
 	//-- org.exolab.castor.xml
 	private static JClass fdImplClass =
 		new JClass("org.exolab.castor.xml.util.XMLFieldDescriptorImpl");
@@ -83,13 +95,31 @@ public class DescriptorSourceFactory {
 	private static final String FIELD_VALIDATOR_NAME = "fieldValidator";
 
 
+    /**
+     * The BuilderConfiguration instance
+     */
+    private BuilderConfiguration _config = null;
+    
+    /**
+     * Creates a new DescriptorSourceFactory with the given configuration
+     *
+     * @param config the BuilderConfiguration instance
+     */
+    public DescriptorSourceFactory(BuilderConfiguration config) {
+        if (config == null) {
+            String err = "The argument 'config' must not be null.";
+            throw new IllegalArgumentException(err);
+        }
+        _config = config;
+    } //-- DescriptorSourceFactory
+    
 	/**
 	 * Creates the Source code of a MarshalInfo for a given XML Schema
 	 * element declaration
-	 * @param element the XML Schema element declaration
+	 * @param classInfo the XML Schema element declaration
 	 * @return the JClass representing the MarshalInfo source code
-	**/
-	public static JClass createSource(ClassInfo classInfo) {
+	 */
+	public JClass createSource(ClassInfo classInfo) {
 
 		JMethod method = null;
 		JSourceCode jsc = null;
@@ -101,7 +131,7 @@ public class DescriptorSourceFactory {
 		String variableName = "_" + className;
 
 		DescriptorJClass classDesc =
-			new DescriptorJClass(className + DESCRIPTOR_NAME, jClass);
+			new DescriptorJClass(_config, className + DESCRIPTOR_NAME, jClass);
 
 		//-- get handle to default constuctor
 
@@ -150,7 +180,7 @@ public class DescriptorSourceFactory {
 
 		//-- declare temp variables
 		jsc.add("org.exolab.castor.xml.util.XMLFieldDescriptorImpl  desc           = null;");
-		jsc.add("org.exolab.castor.xml.XMLFieldHandler              handler        = null;");
+		jsc.add("org.exolab.castor.mapping.FieldHandler             handler        = null;");
 		jsc.add("org.exolab.castor.xml.FieldValidator               fieldValidator = null;");
 
 		//-- handle  content
@@ -224,10 +254,14 @@ public class DescriptorSourceFactory {
 		jsc.add("desc = (org.exolab.castor.xml.util.XMLFieldDescriptorImpl) getFieldDescriptor(\"");
 		jsc.append(member.getNodeName());
 		jsc.append("\"");
+        jsc.append(", nsURI");        
 		if (member.getNodeType() == FieldInfo.ELEMENT_TYPE)
 			jsc.append(", org.exolab.castor.xml.NodeType.Element);");
 		else if (member.getNodeType() == FieldInfo.ATTRIBUTE_TYPE)
 			jsc.append(", org.exolab.castor.xml.NodeType.Attribute);");
+        else {
+            jsc.append("org.exolab.castor.xml.NodeType.Text);");
+        }
 		//--modify the validation code
 		validationCode(member, jsc);
 	}
@@ -236,9 +270,12 @@ public class DescriptorSourceFactory {
 	 * Create a specific descriptor for a given member (whether an attribute or
 	 * an element) represented by a given Class name
 	 */
-	private static void 
-        createDescriptor(DescriptorJClass classDesc,FieldInfo member,String localClassName,
-		                 String nsURI, JSourceCode jsc) 
+	private void createDescriptor
+	    (DescriptorJClass classDesc,
+	     FieldInfo member,
+	     String localClassName, 
+	     String nsURI, 
+	     JSourceCode jsc) 
     {
 
 		XSType xsType = member.getSchemaType();
@@ -265,7 +302,7 @@ public class DescriptorSourceFactory {
 		if ((nodeName != null) && (!isText)) {
 			//-- By default the node name parameter is a literal string
 			nodeNameParam = "\"" + nodeName + "\"";
-			if (SourceGenerator.classDescFieldNames()) {
+			if (_config.classDescFieldNames()) {
 				//-- The node name parameter is a reference to a public static final
 				nodeNameParam = member.getNodeName().toUpperCase();
 				//-- Expose node name as public static final (reused by XMLFieldDescriptorImpl)
@@ -323,9 +360,32 @@ public class DescriptorSourceFactory {
 		//-- handler access methods
 		if (member.getXMLFieldHandler() != null) {
 			String handler = member.getXMLFieldHandler();
-			jsc.add("desc.setHandler(new " + handler + "());");
-		} else
-		    createXMLFieldHandler(member, xsType, localClassName, jsc);
+			jsc.add("handler = new " + handler + "();");
+            jsc.add("//-- test for generalized field handler");
+            jsc.add("if (handler instanceof ");
+            jsc.append(GENERALIZED_FIELD_HANDLER_CLASS.getName());
+            jsc.append(")");
+            jsc.add("{");
+            jsc.indent();
+            jsc.add("//-- save reference to user-specified handler");
+            jsc.add(GENERALIZED_FIELD_HANDLER_CLASS.getName());
+            jsc.append(" gfh = (");
+            jsc.append(GENERALIZED_FIELD_HANDLER_CLASS.getName());
+            jsc.append(")handler;");
+            createXMLFieldHandler(member, xsType, localClassName, jsc, true);
+            jsc.add("gfh.setFieldHandler(handler);");
+            jsc.add("handler = gfh;");
+            jsc.unindent();
+            jsc.add("}");
+            
+		} 
+        else {
+		    createXMLFieldHandler(member, xsType, localClassName, jsc, false);
+            addSpecialHandlerLogic(member, xsType, jsc);
+        }
+        
+        jsc.add("desc.setHandler(handler);");
+        
 
 		//-- container
 		if (member.isContainer()) {
@@ -363,11 +423,17 @@ public class DescriptorSourceFactory {
 		if (member.isRequired()) {
 			jsc.add("desc.setRequired(true);");
 		}
+        
+        //-- nillable
+        if (member.isNillable()) {
+           jsc.add("desc.setNillable(true);");
+        }
+        
 		//-- if any it can match all the names
 		if (any)
 			jsc.add("desc.setMatches(\"*\");");
 		//-- mark as multi or single valued for elements
-		if (isElement) {
+		if (isElement || isAttribute) {
 			jsc.add("desc.setMultivalued(" + member.isMultivalued());
 			jsc.append(");");
 		}
@@ -380,11 +446,16 @@ public class DescriptorSourceFactory {
 
 	} //--CreateDescriptor
 
-	private static void createXMLFieldHandler(
-		FieldInfo member,
-		XSType xsType,
-		String localClassName,
-		JSourceCode jsc) {
+    /**
+     * Creates the XMLFieldHandler for the given FieldInfo
+     */
+	private void createXMLFieldHandler
+	    (FieldInfo member, 
+         XSType xsType, 
+         String localClassName, 
+         JSourceCode jsc,
+         boolean forGeneralizedHandler)
+	{
 
 		boolean any = false;
 		boolean isEnumerated = false;
@@ -399,7 +470,7 @@ public class DescriptorSourceFactory {
 	    if (xsType.getType() == XSType.CLASS)
 			isEnumerated = ((XSClass) xsType).isEnumerated();
 
-		jsc.add("handler = (new org.exolab.castor.xml.XMLFieldHandler() {");
+		jsc.add("handler = new org.exolab.castor.xml.XMLFieldHandler() {");
 		jsc.indent();
 
 		//-- read method
@@ -449,7 +520,7 @@ public class DescriptorSourceFactory {
 		jsc.append(localClassName);
 		jsc.append(") object;");
 		//-- check for null primitives
-		if (xsType.isPrimitive() && !SourceGenerator.usePrimitiveWrapper()) {
+		if (xsType.isPrimitive() && !_config.usePrimitiveWrapper()) {
 			if ((!member.isRequired())
 				&& (!xsType.isEnumerated())
 				&& (!member.isMultivalued())) {
@@ -473,7 +544,7 @@ public class DescriptorSourceFactory {
 		jsc.add("target.");
 		jsc.append(member.getWriteMethodName());
 		jsc.append("( ");
-		if (xsType.isPrimitive() && !SourceGenerator.usePrimitiveWrapper()) {
+		if (xsType.isPrimitive() && !_config.usePrimitiveWrapper()) {
 			jsc.append(xsType.createFromJavaObjectCode("value"));
 		} else if (any) {
 			jsc.append(" value ");
@@ -489,7 +560,7 @@ public class DescriptorSourceFactory {
 			//handle by DateFieldHandler
 			if ( (isAttribute | isContent)
 				&& xsType.isDateTime()
-				&& xsType.getType() != xsType.DATETIME_TYPE) {
+				&& xsType.getType() != XSType.DATETIME_TYPE) {
 				jsc.append(".parse");
 				jsc.append(JavaNaming.toJavaClassName(xsType.getName()));
 				jsc.append("((String) value))");
@@ -516,9 +587,16 @@ public class DescriptorSourceFactory {
 		jsc.add("return ");
         
         boolean isAbstract = false;
+        
         if (member.getDeclaringClassInfo() != null)
 		    isAbstract = member.getDeclaringClassInfo().isAbstract();
+        
+        if (!isAbstract && xsType.getJType() instanceof JClass) {
+        	isAbstract = ((JClass)xsType.getJType()).getModifiers().isAbstract();
+        }
+        
         if (any
+            || forGeneralizedHandler
 			|| isEnumerated
 			|| xsType.isPrimitive()
 			|| xsType.getJType().isArray()
@@ -532,52 +610,66 @@ public class DescriptorSourceFactory {
 		jsc.add("}");
 		//--end of new Instance method
 		jsc.unindent();
-		jsc.add("} );");
-		//--end of XMLFieldDescriptor
-
-		if (isEnumerated) {
-			jsc.add("desc.setHandler( new org.exolab.castor.xml.handlers.EnumFieldHandler(");
-			jsc.append(classType(xsType.getJType()));
-			jsc.append(", handler));");
-			jsc.add("desc.setImmutable(true);");
-		} else if (xsType.getType() == XSType.DATETIME_TYPE) {
-			jsc.add("desc.setHandler( new org.exolab.castor.xml.handlers.DateFieldHandler(");
-			jsc.append("handler));");
-			jsc.add("desc.setImmutable(true);");
-		} else if (xsType.getType() == XSType.DECIMAL_TYPE) {
-			jsc.add("desc.setHandler(handler);");
-			jsc.add("desc.setImmutable(true);");
-		}
-		//-- Handle special Collection Types such as NMTOKENS and IDREFS
-		else if (member.getSchemaType().getType() == XSType.COLLECTION) {
-			switch (xsType.getType()) {
-				case XSType.NMTOKEN_TYPE:
-				case XSType.NMTOKENS_TYPE:
-					//-- use CollectionFieldHandler
-					jsc.add("desc.setHandler( new org.exolab.castor.xml.handlers.CollectionFieldHandler(");
-					jsc.append(
-						"handler, new org.exolab.castor.xml.validators.NameValidator(org.exolab.castor.xml.validators.NameValidator.NMTOKEN)));");
-					break;
-				case XSType.QNAME_TYPE:
-					//-- use CollectionFieldHandler
-					jsc.add("desc.setHandler( new org.exolab.castor.xml.handlers.CollectionFieldHandler(");
-					jsc.append(
-						"handler, null));");
-					break;
-				case XSType.IDREF_TYPE :
-				case XSType.IDREFS_TYPE :
-					//-- uses special code in UnmarshalHandler
-					//-- see UnmarshalHandler#processIDREF
-					jsc.add("desc.setMultivalued(" + member.isMultivalued());
-					jsc.append(");");
-					/* do not break here */
-				default :
-					jsc.add("desc.setHandler(handler);");
-					break;
-			}
-		} else
-			jsc.add("desc.setHandler(handler);");
+		jsc.add("};");
+		//--end of XMLFieldHandler
 	}
+    
+    /**
+     * Adds additional logic or wrappers around the core handler
+     * for special types such as dates, enumerated types, 
+     * collections, etc.
+     * 
+     * @param member
+     * @param xsType
+     * @param jsc
+     */
+    private void addSpecialHandlerLogic
+       (FieldInfo member, 
+        XSType xsType, 
+        JSourceCode jsc)
+    {
+        if (xsType.isEnumerated()) {
+            jsc.add("handler = new org.exolab.castor.xml.handlers.EnumFieldHandler(");
+            jsc.append(classType(xsType.getJType()));
+            jsc.append(", handler);");
+            jsc.add("desc.setImmutable(true);");
+        } else if (xsType.getType() == XSType.DATETIME_TYPE) {
+            jsc.add("handler = new org.exolab.castor.xml.handlers.DateFieldHandler(");
+            jsc.append("handler);");
+            jsc.add("desc.setImmutable(true);");
+        } else if (xsType.getType() == XSType.DECIMAL_TYPE) {
+            jsc.add("desc.setImmutable(true);");
+        }
+        //-- Handle special Collection Types such as NMTOKENS and IDREFS
+        else if (member.getSchemaType().getType() == XSType.COLLECTION) {
+            switch (xsType.getType()) {
+                case XSType.NMTOKEN_TYPE:
+                case XSType.NMTOKENS_TYPE:
+                    //-- use CollectionFieldHandler
+                    jsc.add("handler = new org.exolab.castor.xml.handlers.CollectionFieldHandler(");
+                    jsc.append("handler, new org.exolab.castor.xml.validators.NameValidator(");
+                    jsc.append("org.exolab.castor.xml.validators.NameValidator.NMTOKEN));");
+                    break;
+                case XSType.QNAME_TYPE:
+                    //-- use CollectionFieldHandler
+                    jsc.add("handler = new org.exolab.castor.xml.handlers.CollectionFieldHandler(");
+                    jsc.append("handler, null);");
+                    break;
+                case XSType.IDREF_TYPE :
+                case XSType.IDREFS_TYPE :
+                    //-- uses special code in UnmarshalHandler
+                    //-- see UnmarshalHandler#processIDREF
+                    jsc.add("desc.setMultivalued(");
+                    jsc.append("" + member.isMultivalued());
+                    jsc.append(");");
+                    break;
+                default :
+                    break;
+            }
+        }
+        
+    } //-- addSpecialHandlerLogic
+    
 
 	/**
 	 * Creates the validation code for a given member.
