@@ -38,9 +38,14 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
+ * Copyright 1999-2004 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: FieldInfo.java,v 1.2 2003/03/08 07:35:42 kvisco Exp $
+ * This file was originally developed by Keith Visco during the course
+ * of employment at Intalio Inc.
+ * Portions of this file developed by Keith Visco after Jan 19 2005 are
+ * Copyright (C) 2005 Keith Visco. All Rights Reserverd.
+ * 
+ * $Id: FieldInfo.java,v 1.11 2005/03/05 13:41:42 mfuchs Exp $
  */
 
 package org.exolab.castor.builder;
@@ -49,20 +54,47 @@ import org.exolab.castor.builder.types.*;
 import org.exolab.castor.xml.JavaNaming;
 import org.exolab.javasource.*;
 
-import java.util.Vector;
 
 /**
  * A class for representing field members of a Class. FieldInfo objects
  * hold all the information required about a member in order
  * to be able to produce marshal/unmarshal and validation code.
- * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.2 $ $Date: 2003/03/08 07:35:42 $
-**/
+ * 
+ * @author <a href="mailto:keith AT kvisco DOT com">Keith Visco</a>
+ * @version $Revision: 1.11 $ $Date: 2005/03/05 13:41:42 $
+ */
 public class FieldInfo extends XMLInfo {
 
+    /**
+     * The Read / Getter method flag
+     */
+    public static final int READ_METHOD            = 1;
+    
+    /**
+     * The Write / Setter method flag
+     */
+    public static final int WRITE_METHOD           = 2;
+    
+    /**
+     * The Read and Write methods flags
+     */
+    public static final int READ_WRITE_METHODS     = 3;
+    
+    /**
+     * The method prefixes
+     */
+    private static final String METHOD_PREFIX_ADD    = "add";
+    private static final String METHOD_PREFIX_DELETE = "delete";
+    private static final String METHOD_PREFIX_GET    = "get";
+    private static final String METHOD_PREFIX_HAS    = "has";
+    private static final String METHOD_PREFIX_SET    = "set";
+    
+    
+    
+    
     /**
      * The Java name for Members described by this FieldInfo
-    **/
+     */
     private String name = null;
 
 
@@ -85,6 +117,18 @@ public class FieldInfo extends XMLInfo {
     **/
     private boolean _final     = false;
 
+    /**
+     * The methods flags, indicates which methods
+     * to create
+     */
+    private int _methods = READ_WRITE_METHODS;
+    
+    /**
+     * A reference to this field within the
+     * same class
+     */
+    private String _reference = null;
+    
     /**
      * A flag to indicate a static member
     **/
@@ -112,6 +156,12 @@ public class FieldInfo extends XMLInfo {
      *
      */
      private String _fieldHandler;
+     
+     /**
+      * A boolean to indicate that this field represents
+      * a "nillable" field 
+      */
+     private boolean _nillable = false;
 
     /**
      * The fully qualified name of the Validator (if any)
@@ -123,13 +173,12 @@ public class FieldInfo extends XMLInfo {
     /**
      * Creates a new FieldInfo with the given XML Schema type
      * and the given member name
-     * @param XSType the XML Schema type of this member
+     * @param type the XML Schema type of this member
      * @param name the name of the member
     **/
     public FieldInfo(XSType type, String name) {
         this.name    = name;
         setSchemaType(type);
-        _bound = SourceGenerator.boundPropertiesEnabled();
     } //-- FieldInfo
 
     //------------------/
@@ -139,7 +188,6 @@ public class FieldInfo extends XMLInfo {
     /**
      * Creates the JMembers for this FieldInfo, sometimes a "field"
      * requires more than one java field
-     * @return a JMember which has the appropriate type and name
      * for this FieldInfo
     **/
     public void createJavaField(JClass jClass) {
@@ -173,9 +221,6 @@ public class FieldInfo extends XMLInfo {
         //-- has_field
         if ((!type.isEnumerated()) && jType.isPrimitive()) {
             field = new JField(JType.Boolean, "_has" + name);
-            //indicate that a fixed value has been set for this primitive
-            if (_fixed != null)
-                field.setInitString("true");
             field.setComment("keeps track of state for field: " + name);
             jClass.addField(field);
         }
@@ -201,11 +246,73 @@ public class FieldInfo extends XMLInfo {
 
 
     /**
-     * Creates the access methods for this FieldInfo
+     * Creates the access methods for field associated with
+     * this FieldInfo. The access methods include getters,
+     * setters, and "has" and "delete" methods if necessary.
+     * 
      * @param jClass the JClass to add the methods to
-    **/
+     * @see #createGetterMethod
+     * @see #createSetterMethod
+     * @see #createHasAndDeleteMethods
+     */
     public void createAccessMethods(JClass jClass) {
 
+        if ((_methods & READ_METHOD) > 0) {
+            createGetterMethod(jClass);
+        }
+        if ((_methods & WRITE_METHOD) > 0) {
+            createSetterMethod(jClass);
+        }
+
+        if (isHasAndDeleteMethods()) {
+            createHasAndDeleteMethods(jClass);
+        }
+
+    } //-- createAccessMethods
+    
+    
+
+    /**
+     * Creates the Javadoc comments for the getter method
+     * associated with this FieldInfo.
+     *
+     * @param jDocComment the JDocComment to add the Javadoc
+     * comments to.
+    **/
+    public void createGetterComment(JDocComment jDocComment) {
+
+        String fieldName = this.name;
+        //-- remove '_' if necessary
+        if (fieldName.indexOf('_') == 0) {
+            fieldName = fieldName.substring(1);
+        }
+
+        String at_return = "the value of field '" + fieldName + "'.";
+
+        String mComment = "Returns " + at_return;
+        if ((_comment != null) && (_comment.length() > 0)) {
+            mComment += " The field '" + fieldName +
+                "' has the following description: ";
+
+            // XDoclet support - Add a couple newlines if it's a doclet tag
+            if ( _comment.startsWith( "@" ) )
+            {
+                mComment += "\n\n";
+            }
+
+            mComment += _comment;
+        }
+        jDocComment.setComment(mComment);
+        jDocComment.addDescriptor(JDocDescriptor.createReturnDesc(at_return));
+    } //-- createGetterComment
+    
+    /**
+     * Creates the getter methods for this FieldInfo
+     * 
+     * @param jClass the JClass to add the methods to
+     */
+    public void createGetterMethod(JClass jClass) {
+
         JMethod method    = null;
         JSourceCode jsc   = null;
 
@@ -214,12 +321,8 @@ public class FieldInfo extends XMLInfo {
         XSType xsType = getSchemaType();
         JType jType  = xsType.getJType();
 
-
-        boolean needs_has
-            = ((!xsType.isEnumerated()) && jType.isPrimitive());
-
         //-- create get method
-        method = new JMethod(jType, "get"+mname);
+        method = new JMethod(jType, METHOD_PREFIX_GET+mname);
         jClass.addMethod(method);
         createGetterComment(method.getJDocComment());
         jsc = method.getSourceCode();
@@ -227,8 +330,115 @@ public class FieldInfo extends XMLInfo {
         jsc.append(this.name);
         jsc.append(";");
 
+    } //-- createGetterMethod
+    
+    
+    /**
+     * Creates the "has" and "delete" methods for this field
+     * associated with this FieldInfo. These methods are typically 
+     * only needed for primitive types which cannot be assigned 
+     * a null value.
+     * 
+     * @param jClass the JClass to add the methods to
+     */
+    public void createHasAndDeleteMethods(JClass jClass) {
+
+        JMethod method    = null;
+        JSourceCode jsc   = null;
+
+        String mname = methodSuffix();
+
+        XSType xsType = getSchemaType();
+        JType jType  = xsType.getJType();
+
+        //-- create hasMethod
+        method = new JMethod(JType.Boolean, METHOD_PREFIX_HAS+mname);
+        jClass.addMethod(method);
+        jsc = method.getSourceCode();
+        jsc.add("return this._has");
+        jsc.append(getName());
+        jsc.append(";");
+
+        //-- create delete method
+        method = new JMethod(null, METHOD_PREFIX_DELETE+mname);
+        jClass.addMethod(method);
+        jsc = method.getSourceCode();
+        jsc.add("this._has");
+        jsc.append(getName());
+        jsc.append("= false;");
+        //-- bound properties
+        if (_bound) {
+            //notify listeners
+            jsc.add("notifyPropertyChangeListeners(\"");
+            jsc.append(getName());
+            jsc.append("\", ");
+            //-- 'this.' ensures this refers to the class member not the parameter
+            jsc.append(xsType.createToJavaObjectCode("this."+getName()));
+            jsc.append(", null");
+            jsc.append(");");
+        }
+
+    } //-- createHasAndDeleteMethods
+    
+
+    /**
+     * Creates the Javadoc comments for the setter method
+     * associated with this FieldInfo.
+     *
+     * @param jDocComment the JDocComment to add the Javadoc
+     * comments to.
+    **/
+    public void createSetterComment(JDocComment jDocComment) {
+
+        String fieldName = this.name;
+        //-- remove '_' if necessary
+        if (fieldName.indexOf('_') == 0) {
+            fieldName = fieldName.substring(1);
+        }
+
+        String at_param = "the value of field '" + fieldName + "'.";
+
+        String mComment = "Sets " + at_param;
+        if ((_comment != null) && (_comment.length() > 0)) {
+            mComment += " The field '" + fieldName +
+                "' has the following description: ";
+
+            // XDoclet support - Add a couple newlines if it's a doclet tag
+            if ( _comment.startsWith( "@" ) )
+            {
+                mComment += "\n\n";
+            }
+
+            mComment += _comment;
+        }
+            
+        jDocComment.setComment(mComment);
+
+        JDocDescriptor paramDesc = jDocComment.getParamDescriptor(fieldName);
+        if (paramDesc == null) {
+            paramDesc = JDocDescriptor.createParamDesc(fieldName, null);
+            jDocComment.addDescriptor(paramDesc);
+        }
+        paramDesc.setDescription(at_param);
+    } //-- createSetterComment
+    
+    /**
+     * Creates the setter (mutator) method(s) for this FieldInfo
+     * 
+     * @param jClass the JClass to add the methods to
+     */
+    public void createSetterMethod(JClass jClass) {
+
+        JMethod method    = null;
+        JSourceCode jsc   = null;
+
+        String mname = methodSuffix();
+
+        XSType xsType = getSchemaType();
+        JType jType  = xsType.getJType();
+
         //-- create set method
-        method = new JMethod(null, "set"+mname);
+        method = new JMethod(null, METHOD_PREFIX_SET+mname);
         jClass.addMethod(method);
 
         String paramName = this.name;
@@ -252,7 +462,7 @@ public class FieldInfo extends XMLInfo {
             jsc.add("java.lang.Object old");
             jsc.append(mname);
             jsc.append(" = ");
-			//-- 'this.' ensures this refers to the class member not the parameter
+            //-- 'this.' ensures this refers to the class member not the parameter
             jsc.append(xsType.createToJavaObjectCode("this."+getName()));
             jsc.append(";");
         }
@@ -263,32 +473,22 @@ public class FieldInfo extends XMLInfo {
         jsc.append(" = ");
         jsc.append(paramName);
         jsc.append(";");
+        
+        if (_reference != null) {
+            jsc.add("this.");
+            jsc.append(_reference);
+            jsc.append(" = ");
+            jsc.append(paramName);
+            jsc.append(";");
+        }
 
         //-- hasProperty
-        if (needs_has) {
+        if (isHasAndDeleteMethods()) {
             jsc.add("this._has");
             jsc.append(getName());
             jsc.append(" = true;");
-
-            //-- create hasMethod
-            method = new JMethod(JType.Boolean, "has"+mname);
-            jClass.addMethod(method);
-            jsc = method.getSourceCode();
-            jsc.add("return this._has");
-            jsc.append(getName());
-            jsc.append(";");
-
-			//-- if optional then create delete method
-			if (!isRequired()) {
-                method = new JMethod(null, "delete"+mname);
-                jClass.addMethod(method);
-                jsc = method.getSourceCode();
-                jsc.add("this._has");
-                jsc.append(getName());
-                jsc.append("= false;");
-            }
         }
-
+        
         //-- bound properties
         if (_bound) {
             //notify listeners
@@ -297,74 +497,14 @@ public class FieldInfo extends XMLInfo {
             jsc.append("\", old");
             jsc.append(mname);
             jsc.append(", ");
-			//-- 'this.' ensures this refers to the class member not the parameter
+            //-- 'this.' ensures this refers to the class member not the parameter
             jsc.append(xsType.createToJavaObjectCode("this."+getName()));
             jsc.append(");");
         }
+        
 
-
-    } //-- createAccessMethods
-
-    /**
-     * Creates the Javadoc comments for the getter method
-     * associated with this FieldInfo.
-     *
-     * @param jDocComment the JDocComment to add the Javadoc
-     * comments to.
-    **/
-    public void createGetterComment(JDocComment jDocComment) {
-
-        String fieldName = this.name;
-        //-- remove '_' if necessary
-        if (fieldName.indexOf('_') == 0) {
-            fieldName = fieldName.substring(1);
-        }
-
-        String at_return = "the value of field '" + fieldName + "'.";
-
-        String mComment = "Returns " + at_return;
-        if ((_comment != null) && (_comment.length() > 0)) {
-            mComment += " The field '" + fieldName +
-                "' has the following description: ";
-            mComment += _comment;
-        }
-        jDocComment.setComment(mComment);
-        jDocComment.addDescriptor(JDocDescriptor.createReturnDesc(at_return));
-    } //-- createGetterComment
-
-    /**
-     * Creates the Javadoc comments for the setter method
-     * associated with this FieldInfo.
-     *
-     * @param jDocComment the JDocComment to add the Javadoc
-     * comments to.
-    **/
-    public void createSetterComment(JDocComment jDocComment) {
-
-        String fieldName = this.name;
-        //-- remove '_' if necessary
-        if (fieldName.indexOf('_') == 0) {
-            fieldName = fieldName.substring(1);
-        }
-
-        String at_param = "the value of field '" + fieldName + "'.";
-
-        String mComment = "Sets " + at_param;
-        if ((_comment != null) && (_comment.length() > 0)) {
-            mComment += " The field '" + fieldName +
-                "' has the following description: ";
-            mComment += _comment;
-        }
-            
-        jDocComment.setComment(mComment);
-
-        JDocDescriptor paramDesc = jDocComment.getParamDescriptor(fieldName);
-        if (paramDesc == null) {
-            paramDesc = JDocDescriptor.createParamDesc(fieldName, null);
-            jDocComment.addDescriptor(paramDesc);
-        }
-        paramDesc.setDescription(at_param);
-    } //-- createSetterComment
+    } //-- createSetterMethod
+    
 
     /**
      * Returns the default value for this FieldInfo
@@ -391,7 +531,7 @@ public class FieldInfo extends XMLInfo {
      * @return the name of the delete method for this FieldInfo
     **/
     public String getDeleteMethodName() {
-        return "delete" + methodSuffix();
+        return METHOD_PREFIX_DELETE + methodSuffix();
     } //-- getDeleteMethodName
 
     /**
@@ -399,7 +539,7 @@ public class FieldInfo extends XMLInfo {
      * @return the name of the has method for this FieldInfo
     **/
     public String getHasMethodName() {
-        return "has" + methodSuffix();
+        return METHOD_PREFIX_HAS + methodSuffix();
     } //-- getHasMethodName
 
     /**
@@ -407,7 +547,7 @@ public class FieldInfo extends XMLInfo {
      * @return the name of the read method for this FieldInfo
     **/
     public String getReadMethodName() {
-        return "get" + methodSuffix();
+        return METHOD_PREFIX_GET + methodSuffix();
     } //-- getReadMethodName
 
    /**
@@ -425,9 +565,9 @@ public class FieldInfo extends XMLInfo {
     **/
     public String getWriteMethodName() {
         if (isMultivalued())
-            return "add" + methodSuffix();
+            return METHOD_PREFIX_ADD + methodSuffix();
         else
-            return "set" + methodSuffix();
+            return METHOD_PREFIX_SET + methodSuffix();
     } //-- getWriteMethodName
 
    /**
@@ -459,7 +599,7 @@ public class FieldInfo extends XMLInfo {
                     jsc.add("try {");
                     jsc.indent();
                 }
-                buffer.append("set");
+                buffer.append(METHOD_PREFIX_SET);
                 buffer.append(methodSuffix());
                 buffer.append('(');
                 buffer.append(value);
@@ -487,6 +627,16 @@ public class FieldInfo extends XMLInfo {
         return _comment;
     } //-- getComment
 
+    /**
+     * Returns the methods flag that indicates which
+     * methods will be created 
+     * 
+     * @return the methods flag
+     */
+    public int getMethods() {
+        return _methods;
+    } //-- getMethods
+    
     /**
      * Returns the name of this FieldInfo
      * @return the name of this FieldInfo
@@ -515,6 +665,30 @@ public class FieldInfo extends XMLInfo {
         return _isContainer;
     } //-- isContainer
 
+    /**
+     * Returns true if the "has" and "delete" methods are
+     * needed for the field associated with this FieldInfo.
+     * 
+     * @return true if the has and delete methods are needed.
+     */
+    public boolean isHasAndDeleteMethods() {
+        XSType xsType = getSchemaType();
+        JType jType  = xsType.getJType();
+        return ((!xsType.isEnumerated()) && jType.isPrimitive());
+    } //-- isHasMethod
+    
+    /**
+     * Returns true if this field represents a nillable field.
+     * A nillable field is a field that can have null content
+     * (see XML Schema 1.0 definition of nillable).
+     * 
+     * @return true if nillable, otherwise false.
+     * @see #setNillable(boolean)
+     */
+     public boolean isNillable() {
+     	return _nillable;
+     } //-- isNillable
+     
     /**
      * Returns true if this FieldInfo is a transient member. Transient
      * members are members which should be ignored by the
@@ -598,6 +772,37 @@ public class FieldInfo extends XMLInfo {
         this._fixed = fixedValue;
     } //-- setFixedValue
 
+    /**
+     * Sets which methods to create
+     * 
+     * READ_METHOD, WRITE_METHOD, READ_WRITE_METHODS
+     * 
+     * @param methods
+     */
+    public void setMethods(int methods) {
+        _methods = methods;
+    } //-- setMethods
+    
+    /**
+     * Sets whether or not this field can be nillable.
+     * 
+     * @param nillable a boolean that when true means the
+     * field may be nil.
+     * @see #isNillable()
+     */
+    public void setNillable(boolean nillable) {
+    	_nillable = nillable;
+    } //-- setNillable
+    
+    /**
+     * Sets the name of the field within the same class
+     * that is a reference to this field. 
+     * 
+     * @param fieldName
+     */
+    public void setReference(String fieldName) {
+        _reference = fieldName;
+    } //-- setReference
 
     /**
      * Sets the "static" status of this FieldInfo. Static
