@@ -38,58 +38,88 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Copyright 1999-2003 (C) Intalio Inc. All Rights Reserved.
+ * Copyright 1999-2004 (C) Intalio Inc. All Rights Reserved.
  *
- * $Id: MemberFactory.java,v 1.2 2003/03/03 09:57:14 kvisco Exp $
+ * This file was originally developed by Keith Visco during the course
+ * of employment at Intalio Inc.
+ * Portions of this file developed by Keith Visco after Jan 19 2005 are
+ * Copyright (C) 2005 Keith Visco. All Rights Reserverd.
+ * 
+ * $Id: MemberFactory.java,v 1.15 2005/03/05 13:41:42 mfuchs Exp $
  */
 
 package org.exolab.castor.builder;
 
 import org.exolab.castor.builder.binding.XMLBindingComponent;
 import org.exolab.castor.builder.types.*;
-import org.exolab.castor.xml.JavaNaming;
 import org.exolab.castor.xml.schema.*;
 import org.exolab.castor.xml.schema.simpletypes.ListType;
-import org.exolab.castor.util.Configuration;
-
 import org.exolab.javasource.JClass;
-import org.exolab.javasource.JSourceCode;
 import org.exolab.javasource.JType;
 
 import java.util.Enumeration;
-import java.util.Hashtable;
 
 /**
- * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.2 $ $Date: 2003/03/03 09:57:14 $
-**/
+ * The "Factory" responsible for creating fields for
+ * the given schema components
+ *
+ * @author <a href="mailto:keith AT kvisco DOT com">Keith Visco</a>
+ * @version $Revision: 1.15 $ $Date: 2005/03/05 13:41:42 $
+ */
 public class MemberFactory {
 
 
     /**
      * The FieldInfo factory.
-    **/
+     */
     private FieldInfoFactory _infoFactory = null;
 
+    /**
+     * The BuilderConfiguration instance, for callbacks
+     * to obtain certain configured properties
+     */
+    private BuilderConfiguration _config = null;
+    
     /**
      * Creates a new MemberFactory with default type factory.
-    **/
-    public MemberFactory() {
-        this(new FieldInfoFactory());
+     *
+     * @param config the BuilderConfiguration
+     */
+    public MemberFactory(BuilderConfiguration config) {
+        this(config, new FieldInfoFactory());
     } //-- MemberFactory
 
 
     /**
      * Creates a new MemberFactory using the given FieldInfo factory.
+     *
+     * @param config the BuilderConfiguration
      * @param infoFactory the FieldInfoFactory to use
-    **/
-    public MemberFactory(FieldInfoFactory infoFactory)
+     */
+    public MemberFactory(BuilderConfiguration config, FieldInfoFactory infoFactory)
     {
         super();
+        if (config == null) {
+            String err = "The 'BuilderConfiguration' argument must not be null.";
+            throw new IllegalArgumentException(err);
+        }
+        _config = config;
+        
         if (infoFactory == null)
             _infoFactory = new FieldInfoFactory();
         else
             _infoFactory = infoFactory;
+            
+        if (_config.generateExtraCollectionMethods()) {
+            _infoFactory.setCreateExtraMethods(true);
+        }
+        String suffix = _config.getProperty(CollectionInfo.REFERENCE_SUFFIX_PROPERTY, null);
+        _infoFactory.setReferenceMethodSuffix(suffix);
+        
+        if (_config.boundPropertiesEnabled()) {
+            _infoFactory.setBoundProperties(true);
+        }
+
     } //-- MemberFactory
 
 
@@ -142,7 +172,27 @@ public class MemberFactory {
         return result;
     } //-- createFieldInfoForAny()
 
+    /**
+     * Creates a FieldInfo to hold the value of a choice.
+     * 
+     * @return the new FieldInfo
+     */
+    public FieldInfo createFieldInfoForChoiceValue() {
 
+        String fieldName = "_choiceValue";
+        XSType xsType = new XSClass(SGTypes.Object, "any");
+        FieldInfo fInfo = null;
+        fInfo = _infoFactory.createFieldInfo(xsType,fieldName);
+        fInfo.setNodeType(XMLInfo.ELEMENT_TYPE);
+        fInfo.setComment("Internal choice value storage");
+        fInfo.setRequired(false);
+        fInfo.setTransient(true);
+        fInfo.setNodeName("##any");
+        fInfo.setMethods(FieldInfo.READ_METHOD);
+        return fInfo;
+
+    } //-- createFieldInfoForChoiceValue
+    
     /**
      * Creates a FieldInfo for content.
      * @param xsType the type of content
@@ -171,6 +221,7 @@ public class MemberFactory {
         return fInfo;
 
     } //-- createFieldInfoForContent
+    
 
 
 
@@ -191,6 +242,7 @@ public class MemberFactory {
             memberName = "_"+memberName;
 
         XMLType xmlType = component.getXMLType();
+        
         ClassInfo classInfo = resolver.resolve(component);
 
         XSType   xsType = null;
@@ -201,24 +253,43 @@ public class MemberFactory {
         if (xmlType != null) {
             if (xmlType.isSimpleType()) {
                 SimpleType simpleType = (SimpleType)xmlType;
+                
+                SimpleType baseType = null;
+                String derivationMethod = simpleType.getDerivationMethod();
+                if (derivationMethod != null) {
+                    if (SchemaNames.RESTRICTION.equals(derivationMethod)) {
+                        baseType = (SimpleType)simpleType.getBaseType();
+                    }
+                }
 
                 //-- handle special case for enumerated types
                 if (simpleType.hasFacet(Facet.ENUMERATION)) {
                     //-- LOok FoR CLasSiNfO iF ReSoLvR is NoT NuLL
                     enumeration = true;
                     if (resolver != null) {
-                        classInfo = resolver.resolve(simpleType);
+                        classInfo = resolver.resolve(xmlType);
                     }
-                    if (classInfo != null)
+                    if (classInfo != null) {
                         xsType = classInfo.getSchemaType();
+                    }
                 }
-                else if (simpleType instanceof ListType) {
-                    if (!simpleType.isBuiltInType())
-                        simpleTypeCollection = true;
+                else if ((simpleType instanceof ListType) || 
+                         (baseType instanceof ListType)) 
+                {
+                    if (baseType != null) {
+                    	if (!baseType.isBuiltInType())
+                    		simpleTypeCollection = true;
+                        
+                    }
+                    else {
+                    	if (!simpleType.isBuiltInType())
+                    		simpleTypeCollection = true;
+                    }
                 }
                 
-                if (xsType == null)
+                if (xsType == null) {
                     xsType = component.getJavaType();
+                }
             }//--simpleType
             else if (xmlType.isAnyType()) {
                 //-- Just treat as java.lang.Object.
@@ -230,22 +301,41 @@ public class MemberFactory {
             else if (xmlType.isComplexType() && (xmlType.getName() != null)) {
                 //--if we use the type method then no class is output for
                 //--the element we are processing
-                if (SourceGenerator.mappingSchemaType2Java()) {
-                    XMLBindingComponent temp = new XMLBindingComponent();
+                if (_config.mappingSchemaType2Java()) {
+                    XMLBindingComponent temp = new XMLBindingComponent(_config);
                     temp.setBinding(component.getBinding());
                     temp.setView(xmlType);
-                    String className = temp.getQualifiedName();
-                    if (className != null) {
-                    	xsType = new XSClass(new JClass(className));
-                        className = null;
+                    ClassInfo typeInfo = resolver.resolve(xmlType);
+                    if (typeInfo != null) {
+                        xsType = typeInfo.getSchemaType();
+                    }
+                    else {
+                        String className = temp.getQualifiedName();
+                        if (className != null) {
+                        	xsType = new XSClass(new JClass(className));
+                            className = null;
+                        }
                     }
                 }
             }//--complexType
         }
+        else {
+            
+            //-- patch for bug 1471 (No XMLType specified)
+            //-- treat unspecified type as anyType
+            switch (component.getAnnotated().getStructureType()) {
+                case Structure.ATTRIBUTE:
+                case Structure.ELEMENT:
+                    xsType = new XSClass(SGTypes.Object);
+                    break;
+                default:
+                    // probably a model-group
+                    break;
+            }
+        }
 
         //--is the XSType found?
         if (xsType == null) {
-            
             String className = component.getQualifiedName();
             xsType = new XSClass(new JClass(className));
             className = null;
@@ -286,7 +376,6 @@ public class MemberFactory {
             }
         }
 
-        Form form = null;
         boolean createForElement = false;
         boolean createForAttribute = false;
         //--initialize the field
@@ -296,7 +385,6 @@ public class MemberFactory {
             case Structure.ELEMENT:
                  fieldInfo.setNodeType(XMLInfo.ELEMENT_TYPE);
                  ElementDecl element = (ElementDecl)component.getAnnotated();
-                 form = element.getForm();
                  createForElement = true;
                  break;
             case Structure.ATTRIBUTE:
@@ -311,24 +399,13 @@ public class MemberFactory {
         }
 
         //-- handle namespace URI / prefix
-        Schema schema = component.getSchema();
-        if (schema != null) {
-            if (form == null) {
-                if (createForAttribute)
-                    form = schema.getAttributeFormDefault();
-                else if (createForElement)
-                    form = schema.getElementFormDefault();
-            }
-            if ((form != null) && form.isQualified()) {
-                String nsURI = component.getTargetNamespace();
-                if ((nsURI != null) && (nsURI.length() > 0)) {
-                    fieldInfo.setNamespaceURI(nsURI);
-                    /**
-                     * @todo set the prefix used in the XML Schema in 
-                     * order to use it inside the Marshalling Framework
-                     */
-                }
-            }
+        String nsURI = component.getTargetNamespace();
+        if ((nsURI != null) && (nsURI.length() > 0)) {
+            fieldInfo.setNamespaceURI(nsURI);
+            /**
+             * @todo set the prefix used in the XML Schema in 
+             * order to use it inside the Marshalling Framework
+             */
         }
 
         //--handle default value
@@ -400,6 +477,12 @@ public class MemberFactory {
             else
                 fieldInfo.setDefaultValue(value);
         }
+        
+        //-- handle nillable values
+        if (component.isNillable())
+            fieldInfo.setNillable(true);
+        
+        
 
         //-- add annotated comments
         String comment = createComment(component.getAnnotated());
@@ -424,10 +507,10 @@ public class MemberFactory {
     private String createComment(Annotated annotated) {
 
         //-- process annotations
-        Enumeration enum = annotated.getAnnotations();
-        if (enum.hasMoreElements()) {
+        Enumeration enumeration = annotated.getAnnotations();
+        if (enumeration.hasMoreElements()) {
             //-- just use first annotation
-            return createComment((Annotation) enum.nextElement());
+            return createComment((Annotation) enumeration.nextElement());
         }
         else {
             //-- there were no annotations...try possible references
@@ -458,10 +541,10 @@ public class MemberFactory {
     **/
     private String createComment(Annotation annotation) {
         if (annotation == null) return null;
-        Enumeration enum = annotation.getDocumentation();
-        if (enum.hasMoreElements()) {
+        Enumeration enumeration = annotation.getDocumentation();
+        if (enumeration.hasMoreElements()) {
             //-- just use first <info>
-            Documentation documentation = (Documentation) enum.nextElement();
+            Documentation documentation = (Documentation) enumeration.nextElement();
             return normalize(documentation.getContent());
         }
         return null;
