@@ -38,9 +38,14 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Copyright 1999-2003 (C) Intalio Inc. All Rights Reserved.
+ * Copyright 1999-2004 (C) Intalio Inc. All Rights Reserved.
  *
- * $Id: SourceFactory.java,v 1.5 2003/06/03 18:50:36 kvisco Exp $
+ * This file was originally developed by Keith Visco during the course
+ * of employment at Intalio Inc.
+ * Portions of this file developed by Keith Visco after Jan 19 2005 are
+ * Copyright (C) 2005 Keith Visco. All Rights Reserverd.
+ *  
+ * $Id: SourceFactory.java,v 1.21 2005/06/23 04:14:49 kvisco Exp $
  */
 
 package org.exolab.castor.builder;
@@ -83,19 +88,21 @@ import org.exolab.javasource.JSourceCode;
 import org.exolab.javasource.JType;
 
 import java.util.Enumeration;
-import java.util.Vector;
 
 /**
  * This class creates the Java Source classes for Schema
  * components
- * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @author <a href="mailto:kvisco@intalio.com">Arnaud Blandin</a>
- * @version $Revision: 1.5 $ $Date: 2003/06/03 18:50:36 $
-**/
-public class SourceFactory  {
+ *
+ * @author <a href="mailto:keith AT kvisco DOT com">Keith Visco</a>
+ * @author <a href="mailto:blandin@intalio.com">Arnaud Blandin</a>
+ * @version $Revision: 1.21 $ $Date: 2005/06/23 04:14:49 $
+ */
+public class SourceFactory {
     
     
-
+    private static final String ENUM_ACCESS_INTERFACE =
+        "org.exolab.castor.types.EnumeratedTypeAccess";
+        
     private static final short BASE_TYPE_ENUMERATION   = 0;
     private static final short OBJECT_TYPE_ENUMERATION = 1;
 
@@ -140,28 +147,47 @@ public class SourceFactory  {
     */
     private boolean _sax1 = false;
 
-
+    /**
+     * The BuilderConfiguration instance
+     */
+    private BuilderConfiguration _config = null;
+    
+    /**
+     * The TypeConversion instance to use for mapping
+     * SimpleTypes into XSTypes
+     */
+    private TypeConversion _typeConversion = null;
+    
     /**
      * Creates a new SourceFactory using the default FieldInfo factory.
-    **/
-    public SourceFactory() {
-        this(null);
+     *
+     * @param config the BuilderConfiguration instance (must not be null).
+     */
+    public SourceFactory(BuilderConfiguration config) {
+        this(config, null);
     } //-- SourceFactory
 
 
     /**
      * Creates a new SourceFactory with the given FieldInfoFactory
+     *
+     * @param config the BuilderConfiguration instance (must not be null).
      * @param infoFactory the FieldInfoFactory to use
-    */
-    public SourceFactory(FieldInfoFactory infoFactory) {
+     */
+    public SourceFactory(BuilderConfiguration config, FieldInfoFactory infoFactory) {
         super();
+        if (config == null) {
+            String error = "The argument 'config' must not be null.";
+            throw new IllegalArgumentException(error);
+        }
+        _config = config;
         if (infoFactory == null)
             this.infoFactory = new FieldInfoFactory();
         else
             this.infoFactory = infoFactory;
-
-        this.memberFactory = new MemberFactory(infoFactory);
-
+        
+        this.memberFactory = new MemberFactory(config, infoFactory);
+        _typeConversion = new TypeConversion(_config);
     } //-- SourceFactory
 
    /**
@@ -178,13 +204,36 @@ public class SourceFactory  {
     } //-- setCreateMarshalMethpds
 
     /**
-     * Sets whether or not to create the XML marshalling framework specific
-     * methods (marshall, unmarshall, validate) in the generated classes.
-	 * By default, these methods are generated.
+     * Sets whether or not to create extra collection methods
+     * for accessing the actual collection
+     *
+     * @param extraMethods a boolean that when true indicates that
+     * extra collection accessor methods should be created. False
+     * by default.
+     * @see org.exolab.castor.builder.SourceFactory#setReferenceMethodSuffix
+     */
+    public void setCreateExtraMethods(boolean extraMethods) {
+        infoFactory.setCreateExtraMethods(extraMethods);
+    } //-- setCreateExtraMethods
+    
+    /**
+     * Sets the method suffix (ending) to use when creating
+     * the extra collection methods.
+     *
+     * @param suffix the method suffix to use when creating
+     * the extra collection methods. If null or emtpty the default
+     * value, as specified in CollectionInfo will be used.
+     * @see org.exolab.castor.builder.SourceFactory#setCreateExtraMethods
+     */
+    public void setReferenceMethodSuffix(String suffix) {
+        infoFactory.setReferenceMethodSuffix(suffix);
+    } //-- setReferenceMethodSuffix
+    
+    /**
+     * Sets whether or not to implement CastorTestable
 	 *
      * @param testable a boolean, when true indicates
-     * to generate testing framework methods
-     *
+     * to implement CastorTestable
      */
     public void setTestable(boolean testable) {
            _testable = testable;
@@ -218,6 +267,11 @@ public class SourceFactory  {
             throw new IllegalStateException("SGStateInfo may not be null.");
         }
         
+        
+        //-- check for previous JClass bindings
+        JClass[] classes = sgState.getSourceCode(component.getAnnotated());
+        if (classes != null) return classes;
+        
         _binding = component.getBinding();
         
         if (sgState.verbose()) {
@@ -229,7 +283,6 @@ public class SourceFactory  {
         }
         
         FactoryState state = null;
-        JClass[] classes = null;
 
         //0-- set the packageName
         String packageName = component.getJavaPackage();
@@ -245,7 +298,6 @@ public class SourceFactory  {
             className = JavaNaming.toJavaClassName(className);
             className = resolveClassName(className, packageName);
         }
-
         //2-- check if we have to create an Item class
         boolean createGroupItem = component.createGroupItem();
         if (createGroupItem) {
@@ -265,6 +317,7 @@ public class SourceFactory  {
             state.setParent(sgState.getCurrentFactoryState());
             sgState.setCurrentFactoryState(state);
         }
+        
         //--Prevent endless loop
         if (state.processed(component.getAnnotated())) {
             return new JClass[0];
@@ -292,8 +345,9 @@ public class SourceFactory  {
 
         //5--processing the type
         XMLType type = component.getXMLType();
-        boolean createForGroup = false;
+        boolean createForSingleGroup = false;
         boolean creatingForAnElement = false;
+        
         if (type != null) {
 
             //5a--the type is a complexType
@@ -306,9 +360,11 @@ public class SourceFactory  {
                      //--move the view and keep the structure
                      Annotated saved = component.getAnnotated();
                      String previousPackage = component.getJavaPackage();
-                     XMLBindingComponent baseComponent = new XMLBindingComponent();
+                     XMLBindingComponent baseComponent = new XMLBindingComponent(_config);
                      baseComponent.setBinding(component.getBinding());
                      baseComponent.setView(complexType);
+                     //-- call createSourceCode to pre-process the complexType
+                     createSourceCode(baseComponent, sgState);
                      String baseClassName = null;
                      String basePackage = baseComponent.getJavaPackage();
                      //--if the base class is not in the same package
@@ -356,6 +412,7 @@ public class SourceFactory  {
                 SimpleType simpleType = (SimpleType)type;
                 //-- handle our special case for enumerated types
                 if (simpleType.hasFacet(Facet.ENUMERATION)) {
+                    
 				    //-- Don't create source code for simple types that
 				    //-- don't belong in the elements target namespace
 				    String tns = simpleType.getSchema().getTargetNamespace();
@@ -364,14 +421,23 @@ public class SourceFactory  {
 				        create = (component.getTargetNamespace() == null);
 				    else
                         create = tns.equals(component.getTargetNamespace());
+                        
+                        
                     if (create) {
-					    JClass tmpClass = createSourceCode(simpleType, sgState);
-					    classInfo.setSchemaType(new XSClass(tmpClass));
+                        ClassInfo tmpInfo = sgState.resolve(simpleType);
+                        JClass tmpClass = null;
+                        if (tmpInfo != null) {
+                            tmpClass = tmpInfo.getJClass();
+                        }
+                        else {
+					        tmpClass = createSourceCode(simpleType, sgState);
+					    }
+					    classInfo.setSchemaType(new XSClass(tmpClass));					        
 				    }
                 } else {
                     //////////////////////////////////////////////////////////
                     //NOTE: generate sources if the flag for generating sources
-                    //from imported schemas in on
+                    //from imported schemas is on
                     //////////////////////////////////////////////////////////
                     return new JClass[0];
                 }
@@ -387,8 +453,8 @@ public class SourceFactory  {
         else {
             //--MODEL GROUP OR GROUP
             try{
-                createForGroup = true;
                 Group group = (Group)component.getAnnotated();
+                createForSingleGroup = (group.getMaxOccurs() == 1);                
                 processContentModel(group, state);
                 component.setView(group);
                 //-- Check Group Type
@@ -420,8 +486,8 @@ public class SourceFactory  {
             fname  = JavaNaming.toJavaMemberName(fname, false);
 
             FieldInfo fInfo = null;
-            if (createForGroup) {
-                //A choice Item can occur only once in the nested group
+            if (createForSingleGroup) {
+                //By default a nested group Item can occur only once
                 fInfo = infoFactory.createFieldInfo(new XSClass(jClass),
                                                                 fname);
             } else {
@@ -450,7 +516,7 @@ public class SourceFactory  {
                     else if (complexType.getContentType().getType() == ContentType.SIMPLE) {
                         SimpleContent simpleContent = (SimpleContent)complexType.getContentType();
                         SimpleType temp = simpleContent.getSimpleType();
-                        XSType xsType = TypeConversion.convertType(temp, packageName);
+                        XSType xsType = _typeConversion.convertType(temp, packageName);
                         FieldInfo fieldInfo = memberFactory.createFieldInfoForContent(xsType);
 		                handleField(fieldInfo,state);
 		                temp = null;
@@ -527,7 +593,7 @@ public class SourceFactory  {
                //-- #marshal()
                createMarshalMethods(jClass);
                //-- #unmarshal()
-               createUnmarshalMethods(jClass);
+               createUnmarshalMethods(jClass, sgState);
            }
 		}
 
@@ -536,7 +602,7 @@ public class SourceFactory  {
             createEqualsMethod(jClass);
         //implements CastorTestable?
         if (_testable)
-            SourceFactory.createTestableMethods(jClass);
+            createTestableMethods(jClass, state);
 
 		//-- This boolean is set to create bound properties
         //-- even if the user has set the SUPER CLASS property
@@ -552,11 +618,16 @@ public class SourceFactory  {
 		    if (component.hasBoundProperties()) 
 		        createPropertyChangeMethods(jClass);
 		}
-
+		
         sgState.bindReference(jClass, classInfo);
         sgState.bindReference(component.getAnnotated(), classInfo);
 
         classes[0] = jClass;
+        
+        //-- Save source code bindings to prevent duplicate code
+        //-- generation
+        sgState.bindSourceCode(component.getAnnotated(), classes);
+        
         return classes;
     }
 
@@ -578,7 +649,7 @@ public class SourceFactory  {
         if (sgState == null) {
             throw new IllegalArgumentException("SGStateInfo cannot be null.");
         }
-
+        
         //-- Unions are currently processed as the built-in
         //-- basetype for the member types of the Union, so
         //-- do nothing for now...however we can warn
@@ -591,6 +662,9 @@ public class SourceFactory  {
             }
             return null;
         }
+        
+        ClassInfo cInfo = sgState.resolve(simpleType);
+        if (cInfo != null) return cInfo.getJClass();
 
         boolean enumeration = false;
 
@@ -620,7 +694,9 @@ public class SourceFactory  {
 
         String className   = JavaNaming.toJavaClassName(typeName);
         
-        XMLBindingComponent comp = new XMLBindingComponent();
+        //--XMLBindingComponent is only used to retrieve the java package
+        //-- we need to optimize it by enabling the binding of simpleTypes.
+        XMLBindingComponent comp = new XMLBindingComponent(_config);
         if (_binding != null) {
             comp.setBinding(_binding);
         }
@@ -643,6 +719,9 @@ public class SourceFactory  {
         className = resolveClassName(className, packageName);
 
         FactoryState state = new FactoryState(className, sgState, packageName);
+        state.setParent(sgState.getCurrentFactoryState());
+        
+        
         ClassInfo classInfo = state.classInfo;
         JClass    jClass    = state.jClass;
 
@@ -664,7 +743,7 @@ public class SourceFactory  {
 
         //-- handle enumerated types
         if (enumeration) {
-            xsClass.setAsEnumertated(true);
+            xsClass.setAsEnumerated(true);
             processEnumeration(simpleType, state);
         }
 
@@ -885,15 +964,25 @@ public class SourceFactory  {
 
     } //-- createMarshalMethods
 
-    private void createUnmarshalMethods(JClass parent) {
+    private void createUnmarshalMethods(JClass parent, SGStateInfo sgState) {
 
         //-- mangle method name to avoid compiler errors when this class is extended
 		String methodName = "unmarshal";
-		if (SourceGenerator.mappingSchemaType2Java())
+		if (sgState.getSourceGenerator().mappingSchemaType2Java())
 			methodName+= parent.getName(true);
 
 		//-- create main marshal method
-        JMethod jMethod = new JMethod(parent,methodName);
+        JClass returnType = parent;
+        while (returnType.getSuperClass() != null) {
+            JClass superClass = sgState.getSourceCode(returnType.getSuperClass());
+            if (superClass != null) {
+                returnType = superClass;
+            }
+            else {
+                break;
+            }
+        }
+        JMethod jMethod = new JMethod(returnType ,methodName);
         jMethod.getModifiers().setStatic(true);
         jMethod.addException(SGTypes.MarshalException);
         jMethod.addException(SGTypes.ValidationException);
@@ -1007,7 +1096,9 @@ public class SourceFactory  {
      * @param jclass the JCLass which will implement the CastorTestable Interface
      * @see org.exolab.castor.tests.framework.CastorTestable
      */
-     public static void createTestableMethods(JClass jclass) {
+     public static void createTestableMethods
+         (JClass jclass, FactoryState state) 
+     {
          if (jclass == null)
             throw new IllegalArgumentException("JClass must not be null");
 
@@ -1023,11 +1114,19 @@ public class SourceFactory  {
         jclass.addMethod(jMethod);
         JSourceCode jsc = jMethod.getSourceCode();
         JField[] fields = jclass.getFields();
+        
         for (int i = 0; i <fields.length; i++) {
 
             JField temp = fields[i];
             JType type = temp.getType();
             String name = temp.getName();
+            
+            if (state.fieldInfoForChoice != null) {
+                if (name.equals(state.fieldInfoForChoice.getName())) {
+                    continue;
+                }
+            }
+            
             if (name.startsWith("_"))
                 name = JavaNaming.toJavaClassName(name.substring(1));
             else
@@ -1182,7 +1281,9 @@ public class SourceFactory  {
     //////////////////////////////////////////////
     //Process XML Schema structures
     //Note: This code is XML specific, it has to be
-    //move in XMLBindingComponent
+    //moved somehow in XMLBindingComponent. The aim
+    //of the SourceFactory is to generate code from 
+    //a BindingComponent.
     ///////////////////////////////////////////////
     /**
      * Creates Comments from Schema annotations
@@ -1191,11 +1292,11 @@ public class SourceFactory  {
     **/
     private String processAnnotations(Annotated annotated) {
         //-- process annotations
-        Enumeration enum = annotated.getAnnotations();
-        if (enum.hasMoreElements()) {
+        Enumeration enumeration = annotated.getAnnotations();
+        if (enumeration.hasMoreElements()) {
             StringBuffer comment = new StringBuffer();
-            while (enum.hasMoreElements()) {
-                Annotation ann = (Annotation) enum.nextElement();
+            while (enumeration.hasMoreElements()) {
+                Annotation ann = (Annotation) enumeration.nextElement();
                 Enumeration documentations = ann.getDocumentation();
                 while (documentations.hasMoreElements()) {
                     Documentation documentation =
@@ -1219,11 +1320,11 @@ public class SourceFactory  {
         if (complexType == null)
             return;
 
-        Enumeration enum = complexType.getAttributeDecls();
-        XMLBindingComponent component = new XMLBindingComponent();
+        Enumeration enumeration = complexType.getAttributeDecls();
+        XMLBindingComponent component = new XMLBindingComponent(_config);
         if (_binding != null) component.setBinding(_binding);
-        while (enum.hasMoreElements()) {
-            AttributeDecl attr = (AttributeDecl)enum.nextElement();
+        while (enumeration.hasMoreElements()) {
+            AttributeDecl attr = (AttributeDecl)enumeration.nextElement();
             
             component.setView(attr);
      
@@ -1234,7 +1335,11 @@ public class SourceFactory  {
                 
                 if (sType.getSchema() == component.getSchema())
                 {
-                    createSourceCode(sType, state.getSGStateInfo());
+                    if (state.resolve(sType) == null) {    
+                        if (sType.hasFacet(Facet.ENUMERATION)) {
+                            createSourceCode(sType, state.getSGStateInfo());
+                        }
+                    }
                 }
             }
             FieldInfo fieldInfo = memberFactory.createFieldInfo(component, state);
@@ -1250,7 +1355,7 @@ public class SourceFactory  {
     private void processComplexType
         (ComplexType complexType, FactoryState state)
     {
-        XMLBindingComponent component = new XMLBindingComponent();
+        XMLBindingComponent component = new XMLBindingComponent(_config);
         if (_binding != null) component.setBinding(_binding);
         component.setView(complexType);
         
@@ -1299,17 +1404,56 @@ public class SourceFactory  {
 				component.setView(complexType);
                 //--only set a super class name if the current complexType
                 //--is not a restriction of a simpleContent (--> no object hierarchy, only content hierarchy)
-                if (!(complexType.isSimpleContent() && ((ComplexType)base).isRestricted()) )
-                    state.jClass.setSuperClass(baseClassName);
+                /*
+                 Note: There are times when a simpleContent restriction needs to
+                 extend the hierarchy, such as a restriction of a restriction, so
+                 I'm commenting out the following line for now. see bug 1875
+                 for more details. If this causes any regressions we'll need to
+                 find a more appropriate solution.
+                 if (! ( complexType.isRestricted() && ((ComplexType)base).isSimpleContent() ) )
+                */
+                state.jClass.setSuperClass(baseClassName);
             } //--complexType
 
             //--if the content type is a simpleType create a field info for it.
             if (complexType.getContentType().getType() == ContentType.SIMPLE) {
                 SimpleContent simpleContent = (SimpleContent)complexType.getContentType();
                 SimpleType temp = simpleContent.getSimpleType();
-                XSType xsType = TypeConversion.convertType(temp, state.packageName);
-                FieldInfo fieldInfo = memberFactory.createFieldInfoForContent(xsType);
-		        handleField(fieldInfo,state);
+                SimpleType baseType = (SimpleType)temp.getBaseType();
+                XSType xsType = _typeConversion.convertType(temp, state.packageName);
+                FieldInfo fieldInfo = null;
+                if ((baseType != null)) {
+                    if (xsType.isEnumerated()) {
+                        fieldInfo = memberFactory.createFieldInfoForContent(xsType);
+                        fieldInfo.setBound(false);
+                        handleField(fieldInfo, state);
+                        //-- remove getter since we don't need to
+                        //-- override the original getter
+                        String mname = fieldInfo.getReadMethodName();
+                        JClass jClass = state.jClass;
+                        JMethod method = jClass.getMethod(mname, 0);
+                        jClass.removeMethod(method);
+                        //-- update setter method
+                        mname = fieldInfo.getWriteMethodName();
+                        method = jClass.getMethod(mname, 0);
+                        JSourceCode jsc = method.getSourceCode();
+                        jsc.add("super.");
+                        jsc.append(mname);
+                        jsc.append("(this.");
+                        jsc.append(fieldInfo.getName());
+                        jsc.append(".toString());");
+                    }
+                    //-- else just use superclass setters/getters
+                    //-- do nothing
+                }
+                else {
+                    while (temp.getBaseType() != null) {
+                    	temp = (SimpleType)temp.getBaseType();
+                    }
+                    xsType = _typeConversion.convertType(temp, state.packageName);
+                    fieldInfo = memberFactory.createFieldInfoForContent(xsType);
+    		        handleField(fieldInfo,state);
+                }
             }
 		}//--base not null
 		
@@ -1348,15 +1492,25 @@ public class SourceFactory  {
         //- handle elements and groups -/
         //------------------------------/
 
-        Enumeration enum = contentModel.enumerate();
+        Enumeration enumeration = contentModel.enumerate();
 
+        //-- handle choice item
+        if (state.classInfo.isChoice() && 
+            (state.fieldInfoForChoice == null)) 
+        {
+            state.fieldInfoForChoice 
+                = memberFactory.createFieldInfoForChoiceValue();
+            state.fieldInfoForChoice.createJavaField(state.jClass);
+            state.fieldInfoForChoice.createAccessMethods(state.jClass);
+        }
+        
         FieldInfo fieldInfo = null;
-        XMLBindingComponent component = new XMLBindingComponent();
+        XMLBindingComponent component = new XMLBindingComponent(_config);
         if (_binding != null) component.setBinding(_binding);
         
-        while (enum.hasMoreElements()) {
+        while (enumeration.hasMoreElements()) {
 
-            Annotated annotated = (Annotated)enum.nextElement();
+            Annotated annotated = (Annotated)enumeration.nextElement();
             component.setView(annotated);
 
             switch(annotated.getStructureType()) {
@@ -1370,7 +1524,6 @@ public class SourceFactory  {
                     if (contentModel.getMinOccurs() == 0) { 
                         fieldInfo.setRequired(false); 
                     }
-                    
                     handleField(fieldInfo, state);
                     break;
 
@@ -1393,9 +1546,19 @@ public class SourceFactory  {
                     if (!( (contentModel instanceof ComplexType)||
                             (contentModel instanceof ModelGroup)) )
                     {
-                        fieldInfo = memberFactory.createFieldInfo(component,
-                                                                   state.getSGStateInfo());
-                        handleField(fieldInfo, state);
+                        
+                        if (contentModel instanceof ModelGroup) {
+                            ModelGroup mg = (ModelGroup)contentModel;
+                            if (mg.isReference()) {
+                                contentModel = mg.getReference();
+                            }
+                        }
+                        
+                        if (contentModel.getParticleCount() > 0) {
+	                        fieldInfo = memberFactory.createFieldInfo(component,
+	                                                                   state.getSGStateInfo());
+	                        handleField(fieldInfo, state);
+                        }
                     } else {
                        //--else we just flatten the group
                        processContentModel(group, state);
@@ -1411,9 +1574,14 @@ public class SourceFactory  {
                         //create the field info for the element
                         //that is referring to a model group in order
                         //not to loose the Particle information
-                        fieldInfo = memberFactory.createFieldInfo(component,
-                                                  state.getSGStateInfo());
-                        handleField(fieldInfo, state);
+                        if (modelgroup.isReference())
+                            modelgroup = modelgroup.getReference();
+                            
+                        if (modelgroup.getParticleCount() > 0) {
+	                        fieldInfo = memberFactory.createFieldInfo(component,
+	                                                  state.getSGStateInfo());
+	                        handleField(fieldInfo, state);
+                        }
                         break;
                     } else {
                         //--else we just flatten the group
@@ -1444,6 +1612,12 @@ public class SourceFactory  {
     {
 
 
+ 		// Added by robertlaferla at comcast dot net 01/21/2004
+ 	    if (_config.useEnumeratedTypeInterface()) {
+ 		    state.jClass.addImport(ENUM_ACCESS_INTERFACE);
+ 		    state.jClass.addInterface(ENUM_ACCESS_INTERFACE);
+ 	    } // end enumTypeInterface
+
         switch (enumerationType) {
             case BASE_TYPE_ENUMERATION:
                 processEnumerationAsBaseType(simpleType, state);
@@ -1464,21 +1638,21 @@ public class SourceFactory  {
         (SimpleType simpleType, FactoryState state)
     {
 
-        Enumeration enum = simpleType.getFacets("enumeration");
+        Enumeration enumeration = simpleType.getFacets("enumeration");
 
 
         //-- select naming for types and instances
         boolean useValuesAsName = true;
-        while (enum.hasMoreElements()) {
-            Facet facet = (Facet)enum.nextElement();
-            String value = facet.getValue().toUpperCase();
-            if (!JavaNaming.isValidJavaIdentifier(value)) {
+        while (enumeration.hasMoreElements()) {
+            Facet facet = (Facet)enumeration.nextElement();
+            String possibleId = translateEnumValueToIdentifier(facet.getValue());
+            if (!JavaNaming.isValidJavaIdentifier(possibleId)) {
                 useValuesAsName = false;
                 break;
             }
         }
 
-        enum = simpleType.getFacets("enumeration");
+        enumeration = simpleType.getFacets("enumeration");
 
         JClass jClass = state.jClass;
         String className = jClass.getLocalName();
@@ -1550,20 +1724,30 @@ public class SourceFactory  {
         mInit.getModifiers().setStatic(true);
         mInit.getSourceCode().add("Hashtable members = new Hashtable();");
 
+        //-- #readResolve method
+        JMethod mReadResolve = new JMethod(SGTypes.Object,"readResolve");
+        mReadResolve.getModifiers().makePrivate();        
+        jClass.addMethod(mReadResolve);
+        jdc = mReadResolve.getJDocComment();
+        jdc.appendComment(" will be called during deserialization to replace ");
+        jdc.appendComment("the deserialized object with the correct constant ");
+        jdc.appendComment("instance. <br/>");
+        jsc = mReadResolve.getSourceCode();
+        jsc.add("return valueOf(this.stringValue);");
 
         //-- Loop through "enumeration" facets
         int count = 0;
 
-        while (enum.hasMoreElements()) {
+        while (enumeration.hasMoreElements()) {
 
-            Facet facet = (Facet) enum.nextElement();
+            Facet facet = (Facet) enumeration.nextElement();
 
             String value = facet.getValue();
 
             String typeName = null;
             String objName = null;
 
-            if (useValuesAsName) objName = value.toUpperCase();
+            if (useValuesAsName) objName = translateEnumValueToIdentifier(value);
             else objName = "VALUE_" + count;
 
             //-- create typeName
@@ -1660,7 +1844,7 @@ public class SourceFactory  {
 
 
 
-    } //-- processEnumeration
+    } //-- processEnumerationAsNewObject
 
     /**
      * Creates all the necessary enumeration code from the given
@@ -1698,10 +1882,10 @@ public class SourceFactory  {
         if (base == null)
             baseType = new XSString();
         else
-            baseType = TypeConversion.convertType(base);
+            baseType = _typeConversion.convertType(base);
 
 
-        Enumeration enum = simpleType.getFacets("enumeration");
+        Enumeration enumeration = simpleType.getFacets("enumeration");
 
         JClass jClass = state.jClass;
         String className = jClass.getLocalName();
@@ -1723,9 +1907,9 @@ public class SourceFactory  {
 
         StringBuffer values = new StringBuffer("{\n");
 
-        while (enum.hasMoreElements()) {
+        while (enumeration.hasMoreElements()) {
 
-            Facet facet = (Facet) enum.nextElement();
+            Facet facet = (Facet) enumeration.nextElement();
 
             String value = facet.getValue();
 
@@ -1777,6 +1961,47 @@ public class SourceFactory  {
         jsc.append(": \" + string + \".\");");
 
     } //-- processEnumerationAsBaseType
+    
+
+    /**
+     * Attempts to translate a simpleType enumeration value into a legal
+     * java identifier.  Translation is through a couple of simple rules:
+     *   - if the value parses as a non-negative int, the string 'VALUE_' is
+     *     prepended to it
+     *   - if the value parses as a negative int, the string 'VALUE_NEG_' is
+     *     prepended to it
+     *   - the value is uppercased
+     *   - the characters <code>[](){}<>'`"</code> are removed
+     *   - the characters <code>|\/?~!@#$%^&*-+=:;.,</code> and any whitespace are replaced with <code>_</code>
+     * @author rhett-sutphin@uiowa.edu 
+     */
+    private String translateEnumValueToIdentifier(String enumValue) 
+    {
+        
+        try {
+            int intVal = Integer.parseInt(enumValue);
+            if (intVal >= 0)
+                return "VALUE_" + intVal;
+            else
+                return "VALUE_NEG_" + Math.abs(intVal);
+        } catch (NumberFormatException e) {
+            // just keep going
+        }
+        StringBuffer sb = new StringBuffer(enumValue.toUpperCase());
+        char c;
+        for (int i = 0 ; i < sb.length() ; i++) {
+            c = sb.charAt(i);
+            if ("[](){}<>'`\"".indexOf(c) >= 0) {
+                sb.deleteCharAt(i);
+                i--;
+            }
+            else if (Character.isWhitespace(c) || "\\/?~!@#$%^&*-+=:;.,".indexOf(c) >= 0) {
+                sb.setCharAt(i, '_');
+            }
+        }
+        return sb.toString();   
+    } //-- translateEnumValueToIdentifier    
+    
 
     /**
      * Adds a given FieldInfo to the JClass and ClassInfo
@@ -1810,6 +2035,7 @@ public class SourceFactory  {
                 sInfo.getDialog().notify(warn);
             }
         }
+        
 
         JSourceCode scInitializer
             = state.jClass.getConstructor(0).getSourceCode();
@@ -1829,11 +2055,19 @@ public class SourceFactory  {
             }
         }
 
+        
         state.classInfo.addFieldInfo(fieldInfo);
         present = present && !fieldInfo.isMultivalued();
         //create the relevant Java fields only if the field
         //info is not yet in the base classInfo or if it is not a collection
         if (!present) {
+            
+            if (state.fieldInfoForChoice != null) {
+                if (fieldInfo != state.fieldInfoForChoice) {
+                    fieldInfo.setReference(state.fieldInfoForChoice.getName());
+                }
+            }
+            
             fieldInfo.createJavaField(state.jClass);
             //-- do not create access methods for transient fields
             if (!fieldInfo.isTransient()) {
@@ -1920,179 +2154,3 @@ public class SourceFactory  {
 } //-- SourceFactory
 
 
-/**
- * A class used to save State information for the SourceFactory
- * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
-**/
-class FactoryState implements ClassInfoResolver {
-
-    //--------------------/
-    //- Member Variables -/
-    //--------------------/
-
-    JClass       jClass           = null;
-    ClassInfo    classInfo        = null;
-
-    String packageName         = null;
-
-    private ClassInfoResolver _resolver  = null;
-    private Vector            _processed = null;
-    private SGStateInfo       _sgState   = null;
-    private boolean           _createGroupItem = false;
-    /**
-     * Keeps track of whether or not the BoundProperties
-     * methods have been created
-    **/
-    private boolean           _bound = false;
-
-    /**
-     * Keeps track of the different FactoryState
-     */
-     private FactoryState _parent = null;
-
-    //----------------/
-    //- Constructors -/
-    //----------------/
-
-    /**
-     * Creates a new FactoryState
-    **/
-
-    protected FactoryState
-        (String className, SGStateInfo sgState, String packageName)
-    {
-       if (sgState == null)
-            throw new IllegalArgumentException("SGStateInfo cannot be null.");
-
-        _sgState     = sgState;
-        _processed   = new Vector();
-
-		//keep the elements and complexType already processed
-		//if (resolver instanceof FactoryState)
-		   //_processed = ((FactoryState)resolver)._processed;
-
-        jClass       = new JClass(className);
-        classInfo    = new ClassInfo(jClass);
-
-        _resolver = sgState;
-
-        this.packageName = packageName;
-
-        //-- boundProperties
-        _bound = SourceGenerator.boundPropertiesEnabled();
-    } //-- FactoryState
-
-    //-----------/
-    //- Methods -/
-    //-----------/
-
-    /**
-     * Adds the given Reference to this ClassInfo resolver
-     * @param key the key to bind a reference to
-     * @param classInfo the ClassInfo which is being referenced
-    **/
-    public void bindReference(Object key, ClassInfo classInfo) {
-        _resolver.bindReference(key, classInfo);
-    } //-- bindReference
-
-    /**
-     * Returns the SGStateInfo
-     *
-     * @return the SGStateInfo
-    **/
-    SGStateInfo getSGStateInfo() {
-        return _sgState;
-    } //-- getSGStateInfo
-
-    /**
-     * Marks the given Annotated XML Schema structure as having been processed.
-     *
-     * @param annotated the Annotated XML Schema structure to mark as having
-     * been processed.
-    **/
-    void markAsProcessed(Annotated annotated) {
-        _processed.addElement(annotated);
-    } //-- markAsProcessed
-
-    /**
-     * Returns true if the given Annotated XML Schema structure has been marked as processed.
-     *
-     * @param annotated the Annotated XML Schema structure to check for being marked as processed
-    **/
-    boolean processed(Annotated annotated) {
-        boolean result = _processed.contains(annotated);
-        if (!result) {
-            if (_parent != null) {
-                return _parent.processed(annotated);
-            }
-        }
-        return result;
-    } //-- processed
-
-	/**
-     * Returns true if any bound properties have been found
-     *
-     * @return true if any bound properties have been found
-    **/
-    boolean hasBoundProperties() {
-        return _bound;
-    } //-- hasBoundProperties
-
-    /**
-     * Allows setting the bound properties flag
-     *
-     * @param bound the new value of the bound properties flag
-     * @see #hasBoundProperties
-    **/
-    void setBoundProperties(boolean bound) {
-        _bound = bound;
-    } //-- setBoundProperties
-
-    /**
-     * Returns the ClassInfo which has been bound to the given key
-     * @param key the object to which the ClassInfo has been bound
-     * @return the ClassInfo which has been bound to the given key
-    **/
-    public ClassInfo resolve(Object key) {
-        return _resolver.resolve(key);
-    } //-- resolve
-
-    /**
-     * Returns true if we are currently in the state of
-     * creating a group item class.
-     * @return true if we are currently in the state of
-     * creating a group item class.
-     */
-    boolean isCreateGroupItem() {
-        return _createGroupItem;
-    }
-
-    /**
-     * Sets to true if we are currently generating a class
-     * to represent items in a group.
-     * @param createGroupItem true if we are currently generating a class
-     * to represent items in a group.
-     */
-     void setCreateGroupItem(boolean createGroupItem) {
-         _createGroupItem = createGroupItem;
-     }
-
-     /**
-      * Returns the parent of this FactoryState.
-      * The parent of a factory state is the previous item
-      * of the list that contained all the created factory states.
-      * @return the parent of this FactoryState.
-      */
-     FactoryState getParent() {
-        return _parent;
-     }
-
-     /**
-      * Sets the parent of this FactoryState
-      * @param parent the parent FactoryState
-      * @see #getParent
-      */
-     void setParent(FactoryState parent) {
-         _parent = parent;
-     }
-} //-- FactoryState
