@@ -40,7 +40,7 @@
  *
  * Copyright 1999-2002 (C) Intalio Inc. All Rights Reserved.
  *
- * $Id: TypeConversion.java,v 1.1.1.1 2003/03/03 07:07:48 kvisco Exp $
+ * $Id: TypeConversion.java,v 1.7 2005/03/05 13:41:42 mfuchs Exp $
  */
 
 package org.exolab.castor.builder;
@@ -48,7 +48,6 @@ package org.exolab.castor.builder;
 import org.exolab.castor.builder.types.*;
 import org.exolab.castor.builder.types.XSType;
 import org.exolab.javasource.*;
-import org.exolab.castor.types.TimeDuration;
 
 import org.exolab.castor.xml.schema.Facet;
 import org.exolab.castor.xml.JavaNaming;
@@ -60,32 +59,65 @@ import org.exolab.castor.xml.schema.SimpleTypesFactory;
 import org.exolab.castor.xml.schema.Structure;
 import org.exolab.castor.xml.schema.Union;
 
-import java.text.ParseException;
 import java.util.Enumeration;
 
 /**
  * A class used to convert XML Schema SimpleTypes into
  * the appropriate XSType
  * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:07:48 $
+ * @author <a href="mailto:blandin@intalio.com">Arnaud Blandin</a>
+ * @version $Revision: 1.7 $ $Date: 2005/03/05 13:41:42 $
 **/
 public class TypeConversion {
 
     private static final String TYPES_PACKAGE = "types";
 
+    private BuilderConfiguration _config = null;
+    
+    /**
+     * Creates a new TypeConversion instance
+     *
+     * @param config the BuilderConfiguration instance (must not be null).
+     */
+    public TypeConversion(BuilderConfiguration config) {
+        if (config == null) {
+            String error = "The argument 'config' must not be null.";
+            throw new IllegalArgumentException(error);
+        }
+        _config = config;
+    } //-- TypeConversion
+    
     /**
      * Converts the given Simpletype to the appropriate XSType.
+     *
+     * @param simpleType the SimpleType to convert to an XSType instance
      * @return the XSType which represets the given Simpletype
-    **/
-    public static XSType convertType(SimpleType simpleType) {
-        return convertType(simpleType, SourceGenerator.usePrimitiveWrapper(), null);
+     */
+    public XSType convertType(SimpleType simpleType) {
+        return convertType(simpleType, _config.usePrimitiveWrapper(), null);
     }
 
-    public static XSType convertType(SimpleType simpleType, String packageName) {
-         return convertType(simpleType, SourceGenerator.usePrimitiveWrapper(), packageName);
+    /**
+     * Converts the given Simpletype to the appropriate XSType.
+     *
+     * @param simpleType the SimpleType to convert to an XSType instance
+     * @param packageName the packageName for any new class types
+     * @return the XSType which represets the given Simpletype
+     */
+    public XSType convertType(SimpleType simpleType, String packageName) {
+         return convertType(simpleType, _config.usePrimitiveWrapper(), packageName);
     }
     
-    public static XSType convertType(SimpleType simpleType, boolean useWrapper, String packageName) {
+    /**
+     * Converts the given Simpletype to the appropriate XSType.
+     *
+     * @param simpleType the SimpleType to convert to an XSType instance
+     * @param useWrapper a boolean that when true indicates that primitive wrappers
+     * be used instead of the actual primitives (e.g. java.lang.Integer instead of int)
+     * @param packageName the packageName for any new class types
+     * @return the XSType which represets the given Simpletype
+     */
+    public XSType convertType(SimpleType simpleType, boolean useWrapper, String packageName) {
         if (simpleType == null) return null;
 
         XSType xsType = null;
@@ -143,7 +175,7 @@ public class TypeConversion {
                 //--byte
                 case SimpleTypesFactory.BYTE_TYPE:
                 {
-                    XSByte xsByte = new XSByte();
+                    XSByte xsByte = new XSByte(useWrapper);
                     if (!simpleType.isBuiltInType())
                         xsByte.setFacets(simpleType);
                     return xsByte;
@@ -254,9 +286,8 @@ public class TypeConversion {
                     //-- since we don't actually support this type, yet,
                     //-- we'll simply treat it as a string, but warn the
                     //-- user.
-                    String warning = "Warning: The W3C datatype '"+simpleType.getName();
-                    warning += "' is not currently supported by Castor Source Generator.";
-                    warning += " It will be treated as a java.lang.String.";
+                    String warning = "Warning: Currently, the W3C datatype '"+simpleType.getName();
+                    warning += "' is supported only as a String by Castor Source Generator.";
                     System.out.println(warning);
                     return new XSString();
                 }
@@ -274,7 +305,7 @@ public class TypeConversion {
                 //-- nonPositiveInteger
                 case SimpleTypesFactory.NON_POSITIVE_INTEGER_TYPE:
                 {
-                    XSInteger xsInteger = new XSNonPositiveInteger();
+                    XSInteger xsInteger = new XSNonPositiveInteger(useWrapper);
                     xsInteger.setFacets(simpleType);
                     return xsInteger;
                 }
@@ -282,7 +313,7 @@ public class TypeConversion {
                 //-- nonNegativeInteger
                 case SimpleTypesFactory.NON_NEGATIVE_INTEGER_TYPE:
                 {
-                    XSInteger xsInteger = new XSNonNegativeInteger();
+                    XSInteger xsInteger = new XSNonNegativeInteger(useWrapper);
                     xsInteger.setFacets(simpleType);
                     return xsInteger;
                 }
@@ -290,7 +321,7 @@ public class TypeConversion {
                 //-- negative-integer
                 case SimpleTypesFactory.NEGATIVE_INTEGER_TYPE:
                 {
-                    XSInteger xsInteger = new XSNegativeInteger();
+                    XSInteger xsInteger = new XSNegativeInteger(useWrapper);
                     xsInteger.setFacets(simpleType);
                     return xsInteger;
                 }
@@ -306,7 +337,7 @@ public class TypeConversion {
                 //-- positive-integer
                 case SimpleTypesFactory.POSITIVE_INTEGER_TYPE:
                 {
-                    XSInteger xsInteger = new XSPositiveInteger();
+                    XSInteger xsInteger = new XSPositiveInteger(useWrapper);
                     xsInteger.setFacets(simpleType);
                     return xsInteger;
                 }
@@ -341,7 +372,7 @@ public class TypeConversion {
 
                         if (packageName == null) {
                             String ns = simpleType.getSchema().getTargetNamespace();
-                            packageName = SourceGenerator.lookupPackageNamespace(ns);
+                            packageName = _config.lookupPackageByNamespace(ns);
                         }
                         if ((packageName  != null) && (packageName .length() > 0))
                             packageName  = packageName  + '.' + TYPES_PACKAGE;
@@ -350,7 +381,7 @@ public class TypeConversion {
 
                         className = packageName  + '.' + className;
                         xsType = new XSClass(new JClass(className));
-                        xsType.setAsEnumertated(true);
+                        xsType.setAsEnumerated(true);
                     } //- End Enumeration
                     else {
                         XSString xsString = new XSString();
@@ -376,6 +407,20 @@ public class TypeConversion {
                         xsTime.setFacets(simpleType);
                     return xsTime;
                 }
+                case SimpleTypesFactory.TOKEN_TYPE:
+                {
+                    //-- since we don't actually support this type, yet,
+                    //-- we'll simply treat it as a string, but warn the
+                    //-- user.
+                    String warning = "Warning: Currently, the W3C datatype 'token'";
+                    warning += " is supported only as a String by Castor Source Generator.";
+                    System.out.println(warning);
+                    XSString xsString = new XSString();
+                    if (!simpleType.isBuiltInType()) {
+                        xsString.setFacets(simpleType);
+                    }
+                    return xsString;
+                }
                 default:
                     //-- error
                     String name = simpleType.getName();
@@ -400,7 +445,7 @@ public class TypeConversion {
     /**
      * Returns the XSType that corresponds to the given javaType
      * @param javaType
-     * @return
+     * @return XSType that corresponds to the given javaType
      */
     public static XSType convertType(String javaType) {
         if (javaType == null)
@@ -468,7 +513,7 @@ public class TypeConversion {
         } else
         //--Integer
         if (javaType.equals(TypeNames.INTEGER)) {
-            return new XSInteger();
+            return new XSInteger(true);
         } else
         if (javaType.equals(TypeNames.INT)) {
             return new XSInt();
@@ -497,9 +542,9 @@ public class TypeConversion {
      */
     private static SimpleType findCommonType(Union union) {
         SimpleType common = null;
-        Enumeration enum = union.getMemberTypes();
-        while (enum.hasMoreElements()) {
-            SimpleType type = (SimpleType)enum.nextElement();
+        Enumeration enumeration = union.getMemberTypes();
+        while (enumeration.hasMoreElements()) {
+            SimpleType type = (SimpleType)enumeration.nextElement();
             type = type.getBuiltInBaseType();
             if (common == null) common = type;
             else {
