@@ -38,16 +38,20 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Copyright 2002-2003 (C) Intalio Inc. All Rights Reserved.
+ * Copyright 2002-2004 (C) Intalio Inc. All Rights Reserved.
  *
- * $Id: XMLBindingComponent.java,v 1.4 2003/06/02 13:51:45 blandin Exp $
+ * Any portions of this file developed by Keith Visco after Jan 19 2005 
+ * are Copyright (C) 2005 Keith Visco. All Rights Reserverd.
+ * 
+ *
+ * $Id: XMLBindingComponent.java,v 1.14 2005/05/21 17:30:16 afawcett Exp $
  */
 
 package org.exolab.castor.builder.binding;
 
 import org.exolab.castor.builder.BindingComponent;
+import org.exolab.castor.builder.BuilderConfiguration;
 import org.exolab.castor.builder.GroupNaming;
-import org.exolab.castor.builder.SourceGenerator;
 import org.exolab.castor.builder.TypeConversion;
 import org.exolab.castor.builder.types.XSClass;
 import org.exolab.castor.builder.types.XSType;
@@ -55,6 +59,7 @@ import org.exolab.castor.xml.JavaNaming;
 import org.exolab.castor.xml.schema.Annotated;
 import org.exolab.castor.xml.schema.AttributeDecl;
 import org.exolab.castor.xml.schema.ComplexType;
+import org.exolab.castor.xml.schema.ContentModelGroup;
 import org.exolab.castor.xml.schema.ElementDecl;
 import org.exolab.castor.xml.schema.Form;
 import org.exolab.castor.xml.schema.Group;
@@ -65,6 +70,8 @@ import org.exolab.castor.xml.schema.Structure;
 import org.exolab.castor.xml.schema.XMLType;
 import org.exolab.javasource.JClass;
 
+import java.util.Enumeration;
+
 /**
  * <p>This class is the implementation of BindingComponent from an XML Schema
  * point of view. This specific implementation wraps an XML Schema annotated structure.
@@ -91,7 +98,8 @@ import org.exolab.javasource.JClass;
  * @see org.exolab.castor.builder.BindingComponent
 
  * @author <a href="mailto:blandin@intalio.com">Arnaud Blandin</a>
- * @version $Revision: 1.4 $ $Date: 2003/06/02 13:51:45 $
+ * @author <a href="mailto:keith AT kvisco DOT com">Keith Visco</a>
+ * @version $Revision: 1.14 $ $Date: 2005/05/21 17:30:16 $
  */
 public class XMLBindingComponent implements BindingComponent {
 
@@ -107,6 +115,11 @@ public class XMLBindingComponent implements BindingComponent {
     */
     private ComponentBindingType _compBinding;
 
+    /**
+     * The BuilderConfiguration instance for obtaining default properties
+     */
+    private BuilderConfiguration _config = null;
+    
     /**
      * The XML Schema Annotated structure encapsulated in that XMLBinding object
      */
@@ -131,14 +144,15 @@ public class XMLBindingComponent implements BindingComponent {
     /**
      * caches of several computations
      */
-    private int _hashCode =-1;
-    private String _javaClassName = null;
-    private String _javaMemberName = null;
-    private String _javaPackage = null;
+    private int       _hashCode =-1;
+    private String    _javaClassName = null;
+    private String    _javaMemberName = null;
+    private String    _javaPackage = null;
     private FieldType _member = null;
     private ClassType _class = null;
     private Interface _interface = null;
-    private Schema _schema = null;
+    private Schema    _schema = null;
+    private boolean   _userSpecifiedMemberName = false;
 
     /**
      * A GroupNaming helper class used to named anonymous groups
@@ -146,15 +160,27 @@ public class XMLBindingComponent implements BindingComponent {
     private static GroupNaming _groupNaming = null;
     static {
         _groupNaming = new GroupNaming();
-    }
-
+   }
 
+    /**
+     * The TypeConversion to use when creating XSTypes from
+     * SimpleType
+     */
+    private TypeConversion _typeConversion = null;
+    
     /**
      * Constructs an XMLBindingComponent from an XML Schema Component.
      *
+     * @param config the BuilderConfiguration instance (must not be null).
      */
-    public XMLBindingComponent() {
-    }
+    public XMLBindingComponent(BuilderConfiguration config) {
+        if (config == null) {
+            String error = "The argument 'config' must not be null.";
+            throw new IllegalArgumentException(error);
+        }
+        _config = config;
+        _typeConversion = new TypeConversion(_config);
+    } //-- XMLBindingComponent
 
 
     /**
@@ -195,7 +221,7 @@ public class XMLBindingComponent implements BindingComponent {
         int _hashCode =-1;
         _javaClassName = null;
         _javaMemberName = null;
-        _javaPackage = null;
+        _javaPackage = null;        
         _schema = null;
         _member = null;
         _class = null;
@@ -203,6 +229,8 @@ public class XMLBindingComponent implements BindingComponent {
         _type = -1;
         _prefix = null;
         _suffix = null;
+        _userSpecifiedMemberName = false;
+        _compBinding = null;
         
         
         //--look up for the particular componentBinding relative to the
@@ -297,7 +325,7 @@ public class XMLBindingComponent implements BindingComponent {
      * Returns the hashCode value for this object.
      *
      * @return the hashcode value for this object.
-     * @see java.lang.hashCode()
+     * @see java.lang.Object#hashCode()
      */
     public int hashCode() {
 
@@ -666,7 +694,7 @@ public class XMLBindingComponent implements BindingComponent {
                          ((_annotated.getStructureType() == Structure.GROUP)||
                           (_annotated.getStructureType() == Structure.MODELGROUP)) ) {
 
-                        result = _groupNaming.createClassName((Group)_annotated);
+                        result = _groupNaming.createClassName((Group)_annotated, getJavaPackage());
                         if (result == null) {
                             String err = "Unable to create name for group.";
                             throw new IllegalStateException(err);
@@ -735,7 +763,39 @@ public class XMLBindingComponent implements BindingComponent {
                          temp = _annotated;
                          setView(element.getReference());
                          result = getJavaMemberName();
+                         boolean userSpecified = _userSpecifiedMemberName;
                          setView(temp);
+                         
+                         //-- there might be more than once reference, so we
+                         //-- need to do a little counting here.                         
+                         if (!userSpecified) {
+                            String refName = element.getReferenceName();
+                            int count = 0;
+                            int index = 0;
+                            Structure structure = element.getParent();
+                            if (structure instanceof ContentModelGroup) {
+                                ContentModelGroup cmg = (ContentModelGroup)structure;
+                                Enumeration enumeration = cmg.enumerate();
+                                while (enumeration.hasMoreElements()) {
+                                    Structure tmpStruct = (Structure)enumeration.nextElement();
+                                    if (tmpStruct.getStructureType() == Structure.ELEMENT) {
+                                        ElementDecl tmpDecl = (ElementDecl)tmpStruct;
+                                        if (tmpDecl.isReference()) {
+                                            if (tmpDecl.getReferenceName().equals(refName)) {
+                                                ++count;
+                                                if (tmpDecl == element) {
+                                                    index = count;
+                                                }
+                                            }
+                                        }
+                                    }
+                                }
+                            }
+                            if (count > 1) {
+                                result = result + index;
+                            }
+                         }
+                         
                      }
                      element = null;
                 }
@@ -751,7 +811,7 @@ public class XMLBindingComponent implements BindingComponent {
                           (_annotated.getStructureType() == Structure.MODELGROUP)) )
                     {
 
-                        result = _groupNaming.createClassName((Group)_annotated);
+                        result = _groupNaming.createClassName((Group)_annotated, getJavaPackage());
                         if (result == null) {
                             String err = "Unable to create name for group.";
                             throw new IllegalStateException(err);
@@ -765,6 +825,9 @@ public class XMLBindingComponent implements BindingComponent {
             
                 }
             }
+            else {
+                _userSpecifiedMemberName = true;
+            }
             _javaMemberName = JavaNaming.toJavaMemberName(result);
         }
         /**
@@ -823,7 +886,7 @@ public class XMLBindingComponent implements BindingComponent {
      */
     public String getJavaPackage() {
 
-        if (_javaPackage == null) {
+       if (_javaPackage == null) {
             String packageName = null;
             String schemaLocation = getSchemaLocation();
             String targetNamespace = getTargetNamespace();
@@ -840,14 +903,16 @@ public class XMLBindingComponent implements BindingComponent {
                 }//--switch
             }
             if  ((packageName == null)|| (packageName.length() ==0) ) {
-                //--highest priority is schemaLocation
-                if (schemaLocation != null) {
-                    packageName = SourceGenerator.lookupPackageLocation(schemaLocation);
-                }
-                if (( packageName == null || packageName.length() ==0))
-                {
-                    //--look for a namespace mapping
-                    packageName = SourceGenerator.lookupPackageNamespace(targetNamespace);
+                //--highest priority is targetNamespace
+            	if (( packageName == null || packageName.length() ==0))
+            	{   
+            		//--look for a namespace mapping
+            		packageName = _config.lookupPackageByNamespace(targetNamespace);
+            		
+            	}
+            	
+            	if ( (schemaLocation != null) && ( packageName == null || packageName.length() ==0) ){
+                    packageName = _config.lookupPackageByLocation(schemaLocation);
                 }
             }
             _javaPackage = packageName;
@@ -898,26 +963,7 @@ public class XMLBindingComponent implements BindingComponent {
 	 * from this BindingComponent. 0 is returned by default.
 	 */
     public int getLowerBound() {
-        switch (_annotated.getStructureType()) {
-
-            case Structure.ELEMENT:
-                return ((ElementDecl)_annotated).getMinOccurs();
-
-            case Structure.COMPLEX_TYPE:
-                return ((ComplexType)_annotated).getMinOccurs();
-
-            case Structure.GROUP:
-            case Structure.MODELGROUP:
-                return ((Group)_annotated).getMinOccurs();
-
-            case Structure.ATTRIBUTE:
-                if (((AttributeDecl)_annotated).isRequired())
-                    return 1;
-                break;
-            default:
-               break;
-        }
-        return 0;
+        return getLowerBound(_annotated);        
     }
 
     ////////METHODS RELATED TO A CLASS BINDING
@@ -934,15 +980,14 @@ public class XMLBindingComponent implements BindingComponent {
         if (getType() == CLASS) {
             return _class.getExtends();
         }
-        else return SourceGenerator.getProperty(SourceGenerator.Property.SUPER_CLASS, null);
-
-    }
+        else return _config.getProperty(BuilderConfiguration.Property.SUPER_CLASS, null);
+    } //-- getExtends
 
     /**
      * Returns an array of the different interface names implemented by the class
      * that will represent the current XMLBindingComponent. Null is returned if
      * no class binding is defined for the wrapped XML Schema structure.
-     * @return
+     * @return array of interface names
      */
     public String[] getImplements() {
         if (getType() == CLASS) {
@@ -964,7 +1009,7 @@ public class XMLBindingComponent implements BindingComponent {
             if (_class.hasBound())
                 return _class.getBound();
         }
-        return SourceGenerator.boundPropertiesEnabled();
+        return _config.boundPropertiesEnabled();
     }
 
     /**
@@ -979,7 +1024,7 @@ public class XMLBindingComponent implements BindingComponent {
             if (_class.hasEquals())
                 return _class.getEquals();
         }
-        return SourceGenerator.equalsMethod();
+        return _config.equalsMethod();
     }
 
     /**
@@ -996,8 +1041,21 @@ public class XMLBindingComponent implements BindingComponent {
                result = _class.getAbstract();
         }
         if (!result) {
-            result = (_annotated.getStructureType() == Structure.COMPLEX_TYPE);
-            result = (result && SourceGenerator.mappingSchemaElement2Java());
+            switch(_annotated.getStructureType()) {
+                case Structure.COMPLEX_TYPE:
+                    ComplexType cType = (ComplexType)_annotated;
+                    result = cType.isAbstract();
+                    //-- if we're in element-centric mode, then all 
+                    //--  complexTypes are treated as abstract
+                    result = result || _config.mappingSchemaElement2Java();
+                    break;
+                case Structure.ELEMENT:
+                    ElementDecl eDecl = (ElementDecl)_annotated;
+                    result = eDecl.isAbstract();
+                    break;
+                default:
+                    break;
+            }
         }
         return result;
     }
@@ -1041,6 +1099,22 @@ public class XMLBindingComponent implements BindingComponent {
         }
         return false;
     }
+    
+    /**
+     * Returns true if the wrapped XML Schema component is nillable.
+     *
+     * @return true if the wrapped XML Schema component is nillable.
+     */
+    public boolean isNillable() {
+        switch (_annotated.getStructureType()) {
+            case Structure.ELEMENT:
+                return ((ElementDecl)_annotated).isNillable();
+            default:
+                break;
+        }
+        return false;
+    } //-- isNillable
+    
 
     ////////METHODS RELATED TO A MEMBER BINDING
     /**
@@ -1058,7 +1132,7 @@ public class XMLBindingComponent implements BindingComponent {
                result = _member.getWrapper();
         }
         else {
-            result = SourceGenerator.usePrimitiveWrapper();
+            result = _config.usePrimitiveWrapper();
         }
         return result;
     }
@@ -1096,7 +1170,7 @@ public class XMLBindingComponent implements BindingComponent {
             if (type.isSimpleType()) {
                 String packageName = null;
                 if (((SimpleType)type).getSchema() != getSchema()) {
-                    XMLBindingComponent comp = new XMLBindingComponent();
+                    XMLBindingComponent comp = new XMLBindingComponent(_config);
                     comp.setBinding(_binding);
                     comp.setView(type);
                     packageName = comp.getJavaPackage();
@@ -1105,10 +1179,10 @@ public class XMLBindingComponent implements BindingComponent {
                 
                 if ((packageName == null) || (packageName.length() == 0)) {
                     String ns = ((SimpleType)type).getSchema().getTargetNamespace();
-                    packageName = SourceGenerator.lookupPackageNamespace(ns);
+                    packageName = _config.lookupPackageByNamespace(ns);
                 }
                     
-                result = TypeConversion.convertType((SimpleType)type, useWrapper, packageName);
+                result = _typeConversion.convertType((SimpleType)type, useWrapper, packageName);
             }
         }
         
@@ -1181,5 +1255,63 @@ public class XMLBindingComponent implements BindingComponent {
 	public short getType() {
 	    return _type;
 	}
+	
+	
+   /**
+	 * Returns the lower bound of the collection that is generated from
+	 * this BindingComponent. The lower bound is a positive integer.
+	 * In the case of an XML Schema component, it corresponds to the
+	 * XML Schema minOccurs attribute, if any.
+	 *
+	 * @return an int representing the lower bound of the collection generated
+	 * from this BindingComponent. 0 is returned by default.
+	 */
+    private static int getLowerBound(Annotated annotated) {
+        
+        switch (annotated.getStructureType()) {
 
-}
\ No newline at end of file
+            case Structure.ELEMENT:
+                return ((ElementDecl)annotated).getMinOccurs();
+
+            case Structure.COMPLEX_TYPE:
+                return ((ComplexType)annotated).getMinOccurs();
+
+            case Structure.MODELGROUP:
+            case Structure.GROUP:
+                Group group = (Group)annotated;
+                //-- if the group is top-level, then
+                //-- we always return 0
+                Structure parent = group.getParent();
+                if (parent != null) {
+                    if (parent.getStructureType() == Structure.SCHEMA)
+                        return 0;
+                }                
+                int minOccurs = group.getMinOccurs();                
+                //-- if minOccurs == 1, then check to see
+                //-- if all elements inside group are 
+                //-- optional, if so, we return 0, not 1.
+                if (minOccurs == 1) {
+                    Enumeration enumeration = group.enumerate();                    
+                    while (enumeration.hasMoreElements()) {
+                        if (getLowerBound((Annotated)enumeration.nextElement()) != 0)
+                            return 1;
+                    }
+                    //-- if we make it here, all items in group have a
+                    //-- lowerbound of 0, so the group can be considered
+                    //-- optional
+                    return 0;
+                }
+                return minOccurs;
+                
+            case Structure.ATTRIBUTE:
+                if (((AttributeDecl)annotated).isRequired())
+                    return 1;
+                break;
+            default:
+               break;
+        }
+        return 0;
+        
+    } //-- getLowerBound
+	
+} //-- class: XMLBindingComponent
\ No newline at end of file
