@@ -40,7 +40,7 @@
  *
  * Copyright 1999-2002(C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: XSType.java,v 1.1.1.1 2003/03/03 07:08:01 kvisco Exp $
+ * $Id: XSType.java,v 1.6 2005/03/05 13:41:44 mfuchs Exp $
  */
 
 package org.exolab.castor.builder.types;
@@ -56,7 +56,7 @@ import java.util.Enumeration;
 /**
  * The base XML Schema Type class
  * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:08:01 $
+ * @version $Revision: 1.6 $ $Date: 2005/03/05 13:41:44 $
 **/
 public abstract class XSType {
 
@@ -345,16 +345,16 @@ public abstract class XSType {
 	/**
 	 * Returns a list of Facets from the simpleType
 	 *	(duplicate facets due to extension are filtered out)
-     * @param simpletype the Simpletype we want the facets for
+     * @param simpleType the Simpletype we want the facets for
      * @return Unique list of facets from the simple type
 	 */
 	protected static Enumeration getFacets(SimpleType simpleType)
 	{
 		Hashtable hashTable = new Hashtable();
-        Enumeration enum = simpleType.getFacets();
-		while (enum.hasMoreElements()) {
+        Enumeration enumeration = simpleType.getFacets();
+		while (enumeration.hasMoreElements()) {
 
-            Facet facet = (Facet)enum.nextElement();
+            Facet facet = (Facet)enumeration.nextElement();
             String name = facet.getName();
 			hashTable.put(name, facet);
 		}
@@ -547,7 +547,7 @@ public abstract class XSType {
      * @param enumerated a boolean indicating whether or not this XSClass
      * represents an enumerated type
     **/
-    public void setAsEnumertated(boolean enumerated) {
+    public void setAsEnumerated(boolean enumerated) {
         this.enumerated = enumerated;
     } //-- setAsEnumerated
     
