@@ -40,7 +40,7 @@
  *
  * Copyright 2001 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: QueryAnalyser.java,v 1.1.1.1 2003/03/03 07:08:06 kvisco Exp $
+ * $Id: QueryAnalyser.java,v 1.5 2005/05/02 20:58:59 rjoachim Exp $
  */
 
 
@@ -48,31 +48,27 @@ package org.exolab.castor.gui;
 
 import javax.swing.UIManager;
 import java.awt.*;
-import java.awt.*;
 import java.awt.event.*;
 import javax.swing.*;
 import javax.swing.table.*;
-import org.exolab.castor.jdo.JDO;
+import org.exolab.castor.jdo.JDOManager;
 import org.exolab.castor.jdo.Database;
 import org.exolab.castor.jdo.QueryResults;
 import org.exolab.castor.jdo.OQLQuery;
-import java.util.Properties;
 import java.lang.Class;
 import java.lang.reflect.Method;
 import java.util.Vector;
 import java.util.Iterator;
 import java.util.Date;
-import org.exolab.castor.jdo.oql.*;
-import org.exolab.castor.jdo.engine.DatabaseImpl;
 import org.exolab.castor.jdo.engine.OQLQueryImpl;
 
 import org.exolab.castor.mapping.Mapping;
+import org.exolab.castor.mapping.MappingException;
 import org.exolab.castor.xml.Unmarshaller;
 import org.exolab.castor.xml.Marshaller;
 
 import java.io.FileWriter;
 import java.io.FileReader;
-import org.exolab.castor.xml.*;
 
 
 /**
@@ -80,7 +76,7 @@ import org.exolab.castor.xml.*;
  * OQL Queries against a Castor Database.
  *
  * @author <a href="mauch@imkenberg.de">Thorsten Mauch</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:08:06 $
+ * @version $Revision: 1.5 $ $Date: 2005/05/02 20:58:59 $
  */
 public class QueryAnalyser {
     boolean packFrame = false;
@@ -131,7 +127,7 @@ public class QueryAnalyser {
         BorderLayout borderLayout1 = new BorderLayout();
         DefaultTableModel model;
 
-        JDO jdo;
+        JDOManager jdo;
         String databasename;
         String dbconfig;
         JTabbedPane TabbedPane = new JTabbedPane();
@@ -300,7 +296,7 @@ public class QueryAnalyser {
 
                 // and execute it
                 Date starttime = new Date();
-                r=oql.execute(db.ReadOnly);
+                r=oql.execute(Database.ReadOnly);
                 Date endtime = new Date();
                 // write the status bar
                 statusBar.setText("Query successful, Time: "+ (endtime.getTime()-starttime.getTime()) +" ms");
@@ -376,7 +372,7 @@ public class QueryAnalyser {
                     temp=null;
                     m=(Method)i.next();
                     try{
-                        temp=m.invoke(o,null);
+                        temp=m.invoke(o,(Object[])null);
                     } catch(Exception ie) {
                         temp =null;
                     }
@@ -387,13 +383,16 @@ public class QueryAnalyser {
 
         private void openDB() {
             try{
-                jdo= new JDO();
-                jdo.setDatabaseName(databasename);
-                jdo.setConfiguration(dbconfig);
-                jdo.setClassLoader(ClassLoader.getSystemClassLoader());
+                JDOManager.loadConfiguration (dbconfig, ClassLoader.getSystemClassLoader());
+                jdo = JDOManager.createInstance(databasename);
                 //only to try a connection
                 Database db = jdo.getDatabase();
-            } catch(org.exolab.castor.jdo.PersistenceException pe) {
+            } 
+            catch (MappingException pe) {
+                pe.printStackTrace();
+                System.exit(1);
+            } 
+            catch(org.exolab.castor.jdo.PersistenceException pe) {
                 pe.printStackTrace();
                 System.exit(1);
             }
