@@ -40,15 +40,15 @@
  *
  * Copyright 1999 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: Database.java,v 1.1.1.1 2003/03/03 07:08:06 kvisco Exp $
+ * $Id: Database.java,v 1.12 2005/04/27 09:35:15 wguttmn Exp $
  */
 
 
 package org.exolab.castor.jdo;
 
 
+import org.exolab.castor.mapping.AccessMode;
 import org.exolab.castor.persist.PersistenceInfoGroup;
-import org.exolab.castor.persist.spi.Complex;
 
 /**
  * An open connection to the database. This object represents an open
@@ -96,8 +96,8 @@ import org.exolab.castor.persist.spi.Complex;
  * </pre>
  *
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:08:06 $
- * @see JDO#getDatabase
+ * @version $Revision: 1.12 $ $Date: 2005/04/27 09:35:15 $
+ * @see JDO#getDatabase()
  * @see Query
  */
 public interface Database
@@ -105,18 +105,18 @@ public interface Database
 
 
     /**
-     * Read only access. Used with queries and the {@link #load}
+     * Read only access. Used with queries and the {@link #load(Class,Object)}
      * method to load objects as read-only.
      * <p>
      * Read-only objects are not persistent and changes to these
      * objects are not reflected in the database when the transaction
      * commits.
      */
-    public static final short ReadOnly = 0;
+    public static final AccessMode ReadOnly = AccessMode.ReadOnly;
 
 
     /**
-     * Shared access. Used with queries and the {@link #load}
+     * Shared access. Used with queries and the {@link #load(Class,Object)}
      * method to load objects with shared access.
      * <p>
      * Shared access allows the same record to be accessed by two
@@ -127,11 +127,11 @@ public interface Database
      * modified. Dirty checking is enabled for all fields marked
      * as such, and a cached copy is used to populate the object.
      */
-    public static final short Shared = 1;
+    public static final AccessMode Shared = AccessMode.Shared;
 
 
     /**
-     * Exclusive access. Used with queries and the {@link #load}
+     * Exclusive access. Used with queries and the {@link #load(Class,Object)}
      * method to load objects with exclusive access.
      * <p>
      * Exclusive access prevents two concurrent transactions from
@@ -144,11 +144,11 @@ public interface Database
      * be synchronized with the database and not populated from
      * the cache.
      */
-    public static final short Exclusive = 2;
+    public static final AccessMode Exclusive = AccessMode.Exclusive;
 
 
     /**
-     * Database lock access. Used with queries and the {@link #load}
+     * Database lock access. Used with queries and the {@link #load(Class,Object)}
      * method to load objects with a database lock.
      * <p>
      * Database lock prevents two concurrent transactions from
@@ -161,7 +161,7 @@ public interface Database
      * be synchronized with the database and not populated from
      * the cache. Dirty checking is not required.
      */
-    public static final short DbLocked = 3;
+    public static final AccessMode DbLocked = AccessMode.DbLocked;
 
 
     /**
@@ -172,17 +172,14 @@ public interface Database
      */
     public OQLQuery getOQLQuery();
 
-
     /**
      * Creates an OQL query from the supplied statement.
-     *
-     * @param query An OQL query statement
+     * @param oql An OQL query statement
      * @return An OQL query
-     * @throws QueryException The query syntax is invalid
+     * @throws PersistenceException
      */
     public OQLQuery getOQLQuery( String oql )
-        throws QueryException;
-
+        throws PersistenceException;
 
     /**
      * Creates an empty query. The query must be created before
@@ -198,6 +195,7 @@ public interface Database
 
     /**                              
      * Return the name of the database
+     * @return The database name.
      */                               
     public String getDatabaseName(); 
 
@@ -209,9 +207,9 @@ public interface Database
      * single object. If the identity spans on more than one field, all
      * of the identity fields can be wrapped in a Complex object.
      * 
-     *
      * @param type The object's type
      * @param identity The object's identity
+     * @return The object instance.
      * @throws ObjectNotFoundException No object of the given type and
      *  identity was found in persistent storage
      * @throws LockNotGrantedException Timeout or deadlock occured
@@ -221,16 +219,24 @@ public interface Database
      * @throws PersistenceException An error reported by the
      *  persistence engine
      */
-    public Object load( Class type, Object identity )
-            throws TransactionNotInProgressException, ObjectNotFoundException,
-            LockNotGrantedException, PersistenceException;
+    public Object load(final Class type, final Object identity)
+    throws TransactionNotInProgressException, ObjectNotFoundException,
+    LockNotGrantedException, PersistenceException;
 
     /**
-     * Load an object of the specified type and given identity which
-     * spans on more than one fields.
+     * <p>
+     * Load an object of the specified type and given identity.
+     * Once loaded the object is persistent. Calling this method with
+     * the same identity in the same transaction will return the same
+     * object. This method is equivalent to a query that returns a
+     * single object.
      *
+     * @deprecated Use load(Class, Object, AccessMode) instead.
+     * 
      * @param type The object's type
      * @param identity The object's identity
+     * @param accessMode The access mode
+     * @return The object instance.
      * @throws ObjectNotFoundException No object of the given type and
      *  identity was found in persistent storage
      * @throws LockNotGrantedException Timeout or deadlock occured
@@ -240,12 +246,12 @@ public interface Database
      * @throws PersistenceException An error reported by the
      *  persistence engine
      */
-    public Object load( Class type, Complex identity )
-        throws ObjectNotFoundException, LockNotGrantedException,
-               TransactionNotInProgressException, PersistenceException;
+    public Object load(final Class type, final Object identity,
+                       final short accessMode)
+    throws TransactionNotInProgressException, ObjectNotFoundException,
+    LockNotGrantedException, PersistenceException;
 
     /**
-     * <b>Experimental</b>
      * <p>
      * Load an object of the specified type and given identity.
      * Once loaded the object is persistent. Calling this method with
@@ -255,7 +261,8 @@ public interface Database
      *
      * @param type The object's type
      * @param identity The object's identity
-     * @param accessMode The access mode
+     * @param mode The access mode
+     * @return The object instance.
      * @throws ObjectNotFoundException No object of the given type and
      *  identity was found in persistent storage
      * @throws LockNotGrantedException Timeout or deadlock occured
@@ -265,39 +272,12 @@ public interface Database
      * @throws PersistenceException An error reported by the
      *  persistence engine
      */
-    public Object load( Class type, Object identity, short accessMode )
-            throws TransactionNotInProgressException, ObjectNotFoundException,
-            LockNotGrantedException, PersistenceException;
-
+    public Object load(final Class type, final Object identity,
+                       final AccessMode mode) 
+    throws TransactionNotInProgressException, ObjectNotFoundException,
+    LockNotGrantedException, PersistenceException;
 
     /**
-     * <b>Experimental</b>
-     * <p>
-     * Load an object of the specified type and given identity.
-     * Once loaded the object is persistent. Calling this method with
-     * the same identity in the same transaction will return the same
-     * object. This method is equivalent to a query that returns a
-     * single object. If the identity spans on more than one field, all
-     * of the identity fields can be wrapped in a Complex object.
-     *
-     * @param type The object's type
-     * @param identity The object's identity
-     * @param accessMode The access mode
-     * @throws ObjectNotFoundException No object of the given type and
-     *  identity was found in persistent storage
-     * @throws LockNotGrantedException Timeout or deadlock occured
-     *  attempting to acquire a lock on the object
-     * @throws TransactionNotInProgressException Method called while
-     *   transaction is not in progress
-     * @throws PersistenceException An error reported by the
-     *  persistence engine
-     */
-    public Object load( Class type, Complex identity, short accessMode )
-        throws ObjectNotFoundException, LockNotGrantedException,
-               TransactionNotInProgressException, PersistenceException;
-
-    /**
-     * <b>Experimental</b>
      * <p>
      * Load an object of the specified type and given identity into 
      * a given instance of object.
@@ -311,6 +291,7 @@ public interface Database
      * @param type The object's type
      * @param identity The object's identity
      * @param object The object instance to be loaded into
+     * @return The object instance.
      * @throws ObjectNotFoundException No object of the given type and
      *  identity was found in persistent storage
      * @throws LockNotGrantedException Timeout or deadlock occured
@@ -320,9 +301,10 @@ public interface Database
      * @throws PersistenceException An error reported by the
      *  persistence engine
      */
-    public Object load( Class type, Object identity, Object object )
-        throws ObjectNotFoundException, LockNotGrantedException,
-               TransactionNotInProgressException, PersistenceException;
+    public Object load(final Class type, final Object identity,
+                       final Object object)
+    throws ObjectNotFoundException, LockNotGrantedException,
+    TransactionNotInProgressException, PersistenceException;
 
     /**
      * Creates a new object in persistent storage. The object will be
@@ -453,6 +435,7 @@ public interface Database
      * all reachable object if the object is not loaded from the
      * transaction. If it is turn off, only dependent object will 
      * be created automatically.
+     * @return True if the current transaction is set to 'autoStore'.
      */
     public boolean isAutoStore();
 
@@ -475,6 +458,7 @@ public interface Database
      * If autoStore is not set, Castor will only create/update/store
      * dependent object, and related objects must be created/update 
      * explicitly.
+     * @param autoStore True if this feature should be enabled.
      */
     public void setAutoStore( boolean autoStore );
 
@@ -583,33 +567,23 @@ public interface Database
      * ClassLoader's instance has been explicitely set.
      */
     public ClassLoader getClassLoader ();
-    
 
     /**
-     * @deprecated See {@link #create}
+     * Get's the CacheManager instance.
+     * Call getCacheManager for every Database-instances.
+     * @return the CacheManager-instance.
      */
-    public void makePersistent( Object object )
-        throws ClassNotPersistenceCapableException,
-               DuplicateIdentityException, PersistenceException;
-
+    public CacheManager getCacheManager();
 
     /**
-     * @deprecated See {@link #remove}
+     * Gets the underlying JDBC connection.
+     * 
+     * This is for <b>advanced</b> use only. Please make sure that you <b>never</b>
+     * close this Connection instance, as it will be closed by Castor.
+     * 
+     * @return the underlying JDBC connection, if present; otherwise null 
+     * @throws PersistenceException If the underlying JDBC connection cannot be obtained.
      */
-    public void deletePersistent( Object object )
-        throws ObjectNotPersistentException, LockNotGrantedException, 
-               PersistenceException;
-
-
-    /**
-     * @deprecated Use {@link #commit} and {@link #rollback} instead;
-     *  this method cannot be implemented properly with multiple type
-     *  of locks and will not be supported in future versions of the
-     *  API
-     */
-    public void checkpoint()
-        throws TransactionNotInProgressException, TransactionAbortedException;
-
-
+    public java.sql.Connection getJdbcConnection() throws PersistenceException;
 }
 
