@@ -1,52 +1,25 @@
 
-
 package org.exolab.castor.jdo;
 
-
-
 /**
-
  * This exception is thrown when accessing an object that was deleted.
-
  *
-
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
-
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:08:07 $
-
+ * @version $Revision: 1.3 $ $Date: 2004/10/05 20:24:28 $
  */
-
 public class ObjectDeletedException
-
     extends ObjectNotPersistentException
-
 {
-
     
-
-
-
-    /**
-
-     * Construct an instance of the exception.
-
-     *
-
-     * @param msg A string providing a description of the exception.
-
-     */
-
     public ObjectDeletedException( String message )
-
     {
-
         super( message );
-
     }
 
+    public ObjectDeletedException( String message, Throwable exception )
+    {
+        super( message, exception );
+    }
 
-
-
-
+    
 } 
-
