@@ -3,9 +3,7 @@
 package org.exolab.castor.jdo.drivers;
 
 
-import java.sql.SQLException;
 import org.exolab.castor.persist.spi.QueryExpression;
-import org.exolab.castor.persist.spi.PersistenceQuery;
 
 
 /**
