@@ -40,61 +40,63 @@
  *
  * Copyright 1999 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: DatabaseImpl.java,v 1.1.1.1 2003/03/03 07:08:16 kvisco Exp $
+ * $Id: DatabaseImpl.java,v 1.27 2005/07/24 17:48:18 rjoachim Exp $
  */
-
-
 package org.exolab.castor.jdo.engine;
 
+import java.sql.Connection;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.StringTokenizer;
+
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+
+import org.castor.jdo.engine.AbstractConnectionFactory;
+import org.castor.jdo.engine.DatabaseRegistry;
+
+import org.castor.persist.ProposedObject;
+import org.castor.persist.TransactionContext;
+import org.exolab.castor.jdo.*;
+import org.exolab.castor.mapping.AccessMode;
+import org.exolab.castor.mapping.MappingException;
+import org.exolab.castor.persist.*;
+import org.exolab.castor.persist.spi.CallbackInterceptor;
+import org.exolab.castor.persist.spi.InstanceFactory;
+import org.exolab.castor.util.LocalConfiguration;
+import org.exolab.castor.util.Messages;
 
 import javax.transaction.Status;
-import javax.transaction.Transaction;
-import javax.transaction.SystemException;
 import javax.transaction.Synchronization;
-import org.exolab.castor.jdo.Database;
-import org.exolab.castor.jdo.Query;
-import org.exolab.castor.jdo.OQLQuery;
-import org.exolab.castor.jdo.DatabaseNotFoundException;
-import org.exolab.castor.jdo.ObjectNotPersistentException;
-import org.exolab.castor.jdo.ObjectModifiedException;
-import org.exolab.castor.jdo.ObjectNotFoundException;
-import org.exolab.castor.jdo.DuplicateIdentityException;
-import org.exolab.castor.jdo.ClassNotPersistenceCapableException;
-import org.exolab.castor.jdo.TransactionNotInProgressException;
-import org.exolab.castor.jdo.LockNotGrantedException;
-import org.exolab.castor.jdo.PersistenceException;
-import org.exolab.castor.jdo.TransactionAbortedException;
-import org.exolab.castor.jdo.QueryException;
-import org.exolab.castor.mapping.MappingException;
-import org.exolab.castor.mapping.AccessMode;
-import org.exolab.castor.persist.ClassMolder;
-import org.exolab.castor.persist.TransactionContext;
-import org.exolab.castor.persist.LockEngine;
-import org.exolab.castor.persist.PersistenceInfo;
-import org.exolab.castor.persist.PersistenceInfoGroup;
-import org.exolab.castor.persist.spi.LogInterceptor;
-import org.exolab.castor.persist.spi.InstanceFactory;
-import org.exolab.castor.persist.spi.CallbackInterceptor;
-import org.exolab.castor.persist.spi.Complex;
-import org.exolab.castor.util.Messages;
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.Set;
-
-
-
+import javax.transaction.SystemException;
+import javax.transaction.Transaction;
 
 /**
  * An implementation of the JDO database supporting explicit transaction
- * demaracation.
+ * demarcation.
  *
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:08:16 $
+ * @author <a href="mailto:ferret AT frii dot com">Bruce Snyder</a>
+ * @version $Revision: 1.27 $ $Date: 2005/07/24 17:48:18 $
  */
 public class DatabaseImpl
     implements Database, Synchronization
 {
 
+    /**
+     * Property listing all the available {@link TxSynchronizable}
+     * implementations (<tt>org.exolab.castor.persit.TxSynchronizable</tt>).
+     */
+    private static final String TxSynchronizableProperty = 
+        "org.exolab.castor.persist.TxSynchronizable";
+
+
+    /**
+     * The <a href="http://jakarta.apache.org/commons/logging/">Jakarta
+     * Commons Logging</a> instance used for all logging.
+     */
+    private static Log _log = LogFactory.getFactory().getInstance( DatabaseImpl.class );
+
 
     /**
      * The database engine used to access the underlying SQL database.
@@ -110,6 +112,13 @@ public class DatabaseImpl
     protected TransactionContext       _ctx;
 
 
+    /**
+     * List of TxSynchronizeable implementations that should all be
+     * informed about changes after commit of transactions. 
+     */
+    private ArrayList                  _synchronizables;
+
+
     /**
      * The lock timeout for this database. Zero for immediate
      * timeout, an infinite value for no timeout. The timeout is
@@ -118,12 +127,6 @@ public class DatabaseImpl
     private int                        _lockTimeout;
 
 
-    /**
-     * The log interceptor to which all logging and tracing messages will be sent.
-     */
-    private LogInterceptor             _logInterceptor;
-
-
     /**
      * The default callback interceptor for transaction
      */
@@ -147,7 +150,7 @@ public class DatabaseImpl
     private Transaction                _transaction;
 
 
-    /*
+    /**
      * True if user prefer all reachable object to be stored automatically.
      * False if user want only dependent object to be stored.
      */
@@ -164,40 +167,54 @@ public class DatabaseImpl
      * The transaction to database map
      */
     private TxDatabaseMap              _txMap;
+    
+    /**
+	 * {@link CacheManager} instance.
+	 */
+    private CacheManager cacheManager;
 
-
-    public DatabaseImpl( String dbName, int lockTimeout,
-            LogInterceptor logInterceptor, CallbackInterceptor callback,
-            InstanceFactory instanceFactory, Transaction transaction, 
-            ClassLoader classLoader, boolean autoStore )
-            throws DatabaseNotFoundException {
-        // Locate a suitable datasource and database engine
+    public DatabaseImpl( String dbName, int lockTimeout, CallbackInterceptor callback,
+                         InstanceFactory instanceFactory, Transaction transaction, 
+                         ClassLoader classLoader, boolean autoStore )
+    throws DatabaseNotFoundException {
+        _autoStore = autoStore;
+        
+        // Locate a suitable ConnectionFactory and LockEngine
         // and report if not mapping registered any of the two.
         // A new ODMG engine is created each time with different
         // locking mode.
-        DatabaseRegistry dbs;
-
-        _autoStore = autoStore;
-        dbs = DatabaseRegistry.getDatabaseRegistry( dbName );
-        if ( dbs == null )
-            throw new DatabaseNotFoundException( Messages.format( "jdo.dbNoMapping", dbName ) );
-        LockEngine[] pe = { DatabaseRegistry.getLockEngine( dbs ) };
-        _scope = new PersistenceInfoGroup( pe );
-        _logInterceptor = logInterceptor;
+        AbstractConnectionFactory factory = null;
+        try {
+            factory = DatabaseRegistry.getConnectionFactory(dbName);
+        } catch (MappingException ex) {
+            throw new DatabaseNotFoundException(Messages.format("jdo.dbNoMapping", dbName));
+        }
+        _scope = new PersistenceInfoGroup(new LockEngine[] { factory.getEngine() });
+        
         _callback = callback;
         _instanceFactory = instanceFactory;
         _dbName = dbName;
         _lockTimeout = lockTimeout;
 
         _transaction = transaction;
-        if ( _transaction != null ) {
-            _ctx = new TransactionContextImpl( this, true );
-            _ctx.setLockTimeout( _lockTimeout );
-            _ctx.setAutoStore( _autoStore );
-            _ctx.setCallback( _callback );
-            _ctx.setInstanceFactory( _instanceFactory );
+	
+		if (_transaction != null) {
+			try {
+            	_ctx = new TransactionContextImpl(this , true , transaction );
+			}
+			catch ( javax.transaction.SystemException se ) {
+				throw new DatabaseNotFoundException( se );
+			}
+		} else {
+            _ctx = new TransactionContextImpl(this, false );
         }
+        _ctx.setLockTimeout(_lockTimeout);
+        _ctx.setAutoStore(_autoStore);
+        _ctx.setCallback(_callback);
+        _ctx.setInstanceFactory(_instanceFactory);
         _classLoader = classLoader;
+        
+        loadSynchronizables();
     }
 
     LockEngine getLockEngine()
@@ -209,24 +226,26 @@ public class DatabaseImpl
         return _scope;
     }
 
-    /*
+    /**
      * True if user prefer all reachable object to be stored automatically.
      * False if user want only dependent object to be stored.
+     * @param autoStore True to indicate that 'autoStore' mode should be used.
      */
     public void setAutoStore( boolean autoStore ) {
         _autoStore = autoStore;
     }
 
-    /*
+    /**
      * Return if the current transaction is set to autoStore, it there is
      * transaction active. If there is no active transaction, return if
      * the next transaction will be set to autoStore.
+     * @return True if 'auto-store' mode is in use.
      */
     public boolean isAutoStore() {
         if ( _ctx != null )
             return _ctx.isAutoStore();
-        else
-            return _autoStore;
+
+        return _autoStore;
     }
 
     /**
@@ -242,6 +261,7 @@ public class DatabaseImpl
 
     /**                              }
      * Return the name of the database
+     * @return Name of the database.
      */                               
     public String getDatabaseName()  
     {                                
@@ -282,74 +302,47 @@ public class DatabaseImpl
     }
 
 
-    public boolean isClosed()
-    {
+    public boolean isClosed() {
         return ( _scope == null );
     }
 
-    public Object load( Class type, Object identity, Object object ) 
-            throws TransactionNotInProgressException, ObjectNotFoundException,
-            LockNotGrantedException, PersistenceException {
+    public Object load(final Class type, final Object identity) 
+    throws ObjectNotFoundException, LockNotGrantedException, 
+    TransactionNotInProgressException, PersistenceException {
+        return load(type, identity, null, null);
+    }
 
-        TransactionContext tx;
-        PersistenceInfo    info;
-
-        tx = getTransaction();
-        info = _scope.getPersistenceInfo( type );
-        if ( info == null )
-            throw new ClassNotPersistenceCapableException( Messages.format( "persist.classNotPersistenceCapable", type.getName() ) );
-
-        return tx.load( info.engine, info.molder, identity, object, null );
+    public Object load(final Class type, final Object identity,
+                       final Object object) 
+    throws TransactionNotInProgressException, ObjectNotFoundException,
+    LockNotGrantedException, PersistenceException {
+        return load(type, identity, object, null);
     }
-    public Object load( Class type, Complex identity )
-            throws TransactionNotInProgressException, ObjectNotFoundException,
-            LockNotGrantedException, PersistenceException {
     
-        return load( type, (Object)identity, null );
+    public Object load(final Class type, final Object identity,
+                       final short accessMode)
+    throws ObjectNotFoundException, LockNotGrantedException,
+    TransactionNotInProgressException, PersistenceException {
+        AccessMode mode = AccessMode.valueOf(accessMode);
+        return load(type, identity, null, mode);
     }
 
-    public Object load( Class type, Object identity ) 
-            throws ObjectNotFoundException, LockNotGrantedException, 
-            TransactionNotInProgressException, PersistenceException {
-
-        return load( type, identity, null );
+    public Object load(final Class type, final Object identity,
+                       final AccessMode mode) 
+    throws TransactionNotInProgressException, ObjectNotFoundException,
+    LockNotGrantedException, PersistenceException {
+        return load(type, identity, null, mode);
     }
 
-    public Object load( Class type, Complex identity, short accessMode )
-            throws TransactionNotInProgressException, ObjectNotFoundException,
-            LockNotGrantedException, PersistenceException {
+    private Object load(final Class type, final Object identity,
+                        final Object object, final AccessMode mode) 
+    throws TransactionNotInProgressException, ObjectNotFoundException,
+    LockNotGrantedException, PersistenceException {
+        TransactionContext tx = getTransaction();
+        PersistenceInfo info = _scope.getPersistenceInfo(type);
 
-        return load( type, (Object)identity, accessMode );
-    }
-
-    public Object load( Class type, Object identity, short accessMode) throws ObjectNotFoundException, LockNotGrantedException, TransactionNotInProgressException, PersistenceException {
-        TransactionContext tx;
-        PersistenceInfo    info;
-        AccessMode         mode;
-
-        switch ( accessMode ) {
-        case ReadOnly:
-            mode = AccessMode.ReadOnly;
-            break;
-        case Shared:
-            mode = AccessMode.Shared;
-            break;
-        case Exclusive:
-            mode = AccessMode.Exclusive;
-            break;
-        case DbLocked:
-            mode = AccessMode.DbLocked;
-            break;
-        default:
-            throw new IllegalArgumentException( "Value for 'accessMode' is invalid" );
-        }
-
-        tx = getTransaction();
-        info = _scope.getPersistenceInfo( type );
-        if ( info == null )
-            throw new ClassNotPersistenceCapableException( Messages.format("persist.classNotPersistenceCapable", type.getName()) );
-        
-        return tx.load( info.engine, info.molder, identity, null, mode );
+        ProposedObject proposedObject = new ProposedObject();
+        return tx.load( info.engine, info.molder, identity, proposedObject, mode );
     }
 
     public void create( Object object )
@@ -360,11 +353,18 @@ public class DatabaseImpl
 
         tx = getTransaction();
         info = _scope.getPersistenceInfo( object.getClass() );
-        if ( info == null )
-            throw new ClassNotPersistenceCapableException( Messages.format("persist.classNotPersistenceCapable", object.getClass().getName()) );
 
         tx.create( info.engine, info.molder, object, null );
     }
+    
+    /**
+     * @see Database#getCacheManager()
+     */
+    public CacheManager getCacheManager() {
+        if(cacheManager == null)
+            cacheManager = new CacheManager(this, _ctx, getLockEngine());
+        return cacheManager;
+    }
 
     public void update( Object object )
         throws ClassNotPersistenceCapableException, ObjectModifiedException,
@@ -375,24 +375,11 @@ public class DatabaseImpl
 
         tx = getTransaction();
         info = _scope.getPersistenceInfo( object.getClass() );
-        if ( info == null )
-            throw new ClassNotPersistenceCapableException( Messages.format("persist.classNotPersistenceCapable", object.getClass().getName()) );
 
         tx.update( info.engine, info.molder, object, null );
     }
 
 
-    /**
-     * @deprecated
-     */
-    public synchronized void makePersistent( Object object )
-        throws ClassNotPersistenceCapableException, DuplicateIdentityException,
-               TransactionNotInProgressException, PersistenceException
-    {
-        create( object );
-    }
-
-
     public void remove( Object object )
         throws ObjectNotPersistentException, LockNotGrantedException, 
                TransactionNotInProgressException, PersistenceException
@@ -402,27 +389,13 @@ public class DatabaseImpl
         
         tx = getTransaction();
         info = _scope.getPersistenceInfo( object.getClass() );
-        if ( info == null )
-            throw new ClassNotPersistenceCapableException( Messages.format("persist.classNotPersistenceCapable", object.getClass().getName()) );
 
         tx.delete( object );
     }
 
 
-    /**
-     * @deprecated
-     */
-    public synchronized void deletePersistent( Object object )
-        throws ObjectNotPersistentException, LockNotGrantedException, 
-               PersistenceException
-    {
-        remove( object );
-    }
-
     public boolean isPersistent( Object object )
     {
-        TransactionContext tx;
-        
         if ( _scope == null )
             throw new IllegalStateException( Messages.message( "jdo.dbClosed" ) );
         if ( _ctx != null && _ctx.isOpen()  )
@@ -432,8 +405,6 @@ public class DatabaseImpl
 
     public Object getIdentity(Object object)
     {
-        TransactionContext tx;
-        
         if ( _scope == null )
             throw new IllegalStateException( Messages.message( "jdo.dbClosed" ) );
         if ( _ctx != null && _ctx.isOpen()  )
@@ -459,7 +430,7 @@ public class DatabaseImpl
 
 
     public OQLQuery getOQLQuery( String oql )
-        throws QueryException
+        throws PersistenceException
     {
         OQLQuery oqlImpl;
 
@@ -475,19 +446,32 @@ public class DatabaseImpl
     }
 
 
-    protected void finalize()
-        throws Throwable
-    {
-        if ( _scope != null )
-            close();
-    }
+	/**
+     * Overrides Object.finalize().
+	 * 
+	 * Outputs a warning message to teh logs if the current DatabaseImpl 
+	 * instance still has valid scope. In this condition - a condition that 
+	 * ideally should not occur at all - we close the instance as well to 
+	 * free up resources.
+	 * 
+	 * @see java.lang.Object#finalize()
+	 */
+	protected void finalize() throws Throwable {
+		if (_scope != null) {
+			
+			// retrieve SQL bound to this Database instance
+			OQLQuery oqlQuery = getOQLQuery(); 
+			String sql = ((OQLQueryImpl) oqlQuery).getSQL(); 
+			
+			_log.warn(Messages.format("jdo.finalize_close", this.toString(), _dbName, sql));
+			close();
+		}
+	}
 
 
     protected TransactionContext getTransaction()
         throws TransactionNotInProgressException
     {
-        TransactionContext tx;
-        
         if ( _scope == null )
             throw new TransactionNotInProgressException( Messages.message( "jdo.dbClosed" ) );
         if ( _ctx != null && _ctx.isOpen()  )
@@ -499,23 +483,29 @@ public class DatabaseImpl
     public void begin()
         throws PersistenceException
     {
+        _log.debug( "Beginning tx" );
+
         if ( _transaction != null )
             throw new IllegalStateException( Messages.message( "jdo.txInJ2EE" ) );
 
         if ( _ctx != null && _ctx.isOpen() )
             throw new PersistenceException( Messages.message( "jdo.txInProgress" ) );
 
-        _ctx = new TransactionContextImpl( this, false );
+        //_ctx = new TransactionContextImpl( this, false );
+        ((TransactionContextImpl) _ctx).setStatusActive();
         _ctx.setLockTimeout( _lockTimeout );
         _ctx.setAutoStore( _autoStore );
         _ctx.setCallback( _callback );
         _ctx.setInstanceFactory( _instanceFactory );
+
+        registerSynchronizables();
     }
 
 
     public void commit()
         throws TransactionNotInProgressException, TransactionAbortedException
     {
+        _log.debug( "Committing tx" );
 
         if ( _transaction != null )
             throw new IllegalStateException( Messages.message( "jdo.txInJ2EE" ) );
@@ -523,26 +513,34 @@ public class DatabaseImpl
         if ( _ctx == null || ! _ctx.isOpen() )
             throw new TransactionNotInProgressException( Messages.message( "jdo.txNotInProgress" ) );
         if ( _ctx.getStatus() == Status.STATUS_MARKED_ROLLBACK )
-            throw new TransactionAbortedException( Messages.message( "jdo.txAborted" ) );
+            throw new TransactionAbortedException( Messages.message( "jdo.txRollback" ) );
         try {
             _ctx.prepare();
             _ctx.commit();
         } catch ( TransactionAbortedException except ) {
+            _log.error( Messages.format( "jdo.txAborted", except.getMessage() ), except );
             _ctx.rollback();
             throw except;
         } finally {
-             try {
-                _ctx.close();
-            } catch (Exception ex) {
-            }
-           _ctx = null;
+            try {
+                // TODO [SMH]: Temporary fix, see bug 1491.
+                if(_ctx.isOpen()) {
+                    _ctx.close();
+                }
+           } catch (Exception e) {
+               _log.info(e.getMessage(), e);
+           }
         }
+
+        unregisterSynchronizables();
     }
 
 
     public void rollback()
         throws TransactionNotInProgressException
     {
+        _log.debug( "Rolling back tx" );
+
         if ( _transaction != null )
             throw new IllegalStateException( Messages.message( "jdo.txInJ2EE" ) );
 
@@ -550,33 +548,32 @@ public class DatabaseImpl
         if ( _ctx == null || ! _ctx.isOpen() )
             throw new TransactionNotInProgressException( Messages.message( "jdo.txNotInProgress" ) );
         _ctx.rollback();
-        _ctx = null;
+
+        unregisterSynchronizables();
     }
 
 
     public void beforeCompletion()
     {
+        // XXX [SMH]: Find another test for txNotInProgress
         if ( _transaction == null || _ctx == null || ! _ctx.isOpen() )
             throw new IllegalStateException( Messages.message( "jdo.txNotInProgress" ) );
         if ( _ctx.getStatus() == Status.STATUS_MARKED_ROLLBACK ) {
             try {
                 _transaction.setRollbackOnly();
             } catch ( SystemException except ) {
-                if ( _logInterceptor != null )
-                    _logInterceptor.exception( except );
+                _log.warn( Messages.format( "jdo.warnException", except ) );
             }
             return;
         }
         try {
             _ctx.prepare();
-        } catch ( TransactionAbortedException except ) {
-            if ( _logInterceptor != null )
-                _logInterceptor.exception( except );
+        } catch ( TransactionAbortedException tae ) {
+            _log.error( Messages.format( "jdo.txAbortedMarkRollback", tae.getMessage() ), tae );        
             try {
                 _transaction.setRollbackOnly();
-            } catch ( SystemException except2 ) {
-                if ( _logInterceptor != null )
-                    _logInterceptor.exception( except2 );
+            } catch ( SystemException se ) {
+                _log.fatal( Messages.format( "jdo.txMarkRollbackFailure", se.getMessage() ), se );
             }
             _ctx.rollback();
         }
@@ -586,6 +583,7 @@ public class DatabaseImpl
     public void afterCompletion( int status )
     {
         try {
+            // XXX [SMH]: Find another test for txNotInProgress
             if ( _transaction == null || _ctx == null )
                 throw new IllegalStateException( Messages.message( "jdo.txNotInProgress" ) );
             if ( _ctx.getStatus() == Status.STATUS_ROLLEDBACK )
@@ -597,19 +595,15 @@ public class DatabaseImpl
                 try {
                     _ctx.commit();
                 } catch ( TransactionAbortedException except ) {
-                    if ( _logInterceptor != null )
-                        _logInterceptor.exception( except );
+                    _log.fatal( Messages.format( "jdo.fatalException", except ) );
                     _ctx.rollback();
                 }
-                _ctx = null;
                 return;
             case Status.STATUS_ROLLEDBACK:
                 _ctx.rollback();
-                _ctx = null;
                 return;
             default:
                 _ctx.rollback();
-                _ctx = null;
                 throw new IllegalStateException( "Unexpected state: afterCompletion called with status " + status );
             }
         } finally {
@@ -631,30 +625,109 @@ public class DatabaseImpl
     }
 
 
-    /**
-     * @deprecated Use {@link #commit} and {@link #rollback} instead
-     */
-    public void checkpoint()
-        throws TransactionNotInProgressException, TransactionAbortedException
-    {
-    }
-
-
     public String toString() {
         return super.toString()+":"+_dbName;
     }
 
-
-
     /**
      * Get the underlying JDBC Connection.
-     * Only for internal / advanced use !
-     * Never try to close it (is done by castor).
+     * 
+     * This is for <b>advanced</b> use only. Please make sure that you <b>never</b>
+     * close this Connection instance, as it will be closed by Castor.
+     * 
+     * @return the underlying JDBC connection, if possible
+     * @throws PersistenceException If the underlying JDBC connection cannot be obtained.
      */
-    public Object /* java.sql.Connection */ getConnection()
-            throws org.exolab.castor.jdo.PersistenceException
+    public Connection getJdbcConnection() throws PersistenceException 
     {
         return _ctx.getConnection( _scope.getLockEngine() );
     }
 
-}                                
\ No newline at end of file
+    /**
+     * Load the {@link TxSynchronizable} implementations from the 
+     * properties file, if not loaded before.
+     */
+    private void loadSynchronizables()
+    {
+    	if ( _synchronizables == null ) {
+    		_synchronizables = new ArrayList();
+    		
+    		String syncName = LocalConfiguration.getInstance().getProperty( TxSynchronizableProperty, "" );
+    		StringTokenizer tokenizer = new StringTokenizer( syncName, ", " );
+    		while ( tokenizer.hasMoreTokens() ) {
+    			syncName = tokenizer.nextToken();
+    			try {
+                	Class cls = null;
+                	if (_classLoader != null) {
+                		cls = _classLoader.loadClass( syncName );
+                	} else {
+                		cls = Class.forName( syncName );
+                	}
+    				TxSynchronizable sync = (TxSynchronizable)cls.newInstance();
+    				if ( sync != null ) _synchronizables.add(sync);
+    			} catch ( Exception except ) {
+    				_log.warn(Messages.format( "jdo.missingTxSynchronizable", syncName ));
+    			}
+    		}
+    		
+    		if (_synchronizables.size() == 0) _synchronizables = null;
+    	}
+    }
+    
+    /**
+     * Register the {@link TxSynchronizable} implementations at the
+     * TransactionContect at end of begin().
+     */
+    private void registerSynchronizables()
+    {
+    	if ( _synchronizables != null && _synchronizables.size() > 0) {
+    		Iterator iter = _synchronizables.iterator();
+    		while ( iter.hasNext() ) {
+    			_ctx.addTxSynchronizable((TxSynchronizable)iter.next());
+    		}
+    	}
+    }
+    
+    /**
+     * Unregister the {@link TxSynchronizable} implementations at the
+     * TransactionContect after commit() or rollback().
+     */
+    private void unregisterSynchronizables()
+    {
+    	if ( _synchronizables != null  && _synchronizables.size() > 0) {
+    		Iterator iter = _synchronizables.iterator();
+    		while ( iter.hasNext() ) {
+    			_ctx.removeTxSynchronizable((TxSynchronizable)iter.next());
+    		}
+    	}
+    }
+    /**
+     * Expire objects from the cache.  Objects expired from the cache will be
+     * read from persistent storage, as opposed to being read from the
+     * performance cache, during subsequent load/query operations.
+     *
+     * Objects may be expired from the cache individually, using explicit
+     * type/identity pairs in the argument list, or whole classes of objects
+     * may be expired by specifying a class type without a corresponding 
+     * entry in the identity array.
+     *
+     * Objects contained within a "master" object, for example objects
+     * maintained in a one-to-many relationship, will automatically be expired
+     * from the cache without the need to explicitly identify them.  This does
+     * not apply when expiring objects by type.  Each type, both container and
+     * contained objects need to be specified.
+     * 
+     * @param type An array of class types.
+     * @param identity An array of object identifiers.
+     * @throws PersistenceException Indicates that a problem has occured expiring objects from the cache. 
+     * @deprecated Please use the new {@link org.exolab.castor.jdo.CacheManager} which can be 
+     * obtained by calling {@link #getCacheManager()}.
+     */
+    public void expireCache( Class[] type, Object[] identity )
+        throws PersistenceException 
+    {
+    	throw new PersistenceException ("Please use the new CacheManager to manage Castor performance caches.");
+    }
+    
+}  
+                                
