@@ -40,59 +40,36 @@
  *
  * Copyright 1999 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: OQLQueryImpl.java,v 1.3 2003/04/16 21:53:48 mguessan Exp $
+ * $Id: OQLQueryImpl.java,v 1.15 2005/06/20 12:00:55 wguttmn Exp $
  */
 
 
 package org.exolab.castor.jdo.engine;
 
 
-import java.util.Vector;
-import java.util.Hashtable;
-import java.util.StringTokenizer;
-import java.util.Enumeration;
-import java.util.NoSuchElementException;
-import java.sql.PreparedStatement;
-import java.sql.SQLException;
-import org.exolab.castor.jdo.Query;
-import org.exolab.castor.jdo.OQLQuery;
-import org.exolab.castor.jdo.QueryResults;
-import org.exolab.castor.jdo.Database;
-import org.exolab.castor.jdo.QueryException;
-import org.exolab.castor.jdo.TransactionNotInProgressException;
-import org.exolab.castor.jdo.QueryException;
-import org.exolab.castor.jdo.ObjectNotFoundException;
-import org.exolab.castor.jdo.PersistenceException;
-import org.exolab.castor.jdo.LockNotGrantedException;
-import org.exolab.castor.jdo.oql.Lexer;
-import org.exolab.castor.jdo.oql.Parser;
-import org.exolab.castor.jdo.oql.Token;
-import org.exolab.castor.jdo.oql.TokenTypes;
-import org.exolab.castor.jdo.oql.ParseTreeNode;
-import org.exolab.castor.jdo.oql.ParseTreeWalker;
-import org.exolab.castor.jdo.oql.ParamInfo;
-import org.exolab.castor.persist.TransactionContext;
+import org.castor.persist.TransactionContext;
+import org.exolab.castor.jdo.*;
+import org.exolab.castor.jdo.oql.*;
+import org.exolab.castor.mapping.AccessMode;
+import org.exolab.castor.mapping.FieldHandler;
+import org.exolab.castor.mapping.MappingException;
+import org.exolab.castor.mapping.TypeConvertor;
 import org.exolab.castor.persist.ClassMolder;
 import org.exolab.castor.persist.LockEngine;
-import org.exolab.castor.mapping.AccessMode;
-import org.exolab.castor.mapping.FieldDescriptor;
-import org.exolab.castor.mapping.FieldHandler;
-import org.exolab.castor.mapping.AccessMode;
-import org.exolab.castor.mapping.TypeConvertor;
-import org.exolab.castor.mapping.MappingException;
-import org.exolab.castor.mapping.loader.Types;
 import org.exolab.castor.persist.spi.PersistenceQuery;
 import org.exolab.castor.persist.spi.QueryExpression;
 import org.exolab.castor.util.Messages;
-import org.exolab.castor.util.Logger;
-import org.exolab.castor.persist.OID;
-import org.exolab.castor.persist.spi.Complex;
+
+import java.util.Enumeration;
+import java.util.Hashtable;
+import java.util.NoSuchElementException;
+import java.util.Vector;
 
 /**
  *
  *
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
- * @version $Revision: 1.3 $ $Date: 2003/04/16 21:53:48 $
+ * @version $Revision: 1.15 $ $Date: 2005/06/20 12:00:55 $
  */
 public class OQLQueryImpl
     implements Query, OQLQuery
@@ -146,7 +123,6 @@ public class OQLQueryImpl
 
     public void bind( Object value )
     {
-
         if ( _expr == null && _spCall == null )
             throw new IllegalStateException( "Must create query before using it" );
         if ( _fieldNum == _paramInfo.size() )
@@ -203,16 +179,10 @@ public class OQLQueryImpl
             if ( _bindValues == null )
                 _bindValues = new Object[ _bindTypes.length ];
 
-            for (Enumeration e = info.getParamMap().elements(); e.hasMoreElements(); )
-            {
-                int fieldNum = ( (Integer) e.nextElement() ).intValue();
-                _bindValues[ fieldNum - 1 ] = value;
-            }
-
+            _bindValues[_fieldNum++] = value;
         } catch ( IllegalArgumentException except ) {
             throw except;
         }
-        ++_fieldNum;
     }
 
     public void bind( boolean value )
@@ -251,7 +221,7 @@ public class OQLQueryImpl
     }
 
     public void create( String oql )
-        throws QueryException
+    throws PersistenceException
     {
 
         _fieldNum = 0;
@@ -272,7 +242,10 @@ public class OQLQueryImpl
         if ( _dbEngine == null )
             throw new QueryException( "Could not get a persistence engine" );
 
-        ParseTreeWalker walker = new ParseTreeWalker(_dbEngine, parseTree, _dbImpl.getClassLoader());
+	    TransactionContext trans = _dbImpl.getTransaction();
+	    DbMetaInfo dbInfo = trans.getConnectionInfo(_dbEngine);
+
+	    ParseTreeWalker walker = new ParseTreeWalker(_dbEngine, parseTree, _dbImpl.getClassLoader(), dbInfo);
 
         _objClass = walker.getObjClass();
         _clsDesc = walker.getClassDescriptor();
@@ -281,33 +254,14 @@ public class OQLQueryImpl
         _projectionType = walker.getProjectionType();
         _pathInfo = walker.getPathInfo();
 
-
-        //port param info types back to the format of old bind types.
-        //first get the maximum SQL param.
-        int max = 0;
+        // create the types array and fill it
+        _bindTypes = new Class[_paramInfo.size()];
+        int paramIndex = 0;
         for (Enumeration e = _paramInfo.elements(); e.hasMoreElements(); ) {
             ParamInfo info = (ParamInfo) e.nextElement();
-            for (Enumeration f = info.getParamMap().elements(); f.hasMoreElements(); )
-            {
-                int paramIndex = ( (Integer) f.nextElement() ).intValue();
-                if (  paramIndex > max )
-                    max = paramIndex;
-            }
-        }
 
-        //then create the types array and fill it
-        _bindTypes = new Class[max];
-        for (Enumeration e = _paramInfo.elements(); e.hasMoreElements(); )
-        {
-            ParamInfo info = (ParamInfo) e.nextElement();
-            for (Enumeration f = info.getParamMap().elements(); f.hasMoreElements(); )
-            {
-                int paramIndex = ( (Integer) f.nextElement() ).intValue();
-                _bindTypes[ paramIndex - 1 ] = (info.getSQLType() == null ? info.getTheClass()
-                                                                          : info.getSQLType());
-            }
+            _bindTypes[paramIndex++] = (info.getSQLType()==null? info.getTheClass(): info.getSQLType());
         }
-
     }
 
     public void createCall( String oql ) throws QueryException {
@@ -365,9 +319,9 @@ public class OQLQueryImpl
                         }
                         info = (ParamInfo) _paramInfo.get( paramNo );
                         if ( info == null ) {
-                            info = new ParamInfo( "", "java.lang.Object", null);
+                            info = new ParamInfo( "", "java.lang.Object", null, _dbImpl.getClassLoader());
                         }
-                        info.mapToSQLParam( paramCnt + 1 );
+                        //info.mapToSQLParam( paramCnt + 1 );
                         _paramInfo.put( paramNo , info );
                         paramCnt++;
 
@@ -409,9 +363,9 @@ public class OQLQueryImpl
                     }
                     info = (ParamInfo) _paramInfo.get( paramNo );
                     if ( info == null ) {
-                        info = new ParamInfo( "", "java.lang.Object", null);
+                        info = new ParamInfo( "", "java.lang.Object", null, _dbImpl.getClassLoader());
                     }
-                    info.mapToSQLParam( paramCnt + 1 );
+                    //info.mapToSQLParam( paramCnt + 1 );
                     _paramInfo.put( paramNo , info );
                     paramCnt++;
                 }
@@ -446,51 +400,33 @@ public class OQLQueryImpl
             throw new QueryException( "Could not find an engine supporting class " + objType );
     }
 
-
     public QueryResults execute()
-        throws QueryException, PersistenceException, TransactionNotInProgressException
-    {
+    throws QueryException, PersistenceException, TransactionNotInProgressException {
         return execute( null );
     }
 
-    public QueryResults execute(boolean scrollable)
-        throws QueryException, PersistenceException, TransactionNotInProgressException
-    {
+    public QueryResults execute(final boolean scrollable)
+    throws QueryException, PersistenceException, TransactionNotInProgressException {
         return execute( null, scrollable );
     }
 
-    public QueryResults execute( short accessMode )
-        throws QueryException, PersistenceException, TransactionNotInProgressException
-    {
-      return execute(accessMode, false);
+    public QueryResults execute(final short accessMode)
+    throws QueryException, PersistenceException, TransactionNotInProgressException {
+        return execute(AccessMode.valueOf(accessMode), false);
     }
 
-    public QueryResults execute( short accessMode,  boolean scrollable )
-        throws QueryException, PersistenceException, TransactionNotInProgressException
-    {
-        switch ( accessMode ) {
-        case Database.ReadOnly:
-            return execute( AccessMode.ReadOnly, scrollable );
-        case Database.Shared:
-            return execute( AccessMode.Shared, scrollable );
-        case Database.Exclusive:
-            return execute( AccessMode.Exclusive, scrollable );
-        case Database.DbLocked:
-            return execute( AccessMode.DbLocked, scrollable );
-        default:
-            throw new IllegalArgumentException( "Value for 'accessMode' is invalid" );
-        }
+    public QueryResults execute(final short accessMode, final boolean scrollable)
+    throws QueryException, PersistenceException, TransactionNotInProgressException {
+        return execute(AccessMode.valueOf(accessMode), scrollable);
     }
 
-    private QueryResults execute( AccessMode accessMode )
-        throws QueryException, PersistenceException, TransactionNotInProgressException
-    {
-      return execute(accessMode, false);
+    public QueryResults execute(final AccessMode accessMode)
+    throws QueryException, PersistenceException, TransactionNotInProgressException {
+        return execute(accessMode, false);
     }
 
-    private QueryResults execute( AccessMode accessMode, boolean scrollable )
-        throws QueryException, PersistenceException, TransactionNotInProgressException
-    {
+    public QueryResults execute(final AccessMode accessMode, final boolean scrollable)
+    throws QueryException, PersistenceException, TransactionNotInProgressException {
         org.exolab.castor.persist.QueryResults      results;
         SQLEngine         engine;
 
@@ -504,7 +440,6 @@ public class OQLQueryImpl
                 case ParseTreeWalker.PARENT_OBJECT:
                 case ParseTreeWalker.DEPENDANT_OBJECT:
                 case ParseTreeWalker.DEPENDANT_OBJECT_VALUE:
-
                     try {
                         engine = (SQLEngine) _dbEngine.getPersistence( _objClass );
                         if ( _expr != null ) {
@@ -527,29 +462,37 @@ public class OQLQueryImpl
                     else
                       _results = new OQLEnumeration( results, _pathInfo, _clsDesc);
                     break;
+
                 case ParseTreeWalker.DEPENDANT_VALUE:
                 case ParseTreeWalker.AGGREGATE:
                 case ParseTreeWalker.FUNCTION:
-
-                    SimpleQueryExecutor sqe = new SimpleQueryExecutor( _dbImpl );
-                    _results =  sqe.execute( _expr, _bindValues);
+                    try {
+                        java.sql.Connection conn = (java.sql.Connection)_dbImpl.getTransaction().getConnection(_dbEngine);
+                        SimpleQueryExecutor sqe = new SimpleQueryExecutor( _dbImpl );
+                        _results =  sqe.execute(conn, _expr, _bindValues);
+		            } catch ( QueryException except ) {
+		                throw new QueryException(Messages.message ("persist.simple.query.failed"), except);
+		            }
                     _fieldNum = 0;
-
             }
         } catch ( PersistenceException except ) {
             throw except;
         }
+
         return _results;
     }
 
     /**
      * Get the generated SQL statement for this OQLQuery
+     * @return A SQL statement.
+     * @throws QueryException If the SQL query cannot be generated.
      */
     public String getSQL() throws org.exolab.castor.jdo.QueryException {
-	  if(_expr != null)
+	  if(_expr != null) {
           return _expr.getStatement(true);
-        else
-         return  _spCall;
+	  }
+
+	  return _spCall;
     }
 
     public void close()
@@ -746,7 +689,7 @@ public class OQLQueryImpl
         }
 
         private Object followPath(Object parent) {
-            //System.out.println("Following the path.");
+            // _log.debug ("Following the path.");
             //follow the path
             JDOClassDescriptor curClassDesc = _classDescriptor;
             Object curObject = parent;
