@@ -40,47 +40,47 @@
  *
  * Copyright 1999 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: SQLTypes.java,v 1.1.1.1 2003/03/03 07:08:21 kvisco Exp $
+ * $Id: SQLTypes.java,v 1.15 2005/08/22 16:29:13 wguttmn Exp $
  */
 
 
 package org.exolab.castor.jdo.engine;
 
-import java.io.Reader;
-import java.io.Writer;
-import java.io.InputStream;
-import java.sql.Blob;
-import java.sql.Clob;
-import java.sql.ResultSet;
-import java.sql.Types;
-import java.sql.Time;
-import java.sql.Timestamp;
-import java.sql.PreparedStatement;
-import java.sql.SQLException;
-import java.math.BigDecimal;
-import java.math.BigInteger;
-import java.text.DateFormat;
-import java.text.DecimalFormat;
-import java.text.SimpleDateFormat;
-import java.text.ParseException;
-import java.io.IOException;
-import org.exolab.castor.util.Messages;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+import org.castor.util.Base64Decoder;
+import org.castor.util.Base64Encoder;
 import org.exolab.castor.mapping.MappingException;
 import org.exolab.castor.mapping.TypeConvertor;
 import org.exolab.castor.util.LocalConfiguration;
-import org.exolab.castor.util.MimeBase64Encoder;
-import org.exolab.castor.util.MimeBase64Decoder;
-
+import java.io.IOException;
+import java.io.InputStream;
+import java.math.BigDecimal;
+import java.math.BigInteger;
+import java.sql.*;
+import java.text.DateFormat;
+import java.text.DecimalFormat;
+import java.text.ParseException;
+import java.text.SimpleDateFormat;
+import java.util.Calendar;
+import java.util.GregorianCalendar;
+import java.util.TimeZone;
 
 
 /**
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:08:21 $
+ * @author <a href="mailto:ferret AT frii dot com">Bruce Snyder</a>
+ * @version $Revision: 1.15 $ $Date: 2005/08/22 16:29:13 $
  */
 public final class SQLTypes
 {
 
-
+	/**
+	 * The <a href="http://jakarta.apache.org/commons/logging/">Jakarta
+	 * Commons Logging</a> instance used for all logging.
+	 */
+	private static Log log = LogFactory.getFactory().getInstance(Class.class);
+	
     /**
      * Separator between words in SQL name.
      */
@@ -95,7 +95,46 @@ public final class SQLTypes
 
     private static final char RightParamSeparator = ']';
 
-
+    /**
+     * Time zone based on setting in castor.properties file (or
+     * default time zone if not specified)
+     */
+    private static final TimeZone dbTimeZone;
+    
+    static {
+    	String zone = LocalConfiguration.getInstance().getProperty(
+    			"org.exolab.castor.jdo.defaultTimeZone", "");
+    	
+    	if (zone.length() == 0) {
+    		dbTimeZone = TimeZone.getDefault();
+    	} else {
+    		dbTimeZone = TimeZone.getTimeZone(zone);
+    	}
+    }
+    
+    /**
+     * Thread local Calendar instance pool.
+     */
+    private static final ThreadLocal threadSafeCalendar = new ThreadLocal() {
+    	// The Calendar passed to ResultSet.getTimestamp() etc can actually
+    	// be modified depending on the database driver implementation.  To guard
+    	// against synchronization issues, we need to pass either a local
+    	// instance (which is expensive, creating one for each call), or create
+    	// a thread-local instance (which only gets created once per thread).
+    	// The latter is what this is for.
+    	public Object initialValue() {
+    		return new GregorianCalendar();
+    	}
+    };
+    
+    private static Calendar getCalendar() {
+    	// we have to reset the time zone each time in case the result set
+    	// implementation changes it.
+    	Calendar c = (Calendar) threadSafeCalendar.get();
+    	c.setTimeZone(dbTimeZone);
+    	return c;
+    }
+    
     /**
      * Grabs the LOB buffer size from castor.properties
      */
@@ -114,7 +153,9 @@ public final class SQLTypes
             // eat the exception and just use the old size
             lobSize = 256;
         }
-        // System.out.println( "Using lobSize: " + lobSize );
+        if(log.isDebugEnabled()) {
+            log.debug ( "Using lobSize: " + lobSize );
+        }
         return lobSize;
     }
 
@@ -185,10 +226,11 @@ public final class SQLTypes
         if ( right < 0 )
             right = sqlTypeName.length();
 
-        if ( left >= 0 )
+        if ( left >= 0 ) {
             return sqlTypeName.substring( left + 1, right );
-        else
-            return null;
+        }
+
+        return null;
     }
 
 
@@ -346,11 +388,11 @@ public final class SQLTypes
             intVal = rs.getInt( index );
             return ( rs.wasNull() ? null : new Integer( intVal ) );
         case Types.TIME:
-            return rs.getTime( index );
+            return rs.getTime( index, getCalendar() );
         case Types.DATE:
-            return rs.getDate( index );
+            return rs.getDate( index, getCalendar() );
         case Types.TIMESTAMP:
-            return rs.getTimestamp( index );
+            return rs.getTimestamp( index, getCalendar() );
         case Types.FLOAT:
         case Types.DOUBLE:
             doubleVal = rs.getDouble( index );
@@ -387,8 +429,9 @@ public final class SQLTypes
 
 
     public static void setObject( PreparedStatement stmt, int index, Object value, int sqlType )
-            throws SQLException
+        throws SQLException
     {
+        try {
         if (value == null) {
             stmt.setNull( index, sqlType );
         } else {
@@ -421,6 +464,11 @@ public final class SQLTypes
                 break;
             }
         }
+        }
+        catch (SQLException e) {
+            log.error ("Unexpected SQL exception: ", e);
+            
+        }
     }
 
 
@@ -533,24 +581,67 @@ public final class SQLTypes
 
 
     /**
-     * Date format used by the date convertor.
+     * Date format used by the date convertor. Use the {@link #getDateFormat}
+     * accessor to access this variable.
+     * 
+     * @see #getDateFormat()
      */
     private static DateFormat _dateFormat = new SimpleDateFormat();
 
 
+    /**
+     * Use this accessor to access the <tt>_dateFormat</tt>.
+     */
+    private static SimpleDateFormat getDateFormat()
+    {
+        SimpleDateFormat clone = ( SimpleDateFormat ) _dateFormat.clone();
+        
+        return clone;
+    }
+
+
     /**
      * Date format used by the date convertor when nonempty parameter
-     * is specified.
+     * is specified. Use the {@link #getParamDateFormat} accessor to access
+     * this variable.
+     *
+     * @see #getParamDateFormat()
      */
     private static SimpleDateFormat _paramDateFormat = new SimpleDateFormat();
 
+
     /**
-     * Date format used by the double->date convertor.
+     * Use this accessor to access the <tt>_paramDateFormat</tt>.
+     */
+    private static SimpleDateFormat getParamDateFormat()
+    {
+        SimpleDateFormat clone = ( SimpleDateFormat ) _paramDateFormat.clone();
+        
+        return clone;
+    }
+
+
+    /**
+     * Date format used by the double->date convertor. Use the {@link #getDecimalFormat} 
+     * accessor to access this variable.
+     * 
+     * @see #getDecimalFormat()
      */
     private static DecimalFormat _decimalFormat = new DecimalFormat("#################0");
 
 
-    private abstract static class SQLTypeConvertor implements TypeConvertor {
+    /**
+     * Use this accessor to access the <tt>_decimalFormat</tt>.
+     */
+    private static DecimalFormat getDecimalFormat()
+    {
+        DecimalFormat clone = ( DecimalFormat ) _decimalFormat.clone();
+        
+        return clone;
+    }
+
+
+    public abstract static class SQLTypeConvertor implements TypeConvertor {
 
         Class fromType;
 
@@ -566,6 +657,8 @@ public final class SQLTypes
             return fromType.getName()+"-->"+toType.getName();
         }
     }
+
+
     /**
      * List of all the default convertors between Java types.
      */
@@ -578,7 +671,10 @@ public final class SQLTypes
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.lang.Boolean.class, java.lang.Short.class ) {
             public Object convert( Object obj, String param ) {
-                return new Short( ((Boolean)obj).booleanValue()? (byte)1: (byte)0 );
+                if ( param == null || param.length() != 1 || param.charAt(0) != '-')
+                    return new Short( ((Boolean)obj).booleanValue()? (byte)1: (byte)0 );
+                else
+                    return new Short( ((Boolean)obj).booleanValue()? (byte)-1: (byte)0 );
             }
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.lang.Integer.class, java.lang.Boolean.class ) {
@@ -588,22 +684,29 @@ public final class SQLTypes
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.lang.Boolean.class, java.lang.Integer.class ) {
             public Object convert( Object obj, String param ) {
-                return new Integer( ((Boolean)obj).booleanValue()? 1: 0 );
+                if ( param == null || param.length() != 1 || param.charAt(0) != '-')
+                    return new Integer( ((Boolean)obj).booleanValue()? 1: 0 );
+                else
+                    return new Integer( ((Boolean)obj).booleanValue()? -1: 0 );
             }
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.lang.String.class, java.lang.Boolean.class ) {
             public Object convert( Object obj, String param ) {
-                switch ( ( (String) obj ).length() ) {
-                    case 0: return Boolean.FALSE;
-                    case 1: char ch = ( (String) obj ).charAt( 0 );
-                        if (param == null || param.length() != 2 )
-                            return ( ch == 'T' || ch == 't'  ) ? Boolean.TRUE : Boolean.FALSE;
-                        else
-                            return ( ch == param.charAt( 1 ) ) ? Boolean.TRUE : Boolean.FALSE;
-                    case 4: return ( (String) obj ).equalsIgnoreCase( "true" ) ? Boolean.TRUE : Boolean.FALSE;
-                    case 5: return ( (String) obj ).equalsIgnoreCase( "false" ) ? Boolean.TRUE : Boolean.FALSE;
+                char t = 'T';
+                char f = 'F';
+                if ((param != null) && (param.length() == 2)) {
+                    t = Character.toUpperCase(param.charAt(1));
+                    f = Character.toUpperCase(param.charAt(0));
                 }
-                return Boolean.FALSE;
+                
+                if (((String) obj).length() == 1) {
+                    char c = Character.toUpperCase(((String) obj).charAt(0));
+                    if (c == t) { return Boolean.TRUE; }
+                    if (c == f) { return Boolean.FALSE; }
+                }
+                throw new ClassCastException("Failed to convert string '" + obj
+                        + "' to boolean because it didn't match the expected values '"
+                        + t + "'/'" + f + "' (true/false).");
             }
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.math.BigDecimal.class, java.lang.Boolean.class ) {
@@ -649,8 +752,10 @@ public final class SQLTypes
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.util.Date.class, java.lang.Integer.class ) {
             public Object convert( Object obj, String param ) {
-                _paramDateFormat.applyPattern( org.exolab.castor.mapping.loader.Types.getFullDatePattern( param ) );
-                return new Integer( _paramDateFormat.format( (java.util.Date) obj ) );
+                SimpleDateFormat paramDateFormat = getParamDateFormat();
+                paramDateFormat.applyPattern( 
+                        org.exolab.castor.mapping.loader.Types.getFullDatePattern( param ) );
+                return new Integer( paramDateFormat.format( (java.util.Date) obj ) );
             }
         } ),
         // Convertors to long
@@ -754,8 +859,10 @@ public final class SQLTypes
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.util.Date.class, java.lang.Double.class ) {
             public Object convert( Object obj, String param ) {
-                _paramDateFormat.applyPattern( org.exolab.castor.mapping.loader.Types.getFullDatePattern( param ) );
-                return new Double( _paramDateFormat.format( (java.util.Date) obj ) );
+                SimpleDateFormat paramDateFormat = getParamDateFormat();
+                paramDateFormat.applyPattern( 
+                        org.exolab.castor.mapping.loader.Types.getFullDatePattern( param ) );
+                return new Double( paramDateFormat.format( (java.util.Date) obj ) );
             }
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.lang.String.class, java.lang.Double.class ) {
@@ -829,13 +936,18 @@ public final class SQLTypes
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.util.Date.class, java.math.BigDecimal.class ) {
             public Object convert( Object obj, String param ) {
-                _paramDateFormat.applyPattern( org.exolab.castor.mapping.loader.Types.getFullDatePattern( param ) );
-                return new BigDecimal( new BigInteger( _paramDateFormat.format( (java.util.Date) obj ) ) );
+                SimpleDateFormat paramDateFormat = getParamDateFormat();
+                paramDateFormat.applyPattern( 
+                        org.exolab.castor.mapping.loader.Types.getFullDatePattern( param ) );
+                return new BigDecimal( new BigInteger( paramDateFormat.format( (java.util.Date) obj ) ) );
             }
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.lang.Boolean.class, java.math.BigDecimal.class ) {
             public Object convert( Object obj, String param ) {
-                return BigDecimal.valueOf( ( (Boolean) obj).booleanValue() ? 1 : 0 );
+                if ( param == null || param.length() != 1 || param.charAt(0) != '-')
+                    return BigDecimal.valueOf( ( (Boolean) obj).booleanValue() ? 1 : 0 );
+                else
+                    return BigDecimal.valueOf( ( (Boolean) obj).booleanValue() ? -1 : 0 );
             }
         } ),
         // Convertors to string
@@ -874,8 +986,9 @@ public final class SQLTypes
                 if ( param == null || param.length() == 0 )
                     return obj.toString();
                 else {
-                    _paramDateFormat.applyPattern( param );
-                    return _paramDateFormat.format( (java.util.Date) obj );
+                    SimpleDateFormat paramDateFormat = getParamDateFormat();
+                    paramDateFormat.applyPattern( param );
+                    return paramDateFormat.format( (java.util.Date) obj );
                 }
             }
         } ),
@@ -891,10 +1004,7 @@ public final class SQLTypes
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( byte[].class, java.lang.String.class ) {
             public Object convert( Object obj, String param ) {
-                MimeBase64Encoder encoder;
-                encoder = new MimeBase64Encoder();
-                encoder.translate( (byte[]) obj );
-                return new String( encoder.getCharArray() );
+                return new String(Base64Encoder.encode((byte[]) obj));
             }
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.lang.Boolean.class, java.lang.String.class ) {
@@ -919,10 +1029,7 @@ public final class SQLTypes
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.lang.String.class, byte[].class ) {
             public Object convert( Object obj, String param ) {
-                MimeBase64Decoder decoder;
-                decoder = new MimeBase64Decoder();
-                decoder.translate( (String ) obj );
-                return decoder.getByteArray();
+                return Base64Decoder.decode((String ) obj);
             }
         } ),
         // Convertors to date
@@ -935,10 +1042,11 @@ public final class SQLTypes
             public Object convert( Object obj, String param ) {
                 try {
                     if ( param == null || param.length() == 0 )
-                        return _dateFormat.parse( (String) obj );
+                        return getDateFormat().parse( (String) obj );
                     else {
-                        _paramDateFormat.applyPattern( param );
-                        return _paramDateFormat.parse( (String) obj );
+                        SimpleDateFormat paramDateFormat = getParamDateFormat();
+                        paramDateFormat.applyPattern( param );
+                        return paramDateFormat.parse( (String) obj );
                     }
                 } catch ( ParseException except ) {
                     throw new IllegalArgumentException( except.toString() );
@@ -948,8 +1056,10 @@ public final class SQLTypes
         new TypeConvertorInfo( new SQLTypeConvertor( java.lang.Integer.class, java.util.Date.class ) {
             public Object convert( Object obj, String param ) {
                 try {
-                    _paramDateFormat.applyPattern( org.exolab.castor.mapping.loader.Types.getFullDatePattern( param ) );
-                    return _paramDateFormat.parse( obj.toString() );
+                    SimpleDateFormat paramDateFormat = getParamDateFormat();
+                    paramDateFormat.applyPattern( 
+                            org.exolab.castor.mapping.loader.Types.getFullDatePattern( param ) );
+                    return paramDateFormat.parse( obj.toString() );
                 } catch ( ParseException except ) {
                     throw new IllegalArgumentException( except.toString() );
                 }
@@ -958,8 +1068,10 @@ public final class SQLTypes
         new TypeConvertorInfo( new SQLTypeConvertor( java.math.BigDecimal.class, java.util.Date.class ) {
             public Object convert( Object obj, String param ) {
                 try {
-                    _paramDateFormat.applyPattern( org.exolab.castor.mapping.loader.Types.getFullDatePattern( param ) );
-                    return _paramDateFormat.parse( obj.toString() );
+                    SimpleDateFormat paramDateFormat = getParamDateFormat();
+                    paramDateFormat.applyPattern( 
+                            org.exolab.castor.mapping.loader.Types.getFullDatePattern( param ) );
+                    return paramDateFormat.parse( obj.toString() );
                 } catch ( ParseException except ) {
                     throw new IllegalArgumentException( except.toString() );
                 }
@@ -968,8 +1080,9 @@ public final class SQLTypes
         new TypeConvertorInfo( new SQLTypeConvertor( java.lang.Double.class, java.util.Date.class ) {
             public Object convert( Object obj, String param ) {
                 try {
-                    _paramDateFormat.applyPattern( org.exolab.castor.mapping.loader.Types.getFullDatePattern( param ) );
-                    return _paramDateFormat.parse( _decimalFormat.format(obj).trim() );
+                    SimpleDateFormat paramDateFormat = getParamDateFormat();
+                    paramDateFormat.applyPattern( org.exolab.castor.mapping.loader.Types.getFullDatePattern( param ) );
+                    return paramDateFormat.parse( getDecimalFormat().format(obj).trim() );
                 } catch ( ParseException except ) {
                     throw new IllegalArgumentException( except.toString() );
                 }
@@ -999,7 +1112,7 @@ public final class SQLTypes
             public Object convert( Object obj, String param ) {
                 long time = ( (java.util.Date) obj ).getTime();
                 java.sql.Timestamp timestamp = new java.sql.Timestamp(time);
-                timestamp.setNanos((int) ((time % 1000) * 1000000));
+                //timestamp.setNanos((int) ((time % 1000) * 1000000));
                 //timestamp.setNanos(0);  // this can workaround the bug in SAP DB
                 return timestamp;
             }
@@ -1007,7 +1120,7 @@ public final class SQLTypes
         new TypeConvertorInfo( new SQLTypeConvertor( java.sql.Timestamp.class, java.util.Date.class ) {
             public Object convert( Object obj, String param ) {
                 java.sql.Timestamp timestamp = (java.sql.Timestamp) obj;
-                return new java.util.Date(timestamp.getTime() + timestamp.getNanos() / 1000000);
+                return new java.util.Date(timestamp.getTime());
             }
         } ),
         new TypeConvertorInfo( new SQLTypeConvertor( java.lang.String.class, java.sql.Timestamp.class ) {
@@ -1017,8 +1130,9 @@ public final class SQLTypes
                     param = "yyyy-MM-dd HH:mm:ss.SSS";
                 }
                 try {
-                    _paramDateFormat.applyPattern( param );
-                    time = _paramDateFormat.parse( (String) obj ).getTime();
+                    SimpleDateFormat paramDateFormat = getParamDateFormat();
+                    paramDateFormat.applyPattern( param );
+                    time = paramDateFormat.parse( (String) obj ).getTime();
                 } catch ( ParseException except ) {
                     throw new IllegalArgumentException( except.toString() );
                 }
@@ -1034,8 +1148,9 @@ public final class SQLTypes
                     param = "yyyy-MM-dd HH:mm:ss.SSS";
                 }
                 java.sql.Timestamp timestamp = (java.sql.Timestamp) obj;
-                _paramDateFormat.applyPattern( param );
-                return _paramDateFormat.format( new java.util.Date(timestamp.getTime() + timestamp.getNanos() / 1000000) );
+                SimpleDateFormat paramDateFormat = getParamDateFormat();
+                paramDateFormat.applyPattern( param );
+                return paramDateFormat.format( new java.util.Date(timestamp.getTime() + timestamp.getNanos() / 1000000) );
             }
         } ),
         // InputStream convertors
@@ -1051,7 +1166,6 @@ public final class SQLTypes
                     java.io.ByteArrayOutputStream bos = new java.io.ByteArrayOutputStream();
                     byte[] buffer = new byte[loadLobBufferSize()];
                     int len = 0;
-                    int b;
                     while ( (len = is.read(buffer)) > 0 )
                         bos.write( buffer, 0, len );
                     return bos.toByteArray();
@@ -1080,7 +1194,6 @@ public final class SQLTypes
                     java.io.CharArrayWriter writer = new java.io.CharArrayWriter();
                     char[] buffer = new char[loadLobBufferSize()];
                     int len = 0;
-                    int b;
                     while ( (len = reader.read(buffer)) > 0 )
                         writer.write( buffer, 0, len );
                     return writer.toString();
@@ -1096,7 +1209,6 @@ public final class SQLTypes
                     java.io.CharArrayWriter writer = new java.io.CharArrayWriter();
                     char[] buffer = new char[loadLobBufferSize()];
                     int len = 0;
-                    int b;
                     while ( (len = reader.read(buffer)) > 0 )
                         writer.write( buffer, 0, len );
                     return writer.toCharArray();
@@ -1143,7 +1255,6 @@ public final class SQLTypes
                     java.io.ByteArrayOutputStream bos = new java.io.ByteArrayOutputStream();
                     byte[] buffer = new byte[loadLobBufferSize()];
                     int len = 0;
-                    int b;
                     while ( (len = is.read(buffer)) > 0 )
                         bos.write( buffer, 0, len );
                     return bos.toByteArray();
@@ -1163,6 +1274,16 @@ public final class SQLTypes
             public Object convert( Object obj, String param ) {
                 return obj;
             }
+        } ),
+        new TypeConvertorInfo (new SQLTypeConvertor (org.exolab.castor.types.Duration.class, java.lang.Long.class) {
+            public Object convert (Object obj, String param) {
+                return new Long (((org.exolab.castor.types.Duration) obj).toLong());
+            }
+        } ),
+        new TypeConvertorInfo (new SQLTypeConvertor (java.lang.Long.class, org.exolab.castor.types.Duration.class) {
+            public Object convert (Object obj, String param) {
+                return new org.exolab.castor.types.Duration ((( Long) obj).longValue());
+            }
         } )
     };
 
