@@ -40,32 +40,31 @@
  *
  * Copyright 1999 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: ParseTreeWalker.java,v 1.1.1.1 2003/03/03 07:08:25 kvisco Exp $
+ * $Id: ParseTreeWalker.java,v 1.12 2004/11/29 11:01:01 wguttmann Exp $
  */
 
 
 package org.exolab.castor.jdo.oql;
 
-import java.util.Hashtable;
 import java.util.Enumeration;
+import java.util.Hashtable;
 import java.util.Vector;
-import java.util.ArrayList;
-import java.util.Iterator;
-import org.exolab.castor.persist.LockEngine;
-import org.exolab.castor.persist.spi.QueryExpression;
+
+import org.exolab.castor.jdo.DbMetaInfo;
 import org.exolab.castor.jdo.QueryException;
-import org.exolab.castor.jdo.engine.SQLEngine;
 import org.exolab.castor.jdo.engine.JDOClassDescriptor;
 import org.exolab.castor.jdo.engine.JDOFieldDescriptor;
+import org.exolab.castor.jdo.engine.SQLEngine;
 import org.exolab.castor.mapping.loader.Types;
-import org.exolab.castor.jdo.engine.JDBCSyntax;
+import org.exolab.castor.persist.LockEngine;
+import org.exolab.castor.persist.spi.QueryExpression;
 
 /**
  * A class which walks the parse tree created by the parser to check for errors
  * and translate to SQL.
  *
  * @author <a href="mailto:nissim@nksystems.com">Nissim Karpenstein</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:08:25 $
+ * @version $Revision: 1.12 $ $Date: 2004/11/29 11:01:01 $
  */
 public class ParseTreeWalker implements TokenTypes
 {
@@ -84,8 +83,8 @@ public class ParseTreeWalker implements TokenTypes
   private ClassLoader _classLoader;
   private Class _objClass;
   private QueryExpression _queryExpr;
+  private DbMetaInfo _dbInfo;
 
-  private int _SQLParamIndex; //Alex
   private Hashtable _paramInfo;
   private Hashtable _fieldInfo;
   private Hashtable _pathInfo;
@@ -111,16 +110,17 @@ public class ParseTreeWalker implements TokenTypes
    *
    * @param dbEngine The Persistence Engine
    * @param parseTree The parse tree to walk
+   * @param classLoader A ClassLoader instance to load classes.
    * @throws QueryException Thrown by checkErrors.
    */
-  public ParseTreeWalker(LockEngine dbEngine, ParseTreeNode parseTree, ClassLoader classLoader )
+  public ParseTreeWalker(LockEngine dbEngine, ParseTreeNode parseTree, ClassLoader classLoader, DbMetaInfo dbInfo)
       throws QueryException
   {
     _dbEngine = dbEngine;
     _parseTree = parseTree;
     _classLoader = classLoader;
+    _dbInfo = dbInfo;
 
-    _SQLParamIndex = 1; //Alex
     _paramInfo = new Hashtable();
     _fieldInfo = new Hashtable();
     _pathInfo = new Hashtable();
@@ -130,8 +130,8 @@ public class ParseTreeWalker implements TokenTypes
     if ( ! _parseTree.isRoot() )
       throw (new QueryException( "ParseTreeWalker must be created with the root node of the parse tree."));
 
-    //System.out.println(ParseTest.treeToString(_parseTree, ParseTest.NODE_TYPES));
-    //System.out.println(ParseTest.treeToString(_parseTree, ParseTest.NODE_VALUES));
+    //_log.debug (ParseTest.treeToString(_parseTree, ParseTest.NODE_TYPES));
+    //_log.debug (ParseTest.treeToString(_parseTree, ParseTest.NODE_VALUES));
 
 
     checkErrors();
@@ -188,6 +188,7 @@ public class ParseTreeWalker implements TokenTypes
    * will be used by the QueryResults to follow the path if the object
    * selected is a DEPENDANT_OBJECT or DEPENDANT_OBJECT_VALUE.  Any other
    * projectionTypes do not need this, so null will be returned.
+   * @return Path info for the selected element, null otherwise. 
    */
   public Vector getPathInfo() {
     switch ( _projectionType ) {
@@ -241,6 +242,9 @@ public class ParseTreeWalker implements TokenTypes
         case KEYWORD_LIMIT:
           checkLimitClause( _parseTree.getChild(curChild) );
           break;
+        case KEYWORD_OFFSET:
+            checkOffsetClause( _parseTree.getChild(curChild) );
+            break;
       }
     }
   }
@@ -336,6 +340,9 @@ public class ParseTreeWalker implements TokenTypes
   /**
    * Search for the field in the given class descriptor and descriptors of the superclasses,
    * return null if not found.
+   * @param fieldName The field name.
+   * @param clsDesc A JDO class descriptor.
+   * @return JDOFieldDescriptor for the specified field, null if not found.
    */
   private JDOFieldDescriptor getFieldDesc( String fieldName, JDOClassDescriptor clsDesc ) {
     JDOFieldDescriptor field;
@@ -354,8 +361,8 @@ public class ParseTreeWalker implements TokenTypes
    * Returns new Object[2] {fieldDesc, classDesc}, where classDesc is a descriptor of the class where
    * the field was found.
    * Also adds inner joins to the QueryExpression.
-   * @param tableAlias The path info vector to build the alias with
-   * @param pathIndex Field index in the path info
+   * @param path The path info vector to build the alias with
+   * @param tableIndex Field index in the path info
    */
   private Object[] getFieldAndClassDesc(String fieldName, JDOClassDescriptor clsDesc, QueryExpression expr,
                                         Vector path, int tableIndex) {
@@ -400,7 +407,7 @@ public class ParseTreeWalker implements TokenTypes
    * _projectionType.
    *
    * @param projection The ParseTreeNode containing the projection.
-   * @param firstLevel pass true when calling on the top level of projection
+   * @param topLevel pass true when calling on the top level of projection
    *    in the parse tree.  False when recursing
    * @param onlySimple pass true if you want to throw errors if the
    *    object passed as the ParseTreeNode is not a simple type (use this
@@ -553,6 +560,7 @@ public class ParseTreeWalker implements TokenTypes
    * or (DOT, IDENTIFIER, IDENTIFIER)).  Als creates a Hashtable of paramInfo
    * with type information for query parameters (i.e. $(Class)1 or $1).
    *
+   * @param whereClause WHERE-clause to traverse.
    * @throws QueryException if an error is detected.
    */
   private void checkWhereClause(ParseTreeNode whereClause)
@@ -597,6 +605,7 @@ public class ParseTreeWalker implements TokenTypes
    * Traverses the limit clause sub-tree and checks for errors. Creates
    * a Hashtable of paramInfo with type information for query parameters
    * (i.e. $1).
+   * @param limitClause LIMIT-clause to traverse.
    * @throws QueryException if an error is detected.
    */
   private void checkLimitClause(ParseTreeNode limitClause)
@@ -615,6 +624,29 @@ public class ParseTreeWalker implements TokenTypes
     }
   }
 
+  /**
+   * Traverses the offset clause sub-tree and checks for errors. Creates
+   * a Hashtable of paramInfo with type information for query parameters
+   * (i.e. $1).
+   * @param offsetClause OFFSET-clause to traverse.
+   * @throws QueryException if an error is detected.
+   */
+  private void checkOffsetClause(ParseTreeNode offsetClause)
+        throws QueryException {
+
+    int tokenType = offsetClause.getToken().getTokenType();
+    switch (tokenType) {
+      case DOLLAR:
+        checkParameter(offsetClause);
+        break;
+
+      default:
+        for (Enumeration e = offsetClause.children(); e.hasMoreElements(); ) {
+          checkLimitClause( (ParseTreeNode) e.nextElement() );
+        }
+    }
+  }
+
 
   /**
    * Checks whether the field passed in is valid within this object.  Also
@@ -684,6 +716,9 @@ public class ParseTreeWalker implements TokenTypes
       case KEYWORD_LIMIT: //Alex
         systemType = "java.lang.Number";
         break;
+      case KEYWORD_OFFSET:
+        systemType = "java.lang.Number";
+        break;
       case KEYWORD_LIKE:  case CONCAT:
         systemType = "java.lang.String";
         break;
@@ -692,39 +727,93 @@ public class ParseTreeWalker implements TokenTypes
         break;
       case EQUAL: case NOT_EQUAL: case GT:
       case GTE: case LT: case LTE: case KEYWORD_BETWEEN:
-        systemType = getTypeForComparison(paramTree.getParent());
+        systemType = getParamTypeForComparison(paramTree.getParent());
         desc = getJDOFieldDescriptor(paramTree.getParent());
+        break;
+      case KEYWORD_LIST:
+        systemType = getParamTypeForList(paramTree.getParent());
+        break;
     }
 
     //get the param info for this numbered param
     ParamInfo paramInfo = (ParamInfo) _paramInfo.get(paramNumber);
     if ( paramInfo == null ) {
-      paramInfo = new ParamInfo(userDefinedType, systemType, desc);
-      _paramInfo.put(paramNumber, paramInfo);
+        paramInfo = new ParamInfo(userDefinedType, systemType, desc, _classLoader);
+        _paramInfo.put(paramNumber, paramInfo);
     }
     else
-      paramInfo.check(userDefinedType, systemType);
+        paramInfo.check(userDefinedType, systemType);
 
   }
 
-  private String getTypeForComparison(ParseTreeNode comparisonTree)
+  private String getParamTypeForComparison(ParseTreeNode comparisonTree)
       throws QueryException
   {
-
     for (Enumeration e = comparisonTree.children(); e.hasMoreElements(); ) {
-
       ParseTreeNode curChild = (ParseTreeNode) e.nextElement();
       int tokenType = curChild.getToken().getTokenType();
 
-      if ((tokenType == DOT) || (tokenType == IDENTIFIER)) {
-        JDOFieldDescriptor field = checkField(curChild);
-        return field.getFieldType().getName();
+      switch(tokenType) {
+        case STRING_LITERAL:	return "java.lang.String";
+        case DOUBLE_LITERAL:	return "java.lang.Double";
+        case LONG_LITERAL:		return "java.lang.Long";
+        case BOOLEAN_LITERAL:	return "java.lang.Boolean";
+        case CHAR_LITERAL:		return "java.lang.Character";
+        case DATE_LITERAL:		return "java.util.Date";
+        case TIME_LITERAL:
+        case TIMESTAMP_LITERAL:	return "java.util.Time";
+
+        case DOT:
+        case IDENTIFIER: {
+			JDOFieldDescriptor field = checkField(curChild);
+			return field.getFieldType().getName();
+        }
       }
     }
 
     throw new QueryException( "Could not get type for comparison." );
   }
 
+  /**
+   * Determine type of the list expression from the first contained
+   * constant. If there are only bind variables without user defined
+   * type present in the list, default to String as type.
+   *  
+   * @param listTree non-empty parser tree of list expression
+   * @return class name of the list type
+   * @throws QueryException
+   */
+  private String getParamTypeForList(ParseTreeNode listTree) throws QueryException
+  {
+    for (Enumeration e = listTree.children(); e.hasMoreElements(); ) {
+      ParseTreeNode curChild = (ParseTreeNode) e.nextElement();
+      int tokenType = curChild.getToken().getTokenType();
+
+      switch(tokenType) {
+        case STRING_LITERAL:	return "java.lang.String";
+        case DOUBLE_LITERAL:	return "java.lang.Double";
+        case LONG_LITERAL:		return "java.lang.Long";
+        case BOOLEAN_LITERAL:	return "java.lang.Boolean";
+        case CHAR_LITERAL:		return "java.lang.Character";
+        case DATE_LITERAL:		return "java.util.Date";
+        case TIME_LITERAL:
+        case TIMESTAMP_LITERAL:	return "java.util.Time";
+
+        case DOLLAR:	// look for bind parameters with user defined type
+          if (curChild.getChildCount() == 2) {
+            String userDefinedType = curChild.getChild(0).getToken().getTokenValue();
+            try {
+                return Types.typeFromName(_classLoader, userDefinedType).getName();
+            } catch(ClassNotFoundException e1) {
+                throw new QueryException("Could not find class " + userDefinedType);
+            }
+          }
+      }
+    }
+
+    return "java.lang.String";	// default to String, if only simple bind variables are present in the list 
+  }
+
   private JDOFieldDescriptor getJDOFieldDescriptor(ParseTreeNode comparisonTree)
       throws QueryException
   {
@@ -752,29 +841,37 @@ public class ParseTreeWalker implements TokenTypes
    *    contains non literals.
    */
   private void checkInClauseRightSide(ParseTreeNode theList)
-        throws QueryException {
-
+  		throws QueryException
+  {
     if ( theList.getToken().getTokenType() != KEYWORD_LIST )
       throw new QueryException( "The right side of the IN operator must be a LIST." );
 
     for ( Enumeration e = theList.children(); e.hasMoreElements(); ) {
-      switch (( (ParseTreeNode) e.nextElement() ).getToken().getTokenType()) {
+      ParseTreeNode node = (ParseTreeNode) e.nextElement();
+	  int tokenType = node.getToken().getTokenType();
+
+      switch(tokenType) {
         case KEYWORD_NIL: case KEYWORD_UNDEFINED:
         case BOOLEAN_LITERAL: case LONG_LITERAL:
         case DOUBLE_LITERAL: case CHAR_LITERAL:
         case STRING_LITERAL: case DATE_LITERAL:
         case TIME_LITERAL: case TIMESTAMP_LITERAL:
+	      break;
+
+        case DOLLAR:	// bind variable parameter
+          checkParameter(node);
           break;
+
         default:
-          throw new QueryException( "The LIST can only contain literals and Keywords nil and undefined." );
+          throw new QueryException( "The LIST can only contain literals, bind variables and the keywords 'nil' and 'undefined'." );
       }
     }
-
   }
 
   /**
    * Traverses the order by clause sub-tree and checks for errors.
    *
+   * @param orderClause ORDER-BY clause to traverse.
    * @throws QueryException if an error is detected.
    */
   private void checkOrderClause(ParseTreeNode orderClause)
@@ -828,9 +925,11 @@ public class ParseTreeWalker implements TokenTypes
   /**
    * Generates the QueryExpression which is an SQL representation or the OQL
    * parse tree.
-   *
+   * @throws SyntaxNotSupportedException If a specific OQL feature is not supported by a RDBMS.
    */
-  private void createQueryExpression() {
+  private void createQueryExpression() 
+  	throws SyntaxNotSupportedException 
+  {
 
     //We use the SQLEngine buildfinder for any queries which require
     //us to load the entire object from the database.  Otherwise
@@ -849,6 +948,8 @@ public class ParseTreeWalker implements TokenTypes
         addSelectFromJoins();
     }
 
+    _queryExpr.setDbMetaInfo(_dbInfo);
+
     //check for DISTINCT
     if ( _parseTree.getChild(0).getToken().getTokenType() == KEYWORD_DISTINCT )
       _queryExpr.setDistinct(true);
@@ -867,6 +968,9 @@ public class ParseTreeWalker implements TokenTypes
         case KEYWORD_LIMIT:
           addLimitClause(curChild);
           break;
+        case KEYWORD_OFFSET:
+            addOffsetClause(curChild);
+            break;
       }
     }
   }
@@ -895,8 +999,9 @@ public class ParseTreeWalker implements TokenTypes
    * Builds the alias name for a table from the path info.
    *
    * @param tableName The name of the table to add to the select clause
-   * @param tableAlias The path info vector to build the alias with
-   * @param pathIndex Field index in the path info
+   * @param path The path info vector to build the alias with
+   * @param tableIndex Field index in the path info
+   * @return Alias name for a given table.
    */
   public String buildTableAlias( String tableName, Vector path, int tableIndex )
   {
@@ -947,6 +1052,7 @@ public class ParseTreeWalker implements TokenTypes
   /**
    * Adds joins to the queryExpr for path expressions in the OQL.
    *
+   * @param path Path expression to be added to JOIN clause.
    */
   private void addJoinsForPathExpression( Vector path ) {
     if ( path == null )
@@ -1030,73 +1136,42 @@ public class ParseTreeWalker implements TokenTypes
 
 
   /**
-   * Returns a SQL version of an OQL where clause.
+   * Adds a SQL version of an OQL where clause.
    *
    * @param whereClause the parse tree node with the where clause
-   * @return The SQL translation of the where clause.
    */
-  private void addWhereClause(ParseTreeNode whereClause) {
+  private void addWhereClause(ParseTreeNode whereClause)
+  {
     String sqlExpr = getSQLExpr(whereClause.getChild(0));
 
-    //Map numbered parameters
-    StringBuffer sb = new StringBuffer();
-    int startPos = 0;
-    int pos = sqlExpr.indexOf("?", startPos);
-    int SQLParamIndex = 1;
-    while ( pos != -1 ) {
-      int endPos = sqlExpr.indexOf(" ", pos);
-      Integer paramNumber = null;
-      if ( endPos != -1 )
-        paramNumber = new Integer(sqlExpr.substring(pos + 1, endPos));
-      else
-        paramNumber = new Integer(sqlExpr.substring(pos + 1));
-      ParamInfo paramInfo = (ParamInfo) _paramInfo.get(paramNumber);
-      paramInfo.mapToSQLParam( SQLParamIndex++ );
-      sb.append( sqlExpr.substring( startPos, pos+1 ) );
-      startPos = endPos < 0 ? sqlExpr.length() : endPos;
-      pos = sqlExpr.indexOf("?", startPos);
-    }
-    if ( startPos < sqlExpr.length() )
-      sb.append( sqlExpr.substring( startPos ) );
-
-    _queryExpr.addWhereClause( sb.toString() );
-
-    _SQLParamIndex = SQLParamIndex; //Alex
+	_queryExpr.addWhereClause(sqlExpr);
   }
 
   /**
-   * Returns a SQL version of an OQL limit clause.
+   * Adds a SQL version of an OQL limit clause.
    *
    * @param limitClause the parse tree node with the limit clause
-   * @return The SQL translation of the limit clause.
+   * @throws SyntaxNotSupportedException If the LIMIT clause is not supported by a RDBMS.
    */
-  private void addLimitClause(ParseTreeNode limitClause) {
+  private void addLimitClause(ParseTreeNode limitClause) 
+  	throws SyntaxNotSupportedException 
+  {
     String sqlExpr = getSQLExpr(limitClause/*.getChild(0)*/);
 
-    //Map numbered parameters
-    StringBuffer sb = new StringBuffer();
-    int startPos = 0;
-    int pos = sqlExpr.indexOf("?", startPos);
-    int SQLParamIndex = _SQLParamIndex;
-    while ( pos != -1 ) {
-      int endPos = sqlExpr.indexOf(" ", pos);
-      Integer paramNumber = null;
-      if ( endPos != -1 )
-        paramNumber = new Integer(sqlExpr.substring(pos + 1, endPos));
-      else
-        paramNumber = new Integer(sqlExpr.substring(pos + 1));
-      ParamInfo paramInfo = (ParamInfo) _paramInfo.get(paramNumber);
-      paramInfo.mapToSQLParam( SQLParamIndex++ );
-      sb.append( sqlExpr.substring( startPos, pos+1 ) );
-      startPos = endPos < 0 ? sqlExpr.length() : endPos;
-      pos = sqlExpr.indexOf("?", startPos);
-    }
-    if ( startPos < sqlExpr.length() )
-      sb.append( sqlExpr.substring( startPos ) );
+	_queryExpr.addLimitClause(sqlExpr);
+  }
 
-    _queryExpr.addLimitClause( sb.toString() );
-    // System.out.println(sb.toString());
-    _SQLParamIndex = SQLParamIndex;
+  /**
+   * Adds a SQL version of an OQL offset clause.
+   * @param offsetClause The parse tree node with the offset clause
+   * @throws SyntaxNotSupportedException If the LIMIT clause is not supported by a RDBMS.
+   */
+  private void addOffsetClause(ParseTreeNode offsetClause)
+  	throws SyntaxNotSupportedException
+  {
+    String sqlExpr = getSQLExpr(offsetClause/*.getChild(0)*/);
+
+	_queryExpr.addOffsetClause(sqlExpr);
   }
 
   /**
@@ -1206,10 +1281,10 @@ public class ParseTreeWalker implements TokenTypes
             if ( tokenType == DOT ) {
                 if ( path == null ) {
                     System.err.println( "exprTree=" + exprTree.toStringEx() + "\npathInfo = {" );
-                    Enumeration enum = _pathInfo.keys();
+                    Enumeration enumeration = _pathInfo.keys();
                     ParseTreeNode n;
-                    while ( enum.hasMoreElements() ) {
-                        n = (ParseTreeNode)enum.nextElement();
+                    while ( enumeration.hasMoreElements() ) {
+                        n = (ParseTreeNode)enumeration.nextElement();
                         System.err.println( "\t" + n.toStringEx() );
                     }
                     // Exception follows in addJoinsForPathExpression()
@@ -1307,12 +1382,11 @@ public class ParseTreeWalker implements TokenTypes
       case KEYWORD_UNDEFINED:
         return " NULL ";
       case KEYWORD_LIMIT: //Proceed it with it's own getSQLExpr
+      case KEYWORD_OFFSET:
         return getSQLExprForLimit(exprTree);
     }
 
     return "";
-
-
   }
 
 
@@ -1377,5 +1451,4 @@ public class ParseTreeWalker implements TokenTypes
     return sb.toString();
   }
 
-
 }
