@@ -40,33 +40,34 @@
  *
  * Copyright 1999 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: Mapping.java,v 1.1.1.1 2003/03/03 07:08:27 kvisco Exp $
+ * $Id: Mapping.java,v 1.10 2005/03/07 08:34:49 wguttmn Exp $
  */
 
 
 package org.exolab.castor.mapping;
 
 
-import java.util.Hashtable;
-import java.util.Enumeration;
-import java.util.Vector;
-import java.lang.reflect.Constructor;
-import java.io.PrintWriter;
 import java.io.IOException;
-import java.net.URL;
+import java.io.PrintWriter;
+import java.lang.reflect.Constructor;
 import java.net.MalformedURLException;
+import java.net.URL;
+import java.util.Enumeration;
+import java.util.Hashtable;
+
+import org.exolab.castor.mapping.loader.MappingLoader;
+import org.exolab.castor.mapping.xml.ClassMapping;
+import org.exolab.castor.mapping.xml.Include;
+import org.exolab.castor.mapping.xml.KeyGeneratorDef;
+import org.exolab.castor.mapping.xml.MappingRoot;
+import org.exolab.castor.net.util.URIUtils;
+import org.exolab.castor.util.DTDResolver;
+import org.exolab.castor.util.Messages;
+import org.exolab.castor.xml.UnmarshalListener;
+import org.exolab.castor.xml.Unmarshaller;
 import org.xml.sax.EntityResolver;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
-import org.exolab.castor.xml.Unmarshaller;
-import org.exolab.castor.xml.MarshalException;
-import org.exolab.castor.mapping.loader.MappingLoader;
-import org.exolab.castor.mapping.xml.MappingRoot;
-import org.exolab.castor.mapping.xml.Include;
-import org.exolab.castor.mapping.xml.ClassMapping;
-import org.exolab.castor.mapping.xml.KeyGeneratorDef;
-import org.exolab.castor.util.Messages;
-import org.exolab.castor.util.DTDResolver;
 
 
 /**
@@ -93,7 +94,7 @@ import org.exolab.castor.util.DTDResolver;
  * </pre>
  *
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:08:27 $
+ * @version $Revision: 1.10 $ $Date: 2005/03/07 08:34:49 $
  */
 public class Mapping
 {
@@ -131,7 +132,7 @@ public class Mapping
 
     /**
      * Use this object to obtain the mapping resolver for JDO from
-     * {@link #getResolver}.
+     * {@link #getResolver(EngineMapping)}.
      */
     public static final EngineMapping JDO =
         new EngineMapping( "jdo", "org.exolab.castor.jdo.engine.JDOMappingLoader" );
@@ -139,7 +140,7 @@ public class Mapping
 
     /**
      * Use this object to obtain the mapping resolver for DAX from
-     * {@link #getResolver}.
+     * {@link #getResolver(EngineMapping)}.
      */
     public static final EngineMapping DAX =
         new EngineMapping( "dax", "org.exolab.castor.dax.engine.DAXMappingLoader" );
@@ -147,7 +148,7 @@ public class Mapping
 
     /**
      * Use this object to obtain the mapping resolver for XML from
-     * {@link #getResolver}.
+     * {@link #getResolver(EngineMapping)}.
      */
     public static final EngineMapping XML =
         new EngineMapping( "xml", "org.exolab.castor.xml.XMLMappingLoader" );
@@ -350,6 +351,18 @@ public class Mapping
      */
     public void setBaseURL( String url )
     {
+        //-- remove filename if necessary:
+        if (url != null) {        
+            int idx = url.lastIndexOf('/');
+            if (idx < 0) idx = url.lastIndexOf('\\');
+            if (idx >= 0) {
+                int extIdx = url.indexOf('.', idx);
+                if (extIdx > 0) {
+                    url = url.substring(0, idx);
+                }
+            }
+        }
+        
         try {
           _resolver.setBaseURL( new URL( url ) );
         } catch ( MalformedURLException except ) {
@@ -390,8 +403,10 @@ public class Mapping
     public void loadMapping( String url )
         throws IOException, MappingException
     {
-        if ( _resolver.getBaseURL() == null )
+        if ( _resolver.getBaseURL() == null ) {
             setBaseURL( url );
+            url = URIUtils.getRelativeURI(url);
+        }
         loadMappingInternal( url );
     }
 
@@ -487,16 +502,16 @@ public class Mapping
     {
         MappingRoot  loaded;
         Unmarshaller unm;
-        Enumeration  enum;
+        Enumeration  enumeration;
 
         // Clear all the cached resolvers, so they can be reconstructed a
         // second time based on the new mappings loaded
         _resolvers.clear();
+        
         //check that the mapping has already been processed
         if ((source.getSystemId()!=null) && _state.processed(source.getSystemId()) ) {
-            String err = "The mapping located at:"+source.getSystemId();
-            err += " has already been processed.";
-            throw new MappingException(err);
+            //-- already processed...just return
+            return;
         }
         try {
             if ( _mapping == null ) {
@@ -504,6 +519,10 @@ public class Mapping
                 _idResolver.setMapping(_mapping);
             }
 
+            //mark the mapping as being processed
+            if (source.getSystemId() != null)
+                _state.markAsProcessed(source.getSystemId(), _mapping);
+                
             // Load the specificed mapping source
             unm = new Unmarshaller( MappingRoot.class );
             unm.setEntityResolver( _resolver );
@@ -511,43 +530,43 @@ public class Mapping
                 unm.setLogWriter( _logWriter );
             unm.setClassLoader( Mapping.class.getClassLoader() );
             unm.setIDResolver(_idResolver);
+            unm.setUnmarshalListener(new IncludeListener());
 
             loaded = (MappingRoot) unm.unmarshal( source );
 
-
-            // gather "class" tags
-            enum = loaded.enumerateClassMapping();
-            while ( enum.hasMoreElements() )
-                _mapping.addClassMapping( (ClassMapping) enum.nextElement() );
-
-            // gather "key-generator" tags
-            enum = loaded.enumerateKeyGeneratorDef();
-            while ( enum.hasMoreElements() ) {
-                _mapping.addKeyGeneratorDef( (KeyGeneratorDef) enum.nextElement() );
-            }
-            //mark the mapping as being processed
-            if (source.getSystemId() != null)
-                _state.markAsProcessed(source.getSystemId(), _mapping);
-
+                
             // Load all the included mapping by reference
-            Enumeration   includes;
-
-            includes = loaded.enumerateInclude();
+            //-- note: this is just for processing any
+            //-- includes which may have previously failed
+            //-- using the IncludeListener...and to
+            //-- report any potential errors.
+            Enumeration includes = loaded.enumerateInclude();
             while ( includes.hasMoreElements() ) {
-                try {
-                    loadMappingInternal( ( (Include) includes.nextElement() ).getHref() );
-                } catch ( Exception except ) {
-                    throw new MappingException( except );
+                Include include = (Include) includes.nextElement();
+                if (!_state.processed( include.getHref() )) {
+                    try {
+                        loadMappingInternal( include.getHref() );
+                    } 
+                    catch ( Exception except ) {
+                        throw new MappingException( except );
+                    }
                 }
             }
-        } catch ( MarshalException except ) {
-            if ( except.getException() != null )
-                throw new MappingException( except.getException() );
-            throw new MappingException( except );
+            
+            // gather "class" tags
+            enumeration = loaded.enumerateClassMapping();
+            while ( enumeration.hasMoreElements() )
+                _mapping.addClassMapping( (ClassMapping) enumeration.nextElement() );
+
+            // gather "key-generator" tags
+            enumeration = loaded.enumerateKeyGeneratorDef();
+            while ( enumeration.hasMoreElements() ) {
+                _mapping.addKeyGeneratorDef( (KeyGeneratorDef) enumeration.nextElement() );
+            }
         } catch ( Exception except ) {
             throw new MappingException( except );
         }
-    }
+    } //-- loadMappingInternal
 
     /**
      * An IDResolver to allow us to resolve ClassMappings
@@ -618,7 +637,7 @@ public class Mapping
 
         /**
          * Returns true if the given systemID has been marked as processed
-         * @param schema location the systemID  to check for being marked as processed
+         * @param systemID location the systemID  to check for being marked as processed
          */
         boolean processed(String systemID) {
             return _processed.containsKey(systemID);
@@ -632,7 +651,52 @@ public class Mapping
              return (MappingRoot)_processed.get(systemID);
          }
     }
+    
+    /**
+     * An UnmarshalListener to handle mapping includes
+     */
+    class IncludeListener implements UnmarshalListener {
 
-}
+        /* Not used for includes processing */
+        public void initialized (Object object) {
+            // not used
+        }
+
+        /* Not used for includes processing */
+        public void attributesProcessed(Object object) {
+            // not used ...
+        }
+
+        /* Not used for includes processing */
+        public void fieldAdded (String fieldName, Object parent, Object child) 
+        {
+            //-- do nothing
+        }
+
+        /**
+         * This method is called after an object
+         * has been completely unmarshalled, including
+         * all of its children (if any).
+         *
+         * @param object the Object that was unmarshalled.
+         */
+        public void unmarshalled (Object object) {
+            if (object instanceof Include) {
+                Include include = (Include) object;
+                try {
+                    loadMappingInternal( include.getHref() );
+                } 
+                catch ( Exception except ) {
+                    //-- ignore error, it'll get reported
+                    //-- later when we re-process the
+                    //-- includes of the parent Mapping in
+                    //-- loadMappingInternal
+                }
+            }
+        }
+
+    } //-- UnmarshalListener
+
+} // class: Mapping
 
 
