@@ -40,13 +40,15 @@
  *
  * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: FieldHandlerImpl.java,v 1.2 2003/04/16 07:15:35 kvisco Exp $
+ * $Id: FieldHandlerImpl.java,v 1.7 2005/05/22 14:22:58 wguttmn Exp $
  */
 
 
 package org.exolab.castor.mapping.loader;
 
 
+import java.util.Iterator;
+import java.util.Vector;
 import java.lang.reflect.Array;
 import java.lang.reflect.Field;
 import java.lang.reflect.Method;
@@ -59,7 +61,6 @@ import org.exolab.castor.mapping.TypeConvertor;
 import org.exolab.castor.mapping.CollectionHandler;
 import org.exolab.castor.mapping.MappingException;
 import org.exolab.castor.mapping.MappingRuntimeException;
-import org.exolab.castor.mapping.ValidityException;
 import org.exolab.castor.util.Messages;
 
 
@@ -70,16 +71,14 @@ import org.exolab.castor.util.Messages;
  * Note: the field Java type is obtained from {@link TypeInfo#getFieldType()},
  * but if the field is a collection, the actual field/accessor type is
  * obtained from {@link TypeInfo#getCollectionHandler} and the object to create
- * (with {@link #newInstance}) is the former field type.
+ * (with {@link #newInstance(Object)}) is the former field type.
  *
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
- * @version $Revision: 1.2 $ $Date: 2003/04/16 07:15:35 $
+ * @version $Revision: 1.7 $ $Date: 2005/05/22 14:22:58 $
  */
 public final class FieldHandlerImpl
     extends AbstractFieldHandler
 {
-
-
     /**
      * The prefix for an "add" method
     **/
@@ -95,19 +94,19 @@ public final class FieldHandlerImpl
     /**
      * The Java field described and accessed through this descriptor.
      */
-    private final Field   _field;
+    private final Field	_field;
 
 
     /**
      * The sequence of methods used to obtain the nested field. May be null.
      */
-    private Method[]      _getSequence;
+    private Method[]	_getSequence;
 
 
     /**
      * The sequence of methods used to create the nested object. May be null.
      */
-    private Method[]      _setSequence;
+    private Method[]	_setSequence;
 
 
     /** 
@@ -120,7 +119,7 @@ public final class FieldHandlerImpl
     /**
      * The method used to obtain the value of this field. May be null.
      */
-    private Method        _getMethod;
+    private Method      _getMethod;
 
 
     /**
@@ -132,25 +131,25 @@ public final class FieldHandlerImpl
     /**
      * The method used to check if the value of this field exists. May be null.
      */
-    private Method  _hasMethod;
+    private Method		_hasMethod;
 
 
     /**
      * The method used to delete the value of this field. May be null.
      */
-    private Method  _deleteMethod;
+    private Method		_deleteMethod;
 
 
     /**
      * The method used to create a new instance of the field.
      */
-    private Method  _createMethod;
+    private Method		_createMethod;
 
 
     /**
      * The Java field name.
      */
-    private final String  _fieldName;
+    private final String _fieldName;
 
 
     /**
@@ -210,7 +209,7 @@ public final class FieldHandlerImpl
      * be public, and may not be static or transient. The field name is
      * determined from the Java field, the type from the type information.
      *
-     * @param field The field being described
+     * @param handler
      * @param typeInfo Type information
      * @throws MappingException If the field is not public, is static or
      *    transient
@@ -252,7 +251,7 @@ public final class FieldHandlerImpl
         _handler = null;
         _field = field;
         _fieldType = Types.typeFromPrimitive( typeInfo.getFieldType() );
-        _fieldName = field.getName() + "(" + _fieldType.getName() + ")";
+        _fieldName = field.getName() + "(" + field.getType().getName() + ")";
         _immutable = typeInfo.isImmutable();
         _required = typeInfo.isRequired();
         // If the field is of a primitive type or if it is required
@@ -281,7 +280,7 @@ public final class FieldHandlerImpl
      *  must accept no parameters and have a return type castable to
      *  the field type
      * @param setMethod The method used to set the field value, must
-     *  accept a single paramater that is castable to the field type
+     *  accept a single parameter that is castable to the field type
      * @param typeInfo Type information
      * @throws MappingException If the get or set method are not public,
      *   are static, or do not specify the proper types
@@ -312,9 +311,22 @@ public final class FieldHandlerImpl
                 else 
                     setWriteMethod(setMethod);
             }
+            
+//            for(Iterator iter = setMethods.iterator(); iter.hasNext(); ) {
+//                Method method = (Method) iter.next();
+//
+//	            if (method.getName().startsWith(ADD_PREFIX)) {
+//	                Class paraType = method.getParameterTypes()[0];
+//
+//	                if (paraType != typeInfo.getFieldType()) {
+//	                    addMethods.add(method);
+//	                    iter.remove();
+//	                }
+//	            }
+//            }
             else setWriteMethod( setMethod );
         }
-        
+
         if ( getMethod != null )
             setReadMethod(getMethod);
         
@@ -322,6 +334,7 @@ public final class FieldHandlerImpl
         _fieldName = fieldName + "(" + _fieldType.getName() + ")";
         _immutable = typeInfo.isImmutable();
         _required = typeInfo.isRequired();
+
         // If the field is of a primitive type or if it is required
         // we use the default value 
         if ( setMethod != null && setMethod.getParameterTypes()[0].isPrimitive() )
@@ -353,6 +366,31 @@ public final class FieldHandlerImpl
             // If field is accessed directly, get it's value, if not
             // need to call get method. It's possible to not have a
             // way to access the field.
+            
+            //-- DEBUG
+            /*
+            if ( _handler != null )
+                System.out.println("FieldHandlerImpl: using nested handler.");
+            else if ( _field != null )
+                System.out.println("FieldHandlerImpl: using direct field access.");
+            else if ( _getMethod != null ) {
+                System.out.println("FieldHandlerImpl: using getMethod.");
+                if ( _getSequence != null ) {
+                    System.out.println("FieldHandlerImpl: using getter sequence.");
+                }
+                
+                if (_hasMethod != null) {
+                    System.out.println("FieldHandlerImpl: checking hasMethod.");
+                }
+                
+                System.out.println("FieldHandlerImpl: calling getMethod.");
+                System.out.println("FieldHandlerImpl: getter: " + _getMethod.toString());
+                System.out.println("FieldHandlerImpl: target: " + object.getClass().getName());
+                
+            }
+            */
+            //-- /DEBUG
+             
             if ( _handler != null )
                 value = _handler.getValue( object );
             else if ( _field != null )
@@ -360,7 +398,7 @@ public final class FieldHandlerImpl
             else if ( _getMethod != null ) {
                 if ( _getSequence != null ) 
                     for ( int i = 0; i < _getSequence.length; i++ ) {
-                        object = _getSequence[ i ].invoke( object, null );
+                        object = _getSequence[ i ].invoke( object, (Object[]) null );
                         if ( object == null )
                             break;
                     }
@@ -369,10 +407,10 @@ public final class FieldHandlerImpl
                 // If field has 'has' method, false means field is null
                 // and do not attempt to call getValue. Otherwise, 
                 if ( object == null ||
-                          ( _hasMethod != null && ! ( (Boolean) _hasMethod.invoke( object, null ) ).booleanValue() ) )
+                          ( _hasMethod != null && ! ( (Boolean) _hasMethod.invoke( object, (Object[]) null ) ).booleanValue() ) )
                     value = null;
                 else
-                    value = _getMethod.invoke( object, null );
+                    value = _getMethod.invoke( object, (Object[]) null );
             } else
                 value = null;
         } catch ( IllegalAccessException except ) {
@@ -444,7 +482,7 @@ public final class FieldHandlerImpl
                                 Object last;
 
                                 last = object;
-                                object = _getSequence[ i ].invoke( object, null );
+                                object = _getSequence[ i ].invoke( object, (Object[]) null );
                                 if ( object == null ) {
                                     // if the value is not null, we must instantiate
                                     // the object in the sequence
@@ -458,7 +496,7 @@ public final class FieldHandlerImpl
                             }
                         if ( object != null ) {
                             if ( value == null && _deleteMethod != null )
-                                _deleteMethod.invoke( object, null );
+                                _deleteMethod.invoke( object, (Object[]) null );
                             else
                                 setter.invoke( object, new Object[] { value == null ? _default : value } );
                         }
@@ -470,7 +508,7 @@ public final class FieldHandlerImpl
                 // Graceful way of dealing with unwrapping exception
                 if ( value == null )
                     throw new IllegalArgumentException( Messages.format( "mapping.typeConversionNull", toString() ) );
-                else
+                else 
                     throw new IllegalArgumentException( Messages.format( "mapping.typeConversion",
                                                                          toString(), value.getClass().getName() ) );
             } catch ( IllegalAccessException except ) {
@@ -526,8 +564,8 @@ public final class FieldHandlerImpl
                 else if ( _getMethod != null ) {
                     if ( _getSequence != null ) 
                         for ( int i = 0; i < _getSequence.length; i++ ) 
-                            object = _getSequence[ i ].invoke( object, null );
-                    collect = _getMethod.invoke( object, null );
+                            object = _getSequence[ i ].invoke( object, (Object[]) null );
+                    collect = _getMethod.invoke( object, (Object[]) null );
 
                     // If we deal with a collection who is an array of primitive
                     // and that has not been instantiated, we have to handle the
@@ -580,7 +618,6 @@ public final class FieldHandlerImpl
                 // This should never happen
                 throw new MappingRuntimeException(except.getTargetException());
             }
-
         }
     }
 
@@ -597,13 +634,13 @@ public final class FieldHandlerImpl
                 else if ( _setMethod != null ) {
                     if ( _getSequence != null ) 
                         for ( int i = 0; i < _getSequence.length; i++ ) {
-                            object = _getSequence[ i ].invoke( object, null );
+                            object = _getSequence[ i ].invoke( object, (Object[]) null );
                             if ( object == null )
                                 break;
                         }
                     if ( object != null ) {
                         if ( _deleteMethod != null )
-                            _deleteMethod.invoke( object, null );
+                            _deleteMethod.invoke( object, (Object[]) null );
                         else
                             _setMethod.invoke( object, new Object[] { _default } );
                     }
@@ -638,8 +675,8 @@ public final class FieldHandlerImpl
                 } else if ( _getMethod != null ) {
                     if ( _getSequence != null ) 
                         for ( int i = 0; i < _getSequence.length; i++ ) 
-                            object = _getSequence[ i ].invoke( object, null );
-                    collect = _getMethod.invoke( object, null );
+                            object = _getSequence[ i ].invoke( object, (Object[]) null );
+                    collect = _getMethod.invoke( object, (Object[]) null );
                     collect = _colHandler.clear( collect );
                     if ( collect != null && _setMethod != null)
                         _setMethod.invoke( object, new Object[] { collect } );
@@ -685,8 +722,9 @@ public final class FieldHandlerImpl
         if (_fieldType.isInterface() && _createMethod == null)
             return null;
             
-        if ( _immutable )
+        if (( _immutable ) && ((args == null) || (args.length == 0)))
             throw new IllegalStateException( Messages.format( "mapping.classNotConstructable", _fieldType ) );
+        
         if ( _handler != null ) {
             if (_handler instanceof ExtendedFieldHandler)
                 return ((ExtendedFieldHandler)_handler).newInstance( parent, args );
@@ -892,7 +930,4 @@ public final class FieldHandlerImpl
     {
         return _fieldName;
     }
-    
-
 }
-
