@@ -40,7 +40,10 @@
  *
  * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: MappingLoader.java,v 1.4 2003/04/23 08:50:51 kvisco Exp $
+ * Portions of this file developed by Keith Visco after Jan 19 2005 are
+ * Copyright (C) 2005 Keith Visco. All Rights Reserverd.
+ *
+ * $Id: MappingLoader.java,v 1.14 2005/08/03 21:11:48 rjoachim Exp $
  */
 
 
@@ -55,7 +58,9 @@ import java.io.PrintWriter;
 import java.util.Hashtable;
 import java.util.Vector;
 import java.util.Enumeration;
-import java.util.NoSuchElementException;
+
+//import org.apache.commons.logging.Log;
+//import org.apache.commons.logging.LogFactory;
 import org.exolab.castor.mapping.AccessMode;
 import org.exolab.castor.mapping.ClassDescriptor;
 import org.exolab.castor.mapping.CollectionHandler;
@@ -65,11 +70,11 @@ import org.exolab.castor.mapping.FieldHandler;
 import org.exolab.castor.mapping.GeneralizedFieldHandler;
 import org.exolab.castor.mapping.MappingResolver;
 import org.exolab.castor.mapping.MappingException;
+import org.exolab.castor.mapping.handlers.EnumFieldHandler;
+import org.exolab.castor.mapping.handlers.TransientFieldHandler;
 import org.exolab.castor.mapping.xml.MappingRoot;
 import org.exolab.castor.mapping.xml.ClassMapping;
 import org.exolab.castor.mapping.xml.FieldMapping;
-import org.exolab.castor.mapping.xml.Container;
-import org.exolab.castor.mapping.xml.Include;
 import org.exolab.castor.util.Messages;
 
 
@@ -79,19 +84,19 @@ import org.exolab.castor.util.Messages;
  * scheme typically by extending this class.
  *
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
- * @author <a href="kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.4 $ $Date: 2003/04/23 08:50:51 $
+ * @author <a href="keith AT kvisco DOT com">Keith Visco</a>
+ * @version $Revision: 1.14 $ $Date: 2005/08/03 21:11:48 $
  */
 public abstract class MappingLoader
     implements MappingResolver
 {
 
-
-    /**
-     * The suffix for the name of a compiled class.
-     */
-    private static final String CompiledSuffix = "Descriptor";
-
+	/**
+	 * The <a href="http://jakarta.apache.org/commons/logging/">Jakarta
+	 * Commons Logging</a> instance used for all logging.
+	 */
+	//private static Log log = LogFactory.getFactory().getInstance(MappingLoader.class);
+	
     /**
      * The prefix for the "add" method
     **/
@@ -107,6 +112,25 @@ public abstract class MappingLoader
     **/
     private static final String IS_METHOD_PREFIX = "is";
     
+    /**
+     * Empty array of class types used for reflection
+     */
+    protected static final Class[] EMPTY_ARGS = new Class[0];
+
+
+    /**
+     * The string argument for the valueOf method, used
+     * for introspection when searching for type-safe
+     * enumeration style classes.
+     */
+    protected static final Class[] STRING_ARG = { String.class };
+
+    /**
+     * Factory method name for type-safe enumerations. 
+    **/
+    protected static final String VALUE_OF = "valueOf";
+    
+    
     //----------------------/
     //- Instance Variables -/
     //----------------------/
@@ -157,6 +181,28 @@ public abstract class MappingLoader
         _logWriter = logWriter;
     }
 
+    /**
+     * Returns the ClassDescriptor for the class with the given name.
+     * If no such ClassDescriptor exists, within the set of mappings
+     * for this MappingLoader, null will be returned.
+     *
+     * @param className the className for which to return 
+     * the associated ClassDescriptor.
+     * @return the ClassDescriptor or null if not found.
+     */
+    public ClassDescriptor getDescriptor( String className )
+    {
+        if (className == null) return null;
+        
+        Enumeration enumeration = _clsDescs.keys();
+        while (enumeration.hasMoreElements()) {
+            Class type = (Class) enumeration.nextElement();
+            if (className.equals(type.getName())) {
+                return (ClassDescriptor) _clsDescs.get( type );
+            }
+        }
+        return null;
+    } //-- getDescriptor
 
     public ClassDescriptor getDescriptor( Class type )
     {
@@ -220,17 +266,32 @@ public abstract class MappingLoader
     public void loadMapping( MappingRoot mapping, Object param )
         throws MappingException
     {
-        Enumeration   enum;
+        Enumeration   enumeration;
 
         // Load the mapping for all the classes. This is always returned
         // in the same order as it appeared in the mapping file.
-        enum = mapping.enumerateClassMapping();
-        while ( enum.hasMoreElements() ) {
-            ClassMapping    clsMap;
-            ClassDescriptor clsDesc;
+        enumeration = mapping.enumerateClassMapping();
+        
+        Vector retryList = null;
+        while ( enumeration.hasMoreElements() ) {
 
-            clsMap = (ClassMapping) enum.nextElement();
-            clsDesc = createDescriptor( clsMap );
+            ClassMapping clsMap = (ClassMapping) enumeration.nextElement();
+            ClassDescriptor clsDesc = null;
+            
+            try {
+                clsDesc = createDescriptor( clsMap );
+            }
+            catch(MappingException mx) {
+                //-- save for later for possible out-of-order
+                //-- mapping files...
+                if (retryList == null) {
+                    retryList = new Vector();
+                }
+                retryList.addElement(clsMap); 
+                continue;
+            }
+            
+            
             if ( clsDesc != NoDescriptor )
                 addDescriptor( clsDesc );
             // If the return value is NoDescriptor then the derived
@@ -239,12 +300,33 @@ public abstract class MappingLoader
                 _logWriter.println( Messages.format( "mapping.ignoringMapping", clsMap.getName() ) );
             }
         }
+        
+        //-- handle possible retries, for now we only loop once
+        //-- on the retries, but we should change this to keep
+        //-- looping until we have no more success rate.
+        if (retryList != null) {
+            Vector tmpRetryList = retryList;
+            retryList = null;
+            enumeration = tmpRetryList.elements();
+            while ( enumeration.hasMoreElements() ) {
+                ClassMapping clsMap = (ClassMapping) enumeration.nextElement();
+                ClassDescriptor clsDesc = createDescriptor( clsMap );
+                if ( clsDesc != NoDescriptor )
+                    addDescriptor( clsDesc );
+                // If the return value is NoDescriptor then the derived
+                // class was not successful in constructing a descriptor.
+                if ( clsDesc == NoDescriptor && _logWriter != null ) {
+                    _logWriter.println( Messages.format( "mapping.ignoringMapping", clsMap.getName() ) );
+                }
+            }
+        }
+        
 
-        enum = _clsDescs.elements();
-        while ( enum.hasMoreElements() ) {
+        enumeration = _clsDescs.elements();
+        while ( enumeration.hasMoreElements() ) {
             ClassDescriptor clsDesc;
 
-            clsDesc = (ClassDescriptor) enum.nextElement();
+            clsDesc = (ClassDescriptor) enumeration.nextElement();
             if ( clsDesc != NoDescriptor  )
                 resolveRelations( clsDesc );
         }
@@ -321,12 +403,9 @@ public abstract class MappingLoader
 
         FieldDescriptor[] fields;
         FieldDescriptor[] identities;
-        Enumeration       enum;
         Class             javaClass;
         ClassDescriptor   extend;
         ClassDescriptor   depend;
-        Container         contMaps[];
-        Vector            relations;
         ClassDescriptor   clsDesc;
 
         // See if we have a compiled descriptor.
@@ -500,7 +579,7 @@ public abstract class MappingLoader
         String hack = (clsMap.getAccess() == null)?"shared":clsMap.getAccess().toString();
         
         clsDesc = new ClassDescriptorImpl( javaClass, fields, identities, extend, depend,
-                                           AccessMode.getAccessMode(hack), clsMap.getVerifyConstructable() );
+                                           AccessMode.valueOf(hack), clsMap.getVerifyConstructable() );
 
         if ( clsDesc instanceof ClassDescriptorImpl )
             ((ClassDescriptorImpl)clsDesc).setMapping( clsMap );
@@ -699,7 +778,49 @@ public abstract class MappingLoader
                 ((GeneralizedFieldHandler)exfHandler).setFieldHandler(handler);
                 handler = custom;
             }
-            else typeInfo = typeInfoRef.typeInfo;
+            else {
+                
+                boolean isTypeSafeEnum = false;
+            	//-- check for type-safe enum style classes
+                if ((fieldType != null) && (!isPrimitive(fieldType))) {
+                    //-- make sure no default constructor
+                    Constructor cons = null;
+                    try {
+                        cons = fieldType.getConstructor(EMPTY_ARGS);
+                        if (!Modifier.isPublic(cons.getModifiers())) {
+                            cons = null;
+                        }
+                    }
+                    catch(NoSuchMethodException nsmx) {
+                        //-- Do nothing
+                    }
+                    try {
+                        if (cons == null) {
+                            //-- make sure a valueOf factory method
+                            //-- exists and no user specified handler exists
+                            Method method = fieldType.getMethod(VALUE_OF, STRING_ARG);
+                            Class returnType = method.getReturnType();
+                            if ((returnType != null) && fieldType.isAssignableFrom(returnType)) {
+                                
+                                int mods = method.getModifiers();
+                                if (Modifier.isStatic(mods)) {
+                                    //-- Use EnumFieldHandler
+                                    handler = new EnumFieldHandler(fieldType, handler, method);
+                                    typeInfo.setImmutable(true);
+                                    isTypeSafeEnum = true;
+                                }
+                            }
+                        }
+                    }
+                    catch(NoSuchMethodException nsmx) {
+                        //-- Do nothing
+                    }
+                }
+                //-- reset proper TypeInfo
+                if (!isTypeSafeEnum) typeInfo = typeInfoRef.typeInfo;
+
+            }
+            
         }
                 
         FieldDescriptorImpl fieldDesc 
@@ -730,6 +851,10 @@ public abstract class MappingLoader
         throws MappingException
     {
         
+        //-- prevent introspection of transient fields
+        if (fieldMap.getTransient()) {
+        	return new TransientFieldHandler();
+        }
         
         CollectionHandler colHandler         = null;
         Class             colType            = null;
@@ -808,7 +933,7 @@ public abstract class MappingLoader
                         // * getter for parent field *
                         String parentField = fieldName.substring(0, point);
                         methodName = GET_METHOD_PREFIX + capitalize( parentField );
-                        method = javaClass.getMethod( methodName, null );                        
+                        method = javaClass.getMethod( methodName, (Class[]) null );                        
                         
                         fieldName = fieldName.substring( point + 1 );
                         
@@ -862,6 +987,7 @@ public abstract class MappingLoader
                 } catch ( MappingException except ) {
                     throw except;
                 } catch ( Exception except ) {
+                	// log.warn ("Unexpected exception", except);
                 }
                 if ( getMethod == null )
                     throw new MappingException( "mapping.accessorNotFound",
@@ -941,7 +1067,7 @@ public abstract class MappingLoader
             fieldName = fieldMap.getName(); // Not the same for nested fields
             if ( fieldName == null )
                 fieldName = ( getMethod == null ? setMethod.getName() : getMethod.getName() );
-                
+
             //-- create handler
             handler = new FieldHandlerImpl( fieldName, 
                                             getSequence, 
@@ -959,7 +1085,7 @@ public abstract class MappingLoader
             try {
                 Method method;
 
-                method = javaClass.getMethod( fieldMap.getCreateMethod(), null );
+                method = javaClass.getMethod( fieldMap.getCreateMethod(), (Class[]) null );
                 handler.setCreateMethod( method );
             } catch ( Exception except ) {
                 // No such/access to method
@@ -970,9 +1096,11 @@ public abstract class MappingLoader
             try {
                 Method method;
 
-                method = javaClass.getMethod( "create" + capitalize( fieldName ), null );
+                method = javaClass.getMethod( "create" + capitalize( fieldName ), (Class[]) null );
                 handler.setCreateMethod( method );
-            } catch ( Exception except ) { }
+            } catch ( Exception except ) {
+            	// log.warn ("Unexpected exception", except);
+            }
         }
 
         // If there is an has/delete method, add them to field handler
@@ -982,14 +1110,14 @@ public abstract class MappingLoader
 
             try {
                 if (fieldMap.getHasMethod() != null)
-                    hasMethod = javaClass.getMethod( fieldMap.getHasMethod(), null );
+                    hasMethod = javaClass.getMethod( fieldMap.getHasMethod(), (Class[]) null );
                 else 
-                    hasMethod = javaClass.getMethod( "has" + capitalize( fieldName ), null );
+                    hasMethod = javaClass.getMethod( "has" + capitalize( fieldName ), (Class[]) null );
                 
                 if ((hasMethod.getModifiers() & Modifier.STATIC ) != 0)
                     hasMethod = null;
                 try {
-                    deleteMethod = javaClass.getMethod( "delete" + capitalize( fieldName ), null );
+                    deleteMethod = javaClass.getMethod( "delete" + capitalize( fieldName ), (Class[]) null );
                     if (( deleteMethod.getModifiers() & Modifier.STATIC ) != 0 )
                           deleteMethod = null;
                 } 
@@ -999,7 +1127,9 @@ public abstract class MappingLoader
                 }
                 handler.setHasDeleteMethod( hasMethod, deleteMethod );
             } 
-            catch ( Exception except ) { }
+            catch ( Exception except ) { 
+            	// log.warn("Unexpected exception", except);
+            }
         }
         
         return handler;
@@ -1058,7 +1188,7 @@ public abstract class MappingLoader
      * @param javaClass The class to which the field belongs
      * @param methodName The name of the accessor method
      * @param fieldType The type of the field if known, or null
-     * @param isGetMethod True if get method, false if set method
+     * @param getMethod True if get method, false if set method
      * @return The method, null if not found
      * @throws MappingException The method is not accessible or is not of the
      *  specified type
@@ -1123,6 +1253,7 @@ public abstract class MappingLoader
                         try {
                             method = javaClass.getMethod( methodName, new Class[] { fieldTypeFromPrimitive } );
                         } catch ( Exception except2 ) {
+                        	// log.warn ("Unexpected exception", except2);
                         }
                     }
                 }
@@ -1211,14 +1342,34 @@ public abstract class MappingLoader
             return name;
         return Character.toUpperCase( first ) + name.substring( 1 );
     }
+    
+    /**
+     * Returns true if the given class should be treated as a primitive
+     * type
+     * @return true if the given class should be treated as a primitive
+     * type
+    **/
+    protected static boolean isPrimitive(Class type) {
+
+        if (type.isPrimitive()) return true;
+
+        if ((type == Boolean.class) || (type == Character.class))
+            return true;
+
+        return (type.getSuperclass() == Number.class);
+
+    } //-- isPrimitive
+    
 
 
     /**
      * A class used to by the createFieldHandler method in order to
      * save the reference of the TypeInfo that was used.
      */
-    class TypeInfoReference {
-        TypeInfo typeInfo = null;
+    public class TypeInfoReference {
+        public TypeInfo typeInfo = null;
     } //-- TypeInfoReference
     
+    
+    
 } //-- MappingLoader
