@@ -40,7 +40,7 @@
  *
  * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: Types.java,v 1.4 2003/04/23 08:48:20 kvisco Exp $
+ * $Id: Types.java,v 1.9 2005/08/03 21:11:48 rjoachim Exp $
  */
 
 
@@ -49,19 +49,15 @@ package org.exolab.castor.mapping.loader;
 
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Modifier;
-import java.util.Date;
-import java.text.DateFormat;
-import java.text.DecimalFormat;
-import java.text.SimpleDateFormat;
-import java.text.ParseException;
 import java.io.Serializable;
 import java.math.BigDecimal;
 import java.math.BigInteger;
-import org.exolab.castor.mapping.MappingException;
+import java.util.Vector;
+
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+import org.exolab.castor.types.Duration;
 import org.exolab.castor.util.Messages;
-import org.exolab.castor.util.MimeBase64Encoder;
-import org.exolab.castor.util.MimeBase64Decoder;
-//import org.exolab.castor.util.ClobImpl;
 
 
 /**
@@ -71,12 +67,11 @@ import org.exolab.castor.util.MimeBase64Decoder;
  * column, etc), as well as to create a new instance of a type.
  *
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
- * @version $Revision: 1.4 $ $Date: 2003/04/23 08:48:20 $
+ * @version $Revision: 1.9 $ $Date: 2005/08/03 21:11:48 $
  */
 public class Types
 {
 
-
     /**
      * Returns the class name based on the supplied type name. The type name
      * can be a short name (e.g. int, byte) or any other Java class (e.g.
@@ -97,9 +92,11 @@ public class Types
                 return ( _typeInfos[ i ].primitive != null ? _typeInfos[ i ].primitive :
                          _typeInfos[ i ].javaType );
         }
-        if ( loader != null )
-            return loader.loadClass( typeName );
-        else
+        if ( loader != null ) {
+            Class aClass = loader.loadClass (typeName);
+            // _log.debug ("Loaded class " + aClass);
+            return aClass;
+        } else
             return Class.forName( typeName );
     }
 
@@ -206,11 +203,24 @@ public class Types
     }
 
 
+    private static final Vector _enums = new Vector();
+    
+    public static void addEnumType(Class type) {
+        _enums.add(type);
+    }
+    
+    public static boolean isEnumType(Class type) {
+        return _enums.contains(type);
+    }
+
+    
     /**
      * Constructs a new object from the given class. Does not generate any
      * checked exceptions, since object creation has been proven to work
      * when creating descriptor from mapping.
      *
+     * @param type The class type of the object instance to be constructed.
+     * @return An instance of the class type specified.
      * @throws IllegalStateException The Java object cannot be constructed
      */
     public static Object newInstance( Class type )
@@ -234,6 +244,9 @@ public class Types
      * checked exceptions, since object creation has been proven to work
      * when creating descriptor from mapping.
      *
+     * @param type The class type of the object instance to be constructed.
+     * @param args Arguments to be supplied to constructor call. 
+     * @return An instance of the class type specified.
      * @throws IllegalStateException The Java object cannot be constructed
      */
     public static Object newInstance( Class type, Object args[] )
@@ -286,6 +299,7 @@ public class Types
      * The class must be publicly available and have a default public
      * constructor.
      *
+     * @param allowAbstractOrInterface True to indicate that abstract classes of interfaces are allowed.
      * @param type The Java type
      * @return True if constructable
      */
@@ -300,7 +314,9 @@ public class Types
             if ( ( type.getConstructor( new Class[0] ).getModifiers() & Modifier.PUBLIC ) != 0 )
                 return true;
         } catch ( NoSuchMethodException except ) {
+        	// TODO [WG]: no code available
         } catch ( SecurityException except ) {
+        	// TODO [WG]: no code available
         }
         return false;
     }
@@ -386,10 +402,13 @@ public class Types
     /**
      * Returns the matching constructor for the given arguments
      *
-     * @returns the matching constructor for the given arguments
+     * @param type The class type of the object instance to be constructed.
+     * @param args Arguments to be supplied to constructor call. 
+     * @return the matching constructor for the given arguments
+     * @throws NoSuchMethodException If no constructor with the given number/types of arguments exists.
      */
     private static Constructor findConstructor(Class type, Object[] args) 
-        throws IllegalAccessException, NoSuchMethodException
+        throws NoSuchMethodException
     {
         
         Constructor[] constructors = type.getConstructors();
@@ -510,17 +529,17 @@ public class Types
         new TypeInfo( "int",         java.lang.Integer.TYPE,
                       java.lang.Integer.TYPE,    true,     new Integer( 0 ) ),
         new TypeInfo( "long",        java.lang.Long.TYPE,
-                      java.lang.Long.class,       true,     new Long( 0 ) ),
+                      java.lang.Long.class,       true,     new Long( (long) 0 ) ),
         new TypeInfo( "big-integer", null,
                       java.math.BigInteger.class, true,     BigInteger.valueOf( 0 ) ),
         new TypeInfo( "boolean",     java.lang.Boolean.TYPE,
                       java.lang.Boolean.class,    true,     Boolean.FALSE ),
         new TypeInfo( "double",      java.lang.Double.TYPE,
-                      java.lang.Double.class,     true,     new Double( 0 ) ),
+                      java.lang.Double.class,     true,     new Double( (double) 0 ) ),
         new TypeInfo( "float",       java.lang.Float.TYPE,
-                      java.lang.Float.class,      true,     new Float( 0 ) ),
+                      java.lang.Float.class,      true,     new Float( (float) 0 ) ),
         new TypeInfo( "big-decimal", null,
-                      java.math.BigDecimal.class, true,     new BigDecimal( 0 ) ),
+                      java.math.BigDecimal.class, true,     new BigDecimal( (double) 0 ) ),
         new TypeInfo( "byte",        java.lang.Byte.TYPE,
                       java.lang.Byte.class,       true,     new Byte( (byte) 0 ) ),
         new TypeInfo( "date",        null,
@@ -530,7 +549,7 @@ public class Types
         new TypeInfo( "sqldate",        null,
                       java.sql.Date.class,       true,     null ),
         new TypeInfo( "sqltime",        null,
-                      java.sql.Date.class,       true,     null ),
+                      java.sql.Time.class,       true,     null ),
         new TypeInfo( "short",       java.lang.Short.TYPE,
                       java.lang.Short.class,      true,     new Short( (short) 0 ) ),
         new TypeInfo( "char",        java.lang.Character.TYPE,
@@ -574,11 +593,19 @@ public class Types
           new TypeInfo( XML,        "xml",         org.w3c.dom.Document.class, org.w3c.dom.Element.class ),
           new TypeInfo( Serialized, "ser",         java.io.Serializable.class, null )
         */
+        new TypeInfo( "duration",   null,
+                      org.exolab.castor.types.Duration.class,     false,     new Duration (0) ),
+        new TypeInfo( "xml-date",   null,
+                      org.exolab.castor.types.Date.class,     false,     new org.exolab.castor.types.Date (0) ),
+        new TypeInfo( "xml-time",   null,
+                      org.exolab.castor.types.Time.class,     false,     new org.exolab.castor.types.Time (0) )
+                              
     };
 
     /**
      * A hack for JDK 1.1 Compatibility
-    **/
+     * @return A Class instance for CLOB types.
+     */
     private static final Class getClobClass() {        
         Class type = null;
         try {
