@@ -40,30 +40,29 @@
  *
  * Copyright 2000 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: FieldMolder.java,v 1.2 2003/03/25 16:27:44 mguessan Exp $
+ * $Id: FieldMolder.java,v 1.15 2005/09/06 19:39:44 rjoachim Exp $
  */
 
 
 package org.exolab.castor.persist;
 
 
-import java.util.Hashtable;
-import java.util.Enumeration;
-import java.util.Collection;
-import java.util.Vector;
-import java.util.HashMap;
-import java.util.ArrayList;
 import java.lang.reflect.Field;
+import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
-import java.lang.reflect.InvocationTargetException;
+import java.util.ArrayList;
+import java.util.HashMap;
+
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+import org.exolab.castor.jdo.DataObjectAccessException;
 import org.exolab.castor.mapping.MappingException;
-import org.exolab.castor.mapping.loader.Types;
 import org.exolab.castor.mapping.TypeConvertor;
+import org.exolab.castor.mapping.loader.Types;
 import org.exolab.castor.mapping.xml.FieldMapping;
 import org.exolab.castor.mapping.xml.types.DirtyType;
 import org.exolab.castor.util.Messages;
-import org.exolab.castor.jdo.DataObjectAccessException;
 
 
 /**
@@ -74,16 +73,40 @@ import org.exolab.castor.jdo.DataObjectAccessException;
  */
 public class FieldMolder {
 
-    public static final short PRIMITIVE = 0;
-
+	// accepted collection types
+    private static final String COLLECTION_TYPE_ARRAY = "array";
+    private static final String COLLECTION_TYPE_COLLECTION = "collection";
+    private static final String COLLECTION_TYPE_VECTOR = "vector";
+    private static final String COLLECTION_TYPE_ARRAYLIST = "arraylist";
+    private static final String COLLECTION_TYPE_HASHTABLE = "hashtable";
+    private static final String COLLECTION_TYPE_MAP = "map";
+    private static final String COLLECTION_TYPE_SET = "set";
+    private static final String COLLECTION_TYPE_HASHMAP = "hashmap";
+    private static final String COLLECTION_TYPE_HASHSET = "hashset";
+    
+	private static final String FIELD_TYPE_SERIALIZABLE = "serializable";
+	
+	public static final short PRIMITIVE = 0;
     public static final short SERIALIZABLE = 1;
-
     public static final short PERSISTANCECAPABLE = 2;
-
     public static final short ONE_TO_MANY = 3;
-
     public static final short MANY_TO_MANY = 4;
 
+    /**
+     * The <a href="http://jakarta.apache.org/commons/logging/">Jakarta
+     * Commons Logging</a> instance used for all logging.
+     */
+    private static Log _log = LogFactory.getFactory().getInstance(FieldMolder.class);
+
+    // method prefixes
+    private static final String METHOD_GET_PREFIX = "get";
+    private static final String METHOD_IS_PREFIX = "is";
+    private static final String METHOD_SET_PREFIX = "set";
+    private static final String METHOD_ADD_PREFIX = "add";
+	private static final String METHOD_CREATE_PREFIX = "create";
+	private static final String METHOD_DELETE_PREFIX = "delete";
+	private static final String METHOD_HAS_PREFIX = "has";
+    
     private boolean _lazy;
 
     private boolean _check;
@@ -109,6 +132,8 @@ public class FieldMolder {
     private Object        _default;
 
     private boolean _readonly;
+    
+    private boolean _transient;
 
     /**
      * Collection of reflection service keyed by ClassLoader instance.
@@ -166,7 +191,6 @@ public class FieldMolder {
     public boolean isManyToMany() {
         return _manyToManyLoader != null;
     }
-
     public boolean isDependent() {
         if ( _fMold == null )
             return false;
@@ -202,6 +226,9 @@ public class FieldMolder {
     public boolean isAddable() {
         return _addable;
     }
+    public boolean isTransient() {
+        return _transient;
+    }
     void setFieldClassMolder( ClassMolder fMold ) {
         _fMold = fMold;
     }
@@ -226,7 +253,6 @@ public class FieldMolder {
     }
 
     public Object getValue( Object object, ClassLoader loader ) {
-        Object value;
         ReflectService rf = getContextReflectService ( loader );
         try {
             // If field is accessed directly, get it's value, if not
@@ -240,17 +266,17 @@ public class FieldMolder {
             else if ( rf._getMethod != null ) {
                 if ( rf._getSequence != null )
                     for ( int i = 0; i < rf._getSequence.length; i++ ) {
-                        object = rf._getSequence[ i ].invoke( object, null );
+                        object = rf._getSequence[ i ].invoke( object, (Object[]) null );
                         if ( object == null )
                             break;
                     }
                 // If field has 'has' method, false means field is null
                 // and do not attempt to call getValue. Otherwise,
                 if (  object == null ||
-                        ( rf._hasMethod != null && ! ( (Boolean) rf._hasMethod.invoke( object, null ) ).booleanValue() ) )
+                        ( rf._hasMethod != null && ! ( (Boolean) rf._hasMethod.invoke( object, (Object[]) null ) ).booleanValue() ) )
                     return null;
                 else
-                    return rf._getMethod.invoke( object, null );
+                    return rf._getMethod.invoke( object, (Object[]) null );
             } else
                 return null;
         } catch ( IllegalAccessException except ) {
@@ -266,9 +292,14 @@ public class FieldMolder {
     public void addValue( Object object, Object value, ClassLoader loader ) {
 
         ReflectService rf = getContextReflectService ( loader );
+        
+        if (_log.isDebugEnabled()) {
+        	_log.debug ("Calling " + rf._addMethod.getName() + " on " + object.getClass().getName() + " with value " + value);
+        }
+        
         try {
             if ( rf._addMethod == null )
-                throw new DataObjectAccessException("No add method defined for this field");
+                throw new DataObjectAccessException(Messages.format ("mapping.addMethodNotDefined", this.getName()));
 
             if ( value == null )
                 throw new NullPointerException("Adding null value is not allowed");
@@ -299,7 +330,7 @@ public class FieldMolder {
                         Object last;
 
                         last = object;
-                        object = rf._getSequence[ i ].invoke( object, null );
+                        object = rf._getSequence[ i ].invoke( object, (Object[]) null );
                         if ( object == null ) {
                             // if the value is not null, we must instantiate
                             // the object in the sequence
@@ -313,7 +344,7 @@ public class FieldMolder {
                     }
                 if ( object != null ) {
                     if ( value == null && rf._deleteMethod != null )
-                        rf._deleteMethod.invoke( object, null );
+                        rf._deleteMethod.invoke( object, (Object[]) null );
                     else
                         rf._setMethod.invoke( object, new Object[] { value == null ? _default : value } );
                 }
@@ -361,7 +392,9 @@ public class FieldMolder {
     // ======================================================
     protected Class getCollectionType( String coll, boolean lazy )
             throws MappingException {
-        Class type;
+        /* 
+         * Class type;
+         */
 
         for ( int i=0; i<info.length; i++ ) {
             if ( info[i].name.equals( coll ) )
@@ -392,16 +425,15 @@ public class FieldMolder {
     }
 
     private static CollectionInfo[] info =
-        { new CollectionInfo( "collection", java.util.Collection.class ),
-          new CollectionInfo( "vector", java.util.Vector.class ),
-          new CollectionInfo( "arraylist", java.util.ArrayList.class ),
-          new CollectionInfo( "hashtable", java.util.Hashtable.class ),
-          new CollectionInfo( "hashmap", java.util.HashMap.class ),
-          new CollectionInfo( "set", java.util.Set.class ),
-          new CollectionInfo( "hashset", java.util.HashSet.class ),
-          new CollectionInfo( "map", java.util.Map.class ),
-          new CollectionInfo( "array", Object[].class ) };
-
+        { new CollectionInfo( COLLECTION_TYPE_COLLECTION, java.util.Collection.class ),
+          new CollectionInfo( COLLECTION_TYPE_VECTOR, java.util.Vector.class ),
+          new CollectionInfo( COLLECTION_TYPE_ARRAYLIST, java.util.ArrayList.class ),
+          new CollectionInfo( COLLECTION_TYPE_HASHTABLE, java.util.Hashtable.class ),
+          new CollectionInfo( COLLECTION_TYPE_HASHMAP, java.util.HashMap.class ),
+          new CollectionInfo( COLLECTION_TYPE_SET, java.util.Set.class ),
+          new CollectionInfo( COLLECTION_TYPE_HASHSET, java.util.HashSet.class ),
+          new CollectionInfo( COLLECTION_TYPE_MAP, java.util.Map.class ),
+          new CollectionInfo( COLLECTION_TYPE_ARRAY, Object[].class ) };
 
           //( array | vector | hashtable | collection | set | map )
 
@@ -411,9 +443,8 @@ public class FieldMolder {
      * create a new stock {@link FieldMolder}. Implementations may
      * extend this class to create a more suitable descriptor.
      *
-     * @param javaClass The class to which the field belongs
+     * @param eMold The ClassMolder to which the field belongs
      * @param fieldMap The field mapping information
-     * @return The field descriptor
      * @throws MappingException The field or its accessor methods are not
      *  found, not accessible, not of the specified type, etc
      */
@@ -450,7 +481,7 @@ public class FieldMolder {
                 _check = true;
             }
 
-            if ( "serializable".equals( fieldMap.getType() ) )
+            if ( FIELD_TYPE_SERIALIZABLE.equals( fieldMap.getType() ) )
                 _serial = true;
 
             if ( fieldMap.getSql() == null )
@@ -465,13 +496,24 @@ public class FieldMolder {
             if ( fieldMap.getSql() != null )
                 _readonly = fieldMap.getSql().getReadonly();
 
+            // check whether complete field is declared transient
+            _transient = fieldMap.getTransient();
+            
+            if (fieldMap.getSql() != null) {
+                boolean isSQLTransient = fieldMap.getSql().getTransient();
+                if (_transient == true && isSQLTransient == false) {
+                    throw new MappingException (Messages.message("persist.transient.conflict"));
+                }
+                _transient = isSQLTransient;  
+            }
+            
             if ( fieldMap.getCollection() != null )
                 _multi = true;
 
             // Set collection type
             if ( fieldMap.getCollection() != null ) {
               // simple arrays support
-              if ("array".equals(fieldMap.getCollection().toString())) {
+              if (COLLECTION_TYPE_ARRAY.equals(fieldMap.getCollection().toString())) {
                 String arrayClassName = "[L" + fieldMap.getType() + ";";
                 try {
                   _colClass = ds.resolve(arrayClassName);
@@ -536,7 +578,7 @@ public class FieldMolder {
                 ArrayList setSeq = new ArrayList();
                 String name = fieldMap.getName();
                 Class last;
-                Method method;
+                Method method = null;
                 String methodName = null;
 
                 try {
@@ -545,8 +587,24 @@ public class FieldMolder {
                         if ( point < 0 )
                             break;
                         last = javaClass;
-                        methodName = "get" + capitalize( name.substring( 0, point ) );
-                        method = javaClass.getMethod( methodName, null );
+                        
+                        if (fieldMap.getType().compareTo("boolean") == 0){
+                            try{
+                                methodName = METHOD_IS_PREFIX + capitalize( name.substring( 0, point ) );
+                                method = javaClass.getMethod( methodName, (Class[]) null );
+                            } 
+                            catch (NoSuchMethodException nsme) {
+                                if(_log.isDebugEnabled()) {
+                                    _log.debug (Messages.format("mapping.accessorNotFound", methodName, "boolean", getName()));
+                                }
+                            }
+                        }
+                        
+                        if (method == null) {
+                            methodName = METHOD_GET_PREFIX + capitalize( name.substring(0, point) );
+                            method = javaClass.getMethod( methodName, (Class[]) null );
+                        }
+                        
                         name = name.substring( point + 1 );
                         // Make sure method is not abstract/static
                         // (note: Class.getMethod() returns only public methods).
@@ -557,7 +615,12 @@ public class FieldMolder {
                         getSeq.add( method );
                         javaClass = method.getReturnType();
                         // setter;   Note: javaClass already changed, use "last"
-                        methodName = "set" + methodName.substring(3);
+                        if (fieldMap.getType().compareTo("boolean") == 0) {
+                        	methodName = METHOD_SET_PREFIX + methodName.substring(2);
+                        } else {
+                            methodName = METHOD_SET_PREFIX + methodName.substring(3);
+                        }
+                        	
                         try {
                             method = last.getMethod( methodName, new Class[] { javaClass } );
                             if ( ( method.getModifiers() & Modifier.ABSTRACT ) != 0 ||
@@ -567,44 +630,67 @@ public class FieldMolder {
                             method = null;
                         }
                         setSeq.add( method );
+                        method = null;
+                        
                     }
-                } catch (Exception ex) {
-                    throw new MappingException( "mapping.accessorNotFound",
-                           methodName, null, javaClass.getName() );
+                } 
+                catch (Exception ex) {
+                    throw new MappingException(Messages.format ( "mapping.accessorNotFound",
+                           methodName, null, javaClass.getName() ), ex);
                 }
                 if ( getSeq.size() > 0 ) {
                     _defaultReflectService._getSequence = (Method[]) getSeq.toArray( new Method[ 0 ] );
                     _defaultReflectService._setSequence = (Method[]) setSeq.toArray( new Method[ 0 ] );
                 }
-                Class methodClass = _colClass!=null? _colClass: null;
-                _defaultReflectService._getMethod = findAccessor( javaClass, "get" + capitalize( name ), methodClass, true );
-
-                if ( _defaultReflectService._getMethod == null )
-                    throw new MappingException( "mapping.accessorNotFound",
-                            "get" + name, null, javaClass.getName() );
+                Class methodClass = _colClass != null? _colClass: declaredClass;
+                _defaultReflectService._getMethod = null;
+                
+                // if field is of type boolean, check whether is<Field>() is defined.
+                if(fieldMap.getType() != null && fieldMap.getType().compareTo("boolean") == 0) {
+                	_defaultReflectService._getMethod = 
+                		findAccessor( javaClass, METHOD_IS_PREFIX + capitalize( name ), methodClass, true );
+                }
+                
+                if( _defaultReflectService._getMethod == null ) {
+                	_defaultReflectService._getMethod = 
+                		findAccessor( javaClass, METHOD_GET_PREFIX + capitalize( name ), methodClass, true );
+                }
 
+                if ( _defaultReflectService._getMethod == null ){
+                	if ( fieldMap.getType().compareTo("boolean") == 0){
+                		throw new MappingException( "mapping.accessorNotFound",
+                				METHOD_GET_PREFIX + "/" + METHOD_IS_PREFIX + capitalize( name ), fieldMap.getType(), eMold.getName() );
+                	} else {
+                		throw new MappingException( "mapping.accessorNotFound",
+                				METHOD_GET_PREFIX + capitalize( name ), fieldMap.getType(), eMold.getName() );
+                	}
+                }
+                
                 // update fClass, because we can't tell between primitive
                 // and primitive wrapper from the mapping
                 if ( _colClass == null )
                     _defaultReflectService._fClass = _defaultReflectService._getMethod.getReturnType();
 
-                _defaultReflectService._setMethod = findAccessor( javaClass, "set" + capitalize( name ), methodClass, false );
+                _defaultReflectService._setMethod = findAccessor( javaClass, METHOD_SET_PREFIX + capitalize( name ), methodClass, false );
 
-                if ( _defaultReflectService._setMethod == null )
-                    _defaultReflectService._addMethod
-                        = findAccessor( javaClass, "add" + capitalize( name ), null, false );
-
-                if ( _defaultReflectService._addMethod == null && name.endsWith("s") )
-                    _defaultReflectService._addMethod
-                        = findAccessor( javaClass, "add" + capitalize( name ).substring(0,name.length()-1), declaredClass, false );
+                if ( _defaultReflectService._setMethod == null ) {
+                	_defaultReflectService._addMethod
+					= findAccessor( javaClass, METHOD_ADD_PREFIX + capitalize( name ), declaredClass, false );
+                	
+                	// look again, but this time without a trailing 's'
+                	if ( _defaultReflectService._addMethod == null && name.endsWith("s") )
+                		_defaultReflectService._addMethod
+						= findAccessor( javaClass, METHOD_ADD_PREFIX + capitalize( name ).substring(0,name.length()-1), declaredClass, false );
 
+                	// if add<FieldName>() has been found, set _addable to true 
+                	if ( _defaultReflectService._addMethod != null )
+                        _addable = true;
+                }
+                
                 if ( _defaultReflectService._setMethod == null && _defaultReflectService._addMethod == null )
-                    throw new MappingException( "mapping.accessorNotFound",
-                        "set/add" + capitalize( name ), declaredClass, javaClass.getName() );
-
-                if ( _defaultReflectService._addMethod != null )
-                    _addable = true;
-
+                	throw new MappingException( "mapping.accessorNotFound",
+                			METHOD_SET_PREFIX + "/" + METHOD_ADD_PREFIX + capitalize( name ), declaredClass, javaClass.getName() );
+            	
             } else {
 
                 // Bean type object, map field to get<Method>/set<Method>
@@ -635,14 +721,17 @@ public class FieldMolder {
                 }
 
                 // Second look up the set/add accessor
-                if ( fieldMap.getSetMethod() != null ) {
+                if ( fieldMap.getSetMethod() != null) {
 
                     if ( _colClass != null ) {
                         _defaultReflectService._setMethod = findAccessor( javaClass, fieldMap.getSetMethod(), _colClass, false );
 
                         // find addXXX method only if lazy loading is turned off
-                        if ( _defaultReflectService._setMethod == null && !fieldMap.getLazy() )
-                            _defaultReflectService._addMethod = findAccessor( javaClass, fieldMap.getSetMethod(), methodClass, false );
+                        if ( _defaultReflectService._setMethod == null && !fieldMap.getLazy() ) {
+                            _defaultReflectService._addMethod = 
+                            	findAccessor( javaClass, fieldMap.getSetMethod(), declaredClass, false );
+                            if ( _defaultReflectService._addMethod != null) _addable = true;
+                        }
 
                     } else {
                         // find setXXX method
@@ -650,9 +739,11 @@ public class FieldMolder {
                     }
 
 
-                    if ( _defaultReflectService._setMethod == null && _defaultReflectService._addMethod == null )
-                        throw new MappingException( "mapping.accessorNotFound",
-                                fieldMap.getSetMethod(), methodClass, javaClass.getName() );
+                    if (_defaultReflectService._setMethod == null && 
+                    	_defaultReflectService._addMethod == null)
+						throw new MappingException("mapping.accessorNotFound",
+								fieldMap.getSetMethod(), methodClass, javaClass
+										.getName());
 
                     if ( _defaultReflectService._fClass == null )
                         _defaultReflectService._fClass = _defaultReflectService._setMethod.getParameterTypes()[ 0 ];
@@ -669,9 +760,7 @@ public class FieldMolder {
             // what exact instance to be created.
             if ( fieldMap.getCreateMethod() != null ) {
                 try {
-                    Method method;
-
-                    _defaultReflectService._createMethod = javaClass.getMethod( fieldMap.getCreateMethod(), null );
+                    _defaultReflectService._createMethod = javaClass.getMethod( fieldMap.getCreateMethod(), (Class[]) null );
                 } catch ( Exception except ) {
                     // No such/access to method
                     throw new MappingException( "mapping.createMethodNotFound",
@@ -681,9 +770,11 @@ public class FieldMolder {
                 try {
                     Method method;
 
-                    method = javaClass.getMethod( "create" + capitalize( fieldMap.getName() ), null );
+                    method = javaClass.getMethod( METHOD_CREATE_PREFIX + capitalize( fieldMap.getName() ), (Class[]) null );
                     _defaultReflectService._createMethod = method;
-                } catch ( Exception except ) { }
+                } catch ( Exception except ) {
+                	// no explicit exception handling
+                }
             }
 
             // If there is an has/delete method, add them to field handler
@@ -692,7 +783,7 @@ public class FieldMolder {
                 Method deleteMethod = null;
 
                 try {
-                    hasMethod = javaClass.getMethod( "has" + capitalize( fieldMap.getName() ), null );
+                    hasMethod = javaClass.getMethod( METHOD_HAS_PREFIX + capitalize( fieldMap.getName() ), (Class[]) null );
                     if ( ( hasMethod.getModifiers() & Modifier.PUBLIC ) == 0 ||
                          ( hasMethod.getModifiers() & Modifier.STATIC ) != 0 )
                         hasMethod = null;
@@ -700,11 +791,15 @@ public class FieldMolder {
                         if ( ( hasMethod.getModifiers() & Modifier.PUBLIC ) == 0 ||
                              ( hasMethod.getModifiers() & Modifier.STATIC ) != 0 )
                             deleteMethod = null;
-                        deleteMethod = javaClass.getMethod( "delete" + capitalize( fieldMap.getName() ), null );
-                    } catch ( Exception except ) { }
+                        deleteMethod = javaClass.getMethod( METHOD_DELETE_PREFIX + capitalize( fieldMap.getName() ), (Class[]) null );
+                    } catch ( Exception except ) {
+                    	// no explicit exception handling 
+                    }
                     _defaultReflectService._hasMethod = hasMethod;
                     _defaultReflectService._deleteMethod = deleteMethod;
-                } catch ( Exception except ) { }
+                } catch ( Exception except ) {
+                	// no explicit exception handling
+                }
             }
 
             if ( _defaultReflectService._field == null && _defaultReflectService._setMethod == null && _defaultReflectService._getMethod == null ) {
@@ -713,6 +808,7 @@ public class FieldMolder {
 
             ds.pairFieldClass( this, _fType );
         } catch ( NullPointerException e ) {
+            _log.fatal("Caught unexpected NullPointerException: ", e);
             throw new MappingException("Unexpected Null pointer!\n"+e);
         }
         _fieldName = fieldMap.getName();
@@ -754,7 +850,9 @@ public class FieldMolder {
                 throw new MappingException( "mapping.fieldTypeMismatch", field, fieldType.getName() );
             return field;
         } catch ( NoSuchFieldException except ) {
+        	// no explicit exception handling
         } catch ( SecurityException except ) {
+        	// no explicit exception handling
         }
         return null;
     }
@@ -764,11 +862,16 @@ public class FieldMolder {
      * Returns the named accessor. Uses reflection to return the named
      * accessor and check the return value or parameter type, if
      * specified.
+     * 
+     * TODO:  Most of this code appears to be (badly) duplicated in MappingLoader, which
+     * is probably a closer source to the true behavior of this method.  At some point in
+     * the future, this should get rationalized.
+     * 
      *
      * @param javaClass The class to which the field belongs
      * @param methodName The name of the accessor method
      * @param fieldType The type of the field if known, or null
-     * @param isGetMethod True if get method, false if set method
+     * @param getMethod True if get method, false if set method
      * @return The method, null if not found
      * @throws MappingException The method is not accessible or is not of the
      *  specified type
@@ -777,8 +880,10 @@ public class FieldMolder {
                                         Class fieldType, boolean getMethod )
         throws MappingException
     {
-        Method   method;
+        Method   method = null;
         Method[] methods;
+        Class[] parameterTypes;
+        Class fieldTypeFromPrimitive;
         int      i;
 
         try {
@@ -787,50 +892,102 @@ public class FieldMolder {
                 // method name. Look up the field and potentially check the
                 // return type.
                 method = javaClass.getMethod( methodName, new Class[ 0 ] );
+                
                 if ( fieldType == null ) {
                     fieldType = Types.typeFromPrimitive( method.getReturnType() );
-                } else if ( fieldType == java.io.Serializable.class && java.io.Serializable.class.isAssignableFrom( method.getReturnType() ) ) {
-                    // special case
-                } else if ( ! Types.typeFromPrimitive( fieldType ).isAssignableFrom( Types.typeFromPrimitive( method.getReturnType() ) ) )
-                    throw new MappingException( "mapping.accessorReturnTypeMismatch",
-                                                method, fieldType.getName() );
-            } else {
-                // Set method: look for the named method or prepend set to the
-                // method name. If the field type is know, look up a suitable
-                // method. If the fielf type is unknown, lookup the first
-                // method with that name and one parameter.
-                if ( fieldType != null ) {
-                    try {
-                        method = javaClass.getMethod( methodName, new Class[] { fieldType } );
-                    } catch ( Exception except ) {
-                        method = javaClass.getMethod( methodName, new Class[] { Types.typeFromPrimitive( fieldType ) } );
-                    }
                 } else {
-                    methods = javaClass.getMethods();
-                    method = null;
-                    for ( i = 0 ; i < methods.length ; ++i ) {
-                        if ( methods[ i ].getName().equals( methodName ) &&
-                             methods[ i ].getParameterTypes().length == 1 ) {
-                            method = methods[ i ];
-                            break;
+                        fieldType = Types.typeFromPrimitive(fieldType);
+                        Class returnType = Types.typeFromPrimitive( method.getReturnType());
+                        
+                        //-- First check against whether the declared type is
+                        //-- an interface or abstract class. We also check
+                        //-- type as Serializable for CMP 1.1 compatibility.
+                        if (fieldType.isInterface() ||
+                            ((fieldType.getModifiers() & Modifier.ABSTRACT) != 0) ||
+                            (fieldType == java.io.Serializable.class))
+                        {
+                            if ( ! fieldType.isAssignableFrom( returnType ) )
+                            throw new MappingException("mapping.accessorReturnTypeMismatch",
+                                                        method, fieldType.getName() );
                         }
-                    }
-                    if ( method == null )
-                        return null;
-                }
-            }
-            // Make sure method is public and not abstract/static.
-            if ( ( method.getModifiers() & Modifier.PUBLIC ) == 0 ||
-                 ( method.getModifiers() & Modifier.STATIC ) != 0 )
-                throw new MappingException( "mapping.accessorNotAccessible",
-                                            methodName, javaClass.getName() );
-            return method;
-        } catch ( MappingException except ) {
-            throw except;
-        } catch ( Exception except ) {
-        }
-        return null;
-    }
+                        else {
+                            if ( ! returnType.isAssignableFrom( fieldType ) )
+                            throw new MappingException("mapping.accessorReturnTypeMismatch",
+                                                        method, fieldType.getName() );
+                        }
+                    }      
+             } else {
+                 method = null;
+                 fieldTypeFromPrimitive = null;
+
+                 // Set method: look for the named method or prepend set to the
+                 // method name. If the field type is know, look up a suitable
+                 // method. If the fielf type is unknown, lookup the first
+                 // method with that name and one parameter.
+                 if ( fieldType != null ) {
+                     fieldTypeFromPrimitive = Types.typeFromPrimitive( fieldType );
+                     // first check for setter with reference type (e.g. setXxx(Integer))
+                     try {
+                    	 method = javaClass.getMethod( methodName, new Class[] { fieldTypeFromPrimitive } );
+                     } catch ( Exception except ) {
+                         // if setter for reference type could not be found
+                         // try to find one for primitive type (e.g. setXxx(int))
+                         try {
+                        	 method = javaClass.getMethod( methodName, new Class[] { fieldType } );
+                         } catch ( Exception except2 ) {
+                         	 // log.warn ("Unexpected exception", except2);
+                         }
+                     }
+                 }
+                 if ( null == method ) {
+                     methods = javaClass.getMethods();
+                     method = null;
+                     for ( i = 0 ; i < methods.length ; ++i ) {
+                         if ( methods[ i ].getName().equals( methodName ) ) {
+                             parameterTypes = methods[ i ].getParameterTypes();
+                             if (parameterTypes.length != 1) continue;
+                             
+                             Class paramType = Types.typeFromPrimitive( parameterTypes[0] );
+                             
+                             //-- check straight match
+                             if ((fieldType == null) ||
+                                  paramType.isAssignableFrom( fieldTypeFromPrimitive )) 
+                             {
+                                 method = methods[ i ];
+                                 break;
+                             }
+                             //-- Check against whether the declared type is
+                             //-- an interface or abstract class. 
+                             else if (fieldType.isInterface() ||
+                                     ((fieldType.getModifiers() & Modifier.ABSTRACT) != 0))
+                             {
+                                 if (fieldTypeFromPrimitive.isAssignableFrom( paramType )) 
+                                 {
+                                     method = methods[i];
+                                     break;
+                                 }
+                                     
+                             }
+                         }
+                     }
+                     if ( method == null )
+                         return null;
+                 }
+             }
+             // Make sure method is public and not static.
+             // (note: Class.getMethod() returns only public methods).
+             //if ( ( method.getModifiers() & Modifier.ABSTRACT ) != 0 ||
+             if ( ( method.getModifiers() & Modifier.STATIC ) != 0 )
+                 throw new MappingException( "mapping.accessorNotAccessible",
+                                             methodName, javaClass.getName() );
+             return method;
+         } catch ( MappingException except ) {
+             throw except;
+         } catch ( Exception except ) {
+             //System.out.println(except.toString());
+         }
+         return null;
+     }
 
     private String capitalize( String name )
     {
@@ -873,7 +1030,9 @@ public class FieldMolder {
         /**
          * Default constructor. All fields need to be set one by one.
          */
-        ReflectService () { }
+        ReflectService () {
+        	// no code to execute
+		}
 
         /**
          * Contructs a ReflectService object based on the instance provided
