@@ -40,16 +40,22 @@
  *
  * Copyright 1999 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: ObjectLock.java,v 1.1.1.1 2003/03/03 07:08:46 kvisco Exp $
+ * $Id: ObjectLock.java,v 1.7 2005/07/27 11:54:21 wguttmn Exp $
  */
 
 
 package org.exolab.castor.persist;
 
 
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+
+import org.castor.persist.TransactionContext;
+import org.castor.persist.cache.CacheEntry;
+
 import org.exolab.castor.jdo.LockNotGrantedException;
-import org.exolab.castor.util.Messages;
 import org.exolab.castor.jdo.ObjectDeletedException;
+import org.exolab.castor.util.Messages;
 
 
 /**
@@ -88,13 +94,18 @@ import org.exolab.castor.jdo.ObjectDeletedException;
  *
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
  * @author <a href="yip@intalio.com">Thomas Yip</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:08:46 $
+ * @version $Revision: 1.7 $ $Date: 2005/07/27 11:54:21 $
  * @see TransactionContext
  */
 
-final class ObjectLock implements DepositBox {
-
+public final class ObjectLock implements DepositBox {
 
+	/**
+	 * The <a href="http://jakarta.apache.org/commons/logging/">Jakarta
+	 * Commons Logging</a> instance used for all logging.
+	 */
+	private static Log log = LogFactory.getFactory().getInstance(ObjectLock.class );
+	
     final static short ACTION_READ = 1;
 
     final static short ACTION_WRITE = 2;
@@ -109,9 +120,6 @@ final class ObjectLock implements DepositBox {
 
     private int    _id;
 
-    final static boolean TRACE = false;
-
-
     /**
      * The object being locked.
      */
@@ -174,36 +182,47 @@ final class ObjectLock implements DepositBox {
      */
     private int                _gateCount;
 
-
     private long               _timeStamp;
 
-
     private boolean            _deleted;
 
     private boolean            _invalidated;
 
+    private boolean            _isExpired;
+    
+    private Object             _expiredObject;
+
     /**
      * Create a new lock for the specified object. Must not create two
      * locks for the same object. This will be the object returned from
      * a successful call to one of the <tt>acquire</tt>.
      *
-     * @param obj The object to create a lock for
+     * @param oid The object to create a lock for
      */
-    ObjectLock( OID oid ) {
+    public ObjectLock(OID oid) {
         _oid = oid;
 
         // give each instance of ObjectLock an id, for debug only
-        synchronized ( lock ) {
+        synchronized (lock) {
             _id = idcount;
             idcount++;
         }
     }
+    
+    public ObjectLock(final CacheEntry entry) {
+        this(entry.getOID());
+        
+        _isExpired = false; 
+        _expiredObject = null;
+        _object = entry.getEntry();
+        _timeStamp = entry.getTimeStamp();
+    }
 
 
     /**
      * Return the object's OID.
      */
-    OID getOID() {
+    public OID getOID() {
         return _oid;
     }
 
@@ -238,8 +257,8 @@ final class ObjectLock implements DepositBox {
     }
 
     /**
-     * Return true if there is any transaction called {@ink enter}, 
-     * but not yet called {@link leave}.
+     * Return true if there is any transaction called {@link #enter},
+     * but not yet called {@link #leave}.
      */ 
     boolean isEntered() {
         return _gateCount != 0;
@@ -301,8 +320,6 @@ final class ObjectLock implements DepositBox {
     }
 
     boolean isExclusivelyOwned( TransactionContext tx ) {
-        LinkedTx read;
-
         if ( _writeLock == null && _readLock == null )
             return false;
 
@@ -316,6 +333,40 @@ final class ObjectLock implements DepositBox {
         return false;
     }
 
+    
+    /**
+     * Return true if this entry has been expired from the cache
+     */
+    boolean isExpired() 
+    { 
+        return _isExpired; 
+    }
+
+    public Object getObject() { 
+        if ( (_expiredObject != null) && (_object == null) )
+            return _expiredObject;
+        else
+            return _object; 
+    }
+   
+    /**
+     * Indicate that object needs to be expired from the cache
+     */
+    public void expire() 
+    { 
+        _isExpired = true; 
+    }
+   
+    /**
+     * Indicate that object has been removed from the cache. Perform any 
+     * post expiration cleanup.  In particular, remove the reference to any
+     * saved cached objects.
+     */
+    public void expired() { 
+        _isExpired = false; 
+        _expiredObject = null;
+    }
+
     synchronized void acquireLoadLock( TransactionContext tx, boolean write, int timeout ) 
             throws LockNotGrantedException, ObjectDeletedWaitingForLockException {
         
@@ -385,13 +436,15 @@ final class ObjectLock implements DepositBox {
                     // other transaction holding writeLock, waits for write
                     // or, other transaction holding readLock, waiting for read
                     if ( timeout == 0 ) {
-                        if ( TRACE )
-                            System.out.println( "Timeout on " + this.toString() + " by " + tx );
+                        if (log.isDebugEnabled()) {
+                            log.debug ( "Timeout on " + this.toString() + " by " + tx );
+                        }
                         throw new LockNotGrantedException( (write ? "persist.writeLockTimeout" :
                                                                "persist.readLockTimeout") + _oid + "/" + _id + " by " + tx );
                     }
-                    if ( TRACE )
-                        System.out.println( "Waiting on " + this.toString() + " by " + tx );
+                    if (log.isDebugEnabled()) {
+                        log.debug ( "Waiting on " + this.toString() + " by " + tx );
+                    }
                     // Detect possibility of dead-lock. Must remain in wait-on-lock
                     // position until lock is granted or exception thrown.
                     tx.setWaitOnLock( this );
@@ -504,12 +557,14 @@ final class ObjectLock implements DepositBox {
                     return;
                 } else {
                     if ( timeout == 0 ) {
-                        if ( TRACE )
-                            System.out.println( "Timeout on " + this.toString() + " by " + tx );
-                        throw new LockNotGrantedException( "persist.writeLockTimeout" );
+                        if (log.isDebugEnabled()) {
+                            log.debug ( "Timeout on " + this.toString() + " by " + tx );
+                        }
+                        throw new LockNotGrantedException( Messages.message ("persist.writeLockTimeout") );
+                    }
+                    if (log.isDebugEnabled()) {
+                        log.debug ( "Waiting on " + this.toString() + " by " + tx );
                     }
-                    if ( TRACE )
-                        System.out.println( "Waiting on " + this.toString() + " by " + tx );
                     // Detect possibility of dead-lock. Must remain in wait-on-lock
                     // position until lock is granted or exception thrown.
                     tx.setWaitOnLock( this );
@@ -528,12 +583,12 @@ final class ObjectLock implements DepositBox {
                         wait( waittime<0? 0: waittime );
                     } catch ( InterruptedException except ) {
                         // If the thread is interrupted, come out with the proper message
-                        throw new LockNotGrantedException( "persist.writeLockTimeout" + _oid + "/" + _id + " by " + tx );
+                        throw new LockNotGrantedException( Messages.message ("persist.writeLockTimeout") + _oid + "/" + _id + " by " + tx );
                     }
 
                     if ( _deleted )
                         // If object has been deleted while waiting for lock, report deletion.
-                        throw new ObjectDeletedWaitingForLockException("object deleted" + _oid + "/" + _id + " by " + tx);
+                        throw new ObjectDeletedWaitingForLockException("Object deleted " + _oid + "/" + _id + " by " + tx);
 
                     // Try to re-acquire lock, this time less timeout,
                     // eventually timeout of zero will either succeed or fail
@@ -553,6 +608,9 @@ final class ObjectLock implements DepositBox {
 
     public synchronized void setObject( TransactionContext tx, Object object ) {
 
+        _isExpired = false; // initialize cache expiration flag to false
+        _expiredObject = null;
+
         if ( _confirmWaiting != null && _confirmWaiting == tx ) {
             _timeStamp = System.currentTimeMillis();
             _object = object;
@@ -663,10 +721,8 @@ final class ObjectLock implements DepositBox {
      * cancelled whether or not the write is acquired.
      *
      * @param tx The transaction requesting the lock
-     * @param write The type of lock requested
      * @param timeout Timeout waiting to acquire lock (in milliseconds),
      *  zero for no waiting
-     * @return The locked object
      * @throws LockNotGrantedException Lock could not be granted in
      *  the specified timeout or a dead lock has been detected
      * @throws ObjectDeletedWaitingForLockException The object has
@@ -684,7 +740,7 @@ final class ObjectLock implements DepositBox {
             throw e;
         }
         if ( !hasLock( tx, false ) ) {
-            IllegalStateException e = new IllegalStateException("Transaction doesn't previously acquire this lock");
+            IllegalStateException e = new IllegalStateException("Transaction didn't previously acquire this lock");
             throw e;
         }
 
@@ -701,20 +757,23 @@ final class ObjectLock implements DepositBox {
                     // Upgrading from read to write, no other locks, can upgrade
                     // Order is important in case thread is stopped in the middle
                     //_readLock = null;
-                    if ( TRACE )
-                        System.out.println( "Acquired on " + toString() + " by " + tx );
+                    if (log.isDebugEnabled()) {
+                        log.debug ( "Acquired on " + this.toString() + " by " + tx );
+                    }
                     _writeLock = tx;
                     _readLock = null;
                     return;
                 } else {
                     // Don't wait if timeout is zero
                     if ( timeout == 0 ) {
-                        if ( TRACE )
-                            System.out.println( "Timeout on " + this.toString() + " by " + tx );
+                        if (log.isDebugEnabled()) {
+                            log.debug ( "Timeout on " + this.toString() + " by " + tx );
+                        }
                         throw new LockNotGrantedException( "persist.writeTimeout" + _oid + "/" + _id + " by " + tx );
                     }
-                    if ( TRACE )
-                        System.out.println( "Waiting on " + this.toString() + " by " + tx );
+                    if (log.isDebugEnabled()) {
+                        log.debug ( "Waiting on " + this.toString() + " by " + tx );
+                    }
                     // Detect possibility of dead-lock. Must remain in wait-on-lock
                     // position until lock is granted or exception thrown.
 
@@ -769,8 +828,9 @@ final class ObjectLock implements DepositBox {
      */
     synchronized void release( TransactionContext tx ) {
 
-        if ( TRACE )
-            System.out.println( "Release " + this.toString() + " by " + tx );
+        if (log.isDebugEnabled()) {
+            log.debug ( "Release " + this.toString() + " by " + tx );
+        }
 
         try {
             tx.setWaitOnLock( null );
@@ -778,6 +838,10 @@ final class ObjectLock implements DepositBox {
                 _writeLock = null;
                 if ( _invalidated || _deleted ) {
                     _timeStamp = System.currentTimeMillis();
+                    // save a copy of the expired objects contents;
+                    // this will be used to expire all contained objects
+                    if (_isExpired) 
+                        _expiredObject = _object;
                     _object = null;
                 }
                 _deleted = false;
@@ -795,10 +859,10 @@ final class ObjectLock implements DepositBox {
                         read = read.next;
                     }
                     if ( read == null )
-                        throw new IllegalStateException( "persist.notOwnerLock" + _oid + "/" + _id + " by " + tx );
+                        throw new IllegalStateException( Messages.message ("persist.notOwnerLock") + _oid + "/" + _id + " by " + tx );
                 }
             } else 
-                throw new IllegalStateException( "persist.notOwnerLock" + _oid + "/" + _id + " by " + tx );
+                throw new IllegalStateException( Messages.message ("persist.notOwnerLock") + _oid + "/" + _id + " by " + tx );
 
             // Notify all waiting transactions that they may attempt to
             // acquire lock. First one to succeed wins (or multiple if
@@ -825,10 +889,11 @@ final class ObjectLock implements DepositBox {
     synchronized void delete( TransactionContext tx ) {
 
         if ( tx != _writeLock )
-            throw new IllegalStateException( "persist.notOwnerLock oid:" + _oid + "/" + _id + " by " + tx );
+            throw new IllegalStateException( Messages.message("persist.notOwnerLock") + " oid:" + _oid + "/" + _id + " by " + tx );
 
-        if ( TRACE )
-            System.out.println( "Delete " + this.toString() + " by " + tx );
+        if (log.isDebugEnabled()) {
+            log.debug ( "Delete " + this.toString() + " by " + tx );
+        }
 
         try {
             // Mark lock as unlocked and deleted, notify all waiting transactions
@@ -847,10 +912,11 @@ final class ObjectLock implements DepositBox {
     synchronized void invalidate( TransactionContext tx ) {
         
         if ( tx != _writeLock ) 
-            throw new IllegalStateException( "persist.notOwnerLock oid:" + _oid + "/" + _id + " by " + tx );
+            throw new IllegalStateException( Messages.message ("persist.notOwnerLock") + " oid:" + _oid + "/" + _id + " by " + tx );
 
-        if ( TRACE )
-            System.out.println( "Delete " + this.toString() + " by " + tx );
+        if (log.isDebugEnabled()) {
+            log.debug ( "Delete " + this.toString() + " by " + tx );
+        }
 
         _invalidated = true;
     }
@@ -895,12 +961,12 @@ final class ObjectLock implements DepositBox {
                 // Is the blocked transaction blocked by the transaction locking
                 // this object? This is a deadlock.
                 if ( waitOn._writeLock == waitingTx ) {
-                    throw new LockNotGrantedException( "persist.deadlock" );
+                    throw new LockNotGrantedException( Messages.message("persist.deadlock") );
                 }
                 read = waitOn._readLock;
                 while ( read != null ) {
                     if ( read.tx == waitingTx )
-                        throw new LockNotGrantedException( "persist.deadlock" );
+                        throw new LockNotGrantedException( Messages.message ("persist.deadlock" ));
                     read = read.next;
                 }
                 waitOn.detectDeadlock( waitingTx, numOfRec - 1 );
@@ -920,12 +986,12 @@ final class ObjectLock implements DepositBox {
                     LinkedTx read;
 
                     if ( waitOn._writeLock == waitingTx ) {
-                        throw new LockNotGrantedException( "persist.deadlock" );
+                        throw new LockNotGrantedException( Messages.message ("persist.deadlock") );
                     }
                     read = waitOn._readLock;
                     while ( read != null ) {
                         if ( read.tx == waitingTx )
-                            throw new LockNotGrantedException( "persist.deadlock" );
+                            throw new LockNotGrantedException( Messages.message ("persist.deadlock") );
                         read = read.next;
                     }
                     waitOn.detectDeadlock( waitingTx, numOfRec - 1 );
@@ -1011,7 +1077,5 @@ final class ObjectLock implements DepositBox {
         }
 
     }
-
-
 }
 
