@@ -40,7 +40,7 @@
  *
  * Copyright 2001 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: AnyNode.java,v 1.1.1.1 2003/03/03 07:08:53 kvisco Exp $
+ * $Id: AnyNode.java,v 1.2 2005/03/05 13:41:56 mfuchs Exp $
  * Date         Author              Changes
  * 04/24/2001   Arnaud Blandin      Rewrited from scratch
  * 04/22/2001   Arnaud Blandin      Clean-up and support of comments
@@ -49,17 +49,12 @@
 
 package org.exolab.castor.types;
 
-import org.exolab.castor.util.Configuration;
 import org.exolab.castor.util.LocalConfiguration;
 import org.exolab.castor.util.Stack;
 import org.exolab.castor.xml.util.AnyNode2SAX;
 
 import org.apache.xml.serialize.Serializer;
-import org.apache.xml.serialize.OutputFormat;
 
-import java.util.Enumeration;
-import java.util.Vector;
-import java.io.PrintStream;
 import java.io.StringWriter;
 
 /**
@@ -80,7 +75,7 @@ import java.io.StringWriter;
  * @author <a href="mailto:arkin@intalio.com">Assaf Arkin</a>
  * @author <a href="mailto:blandin@intalio.com">Arnaud Blandin</a>
  * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:08:53 $
+ * @version $Revision: 1.2 $ $Date: 2005/03/05 13:41:56 $
  * @todo Processing Instructions
  * @todo Full handling of namespaces
  */
@@ -190,11 +185,11 @@ public final class AnyNode implements java.io.Serializable {
      * type, localName, prefix, uri and value.
      * This constructor is not user-friendly and launched
      * RunTime exception is you try to instantiate non-valid node.
-     * @param short type the node type.
-     * @param String localName the name of the node.
-     * @param String prefix the prefix if any of the namespace.
-     * @param String uri the namespace uri of this node.
-     * @param String value the value of this node.
+     * @param type the node type.
+     * @param localName the name of the node.
+     * @param prefix the prefix if any of the namespace.
+     * @param uri the namespace uri of this node.
+     * @param value the value of this node.
      */
     public AnyNode(short type, String localName,
                    String prefix,String uri,
