@@ -40,7 +40,7 @@
  *
  * Copyright 2000 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: Date.java,v 1.1.1.1 2003/03/03 07:08:54 kvisco Exp $
+ * $Id: Date.java,v 1.3 2005/03/30 09:46:25 wguttmn Exp $
  * Date         Author              Changes
  * 10/02/2002   Arnaud              Clean up
  * 04/18/2002   Arnaud              String constructor
@@ -54,14 +54,16 @@ package org.exolab.castor.types;
 
 import java.text.SimpleDateFormat;
 import java.text.ParseException;
+import java.util.Calendar;
 import java.util.SimpleTimeZone;
 import java.util.GregorianCalendar;
+import java.util.TimeZone;
 /**
  * Describe an XML schema Date.
  * <p>The format is defined by W3C XML Schema Recommendation and ISO8601
  * i.e <tt>(-)CCYY-MM-DD(Z|(+|-)hh:mm)</tt>
  * @author <a href="mailto:blandin@intalio.com">Arnaud Blandin</a>
- * @version $Revision: 1.1.1.1 $
+ * @version $Revision: 1.3 $
  */
 
 public class Date extends DateTimeBase{
@@ -107,7 +109,18 @@ public class Date extends DateTimeBase{
         setValues(values);
     }
 
-
+    /**
+     * This constructor is used to convert a long value representing a Date
+     * to a new org.exolab.castor.types.Date instance.
+     * <p>Note : all the information concerning the time part of
+     * the java.util.Date is lost since a W3C Schema Date only represents
+     * CCYY-MM-YY
+     * @param dateAsLong Date represented in from of a long value.
+     */
+    public Date (long dateAsLong) {
+    	this (new java.util.Date (dateAsLong));
+    }
+    
     /**
      * This constructor is used to convert a java.util.Date into
      * a new org.exolab.castor.types.Date
@@ -118,13 +131,13 @@ public class Date extends DateTimeBase{
     public Date(java.util.Date dateRef) {
         GregorianCalendar tempCalendar = new GregorianCalendar();
         tempCalendar.setTime(dateRef);
-        setCentury((short) (tempCalendar.get(tempCalendar.YEAR)/100));
-        setYear((short) (tempCalendar.get(tempCalendar.YEAR)%100));
+        setCentury((short) (tempCalendar.get(Calendar.YEAR)/100));
+        setYear((short) (tempCalendar.get(Calendar.YEAR)%100));
 
         //we need to add 1 to the Month value returned by GregorianCalendar
         //because 0<MONTH<11 (i.e January is 0)
-        setMonth((short) (tempCalendar.get(tempCalendar.MONTH)+1));
-        setDay((short) (tempCalendar.get(tempCalendar.DAY_OF_MONTH)));
+        setMonth((short) (tempCalendar.get(Calendar.MONTH)+1));
+        setDay((short) (tempCalendar.get(Calendar.DAY_OF_MONTH)));
     } //Date(java.util.Date)
 
     /**
@@ -194,7 +207,7 @@ public class Date extends DateTimeBase{
             offset = (int) ( (this.getZoneMinute() + this.getZoneHour()*60)*60*1000);
             offset = isZoneNegative() ? -offset : offset;
             timeZone.setRawOffset(offset);
-            timeZone.setID(timeZone.getAvailableIDs(offset)[0]);
+            timeZone.setID(TimeZone.getAvailableIDs(offset)[0]);
             df.setTimeZone(timeZone);
         }
 
@@ -209,6 +222,15 @@ public class Date extends DateTimeBase{
         return date;
     }//toDate()
 
+    /**
+     * Converts this date into a long value.
+     * @return This date instance as a long value.
+     */
+    public long toLong () {
+    	java.util.Date date = toDate();
+    	return date.getTime();
+    }
+    
     /**
      * convert this Date to a string
      * The format is defined by W3C XML Schema recommendation and ISO8601
