@@ -40,7 +40,7 @@
  *
  * Copyright 2001 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: Duration.java,v 1.1.1.1 2003/03/03 07:08:55 kvisco Exp $
+ * $Id: Duration.java,v 1.3 2005/03/05 13:41:56 mfuchs Exp $
  * Date         Author           Changes
  * 07/04/2002   Arnaud           Support for milliseconds
  * 04/18/2002   Arnaud           String constructor
@@ -60,7 +60,7 @@ import java.text.ParseException;
  * <p>For more information see <a href="http://www.w3.org/TR/xmlschema-2/#duration">
  * X3C XML Schema Specification</a>.
  * @author <a href="mailto:blandin@intalio.com">Arnaud Blandin</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:08:55 $
+ * @version $Revision: 1.3 $ $Date: 2005/03/05 13:41:56 $
  **/
 public class Duration
     implements java.io.Serializable
@@ -100,7 +100,7 @@ public class Duration
    /**
     * the potential number of milliseconds
     */
-    private short _millisecond = 0;
+    private long _millisecond = 0;
 
     /**
      * true if the Duration is negative
@@ -127,7 +127,6 @@ public class Duration
      * value of the long by calling setValue.
      * @see #setValue
      * @param l the long value of the Duration
-     * @return A duration which represents the long
      */
     public Duration(long l) {
 
@@ -190,7 +189,7 @@ public class Duration
             System.out.println("nb seconds:"+seconds);
         }
 
-        short milliseconds =  (short)(l);
+        long milliseconds = l;
           if (DEBUG) {
             System.out.println("nb milliseconds:"+milliseconds);
         }
@@ -248,7 +247,7 @@ public class Duration
         _second = second;
     }
 
-    public void setMilli(short milli) {
+    public void setMilli(long milli) {
         if (milli < 0) {
              String err = "In a duration all fields have to be positive.";
              throw new IllegalArgumentException(err);
@@ -270,7 +269,7 @@ public class Duration
      * @param second the second value
      */
      public void setValue(short year, short month, short day,
-                         short hour, short minute, short second, short millisecond)
+                         short hour, short minute, short second, long millisecond)
     {
         this.setYear(year);
         this.setMonth(month);
@@ -308,7 +307,7 @@ public class Duration
         return(_second);
     }
 
-    public short getMilli() {
+    public long getMilli() {
         return(_millisecond);
     }
 
@@ -410,7 +409,7 @@ public class Duration
     /**
      * <p>Parse the given string and return a time duration
      * which represents this string.
-     * @param String str the string to parse
+     * @param str the string to parse
      * @return a TimeDuration instance which represent the string
      * @throws ParseException thrown when the string is not valid
      */
@@ -608,7 +607,7 @@ public class Duration
                                    number = number * 10;
                                 number = number * 10;
                             }
-                            result.setMilli((short)number);
+                            result.setMilli((long)number);
                             hasNumber = false;
                         }
                         else {
@@ -699,7 +698,7 @@ public class Duration
     /**
      * Returns true if the instance of TimeDuration has the same fields
      * of the parameter
-     * @param timeD the time duration to compare
+     * @param duration the time duration to compare
      * @return true if equal, false if not
      */
     public boolean equal(Duration duration) {
@@ -724,7 +723,7 @@ public class Duration
      * the following total order relation is used :
      * <tt>givent t1,t2 timeDuration types
      * t1>t2 iff t1.toLong()>t2.toLong()</tt>
-     * @param timeD the time duration to compare with the present instance
+     * @param duration the time duration to compare with the present instance
      * @return true if the present instance is the greatest, false if not
      */
      public boolean isGreater(Duration duration) {
@@ -733,5 +732,6 @@ public class Duration
         result = this.toLong() > duration.toLong();
         return result;
      } //isGreater
-
+     
+    
 }//Duration
\ No newline at end of file
