@@ -38,41 +38,44 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
+ * Copyright 1999-2004 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: Configuration.java,v 1.2 2003/05/01 07:06:45 kvisco Exp $
+ * $Id: Configuration.java,v 1.8 2005/04/24 15:34:12 rjoachim Exp $
  */
 
 
 package org.exolab.castor.util;
 
 
-import java.util.Properties;
 import java.io.OutputStream;
 import java.io.Writer;
 import java.io.File;
 import java.io.FileInputStream;
-import java.io.InputStream;
 import java.io.IOException;
+import java.util.Properties;
 import java.util.StringTokenizer;
-import java.util.Hashtable;
-import java.net.URL;
-import org.xml.sax.SAXException;
-import org.xml.sax.DocumentHandler;
-import org.xml.sax.Parser;
-import org.xml.sax.XMLReader;
-import org.apache.xml.serialize.Serializer;
-import org.apache.xml.serialize.OutputFormat;
-import org.apache.xml.serialize.Method;
-import org.exolab.castor.util.Messages;
-import org.exolab.castor.xml.JavaNaming;
-import org.exolab.castor.xml.NodeType;
-import org.exolab.castor.xml.XMLNaming;
-import org.exolab.castor.xml.util.DefaultNaming;
+
 import javax.xml.parsers.SAXParser;
 import javax.xml.parsers.SAXParserFactory;
 import javax.xml.parsers.ParserConfigurationException;
 
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+
+import org.apache.xml.serialize.Serializer;
+import org.apache.xml.serialize.OutputFormat;
+import org.apache.xml.serialize.Method;
+
+import org.exolab.castor.xml.NodeType;
+import org.exolab.castor.xml.XMLNaming;
+import org.exolab.castor.xml.util.DefaultNaming;
+import org.exolab.castor.util.Messages;
+
+import org.xml.sax.DocumentHandler;
+import org.xml.sax.Parser;
+import org.xml.sax.SAXException;
+import org.xml.sax.XMLReader;
+
 
 /**
  * Provides default configuration for Castor components from the
@@ -90,12 +93,16 @@ import javax.xml.parsers.ParserConfigurationException;
  * create a new configuration file in the current directory, instead
  * of modifying the global one.
  *
- * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
+ * @author <a href="mailto:kvisco-at-intalio.com">Keith Visco</a>
  * @author <a href="mailto:arkin@intalio.com">Assaf Arkin</a>
- * @version $Revision: 1.2 $ $Date: 2003/05/01 07:06:45 $
+ * @version $Revision: 1.8 $ $Date: 2005/04/24 15:34:12 $
  */
-public abstract class Configuration
-{
+public abstract class Configuration {
+    /**
+     * The <a href="http://jakarta.apache.org/commons/logging/">Jakarta
+     * Commons Logging</a> instance used for all logging.
+     */
+    private static final Log LOG = LogFactory.getLog(Configuration.class);
 
     /**
      * Names of properties used in the configuration file.
@@ -145,6 +152,16 @@ public abstract class Configuration
          * </pre>
          */
         public static final String Namespaces = "org.exolab.castor.parser.namespaces";
+        
+        
+        /**
+         * Property specifying Namespace to Package Mappings
+         * <pre>
+         * org.exolab.castor.xml.nspackages
+         * </pre>
+         */
+        public static final String NamespacePackages = "org.exolab.castor.xml.nspackages";
+        
 
         /**
          * Property specifying the implementation of the naming conventions
@@ -214,6 +231,18 @@ public abstract class Configuration
          */
         public static final String StrictElements = "org.exolab.castor.xml.strictelements";
 
+
+        /**
+         * Property specifying whether or not to save the "keys" of a Hashtable or 
+         * Map during marshalling. By default this is true.
+         * <pre>
+         * org.exolab.castor.xml.saveMapKeys
+         * </pre>
+         * @since 0.9.5.3
+         */
+        public static final String SaveMapKeys = "org.exolab.castor.xml.saveMapKeys";
+        
+        
         /**
          * The name of the configuration file.
          * <pre>
@@ -268,7 +297,7 @@ public abstract class Configuration
      * sub-classes.
      *
      * @returns true if the configuration specifies debugging.
-     * @see getDefaultDebug
+     * @see #getDefaultDebug()
      */
     public boolean debug() {
         return getDefaultDebug();
@@ -296,7 +325,7 @@ public abstract class Configuration
      * sub-classes.
      *
      * @return true if element processing should be "strict".
-     * @see getDefaultStrictElements
+     * @see #getDefaultStrictElements()
      */
     public boolean strictElements() {
         return getDefaultStrictElements();
@@ -309,7 +338,7 @@ public abstract class Configuration
      * If set to false, these 'unknown' elements are ignored
      *
      * @return true if element processing should be "strict".
-     * @see strictElements
+     * @see #strictElements()
      */
     public static boolean getDefaultStrictElements() {
         getDefault();
@@ -325,7 +354,7 @@ public abstract class Configuration
      *
      * @return true if by default validation should be performed during the
      * marshalling and unmarshalling process, otherwise false.
-     * @see getDefaultMarshallingValidation
+     * @see #getDefaultMarshallingValidation()
      */
     public boolean marshallingValidation() {
         return getDefaultMarshallingValidation();
@@ -339,7 +368,7 @@ public abstract class Configuration
      * @return true if by default validation should be performed during the
      * marshalling and unmarshalling process, otherwise false.
      *
-     * @see marshallingValidation
+     * @see #marshallingValidation()
      */
     public static boolean getDefaultMarshallingValidation()
     {
@@ -356,7 +385,7 @@ public abstract class Configuration
      * sub-classes.
      *
      * @return the current set of configuration properties. 
-     * @see getDefault
+     * @see #getDefault()
      */
     public Properties getProperties() {
         return getDefault();
@@ -368,7 +397,7 @@ public abstract class Configuration
      * default configuration.
      *
      * @return the default configuration properties
-     * @see getProperties
+     * @see #getProperties()
      */
     public static synchronized Properties getDefault()
     {
@@ -384,10 +413,10 @@ public abstract class Configuration
      * of {@link #getDefault}.
      *
      * @param name The property name
-     * @param default The property's default value
+     * @param defValue The property's default value
      * @return The property's value
-     * @see getDefaultProperty
-     * @see getProperties
+     * @see #getDefaultProperty(String,String)
+     * @see #getProperties()
      */
     public String getProperty( String name, String defValue )
     {
@@ -402,9 +431,9 @@ public abstract class Configuration
      * of {@link #getDefault}.
      *
      * @param name The property name
-     * @param default The property's default value
+     * @param defValue The property's default value
      * @return The property's value
-     * @see getProperty
+     * @see #getProperty (String, String)
      */
     public static String getDefaultProperty( String name, String defValue )
     {
@@ -420,7 +449,7 @@ public abstract class Configuration
      *
      * @return the currently configured naming conventions to use 
      * for the XML framework     
-     * @see getDefaultXMLNaming
+     * @see #getDefaultXMLNaming()
      */
     public XMLNaming getXMLNaming() {
         return getDefaultXMLNaming();
@@ -466,7 +495,7 @@ public abstract class Configuration
      * sub-classes.
      *
      * @return a suitable XML parser
-     * @see getDefaultParser
+     * @see #getDefaultParser()
      */
     public Parser getParser() {
         return getDefaultParser();
@@ -477,7 +506,7 @@ public abstract class Configuration
      * specified in the default configuration file.
      *
      * @return a suitable XML parser
-     * @see getParser
+     * @see #getParser()
      */
     public static Parser getDefaultParser()
     {
@@ -526,10 +555,10 @@ public abstract class Configuration
                 return saxParser.getParser();
             }
             catch(ParserConfigurationException pcx) {
-                Logger.getSystemLogger().println( Messages.format( "conf.configurationError", pcx ) );
+                LOG.error(Messages.format("conf.configurationError", pcx));
             }
             catch(org.xml.sax.SAXException sx) {
-                Logger.getSystemLogger().println( Messages.format( "conf.configurationError", sx ) );
+                LOG.error(Messages.format("conf.configurationError", sx));
             }
             
         }
@@ -557,7 +586,6 @@ public abstract class Configuration
 
         if ( parser instanceof XMLReader ) {
             StringTokenizer token;
-            boolean         flag;            
             XMLReader xmlReader = (XMLReader)parser;
             try {
                 xmlReader.setFeature( Features.Validation, validation );
@@ -569,9 +597,8 @@ public abstract class Configuration
                         xmlReader.setFeature( token.nextToken(), true );
                     }
                 }
-            } 
-            catch ( SAXException except ) {
-                Logger.getSystemLogger().println( Messages.format( "conf.configurationError", except ) );
+            } catch (SAXException except) {
+                LOG.error(Messages.format("conf.configurationError", except));
             }
         }
         return parser;
@@ -587,7 +614,7 @@ public abstract class Configuration
      *
      * @return the NodeType assigned to Java primitives, or null
      * if no NodeType was specified.
-     * @see getDefaultPrimitiveNodeType
+     * @see #getDefaultPrimitiveNodeType()
      */
     public NodeType getPrimitiveNodeType() {
         return getDefaultPrimitiveNodeType();
@@ -602,7 +629,7 @@ public abstract class Configuration
      * @return the NodeType assigned to Java primitives, or null
      * if no NodeType was specified.
      *
-     * @see getPrimitiveNodeType
+     * @see #getPrimitiveNodeType()
      */
     public static NodeType getDefaultPrimitiveNodeType() {
         
@@ -617,14 +644,129 @@ public abstract class Configuration
             return _defaultValues.primitiveNodeType;
         }
     } //-- getDefaultPrimitiveNodeType
+
     
+    /**
+     * Returns an XML document parser implementing the requested
+     * set of features. The feature list is a comma separated list
+     * of features that parser may or may not support. No errors are
+     * generated for unsupported features. If the feature list is not
+     * null, it overrides the default feature list specified in the
+     * configuration file, including validation and Namespaces.
+     *
+     * @return A suitable XML parser
+     */
+    public XMLReader getXMLReader()
+    {
+        return getDefaultXMLReader( null ) ;
+        
+    } //-- getXMLReader
+    
+    /**
+     * Returns an XML document parser implementing the requested
+     * set of features. The feature list is a comma separated list
+     * of features that parser may or may not support. No errors are
+     * generated for unsupported features. If the feature list is not
+     * null, it overrides the default feature list specified in the
+     * configuration file, including validation and Namespaces.
+     *
+     * @return A suitable XML parser
+     */
+    public static XMLReader getDefaultXMLReader()
+    {
+        return getDefaultXMLReader( null ) ;
+    } //-- getDefaultXMLReader
+    
+    /**
+     * Returns an XML document parser implementing the requested
+     * set of features. The feature list is a comma separated list
+     * of features that parser may or may not support. No errors are
+     * generated for unsupported features. If the feature list is not
+     * null, it overrides the default feature list specified in the
+     * configuration file, including validation and Namespaces.
+     *
+     * @param features The requested feature list, null for the
+     *   defaults
+     * @return A suitable XML parser
+     */
+    public static XMLReader getDefaultXMLReader( String features )
+    {
+        String prop;
+        XMLReader reader = null;
+        
+        //-- validation?
+        prop = getDefault().getProperty( Property.ParserValidation, "false" );
+        boolean validation = ( prop.equalsIgnoreCase( "true" ) || 
+                               prop.equalsIgnoreCase( "on" ) );
+                               
+        //-- namespaces?
+        prop = getDefault().getProperty( Property.Namespaces, "false" );
+        boolean namespaces = ( prop.equalsIgnoreCase( "true" ) || 
+                               prop.equalsIgnoreCase( "on" ) );
+        
+
+        //-- which parser?
+        prop = getDefault().getProperty( Property.Parser );
+        if (( prop == null ) || (prop.length() == 0))  {
+            // If no parser class was specified, check for JAXP
+            // otherwise we default to Xerces.
+            SAXParserFactory factory = SAXParserFactory.newInstance();
+            factory.setNamespaceAware(namespaces);
+            factory.setValidating(validation);
+            try {
+                SAXParser saxParser = factory.newSAXParser();
+                reader = saxParser.getXMLReader();
+            } catch(ParserConfigurationException pcx) {
+                LOG.error(Messages.format("conf.configurationError", pcx));
+            } catch(org.xml.sax.SAXException sx) {
+                LOG.error(Messages.format("conf.configurationError", sx));
+            }
+            
+        }
+        
+        if (reader == null) {
+            if ((prop == null) ||
+                (prop.length() == 0) ||
+                (prop.equalsIgnoreCase("xerces"))) 
+            {
+                prop = "org.apache.xerces.parsers.SAXParser";
+            }
+        
+            // If a parser class was specified, we try to create it and
+            // complain about creation error.
+            try {
+                Class cls;
+                cls = Class.forName( prop );
+                reader = (XMLReader) cls.newInstance();
+            } catch ( Exception except ) {
+                throw new RuntimeException( Messages.format( "conf.failedInstantiateParser",
+                                                            prop, except ) );
+            }
+        }
+
+        StringTokenizer token;
+        try {
+            reader.setFeature( Features.Validation, validation );
+            reader.setFeature( Features.Namespaces, namespaces );
+            features = getDefault().getProperty( Property.ParserFeatures, features );
+            if ( features != null ) {
+                token = new StringTokenizer( features, ", " );
+                while ( token.hasMoreTokens() ) {
+                    reader.setFeature( token.nextToken(), true );
+                }
+            }
+        } catch (SAXException except) {
+            LOG.error(Messages.format("conf.configurationError", except));
+        }
+        return reader;
+    } //-- getDefaultXMLReader
 
     /**
      * Returns a new instance of the specified Regular Expression
      * Evaluator, or null if no validator was specified
      *
      * @return the regular expression evaluator,
-     * @see getDefaultRegExpEvaluator
+     * @see #getDefaultRegExpEvaluator()
      */
     public RegExpEvaluator getRegExpEvaluator() {
         return getDefaultRegExpEvaluator();
@@ -636,7 +778,7 @@ public abstract class Configuration
      *
      * @return the regular expression evaluator,
      *
-     * @see getRegExpEvaluator
+     * @see #getRegExpEvaluator()
      */
     public static RegExpEvaluator getDefaultRegExpEvaluator() {
 
@@ -671,9 +813,8 @@ public abstract class Configuration
      * Design note: This method should be overloaded by any 
      * sub-classes.
      *
-     * @param output The output stream
      * @return A suitable serializer
-     * @see getDefaultSerializer
+     * @see #getDefaultSerializer()
      */
     public abstract Serializer getSerializer();
 
@@ -684,9 +825,8 @@ public abstract class Configuration
      * DOM and SAX events. If such control is not required, it is
      * recommended to call one of the other two methods.
      *
-     * @param output The output stream
      * @return A suitable serializer
-     * @see getSerializer
+     * @see #getSerializer()
      */
     public static Serializer getDefaultSerializer()
     {
@@ -716,7 +856,7 @@ public abstract class Configuration
      * sub-classes.
      *
      * @return the currently configured OutputFormat.
-     * @see getDefaultOutputFormat
+     * @see #getDefaultOutputFormat()
      */
     public abstract OutputFormat getOutputFormat();
 
@@ -859,8 +999,6 @@ public abstract class Configuration
 	public static Properties loadProperties(String resourceName, String fileName)
 	{
         File        file;
-        InputStream is;
-
         Properties properties = new Properties();
 
         boolean found = false;
@@ -888,9 +1026,13 @@ public abstract class Configuration
         String javaHome = null;
         try {
             javaHome = System.getProperty("java.home");
-        }
-        catch(Exception except) {
-            // Ignore probably running in an applet
+        } catch (SecurityException e) {
+            // Not a critical error, but users will need to know if they need to change their config.   
+            LOG.warn(Messages.format("conf.privilegesError", e));
+        } catch (Exception e) {
+            // As we will be trying something else later, record the error for setup purposes,
+            // but do not actually treat as a critical failure.
+            LOG.warn(Messages.format("conf.nonCriticalError", e));
         }
         
         if (javaHome != null) {
@@ -902,9 +1044,12 @@ public abstract class Configuration
                     properties.load( new FileInputStream( file ) );
                     found = true;
                 }      
-            } 
-            catch ( IOException except ) {
-                // Do nothing
+            } catch (SecurityException e) {
+                // Not a critical error, but users will need to know if they need to change their config.   
+                LOG.warn(Messages.format("conf.privilegesError", e));
+            } catch (IOException e) {
+                // Report that we were unable to load the resource.
+                LOG.warn(Messages.format("conf.nonCriticalError", e));
             }
         }
         
