@@ -40,7 +40,7 @@
  *
  * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: LocalConfiguration.java,v 1.3 2003/05/01 07:06:32 kvisco Exp $
+ * $Id: LocalConfiguration.java,v 1.10 2005/03/24 10:45:01 shugubakken Exp $
  */
 
 
@@ -62,6 +62,8 @@ import org.xml.sax.SAXException;
 import org.xml.sax.DocumentHandler;
 import org.xml.sax.Parser;
 import org.xml.sax.XMLReader;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 import org.apache.xml.serialize.Serializer;
 import org.apache.xml.serialize.OutputFormat;
 import org.apache.xml.serialize.Method;
@@ -92,10 +94,15 @@ import javax.xml.parsers.SAXParserFactory;
  *
  * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
  * @author <a href="mailto:arkin@intalio.com">Assaf Arkin</a>
- * @version $Revision: 1.3 $ $Date: 2003/05/01 07:06:32 $
+ * @version $Revision: 1.10 $ $Date: 2005/03/24 10:45:01 $
  */
 public final class LocalConfiguration extends Configuration {
 
+    /**
+     * The <a href="http://jakarta.apache.org/commons/logging/">Jakarta
+     * Commons Logging</a> instance used for all logging.
+     */
+    private static final Log _log = LogFactory.getFactory().getInstance(LocalConfiguration.class);
 
 	/**
      * The properties loaded from the local configuration file.
@@ -328,12 +335,11 @@ public final class LocalConfiguration extends Configuration {
 
         if ( parser instanceof XMLReader ) {
             StringTokenizer token;
-            boolean         flag;            
             XMLReader xmlReader = (XMLReader)parser;
             try {
                 xmlReader.setFeature( Features.Validation, validation );
                 xmlReader.setFeature( Features.Namespaces, namespaces );
-                features = getDefault().getProperty( Property.ParserFeatures, features );
+                features = getProperties().getProperty( Property.ParserFeatures, features );
                 if ( features != null ) {
                     token = new StringTokenizer( features, ", " );
                     while ( token.hasMoreTokens() ) {
@@ -348,6 +354,112 @@ public final class LocalConfiguration extends Configuration {
         return parser;
         
     }
+    
+    /**
+     * Returns an XML document parser implementing the requested
+     * set of features. The feature list is a comma separated list
+     * of features that parser may or may not support. No errors are
+     * generated for unsupported features. If the feature list is not
+     * null, it overrides the default feature list specified in the
+     * configuration file, including validation and Namespaces.
+     *
+     * @return A suitable XML parser
+     */
+    public XMLReader getXMLReader()
+    {
+        return getXMLReader( null ) ;
+        
+    } //-- getXMLReader
+    
+    /**
+     * Returns an XML document parser implementing the requested
+     * set of features. The feature list is a comma separated list
+     * of features that parser may or may not support. No errors are
+     * generated for unsupported features. If the feature list is not
+     * null, it overrides the default feature list specified in the
+     * configuration file, including validation and Namespaces.
+     *
+     * @return A suitable XML parser
+     */
+    public XMLReader getXMLReader( String features )
+    {
+        
+        String prop;
+        XMLReader reader = null;
+        
+        //-- validation?
+        prop = getProperties().getProperty( Property.ParserValidation, "false" );
+        boolean validation = ( prop.equalsIgnoreCase( "true" ) || 
+                               prop.equalsIgnoreCase( "on" ) );
+                               
+        //-- namespaces?
+        prop = getProperties().getProperty( Property.Namespaces, "false" );
+        boolean namespaces = ( prop.equalsIgnoreCase( "true" ) || 
+                               prop.equalsIgnoreCase( "on" ) );
+        
+
+        //-- which parser?
+        prop = getProperties().getProperty( Property.Parser );
+        if (( prop == null ) || (prop.length() == 0)) {
+            // If no parser class was specified, check for JAXP
+            // otherwise we default to Xerces.
+            SAXParserFactory factory = SAXParserFactory.newInstance();
+            factory.setNamespaceAware(namespaces);
+            factory.setValidating(validation);
+            try {
+                SAXParser saxParser = factory.newSAXParser();
+                reader = saxParser.getXMLReader();
+            }
+            catch(ParserConfigurationException pcx) {
+                Logger.getSystemLogger().println( Messages.format( "conf.configurationError", pcx ) );
+            }
+            catch(org.xml.sax.SAXException sx) {
+                Logger.getSystemLogger().println( Messages.format( "conf.configurationError", sx ) );
+            }
+            
+        }
+        
+        if (reader == null) {
+            if ((prop == null) || 
+                (prop.length() == 0) || 
+                (prop.equalsIgnoreCase("xerces"))) 
+            {
+                prop = "org.apache.xerces.parsers.SAXParser";
+            }
+        
+
+            // If a parser class was specified, we try to create it and
+            // complain about creation error.
+            try {
+                Class cls;
+                
+                cls = Class.forName( prop );
+                reader = (XMLReader) cls.newInstance();
+            } catch ( Exception except ) {
+                throw new RuntimeException( Messages.format( "conf.failedInstantiateParser",
+                                                            prop, except ) );
+            }
+        }
+
+        StringTokenizer token;
+        try {
+            reader.setFeature( Features.Validation, validation );
+            reader.setFeature( Features.Namespaces, namespaces );
+            features = getProperties().getProperty( Property.ParserFeatures, features );
+            if ( features != null ) {
+                token = new StringTokenizer( features, ", " );
+                while ( token.hasMoreTokens() ) {
+                    reader.setFeature( token.nextToken(), true );
+                }
+            }
+        } 
+        catch ( SAXException except ) {
+            Logger.getSystemLogger().println( Messages.format( "conf.configurationError", except ) );
+        }
+        return reader;
+        
+    } //-- getXMLReader
+    
 
     /**
      * Returns the NodeType to use for Java primitives.
@@ -410,7 +522,6 @@ public final class LocalConfiguration extends Configuration {
      * DOM and SAX events. If such control is not required, it is
      * recommended to call one of the other two methods.
      *
-     * @param output The output stream
      * @return A suitable serializer
      */
     public Serializer getSerializer()
@@ -509,7 +620,7 @@ public final class LocalConfiguration extends Configuration {
     }
 
     /**
-     * Calls {@link #getDefault} to load the configuration the
+     * Calls {@link #getDefault()} to load the configuration the
      * first time and then looks for a local configuration to
      * merge in with the defaults. Will not complain about inability 
      * to load local configuration file from one of the default 
@@ -567,6 +678,9 @@ public final class LocalConfiguration extends Configuration {
             URL url = getClass().getResource("/" + fileOrResourceName);
             if (url != null) {
                 _resourceUrl = url.toString();
+                if(_log.isDebugEnabled()) {
+                    _log.debug ("Trying to load configuration file from " + _resourceUrl);
+                }
                 _props.load( url.openStream() );
                 //-- debug information
                 //System.out.println("merging local configuration: " + url.toExternalForm());
@@ -576,6 +690,23 @@ public final class LocalConfiguration extends Configuration {
         } catch ( Exception except ) {
             // Do nothing
         }
+        
+        //-- if not found, either it doesn't exist, or "." is not part of the
+        //-- class path, try looking at local working directory
+        if (!found) {
+            try {      
+                File file = new File(fileOrResourceName);
+                if (file.exists() && file.canRead()) {
+                    InputStream is = new FileInputStream(file);
+                    _props.load( is );
+                    is.close();
+                    found = true;
+                }
+            } catch ( Exception except ) {
+                //-- do nothing
+            }
+        }
+        
 
 
         //-- Cannot find any castor.properties file(s).
