@@ -40,7 +40,7 @@
  *
  * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: Introspector.java,v 1.3 2003/04/23 06:42:51 kvisco Exp $
+ * $Id: Introspector.java,v 1.18 2005/05/22 14:22:58 wguttmn Exp $
  */
 
 package org.exolab.castor.xml;
@@ -48,16 +48,20 @@ package org.exolab.castor.xml;
 import org.exolab.castor.xml.descriptors.CoreDescriptors;
 import org.exolab.castor.xml.handlers.ContainerFieldHandler;
 import org.exolab.castor.xml.handlers.DateFieldHandler;
+import org.exolab.castor.xml.handlers.DefaultFieldHandlerFactory;
 import org.exolab.castor.xml.util.ContainerElement;
 import org.exolab.castor.xml.util.DefaultNaming;
 import org.exolab.castor.xml.util.XMLClassDescriptorImpl;
 import org.exolab.castor.xml.util.XMLFieldDescriptorImpl;
 import org.exolab.castor.mapping.CollectionHandler;
 import org.exolab.castor.mapping.FieldHandler;
+import org.exolab.castor.mapping.FieldHandlerFactory;
+import org.exolab.castor.mapping.GeneralizedFieldHandler;
 import org.exolab.castor.mapping.MappingException;
 import org.exolab.castor.mapping.loader.CollectionHandlers;
 import org.exolab.castor.mapping.loader.FieldHandlerImpl;
 import org.exolab.castor.mapping.loader.TypeInfo;
+import org.exolab.castor.mapping.TypeConvertor;
 import org.exolab.castor.util.Configuration;
 import org.exolab.castor.util.LocalConfiguration;
 import org.exolab.castor.util.List;
@@ -77,11 +81,11 @@ import java.util.Vector;
  * descriptors.
  *
  * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.3 $ $Date: 2003/04/23 06:42:51 $
+ * @version $Revision: 1.18 $ $Date: 2005/05/22 14:22:58 $
  */
 public final class Introspector {
     
-          
+    
     /**
      * The property name for enabling collection wrapping.
      * The property controls whether or not collections 
@@ -119,6 +123,10 @@ public final class Introspector {
     private static final String SET     = "set";
     private static final String CREATE  = "create";
     
+    /**
+     * The default FieldHandlerFactory
+     */
+    private FieldHandlerFactory DEFAULT_HANDLER_FACTORY = new DefaultFieldHandlerFactory();
     
     private static final Class[] EMPTY_CLASS_ARGS = new Class[0];
 
@@ -126,6 +134,16 @@ public final class Introspector {
      * Name of the java.util.List collection
      */
     private static final String LIST = "java.util.List";
+    
+    /**
+     * Name of the java.util.Map collection
+     */
+    private static final String MAP = "java.util.Map";
+    
+    /**
+     * Name of the java.util.Map collection
+     */
+    private static final String SET_COLLECTION = "java.util.Set";
 
     /**
      * Used as a prefix for the name of a container field
@@ -185,6 +203,28 @@ public final class Introspector {
      */
     private boolean _wrapCollectionsInContainer = WRAP_COLLECTIONS_DEFAULT;
     
+    
+    /**
+     * The set of registered FieldHandlerFactory instances
+     */
+    private Vector _handlerFactoryList = null;
+    
+    /**
+     * The set of registered FieldHandlerFactory instances
+     * associated with their supported types
+     */
+    private Hashtable _handlerFactoryMap =  null;
+    
+    /**
+     * A flag indicating that MapKeys should be saved. To remain
+     * backward compatible this may be disable via the 
+     * castor.properties.
+     */
+    private boolean _saveMapKeys = true;
+     
+    /**
+     * Creates a new instance of the Introspector
+     */
     public Introspector() {
         super();
         init();
@@ -206,7 +246,37 @@ public final class Introspector {
             _wrapCollectionsInContainer = Boolean.valueOf(wrap).booleanValue();
         }
         
-    }
+        //-- Save Hashtable / Map keys ?
+        String saveKeys = config.getProperty(Configuration.Property.SaveMapKeys, null);
+        if (saveKeys != null) {
+            if ("false".equals(saveKeys) || "off".equals(saveKeys)) {
+                _saveMapKeys = false;
+            }
+        }
+        
+    } //-- init
+    
+    
+    /**
+     * Registers the given "generalized" FieldHandlerFactory with this
+     * Introspector. 
+     *
+     * @param factory the FieldHandlerFactory to add to this 
+     * introspector
+     * @throws IllegalArgumentException if the given factory is null
+     */
+    public synchronized void addFieldHandlerFactory(FieldHandlerFactory factory) {
+        if (factory == null) {
+            String err = "The argument 'factory' must not be null.";
+            throw new IllegalArgumentException(err);
+        }
+        if (_handlerFactoryList == null) {
+            _handlerFactoryList = new Vector();
+        }        
+        _handlerFactoryList.addElement(factory);
+        registerHandlerFactory(factory);
+    } //-- addFieldHandlerFactory
+    
     /**
      * Returns the NodeType for java primitives
      *
@@ -273,12 +343,56 @@ public final class Introspector {
         
         int methodCount = 0;
         
+        Class superClass = c.getSuperclass();
+        Class[] interfaces = c.getInterfaces();
+        
+        
+        
         //-- create method sets
         for (int i = 0; i < methods.length; i++) {
             Method method = methods[i];
             
-            //-- if method comes from the Object base class, ignore
-            if (method.getDeclaringClass() == Object.class) continue;
+            Class owner = method.getDeclaringClass();
+            
+            //-- ignore methods from super-class, that will be
+            //-- introspected separately, if necessary
+            if (owner != c) {
+                //-- if declaring class is anything but
+                //-- an interface, than just continue,
+                //-- the field comes from a super class
+                //-- (e.g. java.lang.Object)
+                if (!owner.isInterface()) continue;
+                
+                //-- owner is an interface, is it an
+                //-- interface this class implements
+                //-- or a parent class?
+                if (interfaces.length > 0) {
+                    boolean found = false;
+                    for (int count = 0; count < interfaces.length; count++) {
+                        if (interfaces[count] == owner) {
+                            found = true;
+                            break;
+                        }
+                    }
+                    if (!found) continue;
+                }
+            }
+            else {
+                //-- look for overloaded methods
+                if (superClass != null) {
+                    Class[] args = method.getParameterTypes();
+                    String name = method.getName();
+                    Method tmpMethod = null;
+                    try {
+                        tmpMethod = superClass.getMethod(name, args);
+                    }
+                    catch(NoSuchMethodException nsme) {
+                        //-- do nothing
+                    }
+                    if (tmpMethod != null) continue;
+                }
+            }
+            
             
             //-- if method is static...ignore
             if ((method.getModifiers() & Modifier.STATIC) != 0) continue;
@@ -387,17 +501,17 @@ public final class Introspector {
         
         //-- Loop Through MethodSets and create
         //-- descriptors
-        Enumeration enum = methodSets.elements();
+        Enumeration enumeration = methodSets.elements();
         
-        while (enum.hasMoreElements()) {
-            
-            MethodSet methodSet = (MethodSet) enum.nextElement();
-            
+        while (enumeration.hasMoreElements()) {
+
+            MethodSet methodSet = (MethodSet) enumeration.nextElement();
+
             //-- create XMLFieldDescriptor
             String xmlName = _naming.toXMLName(methodSet.fieldName);
-                
+
             boolean isCollection = false;
-            
+
             //-- calculate class type
             //-- 1st check for add-method, then set or get method
             Class type = null;
@@ -405,7 +519,7 @@ public final class Introspector {
                 type = methodSet.add.getParameterTypes()[0];
                 isCollection = true;
             }
-            
+
             //-- if there was no add method, use get/set methods
             //-- to calculate type.
             if (type == null) {
@@ -421,7 +535,7 @@ public final class Introspector {
                     continue;
                 }
             }
-            
+
             //-- Handle Collections
             isCollection = (isCollection || isCollection(type));
             
@@ -451,18 +565,13 @@ public final class Introspector {
                 }
             }
             
-            XMLFieldDescriptorImpl fieldDesc 
-                = createFieldDescriptor(type, methodSet.fieldName, xmlName);
-                
-            
             typeInfo = new TypeInfo(type, null, null, false, null, colHandler);
             
-            if (isCollection) {
-                fieldDesc.setMultivalued(true);
-                fieldDesc.setNodeType(NodeType.Element);
-            }
-            
-            FieldHandlerImpl handler = null;
+            //-- Create FieldHandler first, before the XMLFieldDescriptor
+            //-- in case we need to use a custom handler
+
+            FieldHandler handler = null;
+            boolean customHandler = false;
             try {
                 handler = new FieldHandlerImpl(methodSet.fieldName,
                                                 null,
@@ -472,22 +581,54 @@ public final class Introspector {
                                                 typeInfo);
                 //-- clean up
                 if (methodSet.add != null) 
-                    handler.setAddMethod(methodSet.add);
+                    ((FieldHandlerImpl)handler).setAddMethod(methodSet.add);
                                                 
                 if (methodSet.create != null) 
-                    handler.setCreateMethod(methodSet.create);
+                    ((FieldHandlerImpl)handler).setCreateMethod(methodSet.create);
+
+                //-- handle Hashtable/Map 
+                if (isCollection && _saveMapKeys && isMapCollection(type)) {
+                    ((FieldHandlerImpl)handler).setConvertFrom(new IdentityConvertor());
+                }
                     
+                //-- look for GeneralizedFieldHandler
+                FieldHandlerFactory factory = getHandlerFactory(type);
+                if (factory != null) {
+                    GeneralizedFieldHandler gfh = factory.createFieldHandler(type);
+                    if (gfh != null) {
+                        gfh.setFieldHandler(handler);
+                        handler = gfh;
+                        customHandler = true;
+                        //-- swap type with the type specified by the
+                        //-- custom field handler
+                        if (gfh.getFieldType() != null) {
+                            type = gfh.getFieldType();
+                        }
+                    }
+                } 
+                
             }
             catch (MappingException mx) {
                 throw new MarshalException(mx);
             }
                     
+            
+            XMLFieldDescriptorImpl fieldDesc 
+                = createFieldDescriptor(type, methodSet.fieldName, xmlName);
+                        
+            if (isCollection) {
+                fieldDesc.setMultivalued(true);
+                fieldDesc.setNodeType(NodeType.Element);
+            }
+            
             //-- check for instances of java.util.Date
             if (java.util.Date.class.isAssignableFrom(type)) {
                 //handler = new DateFieldHandler(handler);
-                dateDescriptors.add(fieldDesc);
+                if (!customHandler) {
+                    dateDescriptors.add(fieldDesc);
+                }
             }
-                        
+            
             fieldDesc.setHandler(handler);
             
             //-- Wrap collections?
@@ -540,7 +681,42 @@ public final class Introspector {
             for (int i = 0; i < fields.length; i++) {                
                 Field field = fields[i];
                 
-                Class type = field.getType();       
+                Class owner = field.getDeclaringClass();
+                
+                //-- ignore fields from super-class, that will be
+                //-- introspected separately, if necessary
+                if (owner != c) {
+                    //-- if declaring class is anything but
+                    //-- an interface, than just continue,
+                    //-- the field comes from a super class
+                    //-- (e.g. java.lang.Object)
+                    if (!owner.isInterface()) continue;
+                    
+                    //-- owner is an interface, is it an
+                    //-- interface this class implements
+                    //-- or a parent class?
+                    if (interfaces.length > 0) {
+                        boolean found = false;
+                        for (int count = 0; count < interfaces.length; count++) {
+                            if (interfaces[count] == owner) {
+                                found = true;
+                                break;
+                            }
+                        }
+                        if (!found) continue;
+                    }
+                }
+            
+                //-- make sure field is not transient or static final
+                int modifiers = field.getModifiers();
+                if (Modifier.isTransient(modifiers)) continue;
+                if (Modifier.isFinal(modifiers) && 
+                    Modifier.isStatic(modifiers))
+                    continue;
+                
+                Class type = field.getType();
+                
+                
                 
                 if (!isDescriptable(type)) continue;
                 
@@ -580,6 +756,41 @@ public final class Introspector {
                 String fieldName = field.getName();
                 String xmlName = _naming.toXMLName(fieldName);
                 
+                //-- Create FieldHandler first, before the XMLFieldDescriptor
+                //-- in case we need to use a custom handler
+                
+                typeInfo = new TypeInfo(type, null, null, false, null, colHandler);
+                
+                FieldHandler handler = null;
+                boolean customHandler = false;
+                try {
+                    handler = new FieldHandlerImpl(field, typeInfo);
+                    
+                    //-- handle Hashtable/Map 
+                    if (isCollection && _saveMapKeys && isMapCollection(type)) {
+                        ((FieldHandlerImpl)handler).setConvertFrom(new IdentityConvertor());
+                    }
+                        
+                    //-- look for GeneralizedFieldHandler
+                    FieldHandlerFactory factory = getHandlerFactory(type);
+                    if (factory != null) {
+                        GeneralizedFieldHandler gfh = factory.createFieldHandler(type);
+                        if (gfh != null) {
+                            gfh.setFieldHandler(handler);
+                            handler = gfh;
+                            customHandler = true;
+                            //-- swap type with the type specified by the
+                            //-- custom field handler
+                            if (gfh.getFieldType() != null) {
+                                type = gfh.getFieldType();
+                            }
+                        }
+                    } 
+                }
+                catch (MappingException mx) {
+                    throw new MarshalException(mx);
+                }
+                
                 XMLFieldDescriptorImpl fieldDesc = 
                         createFieldDescriptor(type, fieldName, xmlName);
                         
@@ -589,23 +800,14 @@ public final class Introspector {
                 }
                 descriptors.put(xmlName, fieldDesc);
                 classDesc.addFieldDescriptor(fieldDesc);
-                
-                typeInfo = new TypeInfo(type, null, null, false, null, colHandler);
-                
-                FieldHandler handler = null;
-
-                try {
-                    handler = new FieldHandlerImpl(field, typeInfo);
-                }
-                catch (MappingException mx) {
-                    throw new MarshalException(mx);
-                }
-                
                 fieldDesc.setHandler(handler);
                    
                 //-- check for instances of java.util.Date
-                if (java.util.Date.class.isAssignableFrom(type))
-                    dateDescriptors.add(fieldDesc);
+                if (java.util.Date.class.isAssignableFrom(type)) {
+                    if (!customHandler) {
+                        dateDescriptors.add(fieldDesc);
+                    }
+                }
                 
             }            
         } //-- end of direct field access
@@ -618,14 +820,70 @@ public final class Introspector {
                     (XMLFieldDescriptorImpl) dateDescriptors.get(i);
                 FieldHandler handler = fieldDesc.getHandler();
                 fieldDesc.setImmutable(true);
-                fieldDesc.setHandler(new DateFieldHandler(handler));
+                DateFieldHandler dfh = new DateFieldHandler(handler);
+                
+                //-- patch for java.sql.Date
+                Class type = fieldDesc.getFieldType();
+                if (java.sql.Date.class.isAssignableFrom(type)) {
+                    dfh.setUseSQLDate(true);
+                }
+                fieldDesc.setHandler(dfh);
             }
         }
         
-        
+        //-- Add reference to superclass...if necessary
+        if ((superClass != null) &&
+            (superClass != Void.class) &&
+            (superClass != Object.class) &&
+            (superClass != Class.class)) 
+        {
+            try {
+                XMLClassDescriptor parent = generateClassDescriptor(superClass, errorWriter);
+                if (parent != null) {
+                    classDesc.setExtends(parent);
+                }
+            }
+            catch(MarshalException mx) {
+                //-- Ignore for now.
+            }
+
+        }
+             
         return classDesc;
     } //-- generateClassDescriptor
     
+    /**
+     * Removes the given FieldHandlerFactory from this Introspector
+     *
+     * @param factory the FieldHandlerFactory to remove 
+     * @return true if the given FieldHandlerFactory was removed, or
+     * false otherwise.
+     * @throws IllegalArgumentException if the given factory is null
+     */
+    public synchronized boolean removeFieldHandlerFactory(FieldHandlerFactory factory) 
+    {
+        if (factory == null) {
+            String err = "The argument 'factory' must not be null.";
+            throw new IllegalArgumentException(err);
+        }
+        
+        //-- if list is null, just return
+        if (_handlerFactoryList == null) return false;
+        
+        if (_handlerFactoryList.removeElement(factory)) {
+            //-- re-register remaining handlers
+            _handlerFactoryMap.clear();
+            for (int i = 0; i < _handlerFactoryList.size(); i++) {
+                FieldHandlerFactory tmp = 
+                    (FieldHandlerFactory)_handlerFactoryList.elementAt(i);
+                registerHandlerFactory(tmp);
+            }
+            return true;
+        }
+        return false;
+    } //-- removeFieldHandlerFactory
+    
+    
     /**
      * Sets whether or not collections (arrays, vectors, etc) 
      * should be wrapped in a container element. For example:
@@ -688,7 +946,10 @@ public final class Introspector {
                         type.getConstructor( EMPTY_CLASS_ARGS );
                     }
                     catch ( NoSuchMethodException e ) { 
-                        return false;
+                        //-- Allow any built-in descriptor classes
+                        //-- that don't have default constructors
+                        //-- such as java.sql.Date, java.sql.Time, etc.
+                        return (CoreDescriptors.getDescriptor(type) != null);
                     }
                 }
             }
@@ -725,6 +986,20 @@ public final class Introspector {
             _primitiveNodeType = NodeType.Attribute;
     } //-- setPrimitiveNodeType
     
+    /**
+     * Sets whether or not keys from Hastable / Map instances
+     * should be saved in the XML. 
+     * 
+     * <p>Note: This is true by default since Castor 0.9.5.3</p>
+     *
+     * @param saveMapKeys a boolean that when true indicates keys
+     * from Hashtable or Map instances should be saved. Otherwise
+     * only the value object is saved.
+     */
+    public void setSaveMapKeys(boolean saveMapKeys) {
+        _saveMapKeys = saveMapKeys;
+    } //-- setSaveMapKeys
+    
     /**
      * Converts the given xml name to a Java name.
      * @param name the name to convert to a Java Name
@@ -791,6 +1066,48 @@ public final class Introspector {
         return fieldDesc;
     } //-- createFieldDescriptor
 
+
+    /**
+     * Returns the registered FieldHandlerFactory for the
+     * given Class type.
+     *
+     * @param type the Class type to return the registered
+     * FieldHandlerFactory for
+     */
+    private FieldHandlerFactory getHandlerFactory(Class type) {
+        if (_handlerFactoryMap != null) {               
+            Class tmp = type;
+            while (tmp != null) {
+                Object obj = _handlerFactoryMap.get(tmp);
+                if (obj != null) {
+                    return (FieldHandlerFactory)obj;
+                }
+                tmp = tmp.getSuperclass();
+            }
+        }
+        
+        //-- check DefaultFieldHandlerFactory
+        if (DEFAULT_HANDLER_FACTORY.isSupportedType(type))
+            return DEFAULT_HANDLER_FACTORY;
+            
+        return null;
+    } //-- getHandlerFactory
+    
+    /**
+     * Registers the supported class types for the given
+     * FieldHandlerFactory into the map (for faster lookups)
+     */
+    private void registerHandlerFactory(FieldHandlerFactory factory) {
+        if (_handlerFactoryMap == null)
+            _handlerFactoryMap = new Hashtable();
+            
+        Class[] types = factory.getSupportedTypes();
+        for (int i = 0; i < types.length; i++) {
+            _handlerFactoryMap.put(types[i], factory);
+        }
+    } //-- registerHandlerFactory
+
+
     /**
      * Returns true if the given Class is an instance of a
      * collection class.
@@ -818,6 +1135,37 @@ public final class Introspector {
     } //-- isCollection
     
     
+    /**
+     * Returns true if the given Class is an instance of a
+     * collection class.
+     * 
+     * @see loadCollections
+    **/
+    public static boolean isMapCollection(Class clazz) {
+        
+        if (clazz.isArray()) return false;
+        
+        for (int i = 0; i < _collections.length; i++) {
+            //-- check to see if clazz is either the
+            //-- same as or a subclass of one of the
+            //-- available collections. For performance
+            //-- reasons we first check if class is 
+            //-- directly equal to one of the collections
+            //-- instead of just calling isAssignableFrom.
+            if ((clazz == _collections[i]) ||
+                (_collections[i].isAssignableFrom(clazz)))
+            {
+                if (_collections[i] == java.util.Hashtable.class)
+                    return true;
+                //-- For JDK 1.1 compatibility use string name "java.util.Map"
+                if (_collections[i].getName().equals(MAP))
+                    return true;
+            }
+        }
+        return false;
+    } //-- isMapCollection
+    
+    
     /**
      * Returns true if we are allowed to create a descriptor
      * for a given class type
@@ -839,6 +1187,7 @@ public final class Introspector {
             
             //-- make sure we can construct the Object
             if (!type.isArray() ) {
+                
                 //-- try to get the default constructor and make
                 //-- sure we are only looking at classes that can 
                 //-- be instantiated by calling Class#newInstance
@@ -846,7 +1195,11 @@ public final class Introspector {
                     type.getConstructor( EMPTY_CLASS_ARGS );
                 }
                 catch ( NoSuchMethodException e ) { 
-                    return false;
+                    
+                    //-- Allow any built-in descriptor classes
+                    //-- that don't have default constructors
+                    //-- such as java.sql.Date, java.sql.Time, etc.
+                    return (CoreDescriptors.getDescriptor(type) != null);
                 }
             }
         }
@@ -878,15 +1231,19 @@ public final class Introspector {
      * @return a list of available collections
     **/
     private static Class[] loadCollections() {
-        Vector collections = new Vector();
+        
+        
+        Vector collections = new Vector(6);
         
         //-- JDK 1.1
         collections.addElement(Vector.class);
         collections.addElement(Enumeration.class);
+        collections.addElement(Hashtable.class);
         
         //-- JDK 1.2+
         ClassLoader loader = Vector.class.getClassLoader();
         
+        
         Class clazz = null;
         try {
             if (loader != null) {
@@ -899,7 +1256,38 @@ public final class Introspector {
             //-- or some nasty ClassLoader 
             //-- issue has occurred.
         } 
-        if (clazz != null) collections.addElement(clazz);
+        if (clazz != null) {
+            //-- java.util.List found, add to collections, 
+            //-- also add java.util.Map
+            collections.addElement(clazz);
+        
+            clazz = null;
+            try {
+                //-- java.util.Map
+                if (loader != null) {
+                    clazz = loader.loadClass(MAP);
+                }
+                else clazz = Class.forName(MAP);
+                if (clazz != null) {
+                    collections.addElement(clazz);
+                }
+                //-- java.util.Set
+                if (loader != null) {
+                    clazz = loader.loadClass(SET_COLLECTION);
+                }
+                else clazz = Class.forName(SET_COLLECTION);
+                if (clazz != null) {
+                    collections.addElement(clazz);
+                }
+                
+                
+            }
+            catch(ClassNotFoundException cnfx) {
+                //-- just ignore...for now
+                //-- some nasty ClassLoader issue has occurred.
+            } 
+        }
+        
         
         Class[] classes = new Class[collections.size()];
         collections.copyInto(classes);
@@ -908,6 +1296,19 @@ public final class Introspector {
     } //-- loadCollections
     
     
+    /**
+     * A special TypeConvertor that simply returns the object
+     * given. This is used for preventing the FieldHandlerImpl
+     * from using a CollectionHandler when getValue is called.
+     */
+    class IdentityConvertor implements TypeConvertor {
+        public Object convert( Object object, String param )
+            throws ClassCastException
+        {
+            return object;
+        }
+    } //-- class: IdentityConvertor
+    
     /**
      * A simple struct for holding a set of accessor methods
     **/
@@ -917,17 +1318,17 @@ public final class Introspector {
          * A reference to the add method.
         **/
         Method add    = null;
-        
+
         /**
          * A reference to the create method.
         **/
         Method create = null;
-        
+
         /**
          * A reference to the get method.
         **/
         Method get    = null;
-        
+
         /**
          * A reference to the set method.
         **/
@@ -945,6 +1346,9 @@ public final class Introspector {
         }
     } //-- inner class: MethodSet
     
+    
+    
+    
 } //-- Introspector
 
 /**
@@ -975,5 +1379,4 @@ class IntrospectedXMLClassDescriptor
         setIntrospected(true);        
     } //-- XMLClassDescriptorImpl
     
-        
 } //-- IntrospectedClassDescriptor
