@@ -38,9 +38,9 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
+ * Copyright 1999-2004 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: MarshalFramework.java,v 1.4 2003/04/24 02:10:27 kvisco Exp $
+ * $Id: MarshalFramework.java,v 1.12 2005/03/01 00:47:50 kvisco Exp $
  */
 
 package org.exolab.castor.xml;
@@ -51,7 +51,6 @@ import org.exolab.castor.mapping.CollectionHandler;
 import org.exolab.castor.mapping.FieldDescriptor;
 import org.exolab.castor.mapping.MappingException;
 import org.exolab.castor.mapping.loader.CollectionHandlers;
-import org.exolab.castor.mapping.ValidityException;
 
 import java.util.Vector;
 
@@ -59,11 +58,14 @@ import java.util.Vector;
 /**
  * A core class for common code shared throughout the
  * Marshalling Framework
- * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.4 $ $Date: 2003/04/24 02:10:27 $
-**/
+ * 
+ * @author <a href="mailto:kvisco-at-intalio.com">Keith Visco</a>
+ * @version $Revision: 1.12 $ $Date: 2005/03/01 00:47:50 $
+ */
 abstract class MarshalFramework {
 
+    
+    
     //--------------------------/
     //- Public class variables -/
     //--------------------------/
@@ -84,22 +86,68 @@ abstract class MarshalFramework {
     **/
     public static final String XSI_NO_NAMESPACE_SCHEMA_LOCATION 
         = "noNamespaceSchemaLocation";
+   
+    /**
+     * The xml:lang attribute name
+     */
+    public static final String XML_LANG_ATTR = "xml:lang";
     
+    /**
+     * The xml:lang attribute, without the "xml:" prefix.
+     */
+    public static final String LANG_ATTR = "lang";
+    
+    /**
+     * The xsi:nil attribute, without the "xsi:" prefix.
+     */
+    public static final String NIL_ATTR = "nil";
+    
+    public static final String XSI_NIL_ATTR = "xsi:nil";
+    
+    /**
+     * The xml:space attribute name
+     */    
+    public static final String XML_SPACE_ATTR = "xml:space";
+    
+    /**
+     * The xml:space attribute name, without the "xml:" prefix
+     */    
+    public static final String SPACE_ATTR = "space";
+    
+    /**
+     * The xsi:type attribute name, without the "xsi:" prefix
+     */    
+    public static final String TYPE_ATTR = "type";
+    
+    /**
+     * The value of 'true'
+     */
+    public static final String TRUE_VALUE = "true";
+    
+
     //-----------------------------/
     //- Protected class variables -/
     //-----------------------------/
 
+    static final String INTERNAL_XML_NAME = "-error-if-this-is-used-";
+
     /**
      * The default prefix used for specifying the
      * xsi:type as a classname instead of a schema name.
      * This is a Castor specific hack.
-    **/
+     */
     static final String JAVA_PREFIX = "java:";
 
     /**
      * The name of the QName type
-    **/
+     */
     static final String QNAME_NAME = "QName";
+    
+    /**
+     * An empty array of field descriptors
+     */
+    static final XMLFieldDescriptor[] NO_FIELD_DESCRIPTORS 
+        = new XMLFieldDescriptor[0];
 
     /**
      * Returns true if the given Class is a considered a
@@ -261,39 +309,63 @@ abstract class MarshalFramework {
         return false;
     } //-- primitiveOrWrapperEquals
     
+    /**
+     * 
+     */
+    private static final InheritanceMatch[] NO_MATCH_ARRAY = new InheritanceMatch[0];
+    
     /**
      * Search there is a field descriptor which can accept one of the class
      * descriptor which match the given name and namespace.
      * @returns an array of InheritanceMatch.
      */
     public static InheritanceMatch[] searchInheritance
-        (String name, String namespace, XMLClassDescriptor classDesc, ClassDescriptorResolver cdResolver) 
+        (String name, 
+         String namespace, 
+         XMLClassDescriptor classDesc, 
+         ClassDescriptorResolver cdResolver) 
+        throws MarshalException
     {
 
-
-        //-- A little required logic for finding Not-Yet-Loaded
-        //-- descriptors
-        String className =JavaNaming.toJavaClassName(name);
-        //-- should use namespace-to-prefix mappings, but
-        //-- just create package for now.
-        Class clazz = classDesc.getJavaClass();
-        String pkg = null;
-        if (clazz != null) {
-            while (clazz.getDeclaringClass() != null)
-                clazz = clazz.getDeclaringClass();
-            pkg = clazz.getName();
-            int idx = pkg.lastIndexOf('.');
-            if (idx >= 0) {
-                pkg = pkg.substring(0, idx+1);
-                className = pkg + className;
+        ClassDescriptorEnumeration cde = null;
+        
+        try {
+            //-- A little required logic for finding Not-Yet-Loaded
+            //-- descriptors
+            String className =JavaNaming.toJavaClassName(name);
+            //-- should use namespace-to-prefix mappings, but
+            //-- just create package for now.
+            Class clazz = classDesc.getJavaClass();
+            String pkg = null;
+            if (clazz != null) {
+                while (clazz.getDeclaringClass() != null)
+                    clazz = clazz.getDeclaringClass();
+                pkg = clazz.getName();
+                int idx = pkg.lastIndexOf('.');
+                if (idx >= 0) {
+                    pkg = pkg.substring(0, idx+1);
+                    className = pkg + className;
+                }
             }
+            cdResolver.resolve(className, classDesc.getClass().getClassLoader());
+            //-- end Not-Yet-Loaded descriptor logic
+            
+            //-- resolve all by XML name + namespace URI
+            cde = cdResolver.resolveAllByXMLName(name, namespace, null);
+        }
+        catch(ResolverException rx) {
+            Throwable actual = rx.getCause();
+            if (actual instanceof MarshalException) {
+                throw (MarshalException)actual;
+            }
+            if (actual != null) {
+                throw new MarshalException(actual);
+            }
+            throw new MarshalException(rx);
         }
-        cdResolver.resolve(className);
-        //-- end Not-Yet-Loaded descriptor logic
 
-        Vector inheritanceList = new Vector(3);
+        Vector inheritanceList = null;
         XMLFieldDescriptor descriptor  = null;
-        ClassDescriptorEnumeration cde = cdResolver.resolveAllByXMLName(name, namespace, null);
         XMLFieldDescriptor[] descriptors = classDesc.getElementDescriptors();
         XMLClassDescriptor cdInherited = null;
 
@@ -305,12 +377,19 @@ abstract class MarshalFramework {
                 for (int i = 0; i < descriptors.length; i++) {
 
                     if (descriptors[i] == null) continue;
+                    
+                    //-- skip descriptors with special internal name
+                    if (INTERNAL_XML_NAME.equals(descriptors[i].getXMLName())) 
+                        continue;
+                    
                     //-- check for inheritence
                     Class superclass = descriptors[i].getFieldType();
 
                     // It is possible that the superclass is of type object if we use any node.
                     if (superclass.isAssignableFrom(subclass) && (superclass != Object.class)) {
                         descriptor = descriptors[i];
+                        if (inheritanceList == null)
+                            inheritanceList = new Vector(3);
                         inheritanceList.addElement(new InheritanceMatch(descriptor, cdInherited));
                     }
                 }
@@ -318,10 +397,14 @@ abstract class MarshalFramework {
             //-- reset inherited class descriptor, if necessary
             if (descriptor == null) cdInherited = null;
         }
-
-        InheritanceMatch[] result = new InheritanceMatch[inheritanceList.size()];
-        inheritanceList.toArray(result);
-        return result;
+        
+        if (inheritanceList != null) {
+            InheritanceMatch[] result = new InheritanceMatch[inheritanceList.size()];
+            inheritanceList.toArray(result);
+            return result;
+        }
+        return NO_MATCH_ARRAY;
+        
     }
 
      /**
@@ -430,9 +513,9 @@ abstract class MarshalFramework {
          *
          */
         public XMLFieldDescriptor getFieldDescriptor
-            (String name, NodeType nodeType) 
+            (String name, String namespace, NodeType nodeType) 
         {
-            return _classDesc.getFieldDescriptor(name, nodeType);
+            return _classDesc.getFieldDescriptor(name, namespace, nodeType);
         } //-- getFieldDescriptor
 
         /**
@@ -495,22 +578,13 @@ abstract class MarshalFramework {
             return Introspector.introspected(_classDesc);
         } //-- introspected
         
-        /**
-         * <p>Returns true if the given object represented by this XMLClassDescriptor
-         * can accept a member whose name is given.
-         * An XMLClassDescriptor can accept a field if it contains a descriptor that matches
-         * the given name and if the given object can hold this field (i.e a value is not already set for
-         * this field).
-         * Different reasons can change the acceptance criteria, this is the reason why each implementation
-         * of XMLClassDescriptor must define these reasons.
-         *
-         * @param fieldName the name of the field to check
-         * @param object the object represented by this XMLCLassDescriptor
-         * @return true if the given object represented by this XMLClassDescriptor
-         * can accept a member whose name is given.
+        /*
+         *  (non-Javadoc)
+         * @see org.exolab.castor.xml.XMLClassDescriptor#canAccept
+         * (java.lang.String, java.lang.String, java.lang.Object)
          */
-        public boolean canAccept(String fieldName, Object object) {
-            return _classDesc.canAccept(fieldName, object);
+        public boolean canAccept(String name,String namespace, Object object) {
+            return _classDesc.canAccept(name, namespace, object);
         } //-- canAccept
 
 
