@@ -38,9 +38,14 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
+ * Copyright 1999-2004(C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: XMLMappingLoader.java,v 1.5 2003/04/23 08:44:42 kvisco Exp $
+ * This file was originally developed by Keith Visco during the
+ * course of employment at Intalio Inc.
+ * All portions of this file developed by Keith Visco after Jan 19 2005 are
+ * Copyright (C) 2005 Keith Visco. All Rights Reserved.
+ *
+ * $Id: XMLMappingLoader.java,v 1.15 2005/08/03 21:11:48 rjoachim Exp $
  */
 
 
@@ -50,25 +55,25 @@ package org.exolab.castor.xml;
 import org.exolab.castor.mapping.ClassDescriptor;
 import org.exolab.castor.mapping.FieldDescriptor;
 import org.exolab.castor.mapping.FieldHandler;
+import org.exolab.castor.mapping.MapItem;
 import org.exolab.castor.mapping.MappingException;
 import org.exolab.castor.mapping.TypeConvertor;
 import org.exolab.castor.mapping.CollectionHandler;
+import org.exolab.castor.mapping.loader.CollectionHandlers;
 import org.exolab.castor.mapping.loader.MappingLoader;
-import org.exolab.castor.mapping.loader.Types;
-import org.exolab.castor.mapping.loader.FieldDescriptorImpl;
 import org.exolab.castor.mapping.loader.FieldHandlerImpl;
 import org.exolab.castor.mapping.loader.TypeInfo;
 import org.exolab.castor.mapping.xml.ClassMapping;
 import org.exolab.castor.mapping.xml.FieldMapping;
 import org.exolab.castor.mapping.xml.BindXml;
 import org.exolab.castor.mapping.xml.MapTo;
+import org.exolab.castor.mapping.xml.Property;
 import org.exolab.castor.mapping.xml.types.BindXmlAutoNamingType;
 import org.exolab.castor.mapping.xml.types.CollectionType;
-import org.exolab.castor.util.Configuration;
 import org.exolab.castor.util.LocalConfiguration;
 
 import org.exolab.castor.xml.handlers.ContainerFieldHandler;
-import org.exolab.castor.xml.handlers.EnumFieldHandler;
+import org.exolab.castor.xml.handlers.ToStringFieldHandler;
 import org.exolab.castor.xml.util.ClassDescriptorResolverImpl;
 import org.exolab.castor.xml.util.ContainerElement;
 import org.exolab.castor.xml.util.XMLClassDescriptorImpl;
@@ -80,22 +85,26 @@ import java.io.PrintWriter;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
-import java.util.Vector;
-import java.util.Enumeration;
 
 /**
  * An XML implementation of mapping helper. Creates XML class
  * descriptors from the mapping file.
  *
- * @author <a href="kvisco@intalio.com">Keith Visco</a>
+ * @author <a href="keith AT kvisco DOT com">Keith Visco</a>
  * @author <a href="arkin@intalio.com">Assaf Arkin</a>
- * @version $Revision: 1.5 $ $Date: 2003/04/23 08:44:42 $
+ * @version $Revision: 1.15 $ $Date: 2005/08/03 21:11:48 $
  */
 public class XMLMappingLoader
     extends MappingLoader
 {
 
 
+    /**
+     * The default xml prefix used on certain 
+     * attributes such as xml:lang, xml:base, etc.
+     */
+    private static final String XML_PREFIX = "xml:";
+    
 
     /**
      * Empty array of class types used for reflection
@@ -121,14 +130,19 @@ public class XMLMappingLoader
     private static final String VALUE_OF = "valueOf";
 
 
+    /**
+     * A reference to the internal ClassDescriptorResolver
+     */
+    private ClassDescriptorResolverImpl _cdResolver;
+
     /**
      * naming conventions
-    **/
+     */
     private XMLNaming _naming = null;
 
     /**
-     * Introspector
-    **/
+     * The default NodeType for primitives
+     */
     private NodeType _primitiveNodeType = null;
 
 
@@ -147,7 +161,12 @@ public class XMLMappingLoader
 
     } //-- XMLMappingLoader
 
-
+    private void createResolver() {
+        _cdResolver = new ClassDescriptorResolverImpl();
+        _cdResolver.setIntrospection(false);
+        _cdResolver.setLoadPackageMappings(false);
+    }
+    
     protected void resolveRelations( ClassDescriptor clsDesc )
         throws MappingException
     {
@@ -155,8 +174,9 @@ public class XMLMappingLoader
 
         fields = clsDesc.getFields();
         for ( int i = 0 ; i < fields.length ; ++i ) {
+            if (fields[i].getClassDescriptor() != null) continue;
             ClassDescriptor   relDesc;
-
+            
             relDesc = getDescriptor( fields[ i ].getFieldType() );
             if ( relDesc == NoDescriptor ) {
                 // XXX Error message should come here
@@ -182,11 +202,26 @@ public class XMLMappingLoader
         ClassDescriptor clsDesc;
         String          xmlName;
 
-        // See if we have a compiled descriptor.
-        clsDesc = loadClassDescriptor( clsMap.getName() );
-        if ( clsDesc != null && clsDesc instanceof XMLClassDescriptor )
-            return clsDesc;
-
+        if (clsMap.getAutoComplete()) {
+            if ((clsMap.getMapTo() == null) &&
+                (clsMap.getFieldMappingCount() == 0) &&
+                (clsMap.getIdentityCount() == 0))
+            {
+                //-- if we make it here we simply try
+                //-- to load a compiled mapping
+                if (_cdResolver == null) {
+                    createResolver();
+                }
+                try {
+                    clsDesc = _cdResolver.resolve(clsMap.getName());
+                    if (clsDesc != null) 
+                        return clsDesc;
+                }
+                catch(ResolverException rx) {}
+                
+            }
+        }
+        
         // Use super class to create class descriptor. Field descriptors will be
         // generated only for supported fields, see createFieldDesc later on.
         clsDesc = super.createDescriptor( clsMap );
@@ -210,17 +245,29 @@ public class XMLMappingLoader
         if (clsMap.getAutoComplete()) {
 
             XMLClassDescriptor referenceDesc = null;
+            
             Class type = xmlClassDesc.getJavaClass();
             
             //-- check compiled descriptors 
-            ClassDescriptorResolverImpl cdr 
-                = new ClassDescriptorResolverImpl();
-            cdr.setIntrospection(false);
-            referenceDesc = cdr.resolve(type);
+            if (_cdResolver == null) {
+                createResolver();
+            }
+            try {
+                referenceDesc = _cdResolver.resolve(type);
+            }
+            catch(ResolverException rx) {
+                throw new MappingException(rx);
+            }
+            
             if (referenceDesc == null) {
                 Introspector introspector = new Introspector();
                 try {
                     referenceDesc = introspector.generateClassDescriptor(type);
+                    if (clsMap.getExtends() != null) {
+                        //-- clear parent from introspected descriptor since
+                        //-- a mapping was provided in the mapping file
+                        ((XMLClassDescriptorImpl)referenceDesc).setExtends(null);
+                    }
                 } catch (MarshalException mx) {
                     String error = "unable to introspect class '" + 
                         type.getName() + "' for auto-complete: ";
@@ -233,6 +280,7 @@ public class XMLMappingLoader
             if (clsMap.getIdentityCount() > 0)
                 identity = clsMap.getIdentity(0);
 
+            
             FieldDescriptor[] fields = xmlClassDesc.getFields();
 
             // Attributes
@@ -266,7 +314,7 @@ public class XMLMappingLoader
             // Content
             XMLFieldDescriptor field = referenceDesc.getContentDescriptor();
             if (field!= null)
-                if (isMatchFieldName(fields, field.getFieldName()))
+                if (!isMatchFieldName(fields, field.getFieldName()))
                     // If there is no field with this name, we can add
                     xmlClassDesc.addFieldDescriptor(field);
 
@@ -298,7 +346,9 @@ public class XMLMappingLoader
     protected FieldDescriptor createFieldDesc( Class javaClass, FieldMapping fieldMap )
         throws MappingException
     {
+        
         FieldDescriptor        fieldDesc;
+        CollectionType         colType  = fieldMap.getCollection();
         String                 xmlName  = null;
         NodeType               nodeType = null;
         String                 match    = null;
@@ -306,8 +356,16 @@ public class XMLMappingLoader
         boolean                isReference = false;
         boolean                isXMLTransient = false;
 
-        // Create an XML field descriptor
+        //-- handle special case for HashMap/Hashtable
+        if ((fieldMap.getType() == null) && (colType != null)) {
+            if ((colType == CollectionType.HASHTABLE) ||
+                (colType == CollectionType.MAP)) 
+            {
+                fieldMap.setType(MapItem.class.getName());
+            }
+        }
 
+        // Create an XML field descriptor
         fieldDesc = super.createFieldDesc( javaClass, fieldMap );
 
         BindXml xml = fieldMap.getBindXml();
@@ -357,6 +415,10 @@ public class XMLMappingLoader
                 namespace = xmlName.substring(1, idx);
                 xmlName = xmlName.substring(idx+1);
             }
+            else if (xmlName.startsWith(XML_PREFIX)) {
+                namespace = Namespaces.XML_NAMESPACE;
+                xmlName = xmlName.substring(4);
+            }
         }
 
         if (nodeType == null) {
@@ -383,6 +445,9 @@ public class XMLMappingLoader
 
         //--set a default fieldValidator
         xmlDesc.setValidator(new FieldValidator());
+        
+        //-- enable use parent namespace if explicit one doesn't exist
+        xmlDesc.setUseParentsNamespace(true);
 
         //-- If deriveNameByClass we need to reset the name to
         //-- null because XMLFieldDescriptorImpl tries to be smart
@@ -411,6 +476,12 @@ public class XMLMappingLoader
 
         if (xml != null) {
             
+            //-- has class descriptor for type specified
+            if (xml.getClassMapping() != null) {
+                ClassDescriptor cd = createDescriptor(xml.getClassMapping());
+                xmlDesc.setClassDescriptor((XMLClassDescriptor)cd);
+            }
+            
             //-- has location path?
             if (xml.getLocation() != null) {
                 xmlDesc.setLocationPath(xml.getLocation());
@@ -425,14 +496,32 @@ public class XMLMappingLoader
                 validator = new NameValidator(NameValidator.NCNAME);
                 xmlDesc.setValidator(new FieldValidator(validator));
             }
+            
+            //-- special properties?
+            Property[] props = xml.getProperty();
+            if ((props != null) && (props.length > 0)) {
+                for (int pIdx = 0; pIdx < props.length; pIdx++) {
+                    Property prop = props[pIdx];
+                    xmlDesc.setProperty(prop.getName(), prop.getValue());
+                }
+            }
         }
 
+        //-- Get collection type
+        if (colType == null) {
+            //-- just in case user forgot to use collection="..."
+            //-- in the mapping file
+            Class type = fieldDesc.getFieldType();
+            if (CollectionHandlers.hasHandler(type)) {
+                String typeName = CollectionHandlers.getCollectionName(type);
+                colType = CollectionType.valueOf(typeName);
+            }
+        }
+        
         //-- isMapped item
-        CollectionType colType = fieldMap.getCollection();
-        if (colType != null) {
-            
+        if (colType != null) {    
             if ((colType == CollectionType.HASHTABLE) ||
-                                (colType == CollectionType.MAP))
+                (colType == CollectionType.MAP))
             {
                 //-- Make sure user is not using an addMethod
                 //-- before setting the mapped field to true.
@@ -442,13 +531,14 @@ public class XMLMappingLoader
                         xmlDesc.setMapped(true);
                     }
                 }
+                else xmlDesc.setMapped(true);
             }
 
             
             //-- special NodeType.Namespace handling
             //-- prevent FieldHandlerImpl from using CollectionHandler
             //-- during calls to #getValue
-            if (nodeType == NodeType.Namespace) {
+            if ((nodeType == NodeType.Namespace) || (xmlDesc.isMapped())) {
                 Object handler = xmlDesc.getHandler();
                 if (handler instanceof FieldHandlerImpl) {
                     FieldHandlerImpl handlerImpl = (FieldHandlerImpl)handler;
@@ -456,12 +546,13 @@ public class XMLMappingLoader
                 }
             }
             //-- wrap collection in element?
-            else if (nodeType == NodeType.Element) {
+            if (nodeType == NodeType.Element) {
                 if (fieldMap.hasContainer() && (!fieldMap.getContainer())) {
                     xmlDesc = wrapCollection(xmlDesc);
                 }
             }
         }
+        
         //-- is Type-Safe Enumeration?
         //-- This is not very clean, we should have a way
         //-- to specify something is a type-safe enumeration
@@ -489,8 +580,14 @@ public class XMLMappingLoader
                         if ((returnType != null) && fieldType.isAssignableFrom(returnType)) {
                             if (fieldMap.getHandler() == null) {
                                 //-- Use EnumFieldHandler
-                                FieldHandler handler = xmlDesc.getHandler();
-                                handler = new EnumFieldHandler(fieldType, handler);
+                                //-- mapping loader now supports a basic EnumFieldHandler
+                                //-- for xml we simply need to make sure the toString()
+                                //-- method is called during getValue()
+                                //FieldHandler handler = xmlDesc.getHandler();
+                                //handler = new EnumFieldHandler(fieldType, handler);
+                                
+                                FieldHandler handler = new ToStringFieldHandler(fieldType, xmlDesc.getHandler());
+                                
                                 xmlDesc.setHandler(handler);
                                 xmlDesc.setImmutable(true);
                             }
@@ -522,6 +619,21 @@ public class XMLMappingLoader
         return xmlDesc;
     }
 
+    /**
+     * Sets whether or not to look for and load package specific
+     * mapping files (".castor.xml" files).
+     * 
+     * @param loadPackageMappings a boolean that enables or
+     * disables the loading of package specific mapping files
+     */
+    public void setLoadPackageMappings(boolean loadPackageMappings) 
+    {
+        if (_cdResolver == null) {
+            createResolver();
+        }
+        _cdResolver.setLoadPackageMappings(loadPackageMappings);
+    } //-- setLoadPackageMappings
+    
 
     protected TypeInfo getTypeInfo( Class fieldType, CollectionHandler colHandler, FieldMapping fieldMap )
         throws MappingException
@@ -563,6 +675,7 @@ public class XMLMappingLoader
         //-- we can't do this in the constructor because
         //-- XMLFieldDescriptorImpl will create a default one.
         newFieldDesc.setXMLName(null);
+        newFieldDesc.setMatches("*");
         
         //-- add the field descriptor to our new class descriptor
         classDesc.addFieldDescriptor(newFieldDesc);        
@@ -587,7 +700,7 @@ public class XMLMappingLoader
      * @return true if the given class should be treated as a primitive
      * type
     **/
-    private static boolean isPrimitive(Class type) {
+    protected static boolean isPrimitive(Class type) {
 
         if (type.isPrimitive()) return true;
 
