@@ -38,15 +38,14 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Copyright 1999-2002 (C) Intalio Inc. All Rights Reserved.
+ * Copyright 1999-2004 (C) Intalio Inc. All Rights Reserved.
  *
- * $Id: SchemaUnmarshaller.java,v 1.2 2003/03/03 17:26:36 blandin Exp $
+ * $Id: SchemaUnmarshaller.java,v 1.8 2004/10/01 13:25:24 snyder Exp $
  */
 
 package org.exolab.castor.xml.schema.reader;
 
 //-- imported classes and packages
-import java.util.Vector;
 import org.exolab.castor.xml.*;
 import org.exolab.castor.xml.util.AttributeSetImpl;
 import org.exolab.castor.xml.schema.*;
@@ -54,10 +53,15 @@ import org.exolab.castor.net.URIResolver;
 import org.exolab.castor.net.util.URIResolverImpl;
 
 import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.Properties;
+import java.util.StringTokenizer;
+
+import java.io.InputStream;
 
 /**
  * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.2 $ $Date: 2003/03/03 17:26:36 $
+ * @version $Revision: 1.8 $ $Date: 2004/10/01 13:25:24 $
 **/
 public class SchemaUnmarshaller extends ComponentReader {
 
@@ -81,7 +85,7 @@ public class SchemaUnmarshaller extends ComponentReader {
     private static final String XML_PREFIX   = "xml";
     
     /**
-     * is this a included schema?
+     * is this an included schema?
      */
     private boolean _include = false;
     /**
@@ -117,7 +121,10 @@ public class SchemaUnmarshaller extends ComponentReader {
      * The SchemaUnmarsahller state
      */
     private SchemaUnmarshallerState _state = null;
-
+    
+   
+    private RemappedPrefixes _prefixMappings = null;
+    
       //----------------/
      //- Constructors -/
     //----------------/
@@ -160,7 +167,7 @@ public class SchemaUnmarshaller extends ComponentReader {
         _schema = new Schema();
         //--initialize the schema to ensure that the default namespace
         //--is not set
-        _schema.getNamespaces().remove("");
+        _schema.removeNamespace("");
         setResolver(resolver);
         if (uriResolver == null)
             uriResolver = new URIResolverImpl();
@@ -212,10 +219,12 @@ public class SchemaUnmarshaller extends ComponentReader {
         if (nsURI != null &&  nsURI.length() == 0)
            throw new SchemaException("empty string is not a legal namespace.");
         if ((nsURI != null) && (nsURI.length() > 0)) {
-            //if we are including a schema we must take care
-            //that the namespaces are the same
-            if ( (_include) &&(!_schema.getTargetNamespace().equals(nsURI)) ) {
-               throw new SchemaException("The target namespace of the included components must be the same as the target namespace of the including schema");
+        	if (!_state.cacheIncludedSchemas) {
+	        	//if we are including a schema we must take care
+	            //that the namespaces are the same
+	            if ( (_include) &&(!_schema.getTargetNamespace().equals(nsURI)) ) {
+	               throw new SchemaException("The target namespace of the included components must be the same as the target namespace of the including schema");
+	            }
             }
                _schema.setTargetNamespace(nsURI);
         }
@@ -224,7 +233,9 @@ public class SchemaUnmarshaller extends ComponentReader {
         _schema.setVersion(atts.getValue(SchemaNames.VERSION_ATTR));
 
         //set the default locator of this schema
-        _schema.setSchemaLocation(getDocumentLocator().getSystemId());
+        if (!_include || _state.cacheIncludedSchemas) {
+            _schema.setSchemaLocation(getDocumentLocator().getSystemId());
+        }
 
         //-- attributeFormDefault
         String form = atts.getValue(SchemaNames.ATTR_FORM_DEFAULT_ATTR);
@@ -267,11 +278,11 @@ public class SchemaUnmarshaller extends ComponentReader {
 
         if (namespaces == null) return;
 
-        Enumeration enum = namespaces.getLocalNamespaces();
+        Enumeration enumeration = namespaces.getLocalNamespaces();
 
-        while (enum.hasMoreElements()) {
+        while (enumeration.hasMoreElements()) {
 
-            String ns = (String) enum.nextElement();
+            String ns = (String) enumeration.nextElement();
             String[] prefixes = namespaces.getNamespacePrefixes(ns, true);
 
             if (prefixes.length == 0) {
@@ -282,8 +293,55 @@ public class SchemaUnmarshaller extends ComponentReader {
                 throw new XMLException(error);
             }
 
+            boolean hasCollisions = false;
             for (int pIdx = 0; pIdx < prefixes.length; pIdx++) {
                 String prefix = prefixes[pIdx];
+                
+                //-- Since the Schema Object Model does not yet support
+                //-- namespace scoping, we need to checking for namespace
+                //-- prefix collisions...and remap the prefixes
+                String tmpURI = _schema.getNamespace(prefix);
+                if ((tmpURI != null) && (foundSchemaDef)) {
+                    if (!tmpURI.equals(ns)) {
+                        if (!hasCollisions) {
+                            hasCollisions = true;
+                            if (_prefixMappings == null)
+                                _prefixMappings = new RemappedPrefixes();
+                            else
+                                _prefixMappings = _prefixMappings.newRemappedPrefixes();
+                        }
+                        
+                        //-- create a new prefix
+                        if (prefix.length() == 0) 
+                            prefix = "ns";
+                            
+                        int count = 1;
+                        String newPrefix = prefix + count;
+                        tmpURI = _schema.getNamespace(newPrefix);
+                        while (tmpURI != null) {
+                            if (tmpURI.equals(ns)) {
+                                //-- no remapping necessary
+                                break;
+                            }
+                            ++count;
+                            newPrefix = prefix + count;
+                            tmpURI = _schema.getNamespace(newPrefix);
+                        }
+                        _prefixMappings.addMapping(prefix, newPrefix);
+                        prefix = newPrefix;
+                    }
+                    //-- we may need to "reset" a currently mapped prefix
+                    else {
+                        if (_prefixMappings != null) {
+                            if (_prefixMappings.isRemappedPrefix(prefix)) {
+                                //-- reset mapping in this scope
+                                _prefixMappings.addMapping(prefix, prefix);
+                            }
+                        }
+                    }
+                }
+                //-- end collision handling
+                
                 if (prefix.length() == 0) {
                     defaultNS = ns;
                     //register the default namespace with the empty string
@@ -304,6 +362,32 @@ public class SchemaUnmarshaller extends ComponentReader {
 
     } //-- handleNamespaces
 
+    /** 
+     * Remaps any QName attributes for the given element and attributeSet.
+     * This method is a work around for the lack of namespace scoping 
+     * support in the Schema Object Model
+     */
+    private void handleRemapping(String name, String namespace, AttributeSetImpl atts) {
+        
+        if (_prefixMappings == null) return;
+        
+        //-- increase depth for scoping
+        _prefixMappings.depth++;
+        
+        String[] remapAtts = (String[]) RemappedPrefixes.QNAME_TABLE.get(name);
+        
+        if (remapAtts != null) {
+            for (int i = 0; i < remapAtts.length; i++) {
+                String value = atts.getValue(remapAtts[i]);
+                if (value != null) {
+                    value = _prefixMappings.remapQName(value);
+                    atts.setAttribute(remapAtts[i], value);
+                }
+            }
+        }
+        
+    } //-- handleRemapping
+    
 
     public void setResolver(Resolver resolver) {
         if (resolver == null) resolver = new ScopableResolver();
@@ -331,6 +415,10 @@ public class SchemaUnmarshaller extends ComponentReader {
     {
 
         if (skipAll) return;
+        
+        //-- DEBUG
+        //System.out.println("#startElement: " + name + " {" + namespace + "}");
+        //-- /DEBUG
 
 
         //-- process namespaces...unless we are inside an 
@@ -366,9 +454,21 @@ public class SchemaUnmarshaller extends ComponentReader {
             }
         }
 
+        //-- handle namespace prefix remapping
+        if (_annotationDepth == 0) { 
+            if (_prefixMappings != null) {
+                handleRemapping(name, namespace, (AttributeSetImpl)atts);
+            }
+        }
+        
         //-- Do delagation if necessary
         if (unmarshaller != null) {
-            unmarshaller.startElement(name, namespace, atts, nsDecls);
+            try {
+                unmarshaller.startElement(name, namespace, atts, nsDecls);
+            }
+            catch(RuntimeException rtx) {
+                error(rtx);
+            }
             ++depth;
             return;
         }
@@ -423,6 +523,11 @@ public class SchemaUnmarshaller extends ComponentReader {
             unmarshaller
                 = new ImportUnmarshaller(_schema, atts, _resolver, getURIResolver(), getDocumentLocator(), _state);
         }
+        //-- <redefine>
+        else if (name.equals(SchemaNames.REDEFINE)) {
+        	unmarshaller
+			= new RedefineUnmarshaller(_schema, atts, _resolver, getURIResolver(), getDocumentLocator(), _state);
+        }
         else {
             //-- we should throw a new Exception here
             //-- but since we don't support everything
@@ -450,6 +555,10 @@ public class SchemaUnmarshaller extends ComponentReader {
     {
         if (skipAll) return;
 
+        //-- DEBUG
+        //System.out.println("#endElement: " + name + " {" + namespace + "}");
+        //-- /DEBUG
+
         //-- backward compatibility
         if (namespace == null) namespace =  defaultNS;
 
@@ -457,6 +566,14 @@ public class SchemaUnmarshaller extends ComponentReader {
         if (name.equals(SchemaNames.ANNOTATION)) {
             --_annotationDepth;
         }
+        
+        //-- remove namespace remapping, if necessary
+        if (_prefixMappings != null) {
+            if (_prefixMappings.depth == 0) {
+                _prefixMappings = _prefixMappings.getParent();
+            }
+            else --_prefixMappings.depth;
+        }
 
         //-- Do delagation if necessary
         if ((unmarshaller != null) && (depth > 0)) {
@@ -538,6 +655,16 @@ public class SchemaUnmarshaller extends ComponentReader {
             group = (ModelGroup) (((ModelGroupUnmarshaller)unmarshaller).getGroup());
             _schema.addModelGroup(group);
         }
+        //--<redefine>
+        else if (name.equals(SchemaNames.REDEFINE)) {
+        	RedefineSchema redefine = null;
+        	redefine = (RedefineSchema) (((RedefineUnmarshaller)unmarshaller).getObject());
+        	if ((redefine.getSchemaLocation() == null) && (redefine.hasRedefinition()) ) {
+        		_schema.removeRedefineSchema(redefine);
+        		String err = "A <redefine> structure with no 'schemaLocation' attribute must contain only <annotation> elements";
+        		error(err);
+        	}
+        }
 
         unmarshaller = null;
     } //-- endElement
@@ -551,6 +678,167 @@ public class SchemaUnmarshaller extends ComponentReader {
         }
     } //-- characters
 
+    
+    /**
+     * This class handles remapping of namespace prefixes
+     * for attributes of type QName. This is needed to 
+     * work around a limitation in Castor's Schema Object 
+     * Model, which does not support proper namespace
+     * scoping yet.
+     */
+    static class RemappedPrefixes {
+        
+        public static final String RESOURCE_NAME 
+            = "prefixremap.properties";
+        
+        public static final String RESOURCE_LOCATION =
+            "/org/exolab/castor/xml/schema/reader/";
+        
+        public static final HashMap QNAME_TABLE = new HashMap();
+        private static boolean initialized = false;
+        
+        static {
+            
+            synchronized(QNAME_TABLE) {
+                
+                if (!initialized) {
+                    
+                    initialized = true;
+                    
+                    //-- built in mappings
+                    
+                    //-- attribute                    
+                    QNAME_TABLE.put(SchemaNames.ATTRIBUTE, new String [] {
+                        SchemaNames.REF_ATTR, SchemaNames.TYPE_ATTR } );
+                        
+                    //-- attributeGroup
+                    QNAME_TABLE.put(SchemaNames.ATTRIBUTE_GROUP, new String [] {
+                        SchemaNames.REF_ATTR } );
+                        
+                    //-- element
+                    QNAME_TABLE.put(SchemaNames.ELEMENT, new String [] {
+                        SchemaNames.REF_ATTR, SchemaNames.TYPE_ATTR } );
+                        
+                    //-- extension
+                    QNAME_TABLE.put(SchemaNames.EXTENSION, new String [] {
+                        SchemaNames.BASE_ATTR } );
 
+                    //-- group
+                    QNAME_TABLE.put(SchemaNames.GROUP, new String [] {
+                        SchemaNames.REF_ATTR } );
+                        
+                    //-- restriction
+                    QNAME_TABLE.put(SchemaNames.RESTRICTION, new String [] {
+                        SchemaNames.BASE_ATTR } );
+                        
+                    
+                    //-- custom mappings
+                    String filename = RESOURCE_LOCATION + RESOURCE_NAME;
+                    InputStream is = SchemaUnmarshaller.class.getResourceAsStream(filename);
+                    Properties props = new Properties();
+                    if (is != null) {
+                        try {
+                            props.load(is);
+                        }
+                        catch(java.io.IOException iox) {
+                            //-- just use built-in mappings
+                        }
+                    }
+                    
+                                        
+                    Enumeration keys = props.propertyNames();
+                    while (keys.hasMoreElements()) {
+                        String name =  (String) keys.nextElement();
+                        StringTokenizer st = new StringTokenizer(props.getProperty(name), ",");
+                        String[] atts = new String[st.countTokens()];
+                        int index = 0;
+                        while (st.hasMoreTokens()) {
+                            atts[index++] = st.nextToken();
+                        }
+                        QNAME_TABLE.put(name, atts);
+                    }
+                    
+                }
+            }
+        }
+        
+        private HashMap _prefixes = null;
+        
+        private RemappedPrefixes _parent = null;
+        
+        int depth = 0;
+        
+        
+        
+        public boolean isRemappedPrefix(String prefix) {
+            
+            if (prefix == null) prefix = "";
+            
+            if (_prefixes != null) {
+                if (_prefixes.get(prefix) != null) return true;
+            }
+            
+            if (_parent != null) {
+                return _parent.isRemappedPrefix(prefix);
+            }
+            return false;
+        }
+        
+        public RemappedPrefixes getParent() {
+            return _parent;
+        }
+        
+        public String getPrefixMapping(String oldPrefix) {
+            
+            if (_prefixes != null) {
+                String newPrefix = (String)_prefixes.get(oldPrefix);
+                if (newPrefix != null)
+                    return newPrefix;
+            }
+            
+            if (_parent != null) {
+                return _parent.getPrefixMapping(oldPrefix);
+            }
+            
+            return oldPrefix;
+        }
+        
+        public RemappedPrefixes newRemappedPrefixes() {
+            RemappedPrefixes rp = new RemappedPrefixes();
+            rp._parent = this;
+            return rp;
+        }
+        
+        public void addMapping(String oldPrefix, String newPrefix) {
+            if (_prefixes == null) {
+                _prefixes = new HashMap();
+            }
+            _prefixes.put(oldPrefix, newPrefix);
+        }
+
+        public String remapQName(String value) {
+            if (value == null) return null;
+            
+            //-- non-default namespace
+            int idx = value.indexOf(':');
+            String prefix = "";
+            if (idx >= 0) {
+                prefix = value.substring(0, idx);
+            }
+            else idx = -1;            
+            String newPrefix = getPrefixMapping(prefix);
+            if (!prefix.equals(newPrefix)) {
+                if (newPrefix.length() == 0) 
+                    value = value.substring(idx+1);
+                else
+                    value = newPrefix + ":" + value.substring(idx+1);
+            }
+            
+            return value;
+        } //-- remapValue
+        
+    }
+    
+    
 } //-- SchemaUnmarshaller
 
