@@ -38,9 +38,9 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Copyright 2001 (C) Intalio, Inc. All Rights Reserved.
+ * Copyright 2001-2003 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: SAX2ANY.java,v 1.1.1.1 2003/03/03 07:09:48 kvisco Exp $
+ * $Id: SAX2ANY.java,v 1.5 2005/05/09 15:47:49 afawcett Exp $
  * Date         Author              Changes
  * 04/06/2001   Arnaud Blandin      Created
  */
@@ -64,7 +64,7 @@ import org.exolab.castor.xml.Namespaces;
  * This class is a SAX Content Handler that
  * build an AnyNode from a stream of SAX events (either SAX1 for compatibility or SAX2)
  * @author <a href="blandin@intalio.com>Arnaud Blandin</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:09:48 $
+ * @version $Revision: 1.5 $ $Date: 2005/05/09 15:47:49 $
  */
 public class SAX2ANY implements ContentHandler, DocumentHandler, ErrorHandler
 {
@@ -99,6 +99,14 @@ public class SAX2ANY implements ContentHandler, DocumentHandler, ErrorHandler
      * A stack to store the namespaces declaration
      */
     private Stack _namespaces = new Stack();
+    
+    /**
+     * A flag indicating if the SAX2 Parser is processing the 
+     * namespace or not. 'true' will indicate that the code of this
+     * Content Handler will have to deal with Namespaces.This is the default 
+     * value.
+     */
+    private boolean _processNamespace = true;
 
     /**
      * The locator
@@ -167,11 +175,16 @@ public class SAX2ANY implements ContentHandler, DocumentHandler, ErrorHandler
     //--Namespace related (SAX2 Events)
     public void startPrefixMapping(String prefix, String uri) throws SAXException {
         AnyNode temp = new AnyNode(AnyNode.NAMESPACE, null, prefix, uri, null);
-        _namespaces.push(temp);
+       _namespaces.push(temp);
+       if (_processNamespace) { 
+           _context = _context.createNamespaces();
+           _processNamespace = true;
+       }
+       _context.addNamespace(prefix, uri);
     }
 
     public void endPrefixMapping(String prefix) throws SAXException {
-        //nothing to do?
+        _context.removeNamespace(prefix);
     }
 
     //--startElement methods SAX1 and SAX2
@@ -244,24 +257,77 @@ public class SAX2ANY implements ContentHandler, DocumentHandler, ErrorHandler
                             String qName, Attributes atts)
           throws SAXException
     {
-        createNodeElement(namespaceURI, localName, qName);
-        //empty the namespaces stack
-        //that must have been filled by startPrefixMapping()
-        AnyNode tempNode = null;
-        while (!_namespaces.empty()) {
-           tempNode = (AnyNode)_namespaces.pop();
-           _node.addNamespace(tempNode);
+        AnyNode tempNode;            	
+    	
+        //--SAX2 Parser has not processed the namespaces so we need to do it.
+        if (_processNamespace)
+        {
+            //Namespace handling code to be moved once we integrate
+            //the new event API
+            /////////////////NAMESPACE HANDLING/////////////////////
+            _context = _context.createNamespaces();
+            String prefix = "";
+            int idx = qName.indexOf(':');
+            if (idx >= 0) {
+                 prefix = qName.substring(0,idx);
+            }
+            namespaceURI = _context.getNamespaceURI(prefix);
+            //--Overhead here since we process attributes twice
+            for (int i=0; i<atts.getLength(); ++i) {
+                String attrqName = atts.getQName(i);
+                String value = atts.getValue(i);
+                String nsPrefix = null;
+                //handles namespace declaration
+                if (attrqName.startsWith(XMLNS_PREFIX)) {
+                    // Extract the prefix if any
+                    nsPrefix = (attrqName.equals(XMLNS_PREFIX))?null:attrqName.substring(XMLNS_PREFIX_LENGTH);
+                    tempNode = new AnyNode(AnyNode.NAMESPACE, getLocalPart(attrqName), nsPrefix, value, null);
+                    _context.addNamespace(nsPrefix, value);
+                    _namespaces.push(tempNode);
+                    if (prefix.equals(nsPrefix))
+                        namespaceURI = value;
+                }
+            }
+            ////////////////////////END OF NAMESPACE HANDLING///////////////        	
         }
+        
+        //create element
+        createNodeElement(namespaceURI, localName, qName);
+        
         //process attributes
         for (int i=0; i<atts.getLength(); ++i) {
 
             String uri       = atts.getURI(i);
-            String qname     = atts.getQName(i);
-            String localname = atts.getLocalName(i);
+            String attqName  = atts.getQName(i);
             String value     = atts.getValue(i);
-            tempNode = new AnyNode(AnyNode.ATTRIBUTE, localname, null, uri, value);
+            String prefix    = null;
+            
+            //-- skip namespace declarations? (handled above) 
+            if (_processNamespace )
+            	if(attqName.startsWith(XMLNS_PREFIX))
+            		continue;
+            
+            //--attribute namespace prefix?
+            if ((attqName.length() != 0) && (attqName.indexOf(':') != -1 ))
+                prefix = attqName.substring(0,attqName.indexOf(':'));
+                
+            //--namespace not yet processed?
+            if (_processNamespace ) {
+                // attribute namespace
+            	if(prefix!=null)
+            		uri = _context.getNamespaceURI(prefix);
+            }
+            //--add attribute
+            tempNode = new AnyNode(AnyNode.ATTRIBUTE, getLocalPart(attqName), prefix, uri, value);
             _node.addAttribute(tempNode);
         }
+        
+        //--empty the namespace stack and add 
+        //--the namespace nodes to the current node.
+        while (!_namespaces.empty()) {
+            tempNode = (AnyNode)_namespaces.pop();
+            _node.addNamespace(tempNode);
+        }
         tempNode = null;
     }
 
@@ -270,14 +336,9 @@ public class SAX2ANY implements ContentHandler, DocumentHandler, ErrorHandler
     public void endElement(String name)
            throws SAXException
     {
-
-        String prefix = "";
-        String namespaceURI = null;
-        int idx = name.indexOf(':');
-        if (idx >= 0) {
-             prefix = name.substring(0,idx);
-        }
-        namespaceURI = _context.getNamespaceURI(prefix);
+		int idx = name.indexOf(':');
+		String prefix = (idx >= 0) ? name.substring(0,idx) : "";
+		String namespaceURI = _context.getNamespaceURI(prefix);
         endElement(namespaceURI,getLocalPart(name), name);
         _context = _context.getParent();
     }
@@ -286,8 +347,16 @@ public class SAX2ANY implements ContentHandler, DocumentHandler, ErrorHandler
            throws SAXException
     {
         _character = false;
+        String name = null;
+        //-- if namespace processing is disabled then the localName might be null, in that case
+        //-- we use the QName
+        if (localName != null && localName.length() > 0)
+            name = localName;
+        else
+             name = qName;
+        
         //--if it is the starting element just returns
-        if (_startingNode.getLocalName().equals(localName) && _nodeStack.empty())
+        if (_startingNode.getLocalName().equals(name) && _nodeStack.empty())
            return;
 
         //--else just add the node we have built to the previous node
@@ -418,19 +487,32 @@ public class SAX2ANY implements ContentHandler, DocumentHandler, ErrorHandler
     private void createNodeElement(String namespaceURI, String localName,
                                    String qName)
    {
-
+    	
         String prefix = null;
-        //retrieves the prefix
-        if ((qName.length() != 0) && (qName.indexOf(':') != -1 ))
-            prefix = qName.substring(0,qName.indexOf(':'));
-
+        //retrieves the prefix if any
+        if (namespaceURI != null) {
+            prefix = _context.getNamespacePrefix(namespaceURI);
+        }            
+        else if (qName != null) {
+            if ((qName.length() != 0) && (qName.indexOf(':') != -1 ))
+                prefix = qName.substring(0,qName.indexOf(':'));
+        } 
+            
+        String name = null;
+        //-- if namespace processing is disabled then the localName might be null, in that case
+        //-- we use the localpart of the QName
+        if (localName != null && localName.length() > 0)
+            name = localName;
+        else
+             name = getLocalPart(qName);
+             
         //creates the starting ELEMENT node
         //or a default ELEMENT node
         if ( (_nodeStack.empty()) && (_startingNode == null)) {
-           _startingNode = new AnyNode(AnyNode.ELEMENT, localName, prefix, namespaceURI, null);
+           _startingNode = new AnyNode(AnyNode.ELEMENT, name, prefix, namespaceURI, null);
            _node = _startingNode;
         } else {
-          _node = new AnyNode(AnyNode.ELEMENT, localName, prefix, namespaceURI, null);
+          _node = new AnyNode(AnyNode.ELEMENT, name, prefix, namespaceURI, null);
           //push the node in the stack
           _nodeStack.push(_node);
         }
