@@ -38,9 +38,14 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
+ * Copyright 1999-2004 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: XMLClassDescriptorImpl.java,v 1.1.1.1 2003/03/03 07:09:49 kvisco Exp $
+ * This file was originally developed by Keith Visco during the
+ * course of employment at Intalio Inc.
+ * All portions of this file developed by Keith Visco after Jan 19 2005 are
+ * Copyright (C) 2005 Keith Visco. All Rights Reserved.
+ *
+ * $Id: XMLClassDescriptorImpl.java,v 1.9 2005/03/01 06:20:16 kvisco Exp $
  */
 
 
@@ -49,11 +54,12 @@ package org.exolab.castor.xml.util;
 import java.lang.reflect.Array;
 import java.util.Collection;
 
-import org.exolab.castor.mapping.MappingException;
+
+import org.exolab.castor.mapping.AbstractFieldHandler;
+import org.exolab.castor.mapping.AccessMode;
 import org.exolab.castor.mapping.ClassDescriptor;
 import org.exolab.castor.mapping.FieldDescriptor;
 import org.exolab.castor.mapping.FieldHandler;
-import org.exolab.castor.mapping.AccessMode;
 import org.exolab.castor.xml.*;
 
 
@@ -62,8 +68,8 @@ import org.exolab.castor.xml.*;
  * class is used by both generated source code as well
  * as the XMLMappingLoader.
  *
- * @author <a href="kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:09:49 $
+ * @author <a href="keith AT kvisco DOT com">Keith Visco</a>
+ * @version $Revision: 1.9 $ $Date: 2005/03/01 06:20:16 $
  */
 public class XMLClassDescriptorImpl extends Validator
     implements XMLClassDescriptor
@@ -252,34 +258,8 @@ public class XMLClassDescriptorImpl extends Validator
      * @param descriptor the XMLFieldDescriptor to add
      */
     public void addFieldDescriptor(XMLFieldDescriptor descriptor) {
-
-        if (descriptor == null) return;
         
-	    boolean added = false;
-
-        NodeType nodeType = descriptor.getNodeType();
-        switch(nodeType.getType()) {
-            case NodeType.NAMESPACE:
-            case NodeType.ATTRIBUTE:
-                if ((added = _attributes.add(descriptor))) {
-                    _attArray = null;
-                }
-                break;
-            case NodeType.TEXT:
-                contentDescriptor = descriptor;
-                added = true;
-                break;
-            default:
-                if (added = _elements.add(descriptor)) {
-                    _elemArray = null;
-                    if (descriptor.isContainer()) ++_containerCount;
-                }
-                break;
-        }
-        
-        if (added) {
-	        descriptor.setContainingClassDescriptor( this );
-	    }
+        addFieldDescriptor(descriptor, true);
 
     } //-- addFieldDescriptor
 
@@ -353,7 +333,7 @@ public class XMLClassDescriptorImpl extends Validator
      *
      */
     public XMLFieldDescriptor getFieldDescriptor
-        (String name, NodeType nodeType)
+        (String name, String namespace, NodeType nodeType)
     {
         boolean wild = ((nodeType == null) || _introspected);
         XMLFieldDescriptor result = null;
@@ -361,6 +341,16 @@ public class XMLClassDescriptorImpl extends Validator
         XMLFieldDescriptor[] attributes = _attArray;
         XMLFieldDescriptor[] elements   = _elemArray;
         
+        // TODO: clean up location patch
+        String location = null;
+        if (name != null) {
+            int idx = name.lastIndexOf('/');
+            if (idx >= 0) {
+                location = name.substring(0, idx);
+                name = name.substring(idx+1);
+            }
+        }
+        
         if (wild || (nodeType == NodeType.Element)) {
             
             if (elements == null) elements = getElementArray();
@@ -368,20 +358,30 @@ public class XMLClassDescriptorImpl extends Validator
             for (int i = 0; i < elements.length; i++) {
                 XMLFieldDescriptor desc = elements[i];
                 if (desc == null) continue;
+                
+                if (location != null) {
+                    if (!location.equals(desc.getLocationPath())) 
+                        continue;
+                }
 
                 if (desc.matches(name)) {
                       if (!desc.matches(WILDCARD)) return desc;
+                      //--  possible wildcard match check for
+                      //--  exact match (we need to extend the
+                      //--  the descriptor interface to handle this
+                      if (name.equals(desc.getXMLName())) 
+                        return desc;
+                      //-- assume wild-card match
                       result = desc;
-                      break;
                 }
 
                 //handle container
-                if ( (result == null) && desc.isContainer() ) {
+                if ( desc.isContainer() ) {
                     XMLClassDescriptor xcd = (XMLClassDescriptor)desc.getClassDescriptor();
                     //prevent endless loop
                     if (xcd != this) {
                         //is it in this class descriptor?
-                        if (xcd.getFieldDescriptor(name, NodeType.Element) != null) {
+                        if (xcd.getFieldDescriptor(name, namespace, NodeType.Element) != null) {
                             result = desc;
                             break;
                         }
@@ -428,7 +428,7 @@ public class XMLClassDescriptorImpl extends Validator
                     //prevent endless loop
                     if (xcd != this) {
                         //is it in this class descriptor?
-                        XMLFieldDescriptor temp = xcd.getFieldDescriptor(name, NodeType.Attribute);
+                        XMLFieldDescriptor temp = xcd.getFieldDescriptor(name, namespace, NodeType.Attribute);
                         if (temp != null) {
                             return desc;
                         }
@@ -578,7 +578,7 @@ public class XMLClassDescriptorImpl extends Validator
         if (_extends != null) {
             fields = classDesc.getFields();
             for (int i = 0; i < fields.length; i++) {
-                addFieldDescriptor((XMLFieldDescriptor)fields[i]);
+                addFieldDescriptor((XMLFieldDescriptor)fields[i], false);
             }
         }
 
@@ -743,11 +743,15 @@ public class XMLClassDescriptorImpl extends Validator
                 " described by this ClassDecriptor.";
             throw new ValidationException(err);
         }
+        
+        //-- DEBUG
+        //System.out.println("Validating class: " + object.getClass().getName());
+        //-- /DEBUG
 
         XMLFieldDescriptor[] localElements = getElementArray();
         
         if (_extends != null) {
-
+            
             //-- cascade call for validation
             if (_extends instanceof XMLClassDescriptorImpl) {
                 ((XMLClassDescriptorImpl)_extends).validate(object, context);
@@ -790,6 +794,8 @@ public class XMLClassDescriptorImpl extends Validator
                 //-- handle elements, affected by choice
                 for (int i = 0; i < localElements.length; i++) {
                     XMLFieldDescriptor desc = localElements[i];
+                    
+                    if (desc == null) continue;
 
                     FieldHandler handler = desc.getHandler();
 
@@ -840,7 +846,9 @@ public class XMLClassDescriptorImpl extends Validator
                     buffer.append('(');
                     String sep = " | ";
                     for (int i = 0; i < localElements.length; i++) {
-                        XMLFieldDescriptor  desc = localElements[i];
+                        XMLFieldDescriptor  desc = localElements[i];                        
+                        if (desc == null) continue;
+                        
                         FieldValidator fieldValidator = desc.getValidator();
                         if (fieldValidator.getMinOccurs() > 0) {
                             if (error) {
@@ -878,8 +886,11 @@ public class XMLClassDescriptorImpl extends Validator
             case SEQUENCE:
             //-- ALL
             default:
+            
+            
                 //-- handle elements
-                for (int i = 0; i < localElements.length; i++) {
+                for (int i = 0; i < localElements.length; i++) {                                        
+                    if (localElements[i] == null) continue;
                     FieldValidator fieldValidator = localElements[i].getValidator();
                     if (fieldValidator != null)
                         fieldValidator.validate(object, context);
@@ -968,36 +979,20 @@ public class XMLClassDescriptorImpl extends Validator
         return _accessMode;
     } //-- getAccessMode
 
-    /**
-     * <p>Returns true if the given object represented by this XMLClassDescriptor
-     * can accept a member whose name is given.
-     * An XMLClassDescriptor can accept a field if it contains a descriptor that matches
-     * the given name anf if the given object can hold this field (i.e a value is not already set for
-     * this field).
-     * The different criterias for accepting a field with a given name are:
-     * <ul>
-     *    <li>an <tt>XMLFieldDescriptor</tt> of this XMLClassDescriptorImpl that matches
-     *    the given field name exists.</li>
-     *    <li>A value has not already been set for the field.</li>
-     *    <li>If the XMLClassDescriptorImpl represents a CHOICE then no other value must have
-     *    been set.</li>
-     * </ul>
-     *
-     * @param fieldName the name of the field to check
-     * @param object the object represented by this XMLCLassDescriptor
-     * @return true if the given object represented by this XMLClassDescriptor
-     * can accept a member whose name is given.
+    /*
+     *  (non-Javadoc)
+     * @see org.exolab.castor.xml.XMLClassDescriptor#canAccept(java.lang.String, java.lang.String, java.lang.Object)
      */
-    public boolean canAccept(String fieldName, Object object) {
+    public boolean canAccept(String name, String namespace, Object object) {
 
         boolean result = false;
         boolean hasValue = false;
         XMLFieldDescriptor[] fields = null;
         int i = 0;
         //1--direct look up for a field
-        XMLFieldDescriptor fieldDesc = this.getFieldDescriptor(fieldName, NodeType.Element);
+        XMLFieldDescriptor fieldDesc = this.getFieldDescriptor(name, namespace, NodeType.Element);
         if (fieldDesc == null)
-           fieldDesc = this.getFieldDescriptor(fieldName, NodeType.Attribute);
+           fieldDesc = this.getFieldDescriptor(name, namespace, NodeType.Attribute);
 
         //if the descriptor is still null, the field can't be in stored in this classDescriptor
         if (fieldDesc == null)
@@ -1032,18 +1027,40 @@ public class XMLClassDescriptorImpl extends Validator
             if (tempObject == null)
                 result = true;
             else
-                result = ((XMLClassDescriptor)fieldDesc.getClassDescriptor()).canAccept(fieldName, tempObject);
+                result = ((XMLClassDescriptor)fieldDesc.getClassDescriptor()).canAccept(name, namespace, tempObject);
         }
         //4-- Check if the value is set or not
         else {
             FieldHandler handler = fieldDesc.getHandler();
-            hasValue = (handler.getValue(object) != null);
+            
+            boolean checkPrimitiveValue = true;
+            if (handler instanceof AbstractFieldHandler) {
+                hasValue = ((AbstractFieldHandler)handler).hasValue( object );
+                //-- poor man's check for a generated handler, since
+                //-- all generated handlers extend XMLFieldHandler, however
+                //-- this doesn't guarantee that that handler is indeed
+                //-- a generated handler, it does however mean that
+                //-- the handler definately didn't come from the
+                //-- MappingLoader. 
+                //checkPrimitiveValue = (!(handler instanceof XMLFieldHandler));
+            }
+            else {
+                hasValue = (handler.getValue(object) != null);
+            }
             
             //-- patch for primitives, we should check
             //-- for the has-method somehow...but this
-            //-- is good enough for now.
-            if (hasValue && isPrimitive(fieldDesc.getFieldType())) {
-                hasValue = false;
+            //-- is good enough for now. This may break
+            //-- some non-Castor-generated code with
+            //-- primitive values that have been set
+            //-- with the same as the defaults
+            if (hasValue && 
+                checkPrimitiveValue && 
+                fieldDesc.getFieldType().isPrimitive());
+            {
+                if (isDefaultPrimitiveValue(handler.getValue( object ))) {
+                    hasValue = false;
+                }
             }
             //-- end patch
             result = !hasValue;
@@ -1082,6 +1099,7 @@ public class XMLClassDescriptorImpl extends Validator
     //- Protected Methods -/
     //---------------------/
 
+
     /**
      * Returns true if the given class should be treated as a primitive
      * type. This method will return true for all Java primitive
@@ -1105,6 +1123,37 @@ public class XMLClassDescriptorImpl extends Validator
         return (type.getSuperclass() == Number.class);
     } //-- isPrimitive
     
+    /**
+     * Checks to see if the given Object is a java primitive
+     * (does not check for primitive wrappers) and has a 
+     * value that is equal to the default value for that 
+     * primitive. This method will return true if the value
+     * is a java primitive with a default value.
+     * 
+     * @return true if the value is a java primitive with
+     * a default value
+     */
+    static boolean isDefaultPrimitiveValue(Object value) {
+        if (value == null) return false;
+        
+        Class type = value.getClass();
+        if (type.isPrimitive()) {
+            try {
+                return (value.equals(type.newInstance()));
+            }
+            catch(java.lang.IllegalAccessException iax) {
+                //-- Just return false, we should be
+                //-- able to instantiate primitive types
+            }
+            catch(java.lang.InstantiationException ix) {
+                //-- Just return false, we should be
+                //-- able to instantiate primitive types
+            }
+        }
+        return false;        
+    } //-- isDefaultPrimitiveValue
+    
+    
     /**
      * Sets the Class type being described by this descriptor.
      *
@@ -1137,6 +1186,49 @@ public class XMLClassDescriptorImpl extends Validator
         return _naming.toXMLName(name);
     }
     
+    //-------------------/
+    //- Private Methods -/
+    //-------------------/
+    
+    /**
+     * Adds the given XMLFieldDescriptor to the list of descriptors. The
+     * descriptor will be added to the appropriate list by calling
+     * XMLFieldDescriptor#getNodeType() to determine it's type.
+     *
+     * @param descriptor the XMLFieldDescriptor to add
+     */
+    private void addFieldDescriptor(XMLFieldDescriptor descriptor, boolean relink) {
+
+        if (descriptor == null) return;
+        
+	    boolean added = false;
+
+        NodeType nodeType = descriptor.getNodeType();
+        switch(nodeType.getType()) {
+            case NodeType.NAMESPACE:
+            case NodeType.ATTRIBUTE:
+                if ((added = _attributes.add(descriptor))) {
+                    _attArray = null;
+                }
+                break;
+            case NodeType.TEXT:
+                contentDescriptor = descriptor;
+                added = true;
+                break;
+            default:
+                if (added = _elements.add(descriptor)) {
+                    _elemArray = null;
+                    if (descriptor.isContainer()) ++_containerCount;
+                }
+                break;
+        }
+        
+        if ((added) && (relink)) {
+	        descriptor.setContainingClassDescriptor( this );
+	    }
+
+    } //-- addFieldDescriptor
+    
     
     private XMLFieldDescriptor[] getAttributeArray() {
         //-- create local reference to prevent possible
