@@ -38,31 +38,47 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Copyright 1999-2003 (C) Intalio, Inc. All Rights Reserved.
+ * Copyright 1999-2004 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: XMLFieldDescriptorImpl.java,v 1.5 2003/04/24 02:16:19 kvisco Exp $
+ * Portions of this file developed by Keith Visco after Jan 19 2005 are
+ * Copyright (C) 2005 Keith Visco. All Rights Reserverd.
+ *
+ * $Id: XMLFieldDescriptorImpl.java,v 1.12 2005/08/18 04:06:28 kvisco Exp $
  */
 
 
 package org.exolab.castor.xml.util;
 
-
+import org.exolab.castor.mapping.ClassDescriptor;
+import org.exolab.castor.mapping.CollectionHandler;
+import org.exolab.castor.mapping.FieldDescriptor;
+import org.exolab.castor.mapping.FieldHandler;
+import org.exolab.castor.mapping.GeneralizedFieldHandler;
+import org.exolab.castor.mapping.MappingException;
+import org.exolab.castor.mapping.loader.FieldHandlerImpl;
+import org.exolab.castor.xml.FieldValidator;
+import org.exolab.castor.xml.NodeType;
+import org.exolab.castor.xml.XMLClassDescriptor;
+import org.exolab.castor.xml.XMLFieldDescriptor;
+import org.exolab.castor.xml.descriptors.CoreDescriptors;
 import org.exolab.castor.xml.handlers.DateFieldHandler;
-import org.exolab.castor.mapping.*;
-import org.exolab.castor.xml.*;
-import org.exolab.castor.util.List;
+
+import java.util.ArrayList;
 import java.util.StringTokenizer;
+import java.util.Properties;
 
 /**
  * XML field descriptor. Wraps {@link FieldDescriptor} and adds
  * XML-related information, type conversion, etc.
  *
- * @author <a href="kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.5 $ $Date: 2003/04/24 02:16:19 $
+ * @author <a href="keith AT kvisco DOT com">Keith Visco</a>
+ * @version $Revision: 1.12 $ $Date: 2005/08/18 04:06:28 $
  */
 public class XMLFieldDescriptorImpl
     implements XMLFieldDescriptor
 {
+    
+    
 
 
     private static final String WILD_CARD = "*";
@@ -114,6 +130,12 @@ public class XMLFieldDescriptorImpl
      * The field handler for get/set field value.
      */
     private FieldHandler  _handler = null;
+    
+    /**
+     * Cached reference to _handler if the
+     * FieldHandler is a GeneralizedFieldHandler.
+     */
+    private GeneralizedFieldHandler  _gfhandler = null;
 
     /**
      * True if the field type is immutable.
@@ -147,6 +169,11 @@ public class XMLFieldDescriptorImpl
      */
     private boolean _multivalued = false;
 
+    /**
+     * True if the field is allowed to have nil content
+     */
+    private boolean _nillable = false;
+    
     /**
      * The node type (attribute, element, text).
      */
@@ -163,6 +190,11 @@ public class XMLFieldDescriptorImpl
     private String _nsURI = null;
 
 
+    /**
+     * The "user-set" properties of this XMLFieldDescriptor
+     */
+    private Properties _properties = null;
+    
     /**
      * indicates a required field when true
     **/
@@ -184,6 +216,12 @@ public class XMLFieldDescriptorImpl
      */
     private String _qNamePrefix = null;
 
+    /**
+     * A flag which indicates the parent class' namespace
+     * should be used by default
+     */
+    private boolean _useParentClassNamespace = false;
+    
     private FieldValidator _validator = null;
     
     /**
@@ -202,8 +240,7 @@ public class XMLFieldDescriptorImpl
      */
     private String _xmlPath    = null;
 
-
-
+    
     //----------------/
     //- Constructors -/
     //----------------/
@@ -294,8 +331,8 @@ public class XMLFieldDescriptorImpl
               ( (XMLFieldDescriptor)fieldDesc
                 ).getContainingClassDescriptor();
 
-        this._handler         = fieldDesc.getHandler();
-        this._fieldName       = fieldDesc.getFieldName();
+        setHandler(fieldDesc.getHandler());
+        _fieldName            = fieldDesc.getFieldName();
          //////////////Hack for AnyNode//////////////////
         //if the field type is an AnyNode Castor must treat it as
         //an object to avoid changes in the Marshalling framework
@@ -317,9 +354,16 @@ public class XMLFieldDescriptorImpl
         }
 
         //-- check for instances of java.util.Date
-        if (java.util.Date.class.isAssignableFrom(_fieldType)) {
-            if (!(_handler instanceof DateFieldHandler)) {
-                _handler = new DateFieldHandler(_handler);
+        /* Note to self:
+         * This logic really doesn't belong here, as it can
+         * interfere with user specified handlers. It should
+         * go into XMLMappingLoader. 
+         */
+        if (_fieldType != null) {
+            if (java.util.Date.class.isAssignableFrom(_fieldType)) {
+                if (_handler instanceof FieldHandlerImpl) {
+                    _handler = new DateFieldHandler(_handler);
+                }
             }
         }
 
@@ -375,10 +419,29 @@ public class XMLFieldDescriptorImpl
         else if (tmpHandler == null) {
             return false;
         }
+               
+        //-- the following line is causing some issues when
+        //-- used against a FieldHandlerImpl because the
+        //-- equals method for FieldHandlerImpl is the default
+        //-- replacing with a slightly more generic comparison
+        //-- but this should probably change in the future (kv)
+        //return (_handler.equals(tmpHandler));
+        return (_handler.getClass().isInstance(tmpHandler));
         
-        return (_handler.equals(tmpHandler));
     } //-- equals
 
+    /**
+     * Returns the hashCode for this XMLFieldDescriptor
+     */
+    public int hashCode() {
+        int hash = 17;
+        hash = 17 * _fieldName.hashCode();
+        hash = hash * 17 * _fieldType.hashCode();
+        if (_handler != null)
+            hash = hash * 17 * _handler.hashCode();
+        return hash;
+    }
+    
     /**
      * Set the class which contains this field
      */
@@ -447,6 +510,11 @@ public class XMLFieldDescriptorImpl
      * @return Field type
      */
     public Class getFieldType() {
+        if (_gfhandler != null) {
+            if (_gfhandler.getFieldType() != null) {
+            	return _gfhandler.getFieldType();
+            }
+        }
         return _fieldType;
     } //-- getFieldType
 
@@ -488,21 +556,18 @@ public class XMLFieldDescriptorImpl
     **/
     public String getNameSpaceURI() {
 
-        //-- use containing class's namespace if
-        //-- necessary
-        /*
-           KV removed 20030109 (causing wrong namespace to be used,
-           if the field really doesn't have one)
-           
-        if ((nsURI == null) && (this._contClsDescriptor != null)) {
-            if (isPrimitive(_fieldType) && (_nodeType == NodeType.Element))
+        if ((_nsURI == null) && 
+            (_contClsDescriptor != null) &&
+            _useParentClassNamespace)
+        {
+            if ((_nodeType == NodeType.Element) &&
+                (isPrimitive(_fieldType) || (isBuiltInType(_fieldType))))
             {
                 if (_contClsDescriptor instanceof XMLClassDescriptor) {
                     return ((XMLClassDescriptor)_contClsDescriptor).getNameSpaceURI();
                 }
             }
         }
-        */
         return _nsURI;
     } //-- getNameSpaceURI
 
@@ -558,6 +623,25 @@ public class XMLFieldDescriptorImpl
         return _xmlPath;
     } //-- getLocationPath
     
+    
+    /**     
+     * Returns the value property with the given name or null
+     * if no such property exists. This method is useful for
+     * future evolutions of this interface as well as for
+     * user-defined extensions. See class declared properties
+     * for built-in properties.
+     *
+     * @param propertyName the name of the property whose value
+     * should be returned.
+     *
+     * @return the value of the property, or null.
+     */
+    public String getProperty(String propertyName) {
+        if ((_properties == null) || (propertyName == null))
+            return null;
+        return _properties.getProperty(propertyName);
+    } //-- getProperty
+    
     /**
      * Returns the prefix used in case the value of the
      * field described by this descriptor is of type QName.
@@ -654,6 +738,13 @@ public class XMLFieldDescriptorImpl
         return _multivalued;
     } //-- isMultivalued
 
+    /*
+     *  (non-Javadoc)
+     * @see org.exolab.castor.xml.XMLFieldDescriptor#isNillable()
+     */
+    public boolean isNillable() {
+    	return _nillable;
+    }
     /**
      * Returns true if the field described by this descriptor is
      * a reference (ie. IDREF) to another object in the
@@ -737,7 +828,7 @@ public class XMLFieldDescriptorImpl
                 return false;
         }
         else if (_nsURI == null) {
-            if (namespace.length() > 0) {
+            if ((namespace.length() > 0) && (!_isWild)) {
                 return false;
             }
         }
@@ -800,7 +891,13 @@ public class XMLFieldDescriptorImpl
      * by this FieldDescriptor
     **/
     public void setHandler(FieldHandler handler) {
-        this._handler = handler;
+        _handler = handler;
+        if (handler instanceof GeneralizedFieldHandler) {
+        	_gfhandler = (GeneralizedFieldHandler)handler;
+        }
+        else {
+        	_gfhandler = null;
+        }
     } //-- setHandler
 
     /**
@@ -896,7 +993,7 @@ public class XMLFieldDescriptorImpl
         if ((matchExpr == null) || (matchExpr.length() == 0)) return;
 
         StringTokenizer st = new StringTokenizer(matchExpr);
-        List names = new List();
+        ArrayList names = new ArrayList();
         while (st.hasMoreTokens()) {
             String token = st.nextToken();
             if (WILD_CARD.equals(token)) {
@@ -913,6 +1010,48 @@ public class XMLFieldDescriptorImpl
     public void setMultivalued(boolean multivalued) {
         this._multivalued = multivalued;
     } //-- setMultivalued
+    
+    /**
+     * Sets whether or not the described field is allowed
+     * to be nil. A nillable field can have empty content
+     * (text or element content), but may have attribute
+     * values, and still be considered value, even if
+     * the child elements are required.
+     * 
+     * @param nillable a boolean indicating whether or not
+     * the described field may be nillable.
+     */
+    public void setNillable(boolean nillable) {
+    	_nillable = nillable;
+    } //-- setNillable
+    
+    /**     
+     * Sets the value property with the given name
+     *
+     * @param propertyName the name of the property to set
+     * the value of
+     * @param value the value of the property
+     * @see getProperty
+     */
+    public void setProperty(String propertyName, String value) {
+        
+        if (propertyName == null) {
+            String err = "The argument 'propertyName' must not be null.";
+            throw new IllegalArgumentException(err);
+        }
+        
+        if (_properties == null)
+            _properties = new Properties();
+         
+         
+        if (value == null) {
+            _properties.remove(propertyName);
+        }
+        else {
+            _properties.put(propertyName, value);
+        }
+    } //-- setProperty
+    
 
     /**
      * Sets the type of the XML Schema type of the value
@@ -990,6 +1129,15 @@ public class XMLFieldDescriptorImpl
         _transient = isTransient;
     } //-- isTransient
 
+    /**
+     * Sets whether or not the namespace for the parent "containing" 
+     * class should be used during marshalling/unmarshalling when
+     * no specific namespace URI has been set for this field.
+     */
+    public void setUseParentsNamespace(boolean useParentsNamespace) {
+        _useParentClassNamespace = useParentsNamespace;
+    }
+    
     public void setValidator(FieldValidator validator) {
 
         //-- remove reference from current FieldValidator
@@ -1059,8 +1207,27 @@ public class XMLFieldDescriptorImpl
         if ((type == Boolean.class) || (type == Character.class))
             return true;
 
+        //-- Any class which extends Number should be 
+        //-- treated as a primitive
         return (type.getSuperclass() == Number.class);
+        
     } //-- isPrimitive
+    
+    /**
+     * Return true if the given class is a "built-in" type. A built-in
+     * type is one in which Castor provides the default descriptor
+     * for.
+     * 
+     * @param type the class to check
+     * 
+     * @return true if the given class is a built-in type.
+     */
+    private boolean isBuiltInType(Class type) {
+        if (type == null) return false;
+        //-- All built-in Java types, such as java.util.Date,
+        //-- java.sql.Date, various Collection classes, etc.
+        return (CoreDescriptors.getDescriptor(type) != null);
+    } //-- isBuiltInType
 
 } //-- XMLFieldDescriptor
 
