@@ -40,23 +40,26 @@
  *
  * Copyright 1999-2002 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: JClass.java,v 1.1.1.1 2003/03/03 07:09:53 kvisco Exp $
+ * $Id: JClass.java,v 1.7 2005/05/08 18:32:06 afawcett Exp $
+ *
+ * This file was originally developed by Keith Visco during the
+ * course of employment at Intalio Inc.
+ * All portions of this file developed by Keith Visco after Jan 19 2005 are
+ * Copyright (C) 2005 Keith Visco. All Rights Reserverd.
  *
  * Contributors:
  * --------------
- * Keith Visco (kvisco@intalio.com) - Original Author
- * Martin Skopp (skopp@riege.de)    - Moved some core code into JStructure
- *                                    and revised to extend JStructure
+ * Keith Visco (keith AT kvisco DOT com) 
+ *    - Original Author
+ * 
+ * Martin Skopp (skopp AT riege DOT de)  
+ *    - Moved some core code into JStructure and revised to 
+ *      extend JStructure
  *
  */
 
 package org.exolab.javasource;
 
-import java.io.File;
-import java.io.FileWriter;
-import java.io.Writer;
-import java.io.PrintWriter;
-
 import java.util.Enumeration;
 import java.util.Vector;
 
@@ -66,9 +69,9 @@ import java.util.Vector;
  * This package was modelled after the Java Reflection API
  * as much as possible to reduce the learning curve.
  *
- * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @author <a href="mailto:skopp@riege.de">Martin Skopp</a> 
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:09:53 $
+ * @author <a href="mailto:keith AT kvisco DOT com">Keith Visco</a>
+ * @author <a href="mailto:skopp AT riege DOT de">Martin Skopp</a> 
+ * @version $Revision: 1.7 $ $Date: 2005/05/08 18:32:06 $
  */
 public class JClass extends JStructure {
 
@@ -76,26 +79,26 @@ public class JClass extends JStructure {
     /**
      * The list of constructors for this JClass
      */
-    private Vector _constructors  = null;
+    protected Vector _constructors  = null;
 
 
     /**
      * The list of member variables (fields) of this JClass
      */
-    private JNamedMap _fields    = null;
+    protected JNamedMap _fields    = null;
 
 
-    private Vector _innerClasses = null;
+    protected Vector _innerClasses = null;
     
     /**
      * The list of methods of this JClass
      */
-    private Vector _methods       = null;
+    protected Vector _methods       = null;
 
     /**
      * The superclass for this JClass
      */
-    private String _superClass = null;
+    protected JTypeName _superClass = null;
     
 
     
@@ -103,7 +106,7 @@ public class JClass extends JStructure {
     /**
      * The source code for static initialization
     **/
-    private JSourceCode _staticInitializer = new JSourceCode();
+    protected JSourceCode _staticInitializer = new JSourceCode();
     
     /**
      * Creates a new JClass with the given name
@@ -118,7 +121,7 @@ public class JClass extends JStructure {
         _constructors     = new Vector();
         _fields           = new JNamedMap();
         _methods          = new Vector();
-        _innerClasses   = new Vector();
+        _innerClasses     = null;
         //-- initialize default Java doc
         getJDocComment().appendComment("Class " + getLocalName() + ".");
         
@@ -150,6 +153,12 @@ public class JClass extends JStructure {
             err += "by this JClass";
             throw new IllegalArgumentException(err);
         }
+        
+        //-- ensure constructor and parameter annotation classes imported
+        addImport(constructor.getAnnotations());
+        JParameter[] params = constructor.getParameters();
+        for(int i=0; i<params.length; i++)
+        	addImport(params[i].getAnnotations());
     }
 
     /**
@@ -179,10 +188,44 @@ public class JClass extends JStructure {
         JType type = jField.getType();
         while (type.isArray()) type = type.getComponentType();
         if ( !type.isPrimitive() )
-            addImport( ((JClass)type).getName());
-
+            addImport( type.getName());
+            
+        // ensure annotation classes are imported
+        addImport(jField.getAnnotations());
     } //-- addField
 
+    /*
+     *  (non-Javadoc)
+     * @see org.exolab.javasource.JStructure#addImport(java.lang.String)
+     */
+    public void addImport(String name) {
+        
+        if ((name == null) || (name.length() == 0))
+            return;
+        
+        JTypeName jtName = new JTypeName(name);
+        
+        String cls = jtName.getLocalName();
+        String pkg = jtName.getPackageName();
+        
+        if (_superClass != null) {
+            if (_superClass.getLocalName().equals(cls)) {
+                if (_superClass.getPackageName() == null) {
+                    if (pkg != null) {
+                        // if we make it here we are extending
+                        // a class from the default package which
+                        // conflicts with this import, we cannot
+                        // import this class
+                        return;
+                    }
+                }
+            }
+        }
+        
+        super.addImport(name);
+        
+    } //-- adImport
+    
     /**
      * Adds the given JMember to this JClass
      *
@@ -217,9 +260,6 @@ public class JClass extends JStructure {
      * Adds the given JMethod to this JClass
      *
      * @param jMethod, the JMethod to add
-     * @param importReturnType true if we add the importReturnType to
-     * the class import lists. It could be useful to set it to false when 
-     * all types are fully qualified.
      * @exception IllegalArgumentException when the given
      * JMethod has the same name of an existing JMethod.
     **/
@@ -281,8 +321,8 @@ public class JClass extends JStructure {
 
         String[] pkgNames = jMethod.getParameterClassNames();
         for (int i = 0; i < pkgNames.length; i++) {
-            addImport(pkgNames[i]);
-        }
+            addImport(pkgNames[i]);            
+        }        
         //-- check return type to make sure it's included in the
         //-- import list
         JType jType = jMethod.getReturnType();
@@ -299,6 +339,12 @@ public class JClass extends JStructure {
         for (int i = 0; i < exceptions.length; i++) {
             addImport(exceptions[i].getName());
         }
+        //-- ensure method and parameter annotations imported
+        addImport(jMethod.getAnnotations());
+        JParameter[] params = jMethod.getParameters();
+        for (int i = 0; i < params.length; i++) {
+        	addImport(params[i].getAnnotations());
+		}        	
     } //-- addMethod
 
     /**
@@ -366,6 +412,9 @@ public class JClass extends JStructure {
         }
         
         JClass innerClass = new JInnerClass(classname);
+        if (_innerClasses == null) {
+            _innerClasses = new Vector();
+        }
         _innerClasses.addElement(innerClass);
         return innerClass;
         
@@ -429,10 +478,13 @@ public class JClass extends JStructure {
      * @return an array of JClass contained within this JClass
      */
     public JClass[] getInnerClasses() {
-        int size = _innerClasses.size();
-        JClass[] carray = new JClass[size];
-        _innerClasses.copyInto(carray);
-        return carray;
+        if (_innerClasses != null) {
+            int size = _innerClasses.size();
+            JClass[] carray = new JClass[size];
+            _innerClasses.copyInto(carray);
+            return carray;
+        }
+        return new JClass[0];
     } //-- getInnerClasses;
     
     /**
@@ -493,7 +545,8 @@ public class JClass extends JStructure {
      * @return superClass the super Class that this Class extends
      */
     public String getSuperClass() {
-        return _superClass;
+        if (_superClass == null) return null;
+        return _superClass.getQualifiedName();
     } //-- getSuperClass
 
     /**
@@ -526,13 +579,13 @@ public class JClass extends JStructure {
             
             //-- get imports from inner-classes
             Vector removeImports = null;
-            if (_innerClasses.size() > 0) {
+            if ((_innerClasses != null) && (_innerClasses.size() > 0)) {
                 removeImports = new Vector();
                 for (int i = 0; i < _innerClasses.size(); i++) {
                     JClass iClass = (JClass)_innerClasses.elementAt(i);
-                    Enumeration enum = iClass.getImports();
-                    while (enum.hasMoreElements()) {
-                        String classname = (String)enum.nextElement();
+                    Enumeration enumeration = iClass.getImports();
+                    while (enumeration.hasMoreElements()) {
+                        String classname = (String)enumeration.nextElement();
                         if (!hasImport(classname)) {
                             addImport(classname);
                             removeImports.addElement(classname);
@@ -562,6 +615,9 @@ public class JClass extends JStructure {
 
         buffer.setLength(0);
 
+		//-- print annotations
+		getAnnotatedElementHelper().printAnnotations(jsw);
+				
         JModifiers modifiers = getModifiers();
         if (modifiers.isPrivate()) {
             buffer.append("private ");
@@ -591,10 +647,10 @@ public class JClass extends JStructure {
             }
             buffer.append("implements ");
             
-            Enumeration enum = getInterfaces();
-            while (enum.hasMoreElements()) {
-                buffer.append(enum.nextElement());
-                if (enum.hasMoreElements()) buffer.append(", ");
+            Enumeration enumeration = getInterfaces();
+            while (enumeration.hasMoreElements()) {
+                buffer.append(enumeration.nextElement());
+                if (enumeration.hasMoreElements()) buffer.append(", ");
             }
             if (endl) {
                 jsw.writeln(buffer.toString());
@@ -628,6 +684,9 @@ public class JClass extends JStructure {
             JDocComment comment = jField.getComment();
             if (comment != null) comment.print(jsw);
 
+			//-- print Annotations
+			jField.printAnnotations(jsw);
+			
             // -- print member
             jsw.write(jField.getModifiers().toString());
             jsw.write(' ');
@@ -694,17 +753,19 @@ public class JClass extends JStructure {
         }
 
         //-- print inner-classes
-        if (_innerClasses.size() > 0) {
+        if ((_innerClasses != null) && (_innerClasses.size() > 0)) {
             jsw.writeln();
             jsw.writeln("  //-----------------/");
             jsw.writeln(" //- Inner Classes -/");
             jsw.writeln("//-----------------/");
             jsw.writeln();
-        }
-        for (int i = 0; i < _innerClasses.size(); i++) {
-            JClass jClass = (JClass)_innerClasses.elementAt(i);
-            jClass.print(jsw, true);
-            jsw.writeln();
+        
+            for (int i = 0; i < _innerClasses.size(); i++) {
+                JClass jClass = (JClass)_innerClasses.elementAt(i);
+                jClass.print(jsw, true);
+                jsw.writeln();
+            }
+            
         }
         
         jsw.unindent();
@@ -721,13 +782,26 @@ public class JClass extends JStructure {
     public boolean removeConstructor(JConstructor constructor) {
         return _constructors.removeElement(constructor);
     } //-- removeConstructor
+
+
+	/**
+	 * Removes the given method from this JClass
+	 *
+	 * @param method the JMethod to remove
+	 * @return true if the method was removed, otherwise false.
+	 */
+	public boolean removeMethod(JMethod method) 
+	{
+		return _methods.removeElement(method);
+	} //-- removeMethod
     
     /**
      * Removes the field with the given name from this JClass
      *
      * @param name the name of the field to remove
     **/
-    public JField removeField(String name) {
+    public JField removeField(String name) 
+	{
         if (name == null) return null;
         
         JField field = (JField) _fields.remove(name);
@@ -763,7 +837,10 @@ public class JClass extends JStructure {
      * @return true if the JClass was removed, otherwise false.
      */
     public boolean removeInnerClass(JClass jClass) {
-        return _innerClasses.removeElement(jClass);
+        if (_innerClasses != null) {
+            return _innerClasses.removeElement(jClass);
+        }
+        return false;
     } //-- removeInnerClass
 
     /**
@@ -771,7 +848,10 @@ public class JClass extends JStructure {
      * @param superClass the super Class that this Class extends
      */
     public void setSuperClass(String superClass) {
-        _superClass = superClass;
+        if (superClass == null)
+            _superClass = null;
+        else
+            _superClass = new JTypeName(superClass);
     } //-- setSuperClass
 
 
