@@ -42,21 +42,20 @@
  *
  * Contributors:
  * --------------
- * Gary Shea (shea@gtsdesign.com)   - Original Author
- * Keith Visco (kvisco@intalio.com) - Changed JCompElement references to
- *                                    JStructure, some additional tweaking 
- *                                    to get it working with the current 
- *                                    Javasource package.
+ * Gary Shea (shea AT gtsdesign DOT com)   
+ *    - Original Author
+ * 
+ * Keith Visco 
+ *   - Changed JCompElement references to JStructure, some additional
+ *     tweaking to get it working with the current Javasource package.
  *
- * $Id: JCompUnit.java,v 1.1.1.1 2003/03/03 07:09:53 kvisco Exp $
+ * $Id: JCompUnit.java,v 1.4 2005/03/05 13:41:59 mfuchs Exp $
  */
 
 package org.exolab.javasource;
 
 import java.io.File;
 import java.io.FileWriter;
-import java.io.Writer;
-import java.io.PrintWriter;
 
 import java.util.Vector;
 import java.util.Enumeration;
@@ -70,9 +69,10 @@ import java.util.Iterator;
  * a useful utility when creating in memory source code.
  * This package was modelled after the Java Reflection API
  * as much as possible to reduce the learning curve.
- * @author <a href="mailto:shea@gtsdesign.com">Gary Shea</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:09:53 $
-**/
+ * 
+ * @author <a href="mailto:shea AT gtsdesign DOT com">Gary Shea</a>
+ * @version $Revision: 1.4 $ $Date: 2005/03/05 13:41:59 $
+ */
 public class JCompUnit {
 
     /**
@@ -229,17 +229,17 @@ public class JCompUnit {
         for (int i = 0; i < classes.size(); ++i) {
             JClass jClass = (JClass) classes.get(i);
             
-            Enumeration enum = jClass.getImports();
-            while (enum.hasMoreElements()) {
-                allImports.add(enum.nextElement());
+            Enumeration enumeration = jClass.getImports();
+            while (enumeration.hasMoreElements()) {
+                allImports.add(enumeration.nextElement());
             }
         }
         
         for (int i = 0; i < interfaces.size(); ++i) {
             JInterface jInterface = (JInterface) interfaces.get(i);
-            Enumeration enum = jInterface.getImports();
-            while (enum.hasMoreElements()) {
-                allImports.add(enum.nextElement());
+            Enumeration enumeration = jInterface.getImports();
+            while (enumeration.hasMoreElements()) {
+                allImports.add(enumeration.nextElement());
             }
         }
         
@@ -301,7 +301,7 @@ public class JCompUnit {
     /**
      * Prints the source code for this JClass in the current directory 
      * with the default line seperator of the the runtime platform.
-     * @see print(String, String)
+     * @see #print(String, String)
     **/
     public void print() {
         print(null,null);
@@ -311,7 +311,7 @@ public class JCompUnit {
      * Prints the source code for this JClass
      * with the default line seperator of the the runtime platform.
      * @param destDir the destination directory to generate the file.
-     * @see print(String, String)
+     * @see #print(String, String)
     **/
     public void print(String destDir) {
         print(destDir, null);
