@@ -40,16 +40,13 @@
  *
  * Copyright 2001-2002 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: JStructure.java,v 1.1.1.1 2003/03/03 07:09:57 kvisco Exp $
+ * $Id: JStructure.java,v 1.5 2005/03/05 13:41:59 mfuchs Exp $
  */
 
 package org.exolab.javasource;
 
 import java.io.File;
 import java.io.FileWriter;
-import java.io.Writer;
-import java.io.PrintWriter;
-
 import java.util.Enumeration;
 import java.util.Vector;
 
@@ -61,11 +58,13 @@ import java.util.Vector;
  * The code in this package was modelled after the Java Reflection API
  * as much as possible to reduce the learning curve.
  *
- * @author <a href="mailto:skopp@riege.de">Martin Skopp</a> 
- * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:09:57 $
+ * @author <a href="mailto:skopp AT riege DOT de">Martin Skopp</a> 
+ * @author <a href="mailto:keith AT kvisco DOT com">Keith Visco</a>
+ * @version $Revision: 1.5 $ $Date: 2005/03/05 13:41:59 $
  */
-abstract class JStructure extends JType {
+abstract class JStructure extends JType
+	implements JAnnotatedElement 
+{
 
     /**
      * The Id for Source control systems
@@ -117,6 +116,11 @@ abstract class JStructure extends JType {
      * The package to which this JStructure belongs
      */
     private String packageName   = null;
+    
+    /**
+     * Implementation of JAnnoatedElement to delagate to
+     */
+    private JAnnotatedElementHelper annotatedElement = null;
 
     /**
      * Creates a new JStructure with the given name.
@@ -148,6 +152,7 @@ abstract class JStructure extends JType {
         jdc              = new JDocComment();
         methods          = new Vector();
         modifiers        = new JModifiers();
+		annotatedElement = new JAnnotatedElementHelper();
         //-- initialize default Java doc
         jdc.addDescriptor(JDocDescriptor.createVersionDesc(version));
         
@@ -189,7 +194,7 @@ abstract class JStructure extends JType {
     /**
      * Adds the given import to this JStructure
      *
-     * @param the className of the class to import.
+     * @param className name of the class to import.
      */
     public void addImport(String className) {
         if (className == null) return;
@@ -220,6 +225,25 @@ abstract class JStructure extends JType {
             imports.addElement(className);
         }
     } //-- addImport
+    
+    /**
+     * Adds approprite import for this JAnnotation
+     * @param annotation
+     */
+    protected void addImport(JAnnotation annotation)
+    {
+    	addImport(annotation.getAnnotationType().getName());
+    }
+    
+	/**
+	 * Adds approprite imports for these JAnnotation's
+	 * @param annotations
+	 */
+	protected void addImport(JAnnotation[] annotations)
+	{
+		for(int i=0; i<annotations.length; i++)
+			addImport(annotations[i].getAnnotationType().getName());			
+	}
 
     /**
      * Adds the given interface to the list of interfaces this
@@ -411,6 +435,14 @@ abstract class JStructure extends JType {
         return jdc;
     } //-- getJDocComment
 
+	/**
+	 * Returns the object managing the annotations for this JStructure
+	 */
+	protected JAnnotatedElementHelper getAnnotatedElementHelper()
+	{
+		return annotatedElement;	
+	} //-- getAnnotatedElementHelper
+	
     /**
      * Returns an array of all the JMethodSignatures of this JInterface.
      *
@@ -621,10 +653,10 @@ abstract class JStructure extends JType {
             jsw.writeln(" //- Imported classes and packages -/");
             jsw.writeln("//---------------------------------/");
             jsw.writeln();
-            Enumeration enum = imports.elements();
-            while (enum.hasMoreElements()) {
+            Enumeration enumeration = imports.elements();
+            while (enumeration.hasMoreElements()) {
                 jsw.write("import ");
-                jsw.write(enum.nextElement());
+                jsw.write(enumeration.nextElement());
                 jsw.writeln(';');
             }
             jsw.writeln();
@@ -764,6 +796,48 @@ abstract class JStructure extends JType {
         changePackage(packageName);
     } //-- setPackageName
 
+	/* (non-Javadoc)
+	 * @see org.exolab.javasource.JAnnotatedElement#addAnnotation(org.exolab.javasource.JAnnotation)
+	 */
+	public void addAnnotation(JAnnotation annotation) {
+		annotatedElement.addAnnotation(annotation);	
+		addImport(annotation);
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.exolab.javasource.JAnnotatedElement#getAnnotation(org.exolab.javasource.JAnnotationType)
+	 */
+	public JAnnotation getAnnotation(JAnnotationType annotationType) {
+		return annotatedElement.getAnnotation(annotationType);
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.exolab.javasource.JAnnotatedElement#getAnnotations()
+	 */
+	public JAnnotation[] getAnnotations() {
+		return annotatedElement.getAnnotations();
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.exolab.javasource.JAnnotatedElement#isAnnotationPresent(org.exolab.javasource.JAnnotation)
+	 */
+	public boolean isAnnotationPresent(JAnnotationType annotationType) {
+		return annotatedElement.isAnnotationPresent(annotationType);
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.exolab.javasource.JAnnotatedElement#removeAnnotation(org.exolab.javasource.JAnnotation)
+	 */
+	public JAnnotation removeAnnotation(JAnnotationType annotationType) {
+		return annotatedElement.removeAnnotation(annotationType);
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.exolab.javasource.JAnnotatedElement#hasAnnotations()
+	 */
+	public boolean hasAnnotations() {
+		return annotatedElement.hasAnnotations();
+	}	
 
     //---------------------/
     //- Protected Methods -/
