@@ -40,38 +40,21 @@
  *
  * Copyright 1999 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: ManyToMany.java,v 1.1.1.1 2003/03/03 07:10:05 kvisco Exp $
+ * $Id: ManyToMany.java,v 1.5 2005/07/05 13:25:40 rjoachim Exp $
  */
 
 
 package jdo;
 
 
-import java.io.IOException;
-import java.util.Enumeration;
 import java.util.Iterator;
 import java.util.Collection;
-import java.sql.Connection;
-import java.sql.SQLException;
-import java.sql.ResultSet;
-import java.lang.Math;
-import java.util.Vector;
-import java.util.Random;
 import org.exolab.castor.jdo.Database;
 import org.exolab.castor.jdo.OQLQuery;
 import org.exolab.castor.jdo.QueryResults;
 import org.exolab.castor.jdo.PersistenceException;
-import org.exolab.castor.jdo.QueryException;
-import org.exolab.castor.jdo.LockNotGrantedException;
-import org.exolab.castor.jdo.TransactionAbortedException;
-import org.exolab.castor.jdo.TransactionNotInProgressException;
-import org.exolab.castor.jdo.ObjectModifiedException;
-import org.exolab.castor.jdo.DuplicateIdentityException;
 import java.util.ArrayList;
 
-import junit.framework.TestSuite;
-import junit.framework.TestCase;
-import junit.framework.Assert;
 import harness.TestHarness;
 import harness.CastorTestCase;
 
@@ -92,14 +75,14 @@ public class ManyToMany extends CastorTestCase {
      * @param category The test suite of these tests
      */
     public ManyToMany( TestHarness category ) {
-        super( category, "TC23", "ManyToMany" );
+        super( category, "TC123", "ManyToMany" );
         _category = (JDOCategory) category;
     }
 
 
     public void setUp() 
             throws PersistenceException {
-        _db = _category.getDatabase( verbose );
+        _db = _category.getDatabase();
     }
 
     public void runTest() 
@@ -114,7 +97,7 @@ public class ManyToMany extends CastorTestCase {
         ArrayList al, bl;
         OQLQuery oql;
         OQLQuery oqlp;
-        QueryResults enum;
+        QueryResults enumeration;
         int groupAId = 201, groupBId = 202;
         int person1Id = 1, person2Id = 2, person3Id = 3, person4Id = 4;
 
@@ -126,9 +109,9 @@ public class ManyToMany extends CastorTestCase {
         // select an group and delete it, if it exist!
         OQLQuery oqlclean = _db.getOQLQuery( "SELECT object FROM jdo.TestManyGroup object WHERE object.id < $1" );
         oqlclean.bind( Integer.MAX_VALUE );
-        enum = oqlclean.execute();
-        while ( enum.hasMore() ) {
-            groupA = (TestManyGroup) enum.next();
+        enumeration = oqlclean.execute();
+        while ( enumeration.hasMore() ) {
+            groupA = (TestManyGroup) enumeration.next();
             stream.println( "Retrieved object: " + groupA );
             _db.remove( groupA );
             stream.println( "Deleted object: " + groupA );
@@ -138,9 +121,9 @@ public class ManyToMany extends CastorTestCase {
         _db.begin();
         oqlclean = _db.getOQLQuery( "SELECT object FROM jdo.TestManyPerson object WHERE object.id < $1" );
         oqlclean.bind( Integer.MAX_VALUE );
-        enum = oqlclean.execute();
-        while ( enum.hasMore() ) {
-            person1 = (TestManyPerson) enum.next();
+        enumeration = oqlclean.execute();
+        while ( enumeration.hasMore() ) {
+            person1 = (TestManyPerson) enumeration.next();
             stream.println( "Retrieved object: " + person1 );
             _db.remove( person1 );
             stream.println( "Deleted object: " + person1 );
@@ -231,9 +214,9 @@ public class ManyToMany extends CastorTestCase {
         groupA = null;
         person1 = null;
         person2 = null;
-        enum = oql.execute();
-        if ( enum.hasMore() ) {
-            groupA = (TestManyGroup) enum.next();
+        enumeration = oql.execute();
+        if ( enumeration.hasMore() ) {
+            groupA = (TestManyGroup) enumeration.next();
             stream.println( "Retrieved object: " + groupA );
             Collection p = groupA.getPeople();
             if ( p != null ) {
@@ -320,9 +303,9 @@ public class ManyToMany extends CastorTestCase {
         groupA = null;
         person1 = null;
         person2 = null;
-        enum = oql.execute();
-        if ( enum.hasMore() ) {
-            groupA = (TestManyGroup) enum.next();
+        enumeration = oql.execute();
+        if ( enumeration.hasMore() ) {
+            groupA = (TestManyGroup) enumeration.next();
             stream.println( "Retrieved object: " + groupA );
             Collection p = groupA.getPeople();
             if ( p != null ) {
@@ -479,9 +462,9 @@ public class ManyToMany extends CastorTestCase {
         groupA = null;
         person1 = null;
         person2 = null;
-        enum = oql.execute();
-        if ( enum.hasMore() ) {
-            groupA = (TestManyGroup) enum.next();
+        enumeration = oql.execute();
+        if ( enumeration.hasMore() ) {
+            groupA = (TestManyGroup) enumeration.next();
             stream.println( "Retrieved object: " + groupA );
             Collection p = groupA.getPeople();
             if ( p != null ) {
