@@ -40,37 +40,21 @@
  *
  * Copyright 1999 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: ManyToManyKeyGen.java,v 1.1.1.1 2003/03/03 07:10:06 kvisco Exp $
+ * $Id: ManyToManyKeyGen.java,v 1.5 2005/05/11 17:18:49 rjoachim Exp $
  */
 
 
 package jdo;
 
 
-import java.io.IOException;
-import java.util.Enumeration;
 import java.util.Iterator;
 import java.util.Collection;
 import java.util.ArrayList;
-import java.util.Vector;
-import java.util.Random;
-import java.sql.Connection;
-import java.sql.SQLException;
-import java.sql.ResultSet;
 import org.exolab.castor.jdo.Database;
 import org.exolab.castor.jdo.OQLQuery;
 import org.exolab.castor.jdo.QueryResults;
 import org.exolab.castor.jdo.PersistenceException;
-import org.exolab.castor.jdo.QueryException;
-import org.exolab.castor.jdo.LockNotGrantedException;
-import org.exolab.castor.jdo.TransactionAbortedException;
-import org.exolab.castor.jdo.TransactionNotInProgressException;
-import org.exolab.castor.jdo.ObjectModifiedException;
-import org.exolab.castor.jdo.DuplicateIdentityException;
 
-import junit.framework.TestSuite;
-import junit.framework.TestCase;
-import junit.framework.Assert;
 import harness.TestHarness;
 import harness.CastorTestCase;
 
@@ -96,7 +80,7 @@ public class ManyToManyKeyGen extends CastorTestCase {
      * @param category The test suite of these test cases.
      */
     public ManyToManyKeyGen( TestHarness category ) {
-        super( category, "TC31", "ManyToManyKeyGen" );
+        super( category, "TC31a", "ManyToManyKeyGen" );
         _category = (JDOCategory) category;
     }
 
@@ -105,7 +89,7 @@ public class ManyToManyKeyGen extends CastorTestCase {
      */
     public void setUp() 
             throws PersistenceException {
-        _db = _category.getDatabase( verbose );
+        _db = _category.getDatabase();
     }
 
     /**
@@ -127,7 +111,7 @@ public class ManyToManyKeyGen extends CastorTestCase {
         ArrayList al, bl;
         OQLQuery oql;
         OQLQuery oqlp;
-        QueryResults enum;
+        QueryResults enumeration;
         int groupAId, groupBId;
         int person1Id, person2Id, person3Id, person4Id;
 
@@ -140,9 +124,9 @@ public class ManyToManyKeyGen extends CastorTestCase {
         // select an group and delete it, if it exist!
         OQLQuery oqlclean = _db.getOQLQuery( "SELECT object FROM jdo.TestManyGroupKeyGen object WHERE object.id < $1" );
         oqlclean.bind( Integer.MAX_VALUE );
-        enum = oqlclean.execute();
-        while ( enum.hasMore() ) {
-            groupA = (TestManyGroupKeyGen) enum.next();
+        enumeration = oqlclean.execute();
+        while ( enumeration.hasMore() ) {
+            groupA = (TestManyGroupKeyGen) enumeration.next();
             stream.println( "Retrieved object: " + groupA );
             _db.remove( groupA );
             stream.println( "Deleted object: " + groupA );
@@ -152,9 +136,9 @@ public class ManyToManyKeyGen extends CastorTestCase {
         _db.begin();
         oqlclean = _db.getOQLQuery( "SELECT object FROM jdo.TestManyPersonKeyGen object WHERE object.id < $1" );
         oqlclean.bind( Integer.MAX_VALUE );
-        enum = oqlclean.execute();
-        while ( enum.hasMore() ) {
-            person1 = (TestManyPersonKeyGen) enum.next();
+        enumeration = oqlclean.execute();
+        while ( enumeration.hasMore() ) {
+            person1 = (TestManyPersonKeyGen) enumeration.next();
             stream.println( "Retrieved object: " + person1 );
             _db.remove( person1 );
             stream.println( "Deleted object: " + person1 );
@@ -246,9 +230,9 @@ public class ManyToManyKeyGen extends CastorTestCase {
         groupA = null;
         person1 = null;
         person2 = null;
-        enum = oql.execute();
-        if ( enum.hasMore() ) {
-            groupA = (TestManyGroupKeyGen) enum.next();
+        enumeration = oql.execute();
+        if ( enumeration.hasMore() ) {
+            groupA = (TestManyGroupKeyGen) enumeration.next();
             stream.println( "Retrieved object: " + groupA );
             Collection p = groupA.getPeople();
             if ( p != null ) {
@@ -335,9 +319,9 @@ public class ManyToManyKeyGen extends CastorTestCase {
         groupA = null;
         person1 = null;
         person2 = null;
-        enum = oql.execute();
-        if ( enum.hasMore() ) {
-            groupA = (TestManyGroupKeyGen) enum.next();
+        enumeration = oql.execute();
+        if ( enumeration.hasMore() ) {
+            groupA = (TestManyGroupKeyGen) enumeration.next();
             stream.println( "Retrieved object: " + groupA );
             Collection p = groupA.getPeople();
             if ( p != null ) {
@@ -494,9 +478,9 @@ public class ManyToManyKeyGen extends CastorTestCase {
         groupA = null;
         person1 = null;
         person2 = null;
-        enum = oql.execute();
-        if ( enum.hasMore() ) {
-            groupA = (TestManyGroupKeyGen) enum.next();
+        enumeration = oql.execute();
+        if ( enumeration.hasMore() ) {
+            groupA = (TestManyGroupKeyGen) enumeration.next();
             stream.println( "Retrieved object: " + groupA );
             Collection p = groupA.getPeople();
             if ( p != null ) {
